/*
 * Copyright (C) 2001-2006 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#if !defined(FAVORITE_MANAGER_H)
#define FAVORITE_MANAGER_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "SettingsManager.h"

#include "CriticalSection.h"
#include "HttpConnection.h"
#include "User.h"
#include "UserCommand.h"
#include "FavoriteUser.h"
#include "Singleton.h"
#include "ClientManagerListener.h"
#include "ClientManager.h"

class HubEntry {
public:
        typedef vector<HubEntry> List;
        typedef List::const_iterator Iter;

        HubEntry(const string& aName, const string& aServer, const string& aDescription, const string& aUsers) throw() :
        name(aName), server(aServer), description(aDescription), country(Util::emptyString),
        rating(Util::emptyString), reliability(0.0), shared(0), minShare(0), users(Util::toInt(aUsers)), minSlots(0), maxHubs(0), maxUsers(0) { }

        HubEntry(const string& aName, const string& aServer, const string& aDescription, const string& aUsers, const string& aCountry,
                const string& aShared, const string& aMinShare, const string& aMinSlots, const string& aMaxHubs, const string& aMaxUsers,
                const string& aReliability, const string& aRating) : name(aName), server(aServer), description(aDescription), country(aCountry),
                rating(aRating), reliability((float)(Util::toFloat(aReliability) / 100.0)), shared(Util::toInt64(aShared)), minShare(Util::toInt64(aMinShare)),
                users(Util::toInt(aUsers)), minSlots(Util::toInt(aMinSlots)), maxHubs(Util::toInt(aMaxHubs)), maxUsers(Util::toInt(aMaxUsers))
        {

        }

        HubEntry() throw() { }
        HubEntry(const HubEntry& rhs) throw() : name(rhs.name), server(rhs.server), description(rhs.description), country(rhs.country),
                rating(rhs.rating), reliability(rhs.reliability), shared(rhs.shared), minShare(rhs.minShare), users(rhs.users), minSlots(rhs.minSlots),
                maxHubs(rhs.maxHubs), maxUsers(rhs.maxUsers) { }

        ~HubEntry() throw() { }

        GETSET(string, name, Name);
        GETSET(string, server, Server);
        GETSET(string, description, Description);
        GETSET(string, country, Country);
        GETSET(string, rating, Rating);
        GETSET(float, reliability, Reliability);
        GETSET(int64_t, shared, Shared);
        GETSET(int64_t, minShare, MinShare);
        GETSET(int, users, Users);
        GETSET(int, minSlots, MinSlots);
        GETSET(int, maxHubs, MaxHubs)
        GETSET(int, maxUsers, MaxUsers);
};

const string DEF_FAKE_ID = "FakeDC V:1.0"; // !SMT!-S

class FavoriteHubEntry {
public:
        typedef FavoriteHubEntry* Ptr;
        typedef vector<Ptr> List;
        typedef List::const_iterator Iter;

        FavoriteHubEntry() throw() : connect(false), windowposx(0), windowposy(0), windowsizex(0),
                windowsizey(0), windowtype(0), chatusersplit(0), userliststate(true), hideShare(false), showJoins(false), exclChecks(false), mode(0), ip(Util::emptyString), overrideId(0), clientId(DEF_FAKE_ID) { stealth = BOOLSETTING(EMULATE_DCPP); } // !SMT!-S
        FavoriteHubEntry(const HubEntry& rhs) throw() : name(rhs.getName()), server(rhs.getServer()),
                description(rhs.getDescription()), connect(false), windowposx(0), windowposy(0), windowsizex(0),
                windowsizey(0), windowtype(0), chatusersplit(0), stealth(false), userliststate(true), hideShare(false), showJoins(false), exclChecks(false), mode(0), ip(Util::emptyString), overrideId(0), clientId(DEF_FAKE_ID)  { } // !SMT!-S
        FavoriteHubEntry(const FavoriteHubEntry& rhs) throw() : userdescription(rhs.userdescription), email(rhs.email), awaymsg(rhs.awaymsg),
                name(rhs.getName()), server(rhs.getServer()), description(rhs.getDescription()), password(rhs.getPassword()), connect(rhs.getConnect()),
                nick(rhs.nick), windowposx(rhs.windowposx), windowposy(rhs.windowposy), windowsizex(rhs.windowsizex),
                windowsizey(rhs.windowsizey), windowtype(rhs.windowtype), chatusersplit(rhs.chatusersplit), stealth(rhs.stealth),
                userliststate(rhs.userliststate), hideShare(rhs.hideShare), showJoins(rhs.showJoins), exclChecks(rhs.exclChecks), mode(rhs.mode), ip(rhs.ip),
                opChat(rhs.opChat), rawOne(rhs.rawOne), rawTwo(rhs.rawTwo), rawThree(rhs.rawThree), rawFour(rhs.rawFour), rawFive(rhs.rawFive), overrideId(rhs.overrideId), clientId(rhs.clientId) { }
        ~FavoriteHubEntry() throw() { }

        const string& getNick(bool useDefault = true) const {
                return (!nick.empty() || !useDefault) ? nick : SETTING(NICK);
        }

        void setNick(const string& aNick) { nick = aNick; }

        GETSET(string, userdescription, UserDescription);
        GETSET(string, awaymsg, AwayMsg);
        GETSET(string, email, Email);
        GETSET(string, name, Name);
        GETSET(string, server, Server);
        GETSET(string, description, Description);
        GETSET(string, password, Password);
        GETSET(string, headerOrder, HeaderOrder);
        GETSET(string, headerWidths, HeaderWidths);
        GETSET(string, headerVisible, HeaderVisible);
        GETSET(bool, connect, Connect);
        GETSET(int, windowposx, WindowPosX);
        GETSET(int, windowposy, WindowPosY);
        GETSET(int, windowsizex, WindowSizeX);
        GETSET(int, windowsizey, WindowSizeY);
        GETSET(int, windowtype, WindowType);
        GETSET(int, chatusersplit, ChatUserSplit);
        GETSET(bool, stealth, Stealth);
        GETSET(bool, userliststate, UserListState);
        GETSET(bool, hideShare, HideShare); // Hide Share Mod
        GETSET(bool, showJoins, ShowJoins); // Show joins
        GETSET(bool, exclChecks, ExclChecks); // Excl. from client checking
        GETSET(string, rawOne, RawOne);
        GETSET(string, rawTwo, RawTwo);
        GETSET(string, rawThree, RawThree);
        GETSET(string, rawFour, RawFour);
        GETSET(string, rawFive, RawFive);
        GETSET(int, mode, Mode); // 0 = default, 1 = active, 2 = passive
        GETSET(string, ip, IP);
        GETSET(string, opChat, OpChat);
        GETSET(string, clientId, ClientId); // !SMT!-S
        GETSET(bool, overrideId, OverrideId); // !SMT!-S

private:
        string nick;
};

class RecentHubEntry {
public:
        typedef RecentHubEntry* Ptr;
        typedef vector<Ptr> List;
        typedef List::const_iterator Iter;

        ~RecentHubEntry() throw() { }

        GETSET(string, name, Name);
        GETSET(string, server, Server);
        GETSET(string, description, Description);
        GETSET(string, users, Users);
        GETSET(string, shared, Shared);
};

class PreviewApplication {
public:
        typedef PreviewApplication* Ptr;
        typedef vector<Ptr> List;
        typedef List::const_iterator Iter;

        PreviewApplication() throw() {}
        PreviewApplication(string n, string a, string r, string e) : name(n), application(a), arguments(r), extension(e) {};
        ~PreviewApplication() throw() { }

        GETSET(string, name, Name);
        GETSET(string, application, Application);
        GETSET(string, arguments, Arguments);
        GETSET(string, extension, Extension);
};

class FavoriteManagerListener {
public:
        virtual ~FavoriteManagerListener() { }
        template<int I> struct X { enum { TYPE = I };  };

        typedef X<0> DownloadStarting;
        typedef X<1> DownloadFailed;
        typedef X<2> DownloadFinished;
        typedef X<3> FavoriteAdded;
        typedef X<4> FavoriteRemoved;
        typedef X<5> UserAdded;
        typedef X<6> UserRemoved;
        typedef X<7> StatusChanged;
        typedef X<8> RecentAdded;
        typedef X<9> RecentRemoved;
        typedef X<10> RecentUpdated;
        typedef X<11> FavoriteUpdated;
        typedef X<12> FavoriteUserUpdated;

        virtual void on(DownloadStarting, const string&) throw() { }
        virtual void on(DownloadFailed, const string&) throw() { }
        virtual void on(DownloadFinished, const string&) throw() { }
        virtual void on(FavoriteAdded, const FavoriteHubEntry*) throw() { }
        virtual void on(FavoriteRemoved, const FavoriteHubEntry*) throw() { }
        virtual void on(UserAdded, const FavoriteUser&) throw() { }
        virtual void on(UserRemoved, const FavoriteUser&) throw() { }
        virtual void on(StatusChanged, const User::Ptr&) throw() { }
        virtual void on(RecentAdded, const RecentHubEntry*) throw() { }
        virtual void on(RecentRemoved, const RecentHubEntry*) throw() { }
        virtual void on(RecentUpdated, const RecentHubEntry*) throw() { }
        virtual void on(FavoriteUpdated) throw() { }
        virtual void on(FavoriteUserUpdated) throw() { }

};

class SimpleXML;

/**
 * Public hub list, favorites (hub&user). Assumed to be called only by UI thread.
 */
class FavoriteManager : public Speaker<FavoriteManagerListener>, private HttpConnectionListener, public Singleton<FavoriteManager>,
        private SettingsManagerListener, private ClientManagerListener
{
public:
// Public Hubs
        enum HubTypes {
                TYPE_NORMAL,
                TYPE_BZIP2
        };
        StringList getHubLists();
        bool setHubList(int /*aHubList*/);
        unsigned int getSelectedHubList() { return lastServer; }
        void refresh();
        HubTypes getHubListType() { return listType; }
        HubEntry::List getPublicHubs() {
                Lock l(cs);
                return publicListMatrix[publicListServer];
        }
        bool isDownloading() { return running; }

// Favorite Users
        typedef HASH_MAP_X(CID, FavoriteUser, CID::Hash, equal_to<CID>, less<CID>) FavoriteMap;
        FavoriteMap getFavoriteUsers() { Lock l(cs); return users; }
        PreviewApplication::List& getPreviewApps() { return previewApplications; }

        void addFavoriteUser(User::Ptr& aUser);
        bool isFavoriteUser(const User::Ptr& aUser) const { Lock l(cs); return users.find(aUser->getCID()) != users.end(); }
        void removeFavoriteUser(User::Ptr& aUser);

        bool hasSlot(const User::Ptr& aUser) const;

        // !SMT!-S
        bool hasBan(const User::Ptr& aUser) const;
        void setUploadLimit(const User::Ptr& aUser, FavoriteUser::UPLOAD_LIMIT lim);
        bool getFlag(const User::Ptr& aUser, FavoriteUser::Flags) const;
        void setFlag(const User::Ptr& aUser, FavoriteUser::Flags, bool);
        bool hasIgnore(const User::Ptr& aUser) const { return getFlag(aUser, FavoriteUser::FLAG_IGNOREPRIVATE); }
        void setIgnorePrivate(const User::Ptr& aUser, bool grant) { setFlag(aUser, FavoriteUser::FLAG_IGNOREPRIVATE, grant); }
        bool hasFreePM(const User::Ptr& aUser) const { return getFlag(aUser, FavoriteUser::FLAG_FREE_PM_ACCESS); }
        void setFreePM(const User::Ptr& aUser, bool grant) { setFlag(aUser, FavoriteUser::FLAG_FREE_PM_ACCESS, grant); }

        void setUserDescription(const User::Ptr& aUser, const string& description);
        void setAutoGrant(const User::Ptr& aUser, bool grant);
        void setSuperUser(const User::Ptr& aUser, bool superUser);

        void userUpdated(const User::Ptr& info); // !SMT!-fix
        time_t getLastSeen(const User::Ptr& aUser) const;
// Favorite Hubs
        FavoriteHubEntry::List& getFavoriteHubs() { return favoriteHubs; }

        void addFavorite(const FavoriteHubEntry& aEntry);
        void removeFavorite(FavoriteHubEntry* entry);
        bool checkFavHubExists(const FavoriteHubEntry& aEntry);
        FavoriteHubEntry* getFavoriteHubEntry(const string& aServer);

// Favorite Directories
        bool addFavoriteDir(const string& aDirectory, const string& aName);
        bool removeFavoriteDir(const string& aName);
        bool renameFavoriteDir(const string& aName, const string& anotherName);
        StringPairList getFavoriteDirs() { return favoriteDirs; }

// Recent Hubs
        RecentHubEntry::List& getRecentHubs() { return recentHubs; };

        void addRecent(const RecentHubEntry& aEntry);
        void removeRecent(const RecentHubEntry* entry);
        void updateRecent(const RecentHubEntry* entry);

        RecentHubEntry* getRecentHubEntry(const string& aServer) {
                for(RecentHubEntry::Iter i = recentHubs.begin(); i != recentHubs.end(); ++i) {
                        RecentHubEntry* r = *i;
                        if(Util::stricmp(r->getServer(), aServer) == 0) {
                                return r;
                        }
                }
                return NULL;
        }

        PreviewApplication* addPreviewApp(string name, string application, string arguments, string extension){
                PreviewApplication* pa = new PreviewApplication(name, application, arguments, extension);
                previewApplications.push_back(pa);
                return pa;
        }

        PreviewApplication* removePreviewApp(unsigned int index){
                if(previewApplications.size() > index)
                        previewApplications.erase(previewApplications.begin() + index);
                return NULL;
        }

        PreviewApplication* getPreviewApp(unsigned int index, PreviewApplication &pa){
                if(previewApplications.size() > index)
                        pa = *previewApplications[index];
                return NULL;
        }

        PreviewApplication* updatePreviewApp(int index, PreviewApplication &pa){
                *previewApplications[index] = pa;
                return NULL;
        }

        void removeallRecent() {
                recentHubs.clear();
                recentsave();
        }

// User Commands
        UserCommand addUserCommand(int type, int ctx, int flags, const string& name, const string& command, const string& hub);
        bool getUserCommand(int cid, UserCommand& uc);
        int findUserCommand(const string& aName);
        bool moveUserCommand(int cid, int pos);
        void updateUserCommand(const UserCommand& uc);
        void removeUserCommand(int cid);
        void removeUserCommand(const string& srv);
        void removeHubUserCommands(int ctx, const string& hub);

        UserCommand::List getUserCommands() { Lock l(cs); return userCommands; }
        UserCommand::List getUserCommands(int ctx, const StringList& hub, bool& op);

        void load(bool reload=false,const string& file = Util::emptyString );
        void save();
        void recentsave();

		void addDefaultHub(const std::string& name);
		
		const StringList& getDefaultHubs();

private:
        FavoriteHubEntry::List favoriteHubs;
        StringPairList favoriteDirs;
        RecentHubEntry::List recentHubs;
		StringList defaultHubs;

        PreviewApplication::List previewApplications;
        UserCommand::List userCommands;

        int lastId;

        FavoriteMap users;

        mutable CriticalSection cs;

        // Public Hubs
        typedef map<string, HubEntry::List> PubListMap;
        PubListMap publicListMatrix;
        string publicListServer;
        bool running;
        HttpConnection* c;
        unsigned int lastServer;
        HubTypes listType;
        string downloadBuf;

        /** Used during loading to prevent saving. */
        bool dontSave;

        friend class Singleton<FavoriteManager>;

        FavoriteManager() : lastId(0), running(false), c(NULL), lastServer(0), listType(TYPE_NORMAL), dontSave(false) {
                SettingsManager::getInstance()->addListener(this);
                ClientManager::getInstance()->addListener(this);
        }

        virtual ~FavoriteManager() throw() {
                ClientManager::getInstance()->removeListener(this);
                SettingsManager::getInstance()->removeListener(this);
                if(c) {
                        c->removeListener(this);
                        delete c;
                        c = NULL;
                }

                for_each(favoriteHubs.begin(), favoriteHubs.end(), DeleteFunction());
                for_each(recentHubs.begin(), recentHubs.end(), DeleteFunction());
                for_each(previewApplications.begin(), previewApplications.end(), DeleteFunction());
        }

        FavoriteHubEntry::Iter getFavoriteHub(const string& aServer) {
                for(FavoriteHubEntry::Iter i = favoriteHubs.begin(); i != favoriteHubs.end(); ++i) {
                        if(Util::stricmp((*i)->getServer(), aServer) == 0) {
                                return i;
                        }
                }
                return favoriteHubs.end();
        }

        void loadXmlList(const string& xml);

        RecentHubEntry::Iter getRecentHub(const string& aServer) const {
                for(RecentHubEntry::Iter i = recentHubs.begin(); i != recentHubs.end(); ++i) {
                        if(Util::stricmp((*i)->getServer(), aServer) == 0) {
                                return i;
                        }
                }
                return recentHubs.end();
        }

        // ClientManagerListener
        virtual void on(UserUpdated, const User::Ptr& user) throw();
        virtual void on(UserConnected, const User::Ptr& user) throw();
        virtual void on(UserDisconnected, const User::Ptr& user) throw();

        // HttpConnectionListener
        virtual void on(Data, HttpConnection*, const uint8_t*, size_t) throw();
        virtual void on(Failed, HttpConnection*, const string&) throw();
        virtual void on(Complete, HttpConnection*, const string&) throw();
        virtual void on(Redirected, HttpConnection*, const string&) throw();
        virtual void on(TypeNormal, HttpConnection*) throw();
        virtual void on(TypeBZ2, HttpConnection*) throw();

        void onHttpFinished() throw();

        // SettingsManagerListener
        virtual void on(SettingsManagerListener::Load, SimpleXML& xml) throw() {
                load(xml);
                recentload(xml);
                previewload(xml);
        }

        virtual void on(SettingsManagerListener::Save, SimpleXML& xml) throw() {
                previewsave(xml);
        }

        void load(SimpleXML& aXml);
        void recentload(SimpleXML& aXml);
        void previewload(SimpleXML& aXml);
        void previewsave(SimpleXML& aXml);

        string getConfigFile() { return SETTING(FAVORITES_FILE); }
		
};

#endif // !defined(FAVORITE_MANAGER_H)

/**
 * @file
 * $Id: FavoriteManager.h,v 1.19 2006/11/05 15:21:01 bigmuscle Exp $
 */
