/* 
* Copyright (C) 2003 Twink, spm7@waikato.ac.nz
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "stdinc.h"
#include "WebServerManager.h"
#include "QueueManager.h"
#include "FinishedManager.h"
#include "LogManager.h"
#include "ClientManager.h"
#include "UploadManager.h"
#include "StringTokenizer.h"
#include "ResourceManager.h"
#include "../windows/resource.h"

WebServerManager* Singleton<WebServerManager>::instance = NULL;

//static const string WEBSERVER_AREA = "WebServer";

WebServerManager::WebServerManager(void) : started(false), page404(NULL) {
	SettingsManager::getInstance()->addListener(this);
	sended_search = false;
}

WebServerManager::~WebServerManager(void){
	SettingsManager::getInstance()->removeListener(this);
	if(started) Stop();
}

void WebServerManager::Start(){
	if(started)return;
	Lock l(cs);
	started = true;

	socket.listen((short)SETTING(WEBSERVER_PORT));
	socket.addListener(this);
	fire(WebServerListener::Setup());

    page404 = new WebPageInfo(PAGE_404,CSTRING(WS_TITLE_NOT_FOUND),false);
	pages["/style.css"] = new WebPageInfo(STYLE_SHEET,"",false,"text/css");
	pages["/"] = new WebPageInfo(INDEX, CSTRING(WS_TITLE_INDEX));
	pages["/index.html"] = new WebPageInfo(INDEX, CSTRING(WS_TITLE_INDEX));	
	pages["/dlqueue.html"] = new WebPageInfo(DOWNLOAD_QUEUE, CSTRING(WS_TITLE_DOWNLOAD_QUEUE));
	pages["/dlfinished.html"] = new WebPageInfo(DOWNLOAD_FINISHED, CSTRING(WS_TITLE_FINISHED_DOWNLOADS));
	pages["/ulqueue.html"] = new WebPageInfo(UPLOAD_QUEUE, CSTRING(WS_TITLE_UPLOAD_QUEUE));
	pages["/ulfinished.html"] = new WebPageInfo(UPLOAD_FINISHED, CSTRING(WS_TITLE_FINISHED_DOWNLOADS));
	pages["/weblog.html"] = new WebPageInfo(LOG, CSTRING(WS_TITLE_LOGS));
	pages["/syslog.html"] = new WebPageInfo(SYSLOG, CSTRING(WS_TITLE_SYSTEM_LOGS));
	pages["/search.html"] = new WebPageInfo(SEARCH, CSTRING(WS_TITLE_SEARCH));
	pages["/logout.html"] = new WebPageInfo(LOGOUT, CSTRING(WS_TITLE_LOGOUT));
    pages["/shutdown.html"] = new WebPageInfo(SHUTDOWN, CSTRING(WS_TITLE_SHUTDOWN));
    pages["/reboot.html"] = new WebPageInfo(REBOOT, CSTRING(WS_TITLE_REBOOT));
    pages["/suspend.html"] = new WebPageInfo(SUSPEND, CSTRING(WS_TITLE_SUSPEND));
    pages["/logoff.html"] = new WebPageInfo(LOGOFF, CSTRING(WS_TITLE_LOGOFF));
    pages["/switch.html"] = new WebPageInfo(SWITCH, CSTRING(WS_TITLE_SWITCH));
    pages["/images/logo.png"] = new WebPageInfo(IMAGE_LOGO,"",false,"image/png");
    pages["/images/Shutdown.gif"] = new WebPageInfo(IMAGE_SHUTDOWN,"",false,"image/gif");
    pages["/images/Logoff.gif"]   = new WebPageInfo(IMAGE_LOGOFF,"",false,"image/gif");
    pages["/images/LogoffOn.gif"] = new WebPageInfo(IMAGE_LOGOFFON,"",false,"image/gif");
    pages["/images/Reboot.gif"] = new WebPageInfo(IMAGE_REBOOT,"",false,"image/gif");
    pages["/images/RebootOn.gif"] = new WebPageInfo(IMAGE_REBOOTON,"",false,"image/gif");
    pages["/images/Suspend.gif"] = new WebPageInfo(IMAGE_SUSPEND,"",false,"image/gif");
    pages["/images/SuspendOn.gif"] = new WebPageInfo(IMAGE_SUSPENDON,"",false,"image/gif");
    pages["/images/Switch.gif"] = new WebPageInfo(IMAGE_SWITCH,"",false,"image/gif");
    pages["/images/SwitchOn.gif"] = new WebPageInfo(IMAGE_SWITCHON,"",false,"image/gif");

#ifdef _DEBUG  
	//AllocConsole();
	//freopen("con:","w",stdout);
#endif
}

string WebServerManager::getResourceData(int id){
    HINSTANCE hExe = GetModuleHandle(NULL); 
    HRSRC hrInfo = FindResource(hExe, MAKEINTRESOURCE(id), TEXT("WEBSERVER"));    
    HANDLE hr = LoadResource(hExe, hrInfo);
    int datasize = SizeofResource(hExe,hrInfo);
    char* data = (char*) LockResource(hr);
    string str(data,datasize);
    FreeResource(hr);
    FreeLibrary(hExe);
    return str;
}

void WebServerManager::Stop(){
	if(!started)return;
	started = false;
	Lock l(cs);

	delete page404;
	page404 = NULL;

	for(WebPages::iterator p = pages.begin(); p != pages.end(); ++p){
			if(p->second != NULL){
				delete p->second;
				p->second = NULL;

			}
	}

#ifdef _DEBUG 
	//FreeConsole();
#endif
	socket.removeListener(this);
	socket.disconnect();
}

void WebServerManager::on(ServerSocketListener::IncomingConnection) throw() {
	WebServerSocket *wss = new WebServerSocket();
	wss->accept(&socket);	
	wss->start();
}


WebServerManager::WebPageInfo* WebServerManager::getWebPageInfo(string file){
    WebPageInfo* page = page404;
    WebPages::const_iterator find = pages.find(file.c_str());
    if(find != pages.end()) {
        page = find->second;
    }
    return page;
}

string WebServerManager::getLoginPage(){
	string pagehtml = "";
    pagehtml = "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">";
	pagehtml += "<html xmlns='http://www.w3.org/1999/xhtml' xml:lang='en'>";
	pagehtml += "<head>";
    pagehtml += Text::format("    	<title>SabababDC - %s </title>",CSTRING(WS_TITLE_LOGIN));
	pagehtml += "		<meta http-equiv='Content-Type' content='text/html; charset=utf-8' />";
    pagehtml += "		<meta http-equiv='pragma' content='no-cache'>";
    pagehtml += "   	<meta http-equiv='cache-control' content='no-cache, must-revalidate'>";
	pagehtml += "		<link rel='stylesheet' href='/style.css' type='text/css' title='Default style' media='screen' />";
    pagehtml += "</head>";
    pagehtml += "<body>";
    pagehtml += "<table width='100%'>";
    pagehtml += "<tr valign='top'><td>";
    pagehtml += "<div id='index_obsah'>";
    pagehtml += "<h1>SababaDC</h1>";
    pagehtml += "<div id='index_logo'></div>";
    pagehtml += "	<div id='login'>";
    pagehtml += "		<form method='get' action='index.html' enctype='multipart/form-data'>";
    pagehtml += Text::format("&nbsp;&nbsp;<strong>%s: </strong><input type='text' name='user'  size='10'/>",CSTRING(WS_LOGIN_USERNAME));
    pagehtml += Text::format("&nbsp;&nbsp;<strong>%s: </strong><input type='password' name='pass' size='10'>",CSTRING(WS_LOGIN_PASSWORD));
    pagehtml += Text::format("&nbsp;&nbsp;<input class='tlacitko' type='submit' value='%s'>", CSTRING(WS_LOGIN_LOGIN));
    pagehtml += "		</form>";
    pagehtml += "	</div>";
    pagehtml += "	<div id='paticka'>";
    pagehtml += Text::format("		<a href='http://www.sababadc.com/'>SababaDC</a> | %s ",CSTRING(WS_FOOTER));
    pagehtml += "	</div>";
    pagehtml += "</div>";
    pagehtml += Text::format("   <td width='400px'><iframe id='page' src='%s' width='400px' height='500px'></iframe></td>",(LINK_WEB_SERVER).c_str());
    pagehtml += "</tr></table>";
    pagehtml += "</body>";
    pagehtml += "</html>";

	string header = "HTTP/1.0 200 OK\r\nContent-Type: text/html\r\nContent-Length: " + Util::toString(pagehtml.length()) + "\r\n\r\n";

	return header + pagehtml;
}


string WebServerManager::getPage(string file,string IP) {
    WebPageInfo* page = getWebPageInfo(file);    
    if ( page->login ) {
        if(!isloggedin(IP)) {
            return getLoginPage();
        }
    }

    string page_s1 = "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">";
    page_s1 += "<html xmlns='http://www.w3.org/1999/xhtml' xml:lang='en'>";
    page_s1 +=  Text::format("    <title>SababaDC - %s</title>",page->title.c_str());
    page_s1 += "	 <meta http-equiv='Content-Type' content='text/html; charset=utf-8' />";
    page_s1 += "    <meta http-equiv='pragma' content='no-cache'>";
    page_s1 += "    <meta http-equiv='cache-control' content='no-cache, must-revalidate'>";
    page_s1 += "	 <link rel='stylesheet' href='/style.css' type='text/css' title='Default style' media='screen' />";
    page_s1 += "</head>";
    page_s1 += "<body>";
    page_s1 += "<table width='100%'>";
    page_s1 += "<tr valign='top'><td>";
    page_s1 += "<div id='obsah'>";
    page_s1 +=  Text::format("<h1>SababaDC - %s</h1>",page->title.c_str());
    page_s1 += "	<div id='menu'>";
	page_s1 += Text::format("<div id='menu'>");
	page_s1 += Text::format("		<a href='ulfinished.html'>%s</a>",CSTRING(WS_TITLE_FINISHED_UPLOADS));
    page_s1 += Text::format("		<a href='ulqueue.html'>%s</a>",CSTRING(WS_TITLE_UPLOAD_QUEUE));
    page_s1 += Text::format("		<a href='dlfinished.html'>%s</a>",CSTRING(WS_TITLE_FINISHED_DOWNLOADS));
    page_s1 += Text::format("		<a href='dlqueue.html'>%s</a>",CSTRING(WS_TITLE_DOWNLOAD_QUEUE));
	page_s1 += Text::format("</div>");
	page_s1 += Text::format("<div id='menu'>");
    page_s1 += Text::format("		<a href='search.html'>%s</a>",CSTRING(WS_TITLE_SEARCH));
    page_s1 += Text::format("		<a href='weblog.html'>%s</a>",CSTRING(WS_TITLE_LOGS));
    page_s1 += Text::format("		<a href='syslog.html'>%s</a>",CSTRING(WS_TITLE_SYSTEM_LOGS));
    page_s1 += Text::format("		<a href='logout.html'>%s</a>",CSTRING(WS_TITLE_LOGOUT));
	page_s1 += Text::format("</div>");
    page_s1 += "	</div>";
    page_s1 += "	<div id='prava'>";

    string page_s2= "	</div>";
    page_s2 += "	<div id='ikony'>";
    page_s2 += Text::format("			<a href='switch.html' id='switch' title='%s'></a>",CSTRING(WS_TITLE_SWITCH));
    page_s2 += Text::format("			<a href='logoff.html' id='logoff' title='%s'></a>",CSTRING(WS_TITLE_LOGOFF));
    page_s2 += Text::format("			<a href='suspend.html' id='suspend' title='%s'></a>",CSTRING(WS_TITLE_SUSPEND));
    page_s2 += Text::format("			<a href='reboot.html' id='reboot' title='%s'></a>",CSTRING(WS_TITLE_REBOOT));
    page_s2 += Text::format("			<a href='shutdown.html' id='shutdown' title='%s'></a>",CSTRING(WS_TITLE_SHUTDOWN));
    page_s2 += "	</div>";
    page_s2 += "	<div id='paticka'>";
    page_s2 += Text::format("		<a href='http://www.sababadc.com/'>SababaDC</a> | %s ",CSTRING(WS_FOOTER));
    page_s2 += "	</div>";
    page_s2 += "</div></td>";
    string page_s3=Text::format(
        "   <td width='400px'><iframe id='page' src='%s' width='400px' height='500px'></iframe></td>"
        "</tr></table>"
        "</body>"
        "</html>",(LINK_WEB_SERVER).c_str());

    string header = "HTTP/1.0 200 OK\r\n";
    string content = "";
    bool layout = true;
   
	switch(page->id){
        case STYLE_SHEET:
		    content = getResourceData(IDWS_STYLE);
            layout = false;
            break;
		case INDEX:
		case DOWNLOAD_QUEUE:
			content+=getDLQueue();
			break;

		case DOWNLOAD_FINISHED:
			content+=getFinished(false);
			break;

		case UPLOAD_QUEUE:
			content+=getULQueue();
			break;

		case UPLOAD_FINISHED:
			content+=getFinished(true);
			break;

		case SEARCH:
			content+=getSearch();
			break;

		case LOG:
			content+=getLogs();
			break;

		case SYSLOG:
			content+=getSysLogs();
			break;

		case LOGOUT: {
			map<string, uint32_t>::iterator i;
			if((i = loggedin.find(IP)) != loggedin.end())
				loggedin.erase(i);

			content = getLoginPage();
			return content;
		} 
        case SHUTDOWN :
            fire(WebServerListener::ShutdownPC(), 0);
            content = CSTRING(WS_REQUEST_SENT);
            break;
        case LOGOFF:
            fire(WebServerListener::ShutdownPC(), 1);
            content = CSTRING(WS_REQUEST_SENT);
            break;
        case REBOOT:
            fire(WebServerListener::ShutdownPC(), 2);
            content = CSTRING(WS_REQUEST_SENT);
            break;
        case SUSPEND:
            fire(WebServerListener::ShutdownPC(), 3);
            content = CSTRING(WS_REQUEST_SENT);
            break;
        case SWITCH:
            fire(WebServerListener::ShutdownPC(), 5);
            content = CSTRING(WS_REQUEST_SENT);
            break;
        case IMAGE_LOGO:
            content = getResourceData(IDWS_IMAGE_LOGO);
            layout=false;
            break;
        case IMAGE_SHUTDOWN:
            content = getResourceData(IDWS_IMAGE_SHUTDOWN);
            layout=false;
            break;
        case IMAGE_LOGOFF:
            content = getResourceData(IDWS_IMAGE_LOGOFF);
            layout=false;
            break;
        case IMAGE_LOGOFFON:
            content = getResourceData(IDWS_IMAGE_LOGOFFON);
            layout=false;
            break;
        case IMAGE_REBOOT:
            content = getResourceData(IDWS_IMAGE_REBOOT);
            layout=false;
            break;
        case IMAGE_REBOOTON:
            content = getResourceData(IDWS_IMAGE_REBOOTON);
            layout=false;
            break;
        case IMAGE_SUSPEND:
            content = getResourceData(IDWS_IMAGE_SUSPEND);
            layout=false;
            break;
        case IMAGE_SUSPENDON:
            content = getResourceData(IDWS_IMAGE_SUSPENDON);
            layout=false;
            break;
        case IMAGE_SWITCH:
            content = getResourceData(IDWS_IMAGE_SWITCH);
            layout=false;
            break;
        case IMAGE_SWITCHON:
            content = getResourceData(IDWS_IMAGE_SWITCHON);
            layout=false;
            break;
        case PAGE_404:
        default: 
            header = "HTTP/1.0 404 Not Found\r\n";
            content = CSTRING(WS_TITLE_NOT_FOUND);
            layout = false;
            break;
	}


    string pagehtml;
    if ( layout ) {
        pagehtml = page_s1 + content + page_s2 + page_s3;
    } else {
        pagehtml = content;
    }

	header += "Content-Type:" + page->type + "; charset=utf-8" + "\r\nContent-Length: " + Util::toString(pagehtml.length()) + "\r\n\r\n";
    printf("sending: %s\n",( header + pagehtml ).c_str());
    return header + pagehtml;
}

string WebServerManager::getLogs(){
	string ret = "";
	//ret += LOGTAIL(WEBSERVER_AREA, 10);

	/*StringMap params;	
	params["user"] = user->getFirstNick();	
	params["hub"] = user->getClientName();
	params["mynick"] = user->getClientNick();	
	params["mycid"] = user->getClientCID().toBase32();	
	params["cid"] = user->getCID().toBase32();	
	params["hubaddr"] = user->getClientAddressPort();	*/

	string path = SETTING(LOG_DIRECTORY) + SETTING(LOG_FILE_WEBSERVER);
		
	try {
		File f(path, File::READ, File::OPEN);
		
		int64_t size = f.getSize();

		if(size > 32*1024) {
			f.setPos(size - 32*1024);
		}

		if(BOOLSETTING(LOG_WEBSERVER)) {
			StringList lines = StringTokenizer<string>(f.read(32*1024), "\r\n").getTokens();
			int linesCount = lines.size();
			int i = linesCount > 11 ? linesCount - 11 : 0;

			for(; i < (linesCount - 1); ++i){
				ret += "<br />&nbsp;" + lines[i];
			}
		} else {
            ret += Text::format("<br />&nbsp;%s",CSTRING(WS_LOG_DISABLED));
		}

		f.close();
	} catch(FileException & e) {
		if(BOOLSETTING(LOG_WEBSERVER)) {
			ret += Text::format("<br />&nbsp;<strong>%s </strong>",CSTRING(WS_ERROR)) + e.getError();
		} else {
			ret += Text::format("<br />&nbsp;%s",CSTRING(WS_LOG_DISABLED));
		}
	}

/*	string::size_type i = 0;
	while( (i = ret.find('\n', i)) != string::npos) {
		ret.replace(i, 1, "<br>");
		i++;
	}*/

	return ret;
}

string WebServerManager::getSysLogs(){
	string ret = "";

	string path = SETTING(LOG_DIRECTORY) + SETTING(LOG_FILE_SYSTEM);
		
	try {
		File f(path, File::READ, File::OPEN);
		
		int64_t size = f.getSize();

		if(size > 32*1024) {
			f.setPos(size - 32*1024);
		}

		if(BOOLSETTING(LOG_SYSTEM)) {
			StringList lines = StringTokenizer<string>(f.read(32*1024), "\r\n").getTokens();
			int linesCount = lines.size();
			int i = linesCount > 11 ? linesCount - 11 : 0;

			for(; i < (linesCount - 1); ++i){
				ret += "<br />&nbsp;" + lines[i];
			}
		} else {
			ret += Text::format("<br />&nbsp;%s",CSTRING(WS_SYSLOG_DISABLED));
		}

		f.close();
	} catch(FileException & e) {
		if(BOOLSETTING(LOG_SYSTEM)) {
			ret += Text::format("<br />&nbsp;<strong>%s </strong>",CSTRING(WS_ERROR)) + e.getError();
		} else {
			ret += Text::format("<br />&nbsp;%s",CSTRING(WS_SYSLOG_DISABLED));
		}
	}
	return ret;
}

string WebServerManager::getSearch(){
	string ret = "<form method='GET' name='search' ACTION='search.html' enctype='multipart/form-data'>";
	if(sended_search == true) {
        searchResults.clear();
        ret += Text::format("<br /><table width='100%%'><tr><td align='center'><strong>%s</strong>",CSTRING(WS_SEARCHING));
		ret += Text::format("&nbsp;<input  class=\"tlacitko\" type=\"submit\" name=\"%s\" value=\"cancel\"></td></tr></table></form>",CSTRING(WS_STOP_SEARCH));
        ret += "<script>setTimeout(\"window.location='/search.html?stop=true';\",30000)</script>";
		sended_search = false;
	} else {
        ret += Text::format("<br /><table width='100%%'><tr><td align='center'><strong>%s:&nbsp;</strong><input type=text name='search' />",CSTRING(WS_SEARCH));		
		ret += "&nbsp;<select name='type'>";
		ret += "<option value='0' selected>" + STRING(ANY) + "</option>";
		ret += "<option value='1'>" + STRING(AUDIO) + "</option>";
		ret += "<option value='2'>" + STRING(COMPRESSED) + "</option>";
		ret += "<option value='3'>" + STRING(DOCUMENT) + "</option>";
		ret += "<option value='4'>" + STRING(EXECUTABLE) + "</option>";
		ret += "<option value='5'>" + STRING(PICTURE) + "</option>";
		ret += "<option value='6'>" + STRING(VIDEO) + "</option>";
		ret += "<option value='7'>" + STRING(DIRECTORY) + "</option>";
		ret += "<option value='8'>TTH</option>";
        ret += "</select>";
        ret += Text::format("&nbsp;<input class='tlacitko' type='submit' value='%s' />",CSTRING(WS_SEARCH));
		ret += "</td></tr></table></form><hr />";

        row = 0 ;
		ret += "<table width='100%' >"; 
        ret += Text::format("<tr class='tucne'><td width='20px'>#</td><td width='50px'>%s</td><td>%s</td></th>",CSTRING(SIZE),CSTRING(FILENAME));
        for ( SearchResults::const_iterator ii= searchResults.begin() ; ii != searchResults.end() ; ii++ ) {
            string cycle_color= row % 2 ? "#EEEEEE":"#CCCCCC" ;
            const WebServerSearchResult& aResult = *ii;
            ret += "<form method='GET' name=\"form" + Util::toString(row) + "\" ACTION='search.html'>";
			ret += "<input type=\"hidden\" name='file' value='" + aResult.getFile() + "'>";
			ret += "<input type=\"hidden\" name='size' value='" + Util::toString(aResult.getSize()) + "'>";
			ret += "<input type='hidden' name='name' value='" + aResult.getFileName() + "'>";
			ret += "<input type='hidden' name='tth' value='" + aResult.getTTH().toBase32() + "'>";
			ret += "<input type='hidden' name='type' value='" + Util::toString(aResult.getType()) + "'>";
            ret += "<tr bgcolor='"+cycle_color+"' onmouseover=\"this.style.backgroundColor='#316AC5'; this.style.color='#FFF'; this.style.cursor='pointer';\"";
            ret +=     "onmouseout=\"this.style.backgroundColor='"+cycle_color+"'; this.style.color='#000'; this.style.cursor='pointer';\""; 
            ret +=     "onclick=\"form" + Util::toString(row) + ".submit();\">";
            ret += "<td>" + Text::format("%d",aResult.getCount()) + "</td>";
            ret += "<td>" + Util::formatBytes(aResult.getSize()) + "</td>";
            ret += "<td>" + aResult.getFileName() + "</td>";
            ret += "</tr></form>";
			row++;
        }
        ret += "</table>";
	}
	return ret;
}

string WebServerManager::getFinished(bool uploads){
	string ret;

	const FinishedItem::List& fl = FinishedManager::getInstance()->lockList(uploads);
	ret += "	<table  width='100%'>";
	ret += "		<tr class='tucne'>";
    ret += Text::format("			<td>%s</td>",CSTRING(TIME));
	ret += Text::format("			<td>%s</td>",CSTRING(FILENAME));
	ret += Text::format("			<td>%s</td>",CSTRING(SIZE));
	ret += "		</tr>";
	int row = 0 ;
    for(FinishedItem::List::const_iterator i = fl.begin(); i != fl.end(); ++i) {
        string cycle_color= row % 2 ? "#EEEEEE":"#CCCCCC" ;
		ret+="<tr bgcolor='" + cycle_color + "'>";
		ret+="	<td>" + Util::formatTime("%Y-%m-%d %H:%M:%S", (*i)->getTime()) + "</td>";
		ret+="	<td>" + Util::getFileName((*i)->getTarget()) + "</td>";
		ret+="	<td>" + Util::formatBytes((*i)->getSize()) + "</td>";			
		ret+="</tr>";
        row++;
	}
	ret+="</table>";
	FinishedManager::getInstance()->unlockList();

	return ret;
}

string WebServerManager::getDLQueue(){
	string ret;    
	const QueueItem::StringMap& li = QueueManager::getInstance()->lockQueue();
	ret += "	<table  width='100%'>";
	ret += "		<tr class='tucne'>";
    ret += Text::format("			<td>%s</td>",CSTRING(FILENAME));
	ret += Text::format("			<td>%s</td>",CSTRING(SIZE));
	ret += Text::format("			<td>%s</td>",CSTRING(DOWNLOADED));
	ret += Text::format("			<td>%s</td>",CSTRING(SPEED));
	ret += Text::format("			<td>%s</td>",CSTRING(SEGMENTS));
	ret += "		</tr>";
    
    int row = 0 ;
	for(QueueItem::StringMap::const_iterator j = li.begin(); j != li.end(); ++j) {
        string cycle_color= row % 2 ? "#EEEEEE":"#CCCCCC" ;
		QueueItem* aQI = j->second;
		double percent = (aQI->getSize()>0)? aQI->getDownloadedBytes() * 100.0 / aQI->getSize() : 0;
		ret += "	<tr bgcolor='"+ cycle_color +"'>";
		ret += "		<td>" + aQI->getTargetFileName() + "</td>";
		ret += "		<td>" + Util::formatBytes(aQI->getSize()) + "</td>";
		ret += "		<td>" + Util::formatBytes(aQI->getDownloadedBytes()) + " ("+ Util::toString(percent) + "%)</td>";
		ret += "		<td>" + Util::formatBytes(aQI->getAverageSpeed()) + "/s</td>";
		ret += "		<td>" + Util::toString((int)aQI->getCurrents().size())+"/"+Util::toString(aQI->getMaxSegments()) + "</td>";
		ret += "	</tr>";
        row++;
	}
	ret+="</table>";
	QueueManager::getInstance()->unlockQueue();

	return ret;
}

string WebServerManager::getULQueue(){
	string ret = "";
	ret += "	<table  width='100%'>";
	ret += "		<tr class='tucne'>";
    ret += Text::format("			<td>%s</td>",CSTRING(FILENAME));
	ret += "		</tr>";
	UploadQueueItem::UserMap users = UploadManager::getInstance()->getWaitingUsers();
	
    int row = 0 ;
    for(UploadQueueItem::UserMapIter ii = users.begin(); ii != users.end(); ++ii) {
        string cycle_color= row % 2 ? "#EEEEEE":"#CCCCCC" ;
		for(UploadQueueItem::Iter i = ii->second.begin(); i != ii->second.end(); ++i) {
			ret += "	<tr bgcolor='"+ cycle_color +"'><td>" + Util::getFileName((*i)->File) + "</td></tr>";
		}
        row++;
	}
	ret+="</table>";
	return ret;
}	

StringMap WebServerSocket::getArgs(string arguments) {
	StringMap args;

	string::size_type i = 0;
	string::size_type j = 0;
	while((i = arguments.find("=", j)) != string::npos) {
		string key = arguments.substr(j, i-j);
		string value = arguments.substr(i + 1);;

		j = i + 1;
		if((i = arguments.find("&", j)) != string::npos) {
			value = arguments.substr(j, i-j);
			j = i + 1;
		} 

		args[key] = value;
	}

	return args;
}

int WebServerSocket::run(){
	char buff[8192];
	int test = 0;
	ZeroMemory(&buff, sizeof(buff));
	while(true) {

		test++;
		if(test >= 10000)
			break;

		int size = recv(sock,buff,sizeof(buff),0);

		string header = buff;
		header = header.substr(0,size);

		int start = 0, end = 0;

		string IP = Util::toString(from.sin_addr.S_un.S_un_b.s_b1) + string(".") + Util::toString(from.sin_addr.S_un.S_un_b.s_b2) + string(".") + Util::toString(from.sin_addr.S_un.S_un_b.s_b3) + string(".") + Util::toString(from.sin_addr.S_un.S_un_b.s_b4);

		dcdebug("%s\n", header.c_str());	

		if(((start = header.find("GET ")) != string::npos) && (end = header.substr(start+4).find(" ")) != string::npos ){
			if(BOOLSETTING(LOG_WEBSERVER)) {
				StringMap params;
				params["file"] = header.substr(start+4,end);
				params["ip"] = IP;
				if((header.substr(start+4,end)) != "/robots.txt")
					LOG(LogManager::WEBSERVER, params);
			}

			header = header.substr(start+4,end);
			bool check = false;
			dcdebug(header.c_str());
			if((start = header.find("?")) != string::npos) {
				string arguments = header.substr(start+1);
				header = header.substr(0, start);
				StringMap m = getArgs(arguments);

				if(m["user"] == SETTING(WEBSERVER_USER) && m["pass"] == SETTING(WEBSERVER_PASS))
					WebServerManager::getInstance()->login(IP);
				if((m["search"] != Util::emptyString)) {
					WebServerManager::getInstance()->search(m["search"], Util::toInt(m["type"]));
				}
				if((m["stop"] != Util::emptyString)) {
					check = m["stop"] == "true";
					WebServerManager::getInstance()->reset();
				}
				if(m["name"] != Util::emptyString) {
                    WebServerManager::getInstance()->unescape(m["name"]);
					QueueManager::getInstance()->add(m["name"],Util::toInt64(m["size"]),m["tth"]);
				}
			}

			string toSend = Util::emptyString;
            toSend = WebServerManager::getInstance()->getPage(header, IP);
			::send(sock, toSend.c_str(), toSend.size(), 0);
			break;
		}
	}
	::closesocket(sock);
	delete this;
	return 0;

} 

void WebServerManager::onSearchResult(SearchResult* aResult) {
	{
		Lock l(cs);
		if(aResult->getType() == SearchResult::TYPE_FILE) {
            WebServerSearchResult wssr(aResult);
            SearchResults::iterator ii = searchResults.find(wssr);
            if ( ii != searchResults.end() ) {
                wssr.setCount(ii->getCount()+1);
                searchResults.erase(ii);
            }
            searchResults.insert(wssr);
		}
	}
}

