#include "stdafx.h"
#include "Resource.h"
#include "Browser.h"

Browser::Browser()
    : addressContainer( WC_COMBOBOX , this , ADDRESS_MAP ),
      addressEditContainer( WC_EDIT , this , ADDRESS_MAP )
{
    largeImages.CreateFromImage(IDB_BROWSER_TOOLBAR, 20, 15, CLR_DEFAULT, IMAGE_BITMAP, LR_CREATEDIBSECTION | LR_SHARED);
    largeImagesHot.CreateFromImage(IDB_BROWSER_TOOLBAR, 20, 15, CLR_DEFAULT, IMAGE_BITMAP, LR_CREATEDIBSECTION | LR_SHARED);
}
Browser::~Browser() {
    largeImages.Destroy();
    largeImagesHot.Destroy();
    web->Release();
}

LRESULT Browser::OnCreate(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled)
{
    CreateSimpleReBar( ATL_SIMPLE_REBAR_NOBORDER_STYLE );
    AddSimpleReBarBand( createToolbar() , NULL, TRUE ,780,TRUE);
    browser.Create(m_hWnd, rcDefault, Text::toT(version.link_home).c_str() , WS_CHILD | WS_VISIBLE | WS_BORDER | WS_VSCROLL | WS_HSCROLL, 0, 1);
    browser.QueryControl(IID_IWebBrowser2,(void**)&web);
    //AtlAdvise(m_spWebBrowser, GetUnknown(), DIID_DWebBrowserEvents2, &m_dwCookie);

    load();
    bHandled = FALSE;
    return 1;
}

void Browser::UpdateLayout(BOOL bResizeBars /* = TRUE */)
{
    CRect rc;

    GetClientRect(rc);
    UpdateBarsPosition(rc, bResizeBars);

    browser.MoveWindow(rc);
}

HWND Browser::createToolbar() {
    CToolBarCtrl ctrlToolbar;
    int adressitem ;
    largeImages.CreateFromImage(IDB_BROWSER_TOOLBAR, 20, 15, CLR_DEFAULT, IMAGE_BITMAP, LR_CREATEDIBSECTION | LR_SHARED);
    largeImagesHot.CreateFromImage(IDB_BROWSER_TOOLBAR, 20, 15, CLR_DEFAULT, IMAGE_BITMAP, LR_CREATEDIBSECTION | LR_SHARED);

    ctrlToolbar.Create(m_hWnd, NULL, NULL, ATL_SIMPLE_CMDBAR_PANE_STYLE | TBSTYLE_FLAT | TBSTYLE_TOOLTIPS, 0, ATL_IDW_TOOLBAR);
    ctrlToolbar.SetImageList(largeImages);
    ctrlToolbar.SetHotImageList(largeImagesHot);

    const int numButtons = 7;


    TBBUTTON tb[numButtons];
    memset(tb, 0, sizeof(tb));
    int n = 0, bitmap = 0;

    tb[n].iBitmap = bitmap++;
    tb[n].idCommand = IDC_GOHOME;
    tb[n].fsState = TBSTATE_ENABLED ;
    tb[n].fsStyle = TBSTYLE_BUTTON | TBSTYLE_AUTOSIZE;

    n++;
    tb[n].iBitmap = bitmap++;
    tb[n].idCommand = IDC_GOBACK;
    tb[n].fsState = TBSTATE_ENABLED;
    tb[n].fsStyle = TBSTYLE_BUTTON | TBSTYLE_AUTOSIZE;

    n++;
    tb[n].iBitmap = bitmap++;
    tb[n].idCommand = IDC_GOFORWARD;
    tb[n].fsState = TBSTATE_ENABLED;
    tb[n].fsStyle = TBSTYLE_BUTTON | TBSTYLE_AUTOSIZE;

    n++;
    tb[n].iBitmap = bitmap++;
    tb[n].idCommand = IDC_REFRESH;
    tb[n].fsState = TBSTATE_ENABLED;
    tb[n].fsStyle = TBSTYLE_BUTTON | TBSTYLE_AUTOSIZE;

    n++;
    tb[n].iBitmap = bitmap++;
    tb[n].idCommand = IDC_STOP;
    tb[n].fsState = TBSTATE_ENABLED;
    tb[n].fsStyle = TBSTYLE_BUTTON | TBSTYLE_AUTOSIZE  ;

    n++;
    adressitem = n ;
    tb[n].iBitmap = 200;
    tb[n].fsStyle = TBSTYLE_SEP;

    n++;
    tb[n].iBitmap = bitmap++;
    tb[n].idCommand = IDC_GOADDRESS;
    tb[n].fsState = TBSTATE_ENABLED;
    tb[n].fsStyle = TBSTYLE_BUTTON | TBSTYLE_AUTOSIZE  ;

    ctrlToolbar.SetButtonStructSize();
    ctrlToolbar.AddButtons(numButtons, tb);
    ctrlToolbar.AutoSize();

    CRect rect;
    ctrlToolbar.GetItemRect( adressitem,&rect);
    rect.bottom += 100 ;

    address.Create(ctrlToolbar.m_hWnd, rect , NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN |
        WS_VSCROLL | CBS_DROPDOWN | CBS_AUTOHSCROLL , 0);
    address.SetExtendedUI();
    addressContainer.SubclassWindow(address);

    POINT pt; pt.x = 10; pt.y = 10;
    HWND hWnd = address.ChildWindowFromPoint(pt);
    if(hWnd != NULL ) {
        addressEditContainer.SubclassWindow(hWnd);
    }

    return ctrlToolbar.m_hWnd;
}

LRESULT Browser::onAddressChar(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& bHandled){
    if ( wParam == VK_RETURN ) {
        goAddress();
    }
    else {
        bHandled = FALSE;
    }
    return 0;
}

void Browser::goAddress(){
    if(!(address.GetWindowTextLength() > 0)) {
        return ;
    }
    tstring s(address.GetWindowTextLength() + 1, _T('\0'));
    address.GetWindowText(&s[0], s.size());
    if(find(lastAddresses.begin(), lastAddresses.end(), s) == lastAddresses.end()) {
        address.InsertString(0, s.c_str());
        lastAddresses.push_back(s);
    }
    web->Navigate(BSTR(s.c_str()), NULL, NULL, NULL, NULL);
}

void Browser::load() {
    for(TStringIter i = lastAddresses.begin(); i != lastAddresses.end(); ++i) {
        address.InsertString(0, i->c_str());
    }
}