﻿/*
 * Copyright (C) 2001-2006 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "Resource.h"

#include "SearchFrm.h"
#include "LineDlg.h"
#include "MainFrm.h"

#include "../client/QueueManager.h"
#include "../client/StringTokenizer.h"
#include "../client/ClientManager.h"
#include "../client/TimerManager.h"
#include "../client/SearchManager.h"
#include "../client/PGLoader.h"

TStringList SearchFrame::lastSearches;

int SearchFrame::columnIndexes[] = { COLUMN_FILENAME, COLUMN_HITS, COLUMN_NICK, COLUMN_TYPE, COLUMN_SIZE,
        COLUMN_PATH, COLUMN_SLOTS, COLUMN_CONNECTION, COLUMN_HUB, COLUMN_EXACT_SIZE, COLUMN_UPLOAD,
        COLUMN_LOCATION, COLUMN_IP, COLUMN_DNS, // !SMT!-IP
        COLUMN_TTH };
int SearchFrame::columnSizes[] = { 210, 80, 100, 50, 80, 100, 40, 70, 150, 80, 80, 100, 100, 100, 150 }; // !SMT!-IP

static ResourceManager::Strings columnNames[] = {ResourceManager::FILE,  ResourceManager::HIT_COUNT, ResourceManager::USER, ResourceManager::TYPE, ResourceManager::SIZE,
        ResourceManager::PATH, ResourceManager::SLOTS, ResourceManager::CONNECTION,
        ResourceManager::HUB, ResourceManager::EXACT_SIZE, ResourceManager::AVERAGE_UPLOAD,
        ResourceManager::LOCATION_BARE, ResourceManager::IP_BARE, ResourceManager::DNS_BARE, // !SMT!-IP
        ResourceManager::TTH_ROOT };

SearchFrame::FrameMap SearchFrame::frames;

void SearchFrame::openNewWindow(const tstring& str /* = Util::emptyString */, LONGLONG size /* = 0 */, SearchManager::SizeModes mode /* = SearchManager::SIZE_ATLEAST */, SearchManager::TypeModes type /* = SearchManager::TYPE_ANY ( 0 ) */) {
        SearchFrame* pChild = new SearchFrame();
        pChild->setInitial(str, size, mode, type);
        pChild->CreateEx(WinUtil::mdiClient);

        frames.insert( FramePair(pChild->m_hWnd, pChild) );
}

void SearchFrame::closeAll() {
        for(FrameIter i = frames.begin(); i != frames.end(); ++i) {
                ::PostMessage(i->first, WM_CLOSE, 0, 0);
        }
        if ( SearchFrame::frame != NULL ) {
            SearchFrame::frame->PostMessage(WM_CLOSE);
        }
}

LRESULT SearchFrame::onCreate(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled)
{
        CreateSimpleStatusBar(ATL_IDS_IDLEMESSAGE, WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS | SBARS_SIZEGRIP, IDC_STATUSBAR);
        ctrlStatus.Attach(m_hWndStatusBar);
        
        dropDown1.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | SS_NOTIFY | SS_OWNERDRAW | WS_TABSTOP,  0, IDC_DROP1);
        dropDown1.SetFont(WinUtil::systemFont, FALSE);
        dropDown1Container.SubclassWindow(dropDown1.m_hWnd);
        dropDown2.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | SS_NOTIFY | SS_OWNERDRAW | WS_TABSTOP,  0, IDC_DROP2);
        dropDown2.SetFont(WinUtil::systemFont, FALSE);
        dropDown2Container.SubclassWindow(dropDown2.m_hWnd);
        dropDown3.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | SS_NOTIFY | SS_OWNERDRAW | WS_TABSTOP,  0, IDC_DROP3);
        dropDown3.SetFont(WinUtil::systemFont, FALSE);
        dropDown3Container.SubclassWindow(dropDown3.m_hWnd);
        dropDown4.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | SS_NOTIFY | SS_OWNERDRAW | WS_TABSTOP,  0, IDC_DROP4);
        dropDown4.SetFont(WinUtil::systemFont, FALSE);
        dropDown4Container.SubclassWindow(dropDown4.m_hWnd);
        dropDown5.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | SS_NOTIFY | SS_OWNERDRAW | WS_TABSTOP,  0, IDC_DROP5);
        dropDown5.SetFont(WinUtil::systemFont, FALSE);
        dropDown5Container.SubclassWindow(dropDown5.m_hWnd);

        ctrlAutoClose.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | BS_MULTILINE | BS_AUTOCHECKBOX, NULL, IDC_REM_POS);
        ctrlAutoClose.SetFont(WinUtil::systemFont, FALSE);
        ctrlAutoClose.SetWindowText(CTSTRING(AUTO_CLOSE_GROUPS));
        ctrlAutoClose.SetCheck(BOOLSETTING(AUTO_CLOSE_GROUPS));
        autoCloseContainer.SubclassWindow(ctrlAutoClose.m_hWnd);

        slider.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | SS_NOTIFY | SS_OWNERDRAW | WS_TABSTOP,  0, IDC_SEARCH_SLIDER);
        slider.SetFont(WinUtil::systemFont, FALSE);
        sliderContainer.SubclassWindow(slider.m_hWnd);

        
        ctrlResultLimit.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | 
            ES_AUTOHSCROLL | ES_NUMBER, WS_EX_CLIENTEDGE);
	    resultLimitContainer.SubclassWindow(ctrlResultLimit.m_hWnd);        

        resultLimitLabel.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN);
        resultLimitLabel.SetFont(WinUtil::systemFont, FALSE);
        resultLimitLabel.SetWindowText(CTSTRING(RESULT_LIMIT));

        ctrlOneSearchWindow.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN, NULL, IDC_ONESEARCHWINDOW);
        ctrlOneSearchWindow.SetButtonStyle(BS_AUTOCHECKBOX, FALSE);
        ctrlOneSearchWindow.SetFont(WinUtil::systemFont, FALSE);
        ctrlOneSearchWindow.SetWindowText(CTSTRING(SETTINGS_ONE_SEARCH_WINDOW));
        oneSearchWindowContainer.SubclassWindow(ctrlOneSearchWindow.m_hWnd);

        ctrlSearchBox.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN |
                WS_VSCROLL | CBS_DROPDOWN | CBS_AUTOHSCROLL, 0);
        for(TStringIter i = lastSearches.begin(); i != lastSearches.end(); ++i) {
                ctrlSearchBox.InsertString(0, i->c_str());
        }
        searchBoxContainer.SubclassWindow(ctrlSearchBox.m_hWnd);
        ctrlSearchBox.SetExtendedUI();

        ctrlPurge.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN |
                BS_PUSHBUTTON , 0, IDC_PURGE);
        ctrlPurge.SetWindowText(CTSTRING(PURGE));
        ctrlPurge.SetFont(WinUtil::systemFont);
        purgeContainer.SubclassWindow(ctrlPurge.m_hWnd);

        ctrlMode.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN |
                WS_HSCROLL | WS_VSCROLL | CBS_DROPDOWNLIST, WS_EX_CLIENTEDGE);
        modeContainer.SubclassWindow(ctrlMode.m_hWnd);

        ctrlSize.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN |
                ES_AUTOHSCROLL | ES_NUMBER, WS_EX_CLIENTEDGE);
        sizeContainer.SubclassWindow(ctrlSize.m_hWnd);

        ctrlSizeMode.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN |
                WS_HSCROLL | WS_VSCROLL | CBS_DROPDOWNLIST, WS_EX_CLIENTEDGE);
        sizeModeContainer.SubclassWindow(ctrlSizeMode.m_hWnd);

        ctrlFiletype.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN |
                WS_HSCROLL | WS_VSCROLL | CBS_DROPDOWNLIST | CBS_HASSTRINGS | CBS_OWNERDRAWFIXED, WS_EX_CLIENTEDGE, IDC_FILETYPES);

        searchTypes.CreateFromImage(IDB_SEARCH_TYPES, 16, 0, CLR_DEFAULT, IMAGE_BITMAP, LR_CREATEDIBSECTION | LR_SHARED);
        fileTypeContainer.SubclassWindow(ctrlFiletype.m_hWnd);

        if (BOOLSETTING(USE_SYSTEM_ICONS)) {
                ctrlResults.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN |
                        WS_HSCROLL | WS_VSCROLL | LVS_REPORT | LVS_SHOWSELALWAYS | LVS_SHAREIMAGELISTS, WS_EX_CLIENTEDGE, IDC_RESULTS);
        } else {
                ctrlResults.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN |
                        WS_HSCROLL | WS_VSCROLL | LVS_REPORT | LVS_SHOWSELALWAYS, WS_EX_CLIENTEDGE, IDC_RESULTS);
        }
        ctrlResults.SetExtendedListViewStyle(LVS_EX_LABELTIP | LVS_EX_HEADERDRAGDROP | LVS_EX_FULLROWSELECT | 0x00010000 | LVS_EX_INFOTIP);
        resultsContainer.SubclassWindow(ctrlResults.m_hWnd);

        if (BOOLSETTING(USE_SYSTEM_ICONS)) {
                ctrlResults.SetImageList(WinUtil::fileImages, LVSIL_SMALL);
        } else {
                images.CreateFromImage(IDB_SPEEDS, 16, 3, CLR_DEFAULT, IMAGE_BITMAP, LR_CREATEDIBSECTION | LR_SHARED);
                ctrlResults.SetImageList(images, LVSIL_SMALL);
        }

        ctrlHubs.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN |
                WS_HSCROLL | WS_VSCROLL | LVS_REPORT | LVS_NOCOLUMNHEADER, WS_EX_CLIENTEDGE, IDC_HUB);
        ctrlHubs.SetExtendedListViewStyle(LVS_EX_LABELTIP | LVS_EX_CHECKBOXES | LVS_EX_FULLROWSELECT);
        hubsContainer.SubclassWindow(ctrlHubs.m_hWnd);

        ctrlFilter.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN |
                ES_AUTOHSCROLL, WS_EX_CLIENTEDGE);

        ctrlFilterContainer.SubclassWindow(ctrlFilter.m_hWnd);
        ctrlFilter.SetFont(WinUtil::font);

        ctrlFilterSel.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_HSCROLL |
                WS_VSCROLL | CBS_DROPDOWNLIST, WS_EX_CLIENTEDGE);

        ctrlFilterSelContainer.SubclassWindow(ctrlFilterSel.m_hWnd);
        ctrlFilterSel.SetFont(WinUtil::font);

        searchLabel.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN);
        searchLabel.SetFont(WinUtil::systemFont, FALSE);
        searchLabel.SetWindowText(CTSTRING(SEARCH_FOR));

        sizeLabel.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN);
        sizeLabel.SetFont(WinUtil::systemFont, FALSE);
        sizeLabel.SetWindowText(CTSTRING(SIZE));

        typeLabel.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN);
        typeLabel.SetFont(WinUtil::systemFont, FALSE);
        typeLabel.SetWindowText(CTSTRING(FILE_TYPE));

        srLabel.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN);
        srLabel.SetFont(WinUtil::systemFont, FALSE);
        srLabel.SetWindowText(CTSTRING(SEARCH_IN_RESULTS));

        optionLabel.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN);
        optionLabel.SetFont(WinUtil::systemFont, FALSE);
        optionLabel.SetWindowText(CTSTRING(SEARCH_OPTIONS));

        hubsLabel.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN);
        hubsLabel.SetFont(WinUtil::systemFont, FALSE);
        hubsLabel.SetWindowText(CTSTRING(HUBS));

        ctrlSlots.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN, NULL, IDC_FREESLOTS);
        ctrlSlots.SetButtonStyle(BS_AUTOCHECKBOX, FALSE);
        ctrlSlots.SetFont(WinUtil::systemFont, FALSE);
        ctrlSlots.SetWindowText(CTSTRING(ONLY_FREE_SLOTS));
        slotsContainer.SubclassWindow(ctrlSlots.m_hWnd);

        ctrlCollapsed.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN, NULL, IDC_COLLAPSED);
        ctrlCollapsed.SetButtonStyle(BS_AUTOCHECKBOX, FALSE);
        ctrlCollapsed.SetFont(WinUtil::systemFont, FALSE);
        ctrlCollapsed.SetWindowText(CTSTRING(EXPANDED_RESULTS));
        collapsedContainer.SubclassWindow(ctrlCollapsed.m_hWnd);

        if(BOOLSETTING(FREE_SLOTS_DEFAULT)) {
                ctrlSlots.SetCheck(true);
                onlyFree = true;
        }

        if(BOOLSETTING(ONE_SEARCH_WINDOW)) {
                ctrlOneSearchWindow.SetCheck(true);
        }

        ctrlShowUI.Create(ctrlStatus.m_hWnd, rcDefault, _T("+/-"), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN);
        ctrlShowUI.SetButtonStyle(BS_AUTOCHECKBOX, false);
        ctrlShowUI.SetCheck(1);
        showUIContainer.SubclassWindow(ctrlShowUI.m_hWnd);

        ctrlDoSearch.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN |
                BS_PUSHBUTTON , 0, IDC_SEARCH);
        ctrlDoSearch.SetWindowText(CTSTRING(SEARCH));
        ctrlDoSearch.SetFont(WinUtil::systemFont);
        doSearchContainer.SubclassWindow(ctrlDoSearch.m_hWnd);

        ctrlStopSearch.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE |WS_CLIPSIBLINGS | WS_CLIPCHILDREN |
                BS_PUSHBUTTON, 0, IDC_SEARCH_STOP);
        ctrlStopSearch.SetWindowText(CTSTRING(STOP_SEARCH));
        ctrlStopSearch.SetFont(WinUtil::systemFont);

        ctrlSearchBox.SetFont(WinUtil::systemFont, FALSE);
        ctrlSize.SetFont(WinUtil::systemFont, FALSE);
        ctrlMode.SetFont(WinUtil::systemFont, FALSE);
        ctrlSizeMode.SetFont(WinUtil::systemFont, FALSE);
        ctrlFiletype.SetFont(WinUtil::systemFont, FALSE);
        ctrlResultLimit.SetFont(WinUtil::systemFont, FALSE);

        ctrlMode.AddString(CTSTRING(NORMAL));
        ctrlMode.AddString(CTSTRING(AT_LEAST));
        ctrlMode.AddString(CTSTRING(AT_MOST));
        ctrlMode.AddString(CTSTRING(EXACT_SIZE));
        ctrlMode.SetCurSel(1);

        ctrlSizeMode.AddString(CTSTRING(B));
        ctrlSizeMode.AddString(CTSTRING(KB));
        ctrlSizeMode.AddString(CTSTRING(MB));
        ctrlSizeMode.AddString(CTSTRING(GB));
        if(initialSize == 0)
                ctrlSizeMode.SetCurSel(2);
        else
                ctrlSizeMode.SetCurSel(0);

        ctrlFiletype.AddString(CTSTRING(ANY));
        ctrlFiletype.AddString(CTSTRING(AUDIO));
        ctrlFiletype.AddString(CTSTRING(COMPRESSED));
        ctrlFiletype.AddString(CTSTRING(DOCUMENT));
        ctrlFiletype.AddString(CTSTRING(EXECUTABLE));
        ctrlFiletype.AddString(CTSTRING(PICTURE));
        ctrlFiletype.AddString(CTSTRING(VIDEO));
        ctrlFiletype.AddString(CTSTRING(DIRECTORY));
        ctrlFiletype.AddString(_T("TTH"));
        ctrlFiletype.SetCurSel(0);

        // Create listview columns
        WinUtil::splitTokens(columnIndexes, SETTING(SEARCHFRAME_ORDER), COLUMN_LAST);
        WinUtil::splitTokens(columnSizes, SETTING(SEARCHFRAME_WIDTHS), COLUMN_LAST);

        for(int j=0; j<COLUMN_LAST; j++) {
                int fmt = (j == COLUMN_SIZE || j == COLUMN_EXACT_SIZE) ? LVCFMT_RIGHT : LVCFMT_LEFT;
                ctrlResults.InsertColumn(j, CTSTRING_I(columnNames[j]), fmt, columnSizes[j], j);
        }

        ctrlResults.setColumnOrderArray(COLUMN_LAST, columnIndexes);
        ctrlResults.setAscending(false);
        ctrlResults.setVisible(SETTING(SEARCHFRAME_VISIBLE));
        ctrlResults.setSortColumn(COLUMN_HITS);

        ctrlResults.SetBkColor(WinUtil::bgColor);
        ctrlResults.SetTextBkColor(WinUtil::bgColor);
        ctrlResults.SetTextColor(WinUtil::textColor);
        ctrlResults.SetFont(WinUtil::systemFont, FALSE);        // use Util::font instead to obey Appearace settings
        ctrlResults.setFlickerFree(WinUtil::bgBrush);

        ctrlHubs.InsertColumn(0, _T("Dummy"), LVCFMT_LEFT, LVSCW_AUTOSIZE, 0);
        ctrlHubs.SetBkColor(WinUtil::bgColor);
        ctrlHubs.SetTextBkColor(WinUtil::bgColor);
        ctrlHubs.SetTextColor(WinUtil::textColor);
        ctrlHubs.SetFont(WinUtil::systemFont, FALSE);   // use Util::font instead to obey Appearace settings

        initHubs();

        copyMenu.CreatePopupMenu();
        targetDirMenu.CreatePopupMenu();
        targetMenu.CreatePopupMenu();
        resultsMenu.CreatePopupMenu();

        copyMenu.AppendMenu(MF_STRING, IDC_COPY_NICK, CTSTRING(COPY_NICK));
        copyMenu.AppendMenu(MF_STRING, IDC_COPY_FILENAME, CTSTRING(FILENAME));
        copyMenu.AppendMenu(MF_STRING, IDC_COPY_PATH, CTSTRING(PATH));
        copyMenu.AppendMenu(MF_STRING, IDC_COPY_SIZE, CTSTRING(SIZE));
        copyMenu.AppendMenu(MF_STRING, IDC_COPY_TTH, CTSTRING(TTH_ROOT));
        copyMenu.AppendMenu(MF_STRING, IDC_COPY_LINK, CTSTRING(COPY_MAGNET_LINK));
        copyMenu.AppendMenu(MF_STRING, IDC_COPY_WMLINK, CTSTRING(COPY_MLINK_TEMPL)); // !SMT!-UI
        copyMenu.AppendMenu(MF_STRING, IDC_COPY_HTML_LINK, CTSTRING(COPY_HTML_LINK));

        resultsMenu.AppendMenu(MF_STRING, IDC_DOWNLOAD_FAVORITE_DIRS, CTSTRING(DOWNLOAD));
        resultsMenu.AppendMenu(MF_POPUP, (UINT_PTR)(HMENU)targetMenu, CTSTRING(DOWNLOAD_TO));
        resultsMenu.AppendMenu(MF_STRING, IDC_DOWNLOAD_WHOLE_FAVORITE_DIRS, CTSTRING(DOWNLOAD_WHOLE_DIR));
        resultsMenu.AppendMenu(MF_POPUP, (UINT_PTR)(HMENU)targetDirMenu, CTSTRING(DOWNLOAD_WHOLE_DIR_TO));
        resultsMenu.AppendMenu(MF_STRING, IDC_VIEW_AS_TEXT, CTSTRING(VIEW_AS_TEXT));
        resultsMenu.AppendMenu(MF_SEPARATOR);
        resultsMenu.AppendMenu(MF_STRING, IDC_SEARCH_ALTERNATES, CTSTRING(SEARCH_FOR_ALTERNATES));
        resultsMenu.AppendMenu(MF_STRING, IDC_BITZI_LOOKUP, CTSTRING(BITZI_LOOKUP));
        resultsMenu.AppendMenu(MF_POPUP, (UINT_PTR)(HMENU)copyMenu, CTSTRING(COPY));
        resultsMenu.AppendMenu(MF_SEPARATOR);
        appendUserItems(resultsMenu);
        resultsMenu.AppendMenu(MF_SEPARATOR);
        resultsMenu.AppendMenu(MF_STRING, IDC_REMOVE, CTSTRING(REMOVE));
        resultsMenu.SetMenuDefaultItem(IDC_DOWNLOAD_FAVORITE_DIRS);

        UpdateLayout();
        
        ctrlResultLimit.SetWindowText(Text::toT(Util::toString(maxResult)).c_str());
        if(!initialString.empty()) {
                lastSearches.push_back(initialString);
                ctrlSearchBox.InsertString(0, initialString.c_str());
                ctrlSearchBox.SetCurSel(0);
                ctrlMode.SetCurSel(initialMode);
                ctrlSize.SetWindowText(Util::toStringW(initialSize).c_str());
                ctrlFiletype.SetCurSel(initialType);
                onEnter();
        } else {
                if(MainFrame::getMainFrame()->enabled != true)
                        SetWindowText(CTSTRING(SEARCH_DISABLED));
                else
                        SetWindowText(CTSTRING(SEARCH));
                ::EnableWindow(GetDlgItem(IDC_SEARCH_STOP), FALSE);
        }

        for(int j=0; j<COLUMN_LAST; j++) {
                ctrlFilterSel.AddString(CTSTRING_I(columnNames[j]));
        }
        ctrlFilterSel.SetCurSel(0);

	    border1.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | SS_NOTIFY | SS_OWNERDRAW,  0, IDC_SINK1);
	    border1.SetFont(WinUtil::systemFont, FALSE);
	    border2.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | SS_NOTIFY | SS_OWNERDRAW,  0, IDC_SINK2);
	    border2.SetFont(WinUtil::systemFont, FALSE);
	    border3.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | SS_NOTIFY | SS_OWNERDRAW,  0, IDC_SINK3);
	    border3.SetFont(WinUtil::systemFont, FALSE);
	    border4.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | SS_NOTIFY | SS_OWNERDRAW,  0, IDC_SINK4);
	    border4.SetFont(WinUtil::systemFont, FALSE); 
	    border5.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | SS_NOTIFY | SS_OWNERDRAW,  0, IDC_SINK5);
	    border5.SetFont(WinUtil::systemFont, FALSE);
        
        SettingsManager::getInstance()->addListener(this);

        ctrlStatus.SetText(STATUSBAR_PROGRESS_TEXT, 0, SBT_OWNERDRAW);        
        bHandled = FALSE;
        return 1;
}

LRESULT SearchFrame::onMeasure(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
        HWND hwnd = 0;
        if(wParam == IDC_FILETYPES) return ListMeasure(hwnd, wParam, (MEASUREITEMSTRUCT *)lParam);
                else return OMenu::onMeasureItem(hwnd, uMsg, wParam, lParam, bHandled);
}


BOOL SearchFrame::ListMeasure(HWND /*hwnd*/, UINT /*uCtrlId*/, MEASUREITEMSTRUCT *mis) {
        mis->itemHeight = 16;
        return TRUE;
}


BOOL SearchFrame::ListDraw(HWND /*hwnd*/, UINT /*uCtrlId*/, DRAWITEMSTRUCT *dis) {
        TCHAR szText[MAX_PATH+1];
        int idx;

        switch(dis->itemAction) {
                case ODA_FOCUS:
                        if(!(dis->itemState & 0x0200))
                                DrawFocusRect(dis->hDC, &dis->rcItem);
                        break;

                case ODA_SELECT:
                case ODA_DRAWENTIRE:
                        ctrlFiletype.GetLBText(dis->itemID, szText);
                        if(dis->itemState & ODS_SELECTED) {
                                SetTextColor(dis->hDC, GetSysColor(COLOR_HIGHLIGHTTEXT));
                                SetBkColor(dis->hDC, GetSysColor(COLOR_HIGHLIGHT));
                        } else {
                                SetTextColor(dis->hDC, WinUtil::textColor);
                                SetBkColor(dis->hDC, WinUtil::bgColor);
                        }

                        idx = dis->itemData;
                        ExtTextOut(dis->hDC, dis->rcItem.left+22, dis->rcItem.top+1, ETO_OPAQUE, &dis->rcItem, szText, lstrlen(szText), 0);
                        if(dis->itemState & ODS_FOCUS) {
                                if(!(dis->itemState &  0x0200 ))
                                        DrawFocusRect(dis->hDC, &dis->rcItem);
                        }

                        ImageList_Draw(searchTypes, dis->itemID, dis->hDC,
                                dis->rcItem.left + 2,
                                dis->rcItem.top,
                                ILD_TRANSPARENT);

                        break;
        }
        return TRUE;
}


void SearchFrame::onEnter() {		
	    if ( bStopped == false ) { 
			stopSearch();
        }

        // string to search
        tstring s(ctrlSearch.GetWindowTextLength() + 1, _T('\0'));
        ctrlSearch.GetWindowText(&s[0], s.size());
        s.resize(s.size()-1);

        if ( s.empty() ) {
            updateStatusTexts(true,false,CTSTRING(NO_SEARCH_STRING));
            return;
        }        
        
        // clients to search
        StringList clients;
        int n = ctrlHubs.GetItemCount();
        for(int i = 0; i < n; i++) {
                if(ctrlHubs.GetCheckState(i)) {
                        clients.push_back(Text::fromT(ctrlHubs.getItemData(i)->url));
                }
        }
        if( clients.size() == 0 ) {
            updateStatusTexts(true,false,CTSTRING(NO_HUBS_TO_SEARCH));
            return;
        }

        tstring resultLimit(ctrlResultLimit.GetWindowTextLength() + 1, _T('\0'));
        ctrlResultLimit.GetWindowText(&resultLimit[0], resultLimit.size());
        resultLimit.resize(resultLimit.size()-1);
        maxResult = Util::toInt(Text::fromT(resultLimit));

        tstring size(ctrlSize.GetWindowTextLength() + 1, _T('\0'));
        ctrlSize.GetWindowText(&size[0], size.size());
        size.resize(size.size()-1);

        double lsize = Util::toDouble(Text::fromT(size));
        switch(ctrlSizeMode.GetCurSel()) {
                case 1:
                        lsize*=1024.0; break;
                case 2:
                        lsize*=1024.0*1024.0; break;
                case 3:
                        lsize*=1024.0*1024.0*1024.0; break;
        }

        int64_t llsize = (int64_t)lsize;

        {
                Lock l(cs);
                search = StringTokenizer<tstring>(s, ' ').getTokens();
        }

        //strip out terms beginning with -
        tstring fullSearch = s;
        s.clear();
        for (TStringList::const_iterator si = search.begin(); si != search.end(); ++si)
                if ((*si)[0] != _T('-') || si->size() == 1) s += *si + _T(' '); //Shouldn't get 0-length tokens, so safely assume at least a first char.
        s = s.substr(0, max(s.size(), static_cast<tstring::size_type>(1)) - 1);

        SearchManager::SizeModes mode((SearchManager::SizeModes)ctrlMode.GetCurSel());
        if(llsize == 0)
                mode = SearchManager::SIZE_DONTCARE;

        int ftype = ctrlFiletype.GetCurSel();
        exactSize1 = (mode == SearchManager::SIZE_EXACT);
        exactSize2 = llsize;

        if(BOOLSETTING(CLEAR_SEARCH)){
                ctrlSearch.SetWindowText(_T(""));
        }

        // Add new searches to the last-search dropdown list
        if(find(lastSearches.begin(), lastSearches.end(), s) == lastSearches.end())
        {
                int i = max(SETTING(SEARCH_HISTORY)-1, 0);

                if(ctrlSearchBox.GetCount() > i)
                        ctrlSearchBox.DeleteString(i);
                ctrlSearchBox.InsertString(0, s.c_str());

                while(lastSearches.size() > (TStringList::size_type)i) {
                        lastSearches.erase(lastSearches.begin());
                }
                lastSearches.push_back(s);
        }

        ctrlStopSearch.EnableWindow(true);
        ctrlDoSearch.EnableWindow(false);
        
        // Start the countdown timer...
        // Can this be done in a better way?
        TimerManager::getInstance()->addListener(this);

        if(MainFrame::getMainFrame()->enabled) {
                SearchManager::getInstance()->search(clients, Text::fromT(s), llsize,
                        (SearchManager::TypeModes)ftype, mode, "manual", (int*)this, fullSearch);
                searches++;
        }
}

void SearchFrame::on(SearchManagerListener::SR, SearchResult* aResult) throw() {
        if( bStopped ) {
            return;
        }
        if ( ( maxResult >0 && int(resultsCount+droppedResults) >= maxResult ) ||
			 ( maxResult==0 && searchEndTime <= GET_TICK() ) || 
			 GetAsyncKeyState(VK_ESCAPE) & 0x8000) {
            ctrlDoSearch.EnableWindow(true);
            ctrlStopSearch.EnableWindow(false); 
			stopSearch();
		    return;
	    }

        // Check that this is really a relevant search result...
        {
                Lock l(cs);

                if(search.empty() || MainFrame::getMainFrame()->enabled != true) {
                        return;
                }

                if(isHash) {
                        if(aResult->getType() != SearchResult::TYPE_FILE || TTHValue(Text::fromT(search[0])) != aResult->getTTH()) {
                                droppedResults++;
                                PostMessage(WM_SPEAKER, FILTER_RESULT);
                                return;
                        }
                } else {
                        // match all here
                        for(TStringIter j = search.begin(); j != search.end(); ++j) {
								if((*j->begin() != _T('-') && Util::findSubString(aResult->getFile(), Text::fromT(*j)) == -1)) {
									return;
								}
								if((*j->begin() == _T('-') && j->size() != 1 && Util::findSubString(aResult->getFile(), Text::fromT(j->substr(1))) != -1))
                                {
                                        droppedResults++;										
                                        PostMessage(WM_SPEAKER, FILTER_RESULT);
                                        return;
                                }
                        }
                }
        }

        if(SETTING(PG_ENABLE) && SETTING(PG_SEARCH) && PGLoader::getInstance()->notAbused()) {
                // Reject these, not done in proper place but who cares
                // searching for a file isn't illegal anyway
                string company = PGLoader::getInstance()->getIPBlock(aResult->getIP());
                if (!company.empty()) {
                        droppedResults++;
                        PostMessage(WM_SPEAKER, FILTER_RESULT, NULL);
                        return;
                }
        }

        // Reject results without free slots
        if( (onlyFree && aResult->getFreeSlots() < 1) ||
           (exactSize1 && (aResult->getSize() != exactSize2)))
        {
                droppedResults++;
                PostMessage(WM_SPEAKER, FILTER_RESULT);
                return;
        }

        SearchInfo* i = new SearchInfo(aResult);
        PostMessage(WM_SPEAKER, ADD_RESULT, (LPARAM)i);
}

void SearchFrame::on(SearchManagerListener::Searching, SearchQueueItem* aSearch) throw() {
        if((searches >= 0) && (aSearch->getWindow() == (int*)this)) {
                searches--;
                dcassert(searches >= 0);
                PostMessage(WM_SPEAKER, SEARCH_START, (LPARAM)new SearchQueueItem(*aSearch));
        }
}

void SearchFrame::on(TimerManagerListener::Second, uint32_t aTick) throw() {
        if(searches > 0) {                
                uint32_t waitFor = (((SearchManager::getInstance()->getLastSearch() + (SETTING(MINIMUM_SEARCH_INTERVAL)*1000)) - aTick)/1000) + SETTING(MINIMUM_SEARCH_INTERVAL) * SearchManager::getInstance()->getSearchQueueNumber((int*)this);
                PostMessage(WM_SPEAKER, QUEUE_STATS, (LPARAM)new tstring(Text::tformat(CSTRING(WAITING_FOR), waitFor)));
                PostMessage(WM_SPEAKER, NEXT_SEARCH, (LPARAM)new tstring(Util::formatSeconds(waitFor,true)));
        }        
}

void SearchFrame::SearchInfo::view() {
        try {
                if(sr->getType() == SearchResult::TYPE_FILE) {
                        QueueManager::getInstance()->add(Util::getTempPath() + sr->getFileName(),
                                sr->getSize(), sr->getTTH(), sr->getUser(),
                                QueueItem::FLAG_CLIENT_VIEW | QueueItem::FLAG_TEXT);
                }
        } catch(const Exception&) {
        }
}

void SearchFrame::SearchInfo::Download::operator()(SearchInfo* si) {
        try {
                if(si->sr->getType() == SearchResult::TYPE_FILE) {
                        string target = Text::fromT(tgt + si->columns[COLUMN_FILENAME]);
                        QueueManager::getInstance()->add(target, si->sr->getSize(),
                                si->sr->getTTH(), si->sr->getUser());

                        for(SearchInfo::Iter i = si->subItems.begin(); i != si->subItems.end(); i++) {
                                SearchInfo* j = *i;
                                try {
                                        QueueManager::getInstance()->add(Text::fromT(tgt + si->columns[COLUMN_FILENAME]), j->sr->getSize(), j->sr->getTTH(), j->sr->getUser());
                                } catch(const Exception&) {
                                }
                        }
                        if(WinUtil::isShift())
                                QueueManager::getInstance()->setPriority(target, QueueItem::HIGHEST);
                } else {
                        QueueManager::getInstance()->addDirectory(si->sr->getFile(), si->sr->getUser(), Text::fromT(tgt),
                        WinUtil::isShift() ? QueueItem::HIGHEST : QueueItem::DEFAULT);
                }
        } catch(const Exception&) {
        }
}

void SearchFrame::SearchInfo::DownloadWhole::operator()(SearchInfo* si) {
        try {
                QueueItem::Priority prio = WinUtil::isShift() ? QueueItem::HIGHEST : QueueItem::DEFAULT;
                if(si->sr->getType() == SearchResult::TYPE_FILE) {
                        QueueManager::getInstance()->addDirectory(Text::fromT(si->columns[COLUMN_PATH]),
                        si->sr->getUser(), Text::fromT(tgt), prio);
                } else {
                        QueueManager::getInstance()->addDirectory(si->sr->getFile(), si->sr->getUser(),
                        Text::fromT(tgt), prio);
                }
        } catch(const Exception&) {
        }
}

void SearchFrame::SearchInfo::DownloadTarget::operator()(SearchInfo* si) {
        try {
                if(si->sr->getType() == SearchResult::TYPE_FILE) {
                        string target = Text::fromT(tgt);
                        QueueManager::getInstance()->add(target, si->sr->getSize(),
                                si->sr->getTTH(), si->sr->getUser());

                        if(WinUtil::isShift())
                                QueueManager::getInstance()->setPriority(target, QueueItem::HIGHEST);
                } else {
                        QueueManager::getInstance()->addDirectory(si->sr->getFile(), si->sr->getUser(), Text::fromT(tgt),
                        WinUtil::isShift() ? QueueItem::HIGHEST : QueueItem::DEFAULT);
                }
        } catch(const Exception&) {
        }
}

void SearchFrame::SearchInfo::getList() {
        try {
                WinUtil::addInitalDir(sr->getUser(), Text::fromT(columns[COLUMN_PATH]));
                QueueManager::getInstance()->addList(sr->getUser(), QueueItem::FLAG_CLIENT_VIEW);
        } catch(const Exception&) {
                // Ignore for now...
        }
}

void SearchFrame::SearchInfo::browseList() {
        try {
                QueueManager::getInstance()->addPfs(sr->getUser(), Text::fromT(columns[COLUMN_PATH]));
        } catch(const Exception&) {
                // Ignore for now...
        }
}

void SearchFrame::SearchInfo::CheckTTH::operator()(SearchInfo* si) {
        if(firstTTH) {
                tth = si->columns[COLUMN_TTH];
                hasTTH = true;
                firstTTH = false;
        } else if(hasTTH) {
                if(tth != si->columns[COLUMN_TTH]) {
                        hasTTH = false;
                }
        }

        if(firstHubs && hubs.empty()) {
                hubs = ClientManager::getInstance()->getHubs(si->sr->getUser()->getCID());
                firstHubs = false;
        } else if(!hubs.empty()) {
                // we will merge hubs of all users to ensure we can use OP commands in all hubs
                StringList sl = ClientManager::getInstance()->getHubs(si->sr->getUser()->getCID());
                hubs.insert( hubs.end(), sl.begin(), sl.end() );
                //Util::intersect(hubs, ClientManager::getInstance()->getHubs(si->sr->getUser()->getCID()));
        }
}

tstring SearchFrame::getDownloadDirectory(WORD wID) {
        TargetsMap::const_iterator i = dlTargets.find(wID);
        tstring dir = Text::toT(SETTING(DOWNLOAD_DIRECTORY));
        if (i == dlTargets.end()) return dir;
        if (i->second == _T("‭")) {
                if (WinUtil::browseDirectory(dir, m_hWnd)) {
                        WinUtil::addLastDir(dir);
                        return dir;
                } else {
                        return Util::emptyStringT;
                }
        }
        if (!i->second.empty()) dir = i->second;
        return dir;
}

LRESULT SearchFrame::onDownloadTo(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
        if(ctrlResults.GetSelectedCount() == 1) {
                int i = ctrlResults.GetNextItem(-1, LVNI_SELECTED);
                dcassert(i != -1);
                SearchInfo* si = ctrlResults.getItemData(i);
                SearchResult* sr = si->sr;

                if(sr->getType() == SearchResult::TYPE_FILE) {
                        tstring target = Text::toT(SETTING(DOWNLOAD_DIRECTORY)) + si->columns[COLUMN_FILENAME];
                        if(WinUtil::browseFile(target, m_hWnd)) {
                                WinUtil::addLastDir(Util::getFilePath(target));
                                ctrlResults.forEachSelectedT(SearchInfo::DownloadTarget(target));
                        }
                } else {
                        tstring target = Text::toT(SETTING(DOWNLOAD_DIRECTORY));
                        if(WinUtil::browseDirectory(target, m_hWnd)) {
                                WinUtil::addLastDir(target);
                                ctrlResults.forEachSelectedT(SearchInfo::Download(target));
                        }
                }
        } else {
                tstring target = Text::toT(SETTING(DOWNLOAD_DIRECTORY));
                if(WinUtil::browseDirectory(target, m_hWnd)) {
                        WinUtil::addLastDir(target);
                        ctrlResults.forEachSelectedT(SearchInfo::Download(target));
                }
        }
        return 0;
}

LRESULT SearchFrame::onDownload(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
        tstring dir = getDownloadDirectory(wID);
        if (!dir.empty())
                ctrlResults.forEachSelectedT(SearchInfo::Download(dir));
        return 0;
}

LRESULT SearchFrame::onDownloadWhole(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
        tstring dir = getDownloadDirectory(wID);
        if (!dir.empty())
                ctrlResults.forEachSelectedT(SearchInfo::DownloadWhole(dir));
        return 0;
}

LRESULT SearchFrame::onDownloadTarget(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
        tstring dir = getDownloadDirectory(wID);
        if (!dir.empty())
                ctrlResults.forEachSelectedT(SearchInfo::DownloadTarget(dir));
        return 0;
}

LRESULT SearchFrame::onDoubleClickResults(int /*idCtrl*/, LPNMHDR pnmh, BOOL& /*bHandled*/) {
        LPNMITEMACTIVATE item = (LPNMITEMACTIVATE)pnmh;

        if (item->iItem != -1) {
                CRect rect;
                ctrlResults.GetItemRect(item->iItem, rect, LVIR_ICON);

                // if double click on state icon, ignore...
                if (item->ptAction.x < rect.left)
                        return 0;

                ctrlResults.forEachSelectedT(SearchInfo::Download(Text::toT(SETTING(DOWNLOAD_DIRECTORY))));
        }
        return 0;
}

LRESULT SearchFrame::onClose(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled)
{
        if(!closed) {
				stopSearch();
                SearchManager::getInstance()->removeListener(this);
                ClientManager::getInstance()->removeListener(this);
                SettingsManager::getInstance()->removeListener(this);
                frames.erase(m_hWnd);

                WinUtil::UnlinkStaticMenus(resultsMenu); // !SMT!-UI
                closed = true;
                PostMessage(WM_CLOSE);
                return 0;
        } else {
                ctrlResults.SetRedraw(FALSE);
                ctrlResults.deleteAllItems();
                ctrlResults.SetRedraw(TRUE);

                for(int i = 0; i < ctrlHubs.GetItemCount(); i++) {
                        delete ctrlHubs.getItemData(i);
                }
                ctrlHubs.DeleteAllItems();

                ctrlResults.saveHeaderOrder(SettingsManager::SEARCHFRAME_ORDER, SettingsManager::SEARCHFRAME_WIDTHS,
                        SettingsManager::SEARCHFRAME_VISIBLE);

                bHandled = FALSE;
        return 0;
        }
}

void SearchFrame::UpdateLayout(BOOL bResizeBars)
{
        RECT rect;
        GetClientRect(&rect);
        // position bars and offset their dimensions
        UpdateBarsPosition(rect, bResizeBars);

        if(ctrlStatus.IsWindow()) {
                CRect sr;
                int w[6];
                ctrlStatus.GetClientRect(sr);
                int tmp = (sr.Width()) > 420 ? 376 : ((sr.Width() > 116) ? sr.Width()-100 : 16);
                w[0] = 15;
                w[1] = sr.right - tmp;
                w[2] = w[1] + (tmp-16) / 4;
                w[3] = w[2] + (tmp-16) / 4;
                w[4] = w[3] + (tmp-16) / 4;		
                w[5] = w[4] + (tmp-16) / 4;		
                ctrlStatus.SetParts(6, w);                
                // Layout showUI button in statusbar part #0
                ctrlStatus.GetRect(STATUSBAR_SHOWUI, sr);
                ctrlShowUI.MoveWindow(sr);
        }

        CRect dispose(0,0,0,0);
        if(showUI)
        {            
            int optionsBorder = 0;
	        int const spacing = 42, separator = 25, labelH = 16, optionH = 19, fieldH = 21, optionSpacer = 12, fillerLR = 32,
	            comboH = 140, lMargin = 2, lMarginGrouped = 8, rMargin = 4, rMarginGrouped = 10, midPoint = 6,
	            borderSmall = 10, dropDownH = 17, dropDownSep = 4, dropDownL = 2, dropDownR = 4, contentHearder = 20;
	        
	        int w2 = width - rMargin - lMargin;
	        int wg = width - rMarginGrouped - lMarginGrouped;
	        CRect rc = rect;
	        
	        int const BORDER1H = 250; // "refine results"
	        int const BORDER3H = 220; // "search"
	        int Border4H = 120 + optionsBorder;  // "options"  20-per + 40(top/bottom margins)
	        int const BORDER5H = 110;  // "download options"

	        // draw the result list
	        rc.left += width + 3;
	        ctrlResults.MoveWindow(rc);

	        // draw the slider
	        rc.right = rc.left;
	        rc.left -= 3;
	        rc.top -= lMargin;
	        rc.bottom += lMargin;
	        slider.MoveWindow(rc);
	        slider.Invalidate();

	        rc = rect;

	        // dropdown3  "Search"
	        if(dd3_Checked){
	            rc.top += dropDownSep + midPoint;
	            rc.bottom = rc.top + BORDER3H;
	            rc.left = lMargin;
	            rc.right = width - rMargin;
	        }else{
	            rc.top += dropDownSep + midPoint;
	            rc.bottom = rc.top + borderSmall;
	            rc.left = lMargin;
	            rc.right = width - rMargin;
	        }
	        border3.MoveWindow(rc);

	        int dd3Top = rc.top;
	        int dd3Bottom = rc.bottom;

	        rc.top -= midPoint;
	        rc.bottom = rc.top + dropDownH;
	        rc.left = lMargin + dropDownL;
	        rc.right = width - rMargin - dropDownR;
	        dropDown3.MoveWindow(rc);
	        
	        border3.Invalidate();
	        dropDown3.Invalidate();

	        // dropdown1  "Refine Results"
	        if(dd1_Checked){
	            rc.top = dd3Bottom + dropDownSep + midPoint;
	            rc.bottom = rc.top + BORDER1H;
	            rc.left = lMargin;
	            rc.right = width - rMargin;
	        }else{
	            rc.top = dd3Bottom + dropDownSep + midPoint;
	            rc.bottom = rc.top + borderSmall;
	            rc.left = lMargin;
	            rc.right = width - rMargin;
	        }
	        border1.MoveWindow(rc);

	        int dd1Top = rc.top;
	        int dd1Bottom = rc.bottom;

	        rc.top -= midPoint;
	        rc.bottom = rc.top + dropDownH;
	        rc.left = lMargin + dropDownL;
	        rc.right = width - rMargin - dropDownR;
	        dropDown1.MoveWindow(rc);
	        
	        border1.Invalidate();
	        dropDown1.Invalidate();

	        // see how much space is left under the first two dropdowns
	        int clientBottom = rect.bottom - 5;  
	        int diff = clientBottom - dd1Bottom;
	        
	        // now we calculate the space requirements of our
	        // bottom two dropdowns

	        // start with the size of hublist dropdown + spacer
	        int lowerSize = 20;  

	        // if hublist is opened then add space for minimum
	        // one item of list
	        if(dd2_Checked)
	            lowerSize += 40;
	    
	        // if search options are open then add space for entire dialog
	        // plus remaining size of dropdown + spacer
	        // else just add size of dropdown + spacer
	        if(dd4_Checked)
	            lowerSize += Border4H + borderSmall;
	        else
	            lowerSize += 20;

	        // if download options are open then add space for entire dialog
	        // plus remaining size of dropdown + spacer
	        // else just add size of dropdown + spacer
	        if(dd5_Checked)
	            lowerSize += BORDER5H + borderSmall;
	        else
	            lowerSize += 20;
	        
	        // check if there is space for our drop downs
	        // if not then lower the bottom margin of whole
	        // area to compensate
	        if(diff < lowerSize)
	            clientBottom += (lowerSize - diff);

	        // start with lowest group

	        // dropdown4  "Options"
	        if(dd4_Checked){
	            rc.bottom = clientBottom;
	            rc.top =  rc.bottom - Border4H;
	            rc.left = lMargin;
	            rc.right = width - rMargin;
	        }else{
	            rc.bottom = clientBottom;
	            rc.top = rc.bottom - borderSmall;
	            rc.left = lMargin;
	            rc.right = width - rMargin;
	        }
	        border4.MoveWindow(rc);

	        int dd4Top = rc.top;
	    
	        rc.top -= midPoint;
	        rc.bottom = rc.top + dropDownH;
	        rc.left = lMargin + dropDownL;
	        rc.right = width - rMargin - dropDownR;
	        dropDown4.MoveWindow(rc);

	        border4.Invalidate();
	        dropDown4.Invalidate();

	        // dropdown5  "Download Options"
	        if(dd5_Checked){
	            rc.bottom = dd4Top - midPoint - dropDownSep;
	            rc.top =  rc.bottom - BORDER5H;
	            rc.left = lMargin;
	            rc.right = width - rMargin;
	        }else{
	            rc.bottom = dd4Top - borderSmall;
	            rc.top = rc.bottom - borderSmall;
	            rc.left = lMargin;
	            rc.right = width - rMargin;
	        }
	        border5.MoveWindow(rc);

	        int dd5Top = rc.top;
	    
	        rc.top -= midPoint;
	        rc.bottom = rc.top + dropDownH;
	        rc.left = lMargin + dropDownL;
	        rc.right = width - rMargin - dropDownR;
	        dropDown5.MoveWindow(rc);

	        border5.Invalidate();
	        dropDown5.Invalidate();
	    
	        // dropdown2  "HubList"
	        if(dd2_Checked){
	            rc.bottom = dd5Top - midPoint - dropDownSep;
	            rc.top = dd1Bottom + dropDownSep +  midPoint;
	            rc.left = lMargin;
	            rc.right = width - rMargin;
	        }else{
	            rc.bottom = dd5Top - borderSmall;
	            rc.top = rc.bottom - borderSmall;
	            rc.left = lMargin;
	            rc.right = width - rMargin;
	        }
	        border2.MoveWindow(rc);

	        int dd2Top = rc.top;
	        int dd2Bottom = rc.bottom;

	        rc.top -= midPoint;
	        rc.bottom = rc.top + dropDownH;
	        rc.left = lMargin + dropDownL;
	        rc.right = width - rMargin - dropDownR;
	        dropDown2.MoveWindow(rc);

	        border2.Invalidate();
	        dropDown2.Invalidate();


	        // "Search" contents
	        if(dd3_Checked){
	            
	            rc.top = dd3Top + contentHearder + labelH;

	            // "Search for"
	            rc.right = width - rMarginGrouped;
	            rc.left = lMarginGrouped;
	            rc.bottom = rc.top + fieldH;
	            ctrlSearchBox.MoveWindow(rc);

	            searchLabel.MoveWindow(rc.left + lMargin, rc.top - labelH, width - rMarginGrouped - lMargin - lMarginGrouped, labelH-1);
	                        
	            // "Size"
	            rc.top += spacing;
	            rc.bottom = rc.top + comboH + fieldH;
	            rc.left = lMarginGrouped;
	            rc.right = (LONG)(w2/2.5);
	            ctrlMode.MoveWindow(rc);
	            
	            sizeLabel.MoveWindow(rc.left + lMargin, rc.top - labelH, width - rMarginGrouped - lMargin - rMarginGrouped, labelH-1);
	            
	            // "Size Edit"
	            rc.bottom -= comboH;
	            rc.left = rc.right + lMargin;
	            rc.right += w2/3;            
	            ctrlSize.MoveWindow(rc);
	            
	            // "Size Mode"
	            rc.left = rc.right + lMargin;
	            rc.right = width - rMarginGrouped;
	            rc.bottom += comboH;
	            ctrlSizeMode.MoveWindow(rc);
	            
	            // "File type"
	            rc.left = lMarginGrouped;
	            rc.right = ((width - rMarginGrouped)/3)*2;
	            rc.top += spacing;
	            rc.bottom = rc.top + comboH + fieldH;
	            ctrlFiletype.MoveWindow(rc);

	            typeLabel.MoveWindow(rc.left + lMargin, rc.top - labelH, width - rMarginGrouped - lMargin - rMarginGrouped, labelH-1);

	            // "Purge"
	            rc.left = rc.right + 5;
	            rc.right = width - rMarginGrouped;
	            rc.bottom -= comboH;
	            ctrlPurge.MoveWindow(rc);

	             // "result limit"
	            rc.top += spacing;
	            rc.bottom = rc.top + fieldH;
	            rc.left = lMarginGrouped ;
	            rc.right = width - rMarginGrouped;
	            ctrlResultLimit.MoveWindow(rc);
	            resultLimitLabel.MoveWindow(rc.left + lMargin, rc.top - labelH, width - rMarginGrouped - lMargin - lMarginGrouped, labelH-1);
	            // "Search"
	            rc.top = rc.bottom + 10 ;
	            rc.bottom = rc.top + fieldH;
	            rc.left = lMarginGrouped;
	            rc.right = (width - lMarginGrouped)/2;
	            ctrlDoSearch.MoveWindow(rc);
                
                rc.left = rc.right + lMargin;
                rc.right = width - rMarginGrouped ;
                ctrlStopSearch.MoveWindow(rc);
	        }else{
	            ctrlSearchBox.MoveWindow(dispose);
	            ctrlMode.MoveWindow(dispose);
	            ctrlSize.MoveWindow(dispose);
	            ctrlResultLimit.MoveWindow(dispose);
	            ctrlSizeMode.MoveWindow(dispose);
	            ctrlFiletype.MoveWindow(dispose);
	            ctrlPurge.MoveWindow(dispose);
	            ctrlDoSearch.MoveWindow(dispose);
                ctrlStopSearch.MoveWindow(dispose);
	            searchLabel.MoveWindow(dispose);
	            sizeLabel.MoveWindow(dispose);
	            resultLimitLabel.MoveWindow(dispose);
	            typeLabel.MoveWindow(dispose);
	            
	        }

	        //"Refine results" contents
	        if(dd1_Checked){
	            
	            rc.top = dd1Top + contentHearder;

	            // "Remove items"
	            rc.bottom = rc.top + fieldH;
	            rc.left = lMargin + 8;
	            rc.right = (w2/2) + 8;
//	            removeItemsLabel.MoveWindow(rc);

	            // "with/without"
	            rc.bottom = rc.bottom + comboH;
	            rc.left = rc.right + lMargin;
	            rc.right = width - rMargin - 6;
//	            ctrlFilterBool.MoveWindow(rc);

	            // "Edit"
	            rc.top += separator;
	            rc.bottom = rc.top + fieldH;
	            rc.left = lMarginGrouped;
	            rc.right = (LONG)(w2/2.25) - 5;
	            ctrlFilter.MoveWindow(rc);

	            // "in"
	            rc.left = rc.right + lMargin;
	            rc.right = rc.left +  15;
//	            inLabel.MoveWindow(rc);

	            // "columns"
	            rc.left = rc.right + lMargin;
	            rc.right = width - rMarginGrouped;
	            ctrlFilterSel.MoveWindow(rc);
	                
	            // "remove button"
	            rc.top += separator;
	            rc.bottom = rc.top + fieldH;
	            rc.left = lMarginGrouped;
	            rc.right = width - rMarginGrouped;
//	            ctrlGo.MoveWindow(rc);

	            // operations list
	            rc.top += spacing;
	            rc.bottom = rc.top + 100;
	            rc.left = lMarginGrouped;
	            rc.right = width - rMarginGrouped;
//	            ctrlList.MoveWindow(rc);
	            
	            // "filter title"
//	            filterTitle.MoveWindow(rc.left + lMargin, rc.top - labelH, width - rMarginGrouped - lMargin - lMarginGrouped, labelH-1);
	        
//	            int listWidth = rc.right - rc.left - 3;
//	            ctrlList.SetColumnWidth(0,listWidth/3);
//	            ctrlList.SetColumnWidth(1,listWidth/3);
//	            ctrlList.SetColumnWidth(2,listWidth/3);

	            // "Results label"
	            rc.top = rc.bottom + 6;
	            rc.bottom = rc.top + fieldH;
	            rc.left = lMarginGrouped;
	            rc.right = (wg/3);
//	            resultsLabel.MoveWindow(rc);

	            // "Undo all"
	            rc.left = rc.right + lMargin;
	            rc.right = rc.left + (wg/3) + 5;
//	            ctrlRevert.MoveWindow(rc);

	            // "delete"
	            rc.left = rc.right + lMargin;
	            rc.right = width - rMarginGrouped;
//	            ctrlTrash.MoveWindow(rc);

	        }else{
	            ctrlFilterSel.MoveWindow(dispose);
//	            ctrlGo.MoveWindow(dispose);
//	            ctrlFilterBool.MoveWindow(dispose);
	            ctrlFilter.MoveWindow(dispose);
//	            resultsLabel.MoveWindow(dispose);
//	            ctrlRevert.MoveWindow(dispose);
//	            ctrlTrash.MoveWindow(dispose);
//	            removeItemsLabel.MoveWindow(dispose);
//	            inLabel.MoveWindow(dispose);
//	            filterTitle.MoveWindow(dispose);
//	            ctrlList.MoveWindow(dispose);
	        }
	    

	        //"Hub List" contents
	        if(dd2_Checked){

	            rc.top = dd2Top + labelH;
	                        
	            // "Hubs"
	            rc.bottom = dd2Bottom - midPoint;
	            rc.left = lMarginGrouped;
	            rc.right = width - rMarginGrouped;
	            
	            if(rc.bottom < rc.top + (labelH*3)/2)
	                rc.bottom = rc.top + (labelH*3)/2;
	            
	            ctrlHubs.MoveWindow(rc);
	        }else{
	            ctrlHubs.MoveWindow(dispose);
	        }

	        //"Download Options" contents
	        if(dd5_Checked){
	          
	            rc.top = dd5Top + contentHearder + labelH;

	            // "Prio type"
	            rc.left = lMarginGrouped;
	            rc.right =  width - rMarginGrouped;
	            rc.bottom = rc.top + comboH + fieldH;
//	            ctrlPriority.MoveWindow(rc);

//	            prioLabel.MoveWindow(rc.left + lMargin, rc.top - labelH, width - rMarginGrouped - lMargin - rMarginGrouped, labelH-1);

	            // "Combo Dirs"
	            rc.left = lMarginGrouped;
	            rc.right = ((width - rMarginGrouped)/3)*2 + 12;
	            rc.top += spacing;
	            rc.bottom = rc.top + comboH + fieldH;
//	            ctrlDirs.MoveWindow(rc);

//	            dirsLabel.MoveWindow(rc.left + lMargin, rc.top - labelH, width - rMarginGrouped - lMargin - rMarginGrouped, labelH-1);

	            // "Add"
	            rc.left = rc.right + 5;
	            rc.right = width - rMarginGrouped;
	            rc.bottom -= comboH;
//	            ctrlAdd.MoveWindow(rc);

	        }else{
//	            ctrlDirs.MoveWindow(dispose);
//	            ctrlAdd.MoveWindow(dispose);
//	            dirsLabel.MoveWindow(dispose);
//	            ctrlPriority.MoveWindow(dispose);
//	            prioLabel.MoveWindow(dispose);
	        }


	        //"Options" contents
	        if(dd4_Checked){
	            rc.top = dd4Top + labelH;
	            optionsBorder = 0;
	            
	            // "Free Slots"
	            rc.bottom = rc.top + optionH ;
	            rc.left = lMarginGrouped;
	            rc.right = width - rMarginGrouped;

	            if((rc.right - rc.left) + fillerLR < WinUtil::getTextWidth(CTSTRING(ONLY_FREE_SLOTS), m_hWnd)){
	                rc.bottom += optionSpacer;
	                optionsBorder += optionSpacer;
	            }
	            ctrlSlots.MoveWindow(rc);

	            // "OnlyTTH"
	            //rc.left = lMarginGrouped ;
	            //rc.right = width - rMarginGrouped;
	            //rc.top = rc.bottom;
	            //rc.bottom = rc.top + optionH;
	            //if((rc.right - rc.left) + fillerLR < WinUtil::getTextWidth(CTSTRING(ONLY_TTH), m_hWnd)){
	            //    rc.bottom += optionSpacer;
	            //    optionsBorder += optionSpacer;
	            //}
	            //ctrlTTH.MoveWindow(rc);
	    
	            // "Auto Close groups"
	            rc.left = lMarginGrouped ;
	            rc.right = width - rMarginGrouped;
	            rc.top = rc.bottom;
	            rc.bottom = rc.top + optionH;

	            if((rc.right - rc.left) + fillerLR < WinUtil::getTextWidth(CTSTRING(AUTO_CLOSE_GROUPS), m_hWnd)){
	                rc.bottom += optionSpacer;
	                optionsBorder += optionSpacer;
	            }
	            ctrlAutoClose.MoveWindow(rc);
	            
	            rc.left = lMarginGrouped ;
	            rc.right = width - rMarginGrouped;
	            rc.top = rc.bottom;
	            rc.bottom = rc.top + optionH;
                ctrlOneSearchWindow.MoveWindow(rc);

                // "Remember Group Positions"
	            rc.left = lMarginGrouped ;
	            rc.right = width - rMarginGrouped;
	            rc.top = rc.bottom;
	            rc.bottom = rc.top + optionH;
	            
//	            if((rc.right - rc.left) + fillerLR < WinUtil::getTextWidth(CTSTRING(REM_POS), m_hWnd)){
//	                rc.bottom += optionSpacer;
//	                optionsBorder += optionSpacer;
//	            }
//	            ctrlRememberPos.MoveWindow(rc);

	            // "Group Wise"
	            rc.left = lMarginGrouped ;
	            rc.right = width - rMarginGrouped;
	            rc.top = rc.bottom;
	            rc.bottom = rc.top + optionH;
	            
//	            if((rc.right - rc.left) + fillerLR < WinUtil::getTextWidth(CTSTRING(GROUP_WISE), m_hWnd)){
//	                rc.bottom += optionSpacer;
//	                optionsBorder += optionSpacer;
//	            }
//	            ctrlGroupWise.MoveWindow(rc);
	        }else{
//	            ctrlGroupWise.MoveWindow(dispose);
//	            ctrlRememberPos.MoveWindow(dispose);
//	            ctrlTTH.MoveWindow(dispose);
	            ctrlSlots.MoveWindow(dispose);
                ctrlOneSearchWindow.MoveWindow(dispose);
	            ctrlAutoClose.MoveWindow(dispose);
	        }
        }
        else
        {
                CRect rc = rect;
                ctrlResults.MoveWindow(rc);
                ctrlStopSearch.MoveWindow(dispose);
    		    //ctrlGroupWise.MoveWindow(dispose);
    		    //ctrlDirs.MoveWindow(dispose);
    		    //ctrlAdd.MoveWindow(dispose);
    		    //dirsLabel.MoveWindow(dispose);
    		    //ctrlRememberPos.MoveWindow(dispose);
    		    //ctrlTTH.MoveWindow(dispose);
    		    ctrlSlots.MoveWindow(dispose);
                ctrlOneSearchWindow.MoveWindow(dispose);
    		    ctrlDoSearch.MoveWindow(dispose);
    		    searchLabel.MoveWindow(dispose);
    		    sizeLabel.MoveWindow(dispose);
    		    resultLimitLabel.MoveWindow(dispose);
    		    //prioLabel.MoveWindow(dispose);
    		    typeLabel.MoveWindow(dispose);
    		    //ctrlGo.MoveWindow(dispose);
    		    ctrlFilter.MoveWindow(dispose);
    		    ctrlFilterSel.MoveWindow(dispose);
    		    //ctrlFilterBool.MoveWindow(dispose);
    		    //ctrlRevert.MoveWindow(dispose);
    		    //resultsLabel.MoveWindow(dispose);
    		    border1.MoveWindow(dispose);
    		    //removeItemsLabel.MoveWindow(dispose);
    		    //inLabel.MoveWindow(dispose);
    		    //ctrlTrash.MoveWindow(dispose);
    		    ctrlHubs.MoveWindow(dispose);
    		    ctrlSearchBox.MoveWindow(dispose);
    		    ctrlMode.MoveWindow(dispose);
    		    ctrlSize.MoveWindow(dispose);
    		    ctrlResultLimit.MoveWindow(dispose);
    		    ctrlSizeMode.MoveWindow(dispose);
    		    ctrlFiletype.MoveWindow(dispose);
    		    //ctrlPriority.MoveWindow(dispose);
    		    ctrlPurge.MoveWindow(dispose);
    		    dropDown1.MoveWindow(dispose);
    		    border1.MoveWindow(dispose);
    		    dropDown2.MoveWindow(dispose);
    		    border2.MoveWindow(dispose);
    		    dropDown3.MoveWindow(dispose);
    		    border3.MoveWindow(dispose);
    		    dropDown4.MoveWindow(dispose);
    		    border4.MoveWindow(dispose);
    		    dropDown5.MoveWindow(dispose);
    		    border5.MoveWindow(dispose);
    		    //ctrlTTH.MoveWindow(dispose);
    		    ctrlSlots.MoveWindow(dispose);
                ctrlOneSearchWindow.MoveWindow(dispose);
    		    slider.MoveWindow(dispose);
    		    //filterTitle.MoveWindow(dispose);
    		    //ctrlList.MoveWindow(dispose);
    		    ctrlAutoClose.MoveWindow(dispose);

        }

        POINT pt;
        pt.x = 10;
        pt.y = 10;
        HWND hWnd = ctrlSearchBox.ChildWindowFromPoint(pt);
        if(hWnd != NULL && !ctrlSearch.IsWindow() && hWnd != ctrlSearchBox.m_hWnd) {
                ctrlSearch.Attach(hWnd);
                searchContainer.SubclassWindow(ctrlSearch.m_hWnd);
        }
}

void SearchFrame::runUserCommand(UserCommand& uc) {
        if(!WinUtil::getUCParams(m_hWnd, uc, ucLineParams))
                return;

        StringMap ucParams = ucLineParams;

        set<CID> users;

        int sel = -1;
        while((sel = ctrlResults.GetNextItem(sel, LVNI_SELECTED)) != -1) {
                SearchResult* sr = ctrlResults.getItemData(sel)->sr;

                if(!sr->getUser()->isOnline())
                        continue;

                if(uc.getType() == UserCommand::TYPE_RAW_ONCE) {
                        if(users.find(sr->getUser()->getCID()) != users.end())
                                continue;
                        users.insert(sr->getUser()->getCID());
                }

                ucParams["fileFN"] = sr->getFile();
                ucParams["fileSI"] = Util::toString(sr->getSize());
                ucParams["fileSIshort"] = Util::formatBytes(sr->getSize());
                if(sr->getType() == SearchResult::TYPE_FILE) {
                        ucParams["fileTR"] = sr->getTTH().toBase32();
                }

                // compatibility with 0.674 and earlier
                ucParams["file"] = ucParams["fileFN"];
                ucParams["filesize"] = ucParams["fileSI"];
                ucParams["filesizeshort"] = ucParams["fileSIshort"];
                ucParams["tth"] = ucParams["fileTR"];

                StringMap tmp = ucParams;
                ClientManager::getInstance()->userCommand(sr->getUser(), uc, tmp, true);
        }
}

LRESULT SearchFrame::onCtlColor(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/) {
        HWND hWnd = (HWND)lParam;
        HDC hDC = (HDC)wParam;

        if(hWnd == searchLabel.m_hWnd || hWnd == sizeLabel.m_hWnd || hWnd == optionLabel.m_hWnd || hWnd == typeLabel.m_hWnd
                || hWnd == hubsLabel.m_hWnd || hWnd == ctrlSlots.m_hWnd || hWnd == ctrlCollapsed.m_hWnd || hWnd == srLabel.m_hWnd
                || hWnd == resultLimitLabel.m_hWnd || hWnd ==  ctrlAutoClose.m_hWnd || hWnd == ctrlOneSearchWindow.m_hWnd ) {
                ::SetBkColor(hDC, ::GetSysColor(COLOR_3DFACE));
                ::SetTextColor(hDC, ::GetSysColor(COLOR_BTNTEXT));
                return (LRESULT)::GetSysColorBrush(COLOR_3DFACE);
        } else {
                ::SetBkColor(hDC, WinUtil::bgColor);
                ::SetTextColor(hDC, WinUtil::textColor);
                return (LRESULT)WinUtil::bgBrush;
        }
}

LRESULT SearchFrame::onChar(UINT uMsg, WPARAM wParam, LPARAM /*lParam*/, BOOL& bHandled) {
        switch(wParam) {
        case VK_TAB:
                if(uMsg == WM_KEYDOWN) {
                        onTab(WinUtil::isShift());
                }
                break;
        case VK_RETURN:
                if( WinUtil::isShift() || WinUtil::isCtrl() || WinUtil::isAlt() ) {
                        bHandled = FALSE;
                } else {
                        if(uMsg == WM_KEYDOWN) {
                                onEnter();
                        }
                }
                break;
        default:
                bHandled = FALSE;
        }
        return 0;
}

void SearchFrame::onTab(bool shift) {
        HWND wnds[] = {
                ctrlSearch.m_hWnd, ctrlPurge.m_hWnd, ctrlMode.m_hWnd, ctrlSize.m_hWnd, ctrlSizeMode.m_hWnd,
                ctrlFiletype.m_hWnd, ctrlSlots.m_hWnd, ctrlCollapsed.m_hWnd, ctrlDoSearch.m_hWnd, ctrlSearch.m_hWnd,
                ctrlResults.m_hWnd
        };

        HWND focus = GetFocus();
        if(focus == ctrlSearchBox.m_hWnd)
                focus = ctrlSearch.m_hWnd;

        static const int size = sizeof(wnds) / sizeof(wnds[0]);
        int i;
        for(i = 0; i < size; i++) {
                if(wnds[i] == focus)
                        break;
        }

        ::SetFocus(wnds[(i + (shift ? -1 : 1)) % size]);
}

LRESULT SearchFrame::onSearchByTTH(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
        // !SMT!-UI
        int i = -1;
        while((i = ctrlResults.GetNextItem(i, LVNI_SELECTED)) != -1) {
                SearchResult* sr = ctrlResults.getItemData(i)->sr;
                if(sr->getType() == SearchResult::TYPE_FILE) {
                        WinUtil::searchHash(sr->getTTH());
                }
        }
        return 0;
}

LRESULT SearchFrame::onBitziLookup(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
        if(ctrlResults.GetSelectedCount() == 1) {
                int i = ctrlResults.GetNextItem(-1, LVNI_SELECTED);
                SearchResult* sr = ctrlResults.getItemData(i)->sr;
                if(sr->getType() == SearchResult::TYPE_FILE) {
                        WinUtil::bitziLink(sr->getTTH());
                }
        }
        return 0;
}

LRESULT SearchFrame::onSpeaker(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/) {
        switch(wParam) {
        case ADD_RESULT:
                {
                        SearchInfo* si = (SearchInfo*)lParam;
                        SearchResult* sr = si->sr;
            // Check previous search results for dupes
                        if(!si->columns[COLUMN_TTH].empty() && useGrouping) {
                                SearchInfo* main = ctrlResults.findMainItem(si->columns[COLUMN_TTH]);
                                if(main) {
                                        if((sr->getUser()->getCID() == main->sr->getUser()->getCID()) && (sr->getFile() == main->sr->getFile())) {
                                                delete si;
                                                return 0;
                                        }
                                        for(SearchInfo::Iter k = main->subItems.begin(); k != main->subItems.end(); k++){
                                                if((sr->getUser()->getCID() == (*k)->getUser()->getCID()) && (sr->getFile() == (*k)->sr->getFile())) {
                                                        delete si;
                                                        return 0;
                                                }
                                        }
                                }
                        } else {
                                for(vector<SearchInfo*>::const_iterator s = ctrlResults.mainItems.begin(); s != ctrlResults.mainItems.end(); ++s) {
                                        SearchInfo* si2 = *s;
                        SearchResult* sr2 = si2->sr;
                                        if((sr->getUser()->getCID() == sr2->getUser()->getCID()) && (sr->getFile() == sr2->getFile())) {
                                                delete si;
                                        return 0;
                                        }
                                }
                        }
                        if(bStopped == false) {
                                bool resort = false;
                                if(resultsCount++ % 13 == 0) {
                                        //ctrlResults.SetRedraw(FALSE);
                                        resort = true;
                                }

                                if(!si->columns[COLUMN_TTH].empty() && useGrouping) {
                                        ctrlResults.insertGroupedItem(si, expandSR);
                                } else {
                                        ctrlResults.insertItem(si, si->imageIndex());
                                        ctrlResults.mainItems.push_back(si);
                                }

                                if(!filter.empty())
                                        updateSearchList(si);

                                if (BOOLSETTING(BOLD_SEARCH)) {
                                        setDirty();
                                }

                                if(resort) {
                                        ctrlResults.resort();
                                        //ctrlResults.SetRedraw(TRUE);
                                }
                        } else {
                        }
                }
                updateStatusTexts();
                break;
        case FILTER_RESULT:
                updateStatusTexts();
                break;
        case HUB_ADDED:
                        onHubAdded((HubInfo*)(lParam));
                break;
        case HUB_CHANGED:
                        onHubChanged((HubInfo*)(lParam));
                break;
        case HUB_REMOVED:
                        onHubRemoved((HubInfo*)(lParam));
                break;
        case QUEUE_STATS:
                {
                        tstring* t = (tstring*)(lParam);                        
                        updateStatusTexts(true,true,*t);
                        delete t;
                }
                break;
        case NEXT_SEARCH:
                {
                        tstring* t = (tstring*)(lParam);                        
                        ctrlStatus.SetText(STATUSBAR_NEXT_SEARCH, t->c_str());
                        delete t;
                }
                break;
        case SEARCH_START:
                {
                        SearchQueueItem* aSearch = (SearchQueueItem*)(lParam);

                        ctrlResults.deleteAllItems();
                        resultsCount = 0;
                        bStopped = false;
                        searchStartTime = GET_TICK();
                        searchEndTime = searchStartTime + SETTING(MINIMUM_SEARCH_INTERVAL)*1000;
                        {
                                Lock l(cs);
                                search = StringTokenizer<tstring>(aSearch->getSearch(), ' ').getTokens();
                        }
                        isHash = (aSearch->getTypeMode() == SearchManager::TYPE_TTH);

                        // Turn off the countdown timer if no more manual searches left
                        if(searches == 0)
                                TimerManager::getInstance()->removeListener(this);

                        // Update the status bar                                                
                        if(MainFrame::getMainFrame()->enabled) {
                            updateStatusTexts(true,true,Text::tformat("%s %s...",STRING(SEARCHING_FOR).c_str(),aSearch->getTarget().c_str()));
                        }
                        else {
                            updateStatusTexts(true,true,Text::toT("Search is disabled!"));
                        }

                        if(MainFrame::getMainFrame()->enabled)
                                SetWindowText(Text::toT(STRING(SEARCH) + " - " + aSearch->getTarget()).c_str());
                        delete aSearch;

                        droppedResults = 0;
                }
                break;
        }

        return 0;
}

LRESULT SearchFrame::onContextMenu(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
        if (reinterpret_cast<HWND>(wParam) == ctrlResults && ctrlResults.GetSelectedCount() > 0) {
                POINT pt = { GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam) };

                if(pt.x == -1 && pt.y == -1) {
                        WinUtil::getContextMenuPos(ctrlResults, pt);
                }

                if(ctrlResults.GetSelectedCount() > 0) {

                        updateSummary(); // !SMT!-UI
                        dlTargets.clear(); // !SMT!-S

                        SearchInfo* si = NULL;
                        SearchResult* sr = NULL;
                        if(ctrlResults.GetSelectedCount() == 1) {
                                int i = ctrlResults.GetNextItem(-1, LVNI_SELECTED);
                                dcassert(i != -1);
                                si = ctrlResults.getItemData(i);
                                sr = si->sr;
                        }

                        // first sub-menu
                        while(targetMenu.GetMenuItemCount() > 0) {
                                targetMenu.DeleteMenu(0, MF_BYPOSITION);
                        }

                        dlTargets[IDC_DOWNLOAD_FAVORITE_DIRS + 0] = Util::emptyStringT; // for 'Download' without options

                        targetMenu.InsertSeparatorFirst(TSTRING(DOWNLOAD_TO));
                        //Append favorite download dirs
                        StringPairList spl = FavoriteManager::getInstance()->getFavoriteDirs();
                        int n = 1;
                        if (spl.size() > 0) {
                                for(StringPairIter i = spl.begin(); i != spl.end(); i++) {
                                        const tstring tar = Text::toT(i->second); // !SMT!-S
                                        dlTargets[IDC_DOWNLOAD_FAVORITE_DIRS + n] = Text::toT(i->first);
                                        targetMenu.AppendMenu(MF_STRING, IDC_DOWNLOAD_FAVORITE_DIRS + n, tar.c_str());
                                        n++;
                                }
                                targetMenu.AppendMenu(MF_SEPARATOR);
                        }

                        // !SMT!-S: Append special folder, like in source share
                        if (si) {
                                tstring srcpath = si->columns[COLUMN_PATH];
                                if (srcpath.length() > 2) {
                                        unsigned start = srcpath.substr(0,srcpath.length()-1).find_last_of(_T('\\'));
                                        if (start == srcpath.npos) start = 0;
                                        srcpath = Text::toT(SETTING(DOWNLOAD_DIRECTORY)) + srcpath.substr(start+1);
                                        dlTargets[IDC_DOWNLOAD_FAVORITE_DIRS + n] = srcpath;
                                        targetMenu.AppendMenu(MF_STRING, IDC_DOWNLOAD_FAVORITE_DIRS + n, srcpath.c_str());
                                        n++;
                                }
                        }

                        targetMenu.AppendMenu(MF_STRING, IDC_DOWNLOADTO, CTSTRING(BROWSE));
                        n++;

                        //Append last favorite download dirs
                        if(WinUtil::lastDirs.size() > 0) {
                                targetMenu.InsertSeparatorLast(TSTRING(PREVIOUS_FOLDERS));
                                for(TStringIter i = WinUtil::lastDirs.begin(); i != WinUtil::lastDirs.end(); ++i) {
                                        const tstring& tar = *i; // !SMT!-S
                                        dlTargets[IDC_DOWNLOAD_FAVORITE_DIRS + n] = tar;
                                        targetMenu.AppendMenu(MF_STRING, IDC_DOWNLOAD_FAVORITE_DIRS + n, tar.c_str());
                                        n++;
                                }
                        }

                        SearchInfo::CheckTTH cs = ctrlResults.forEachSelectedT(SearchInfo::CheckTTH());

                        if(cs.hasTTH) {
                                targets.clear();
                                QueueManager::getInstance()->getTargets(TTHValue(Text::fromT(cs.tth)), targets);

                                if(targets.size() > 0) {
                                        targetMenu.InsertSeparatorLast(TSTRING(ADD_AS_SOURCE));
                                        for(StringIter i = targets.begin(); i != targets.end(); ++i) {
                                                const tstring& tar = Text::toT(*i); // !SMT!-S
                                                dlTargets[IDC_DOWNLOAD_TARGET + n] = tar;
                                                targetMenu.AppendMenu(MF_STRING, IDC_DOWNLOAD_TARGET + n, tar.c_str());
                                                n++;
                                        }
                                }
                        }

                        // second sub-menu
                        while(targetDirMenu.GetMenuItemCount() > 0) {
                                targetDirMenu.DeleteMenu(0, MF_BYPOSITION);
                        }

                        dlTargets[IDC_DOWNLOAD_WHOLE_FAVORITE_DIRS + 0] = Util::emptyStringT; // for 'Download whole dir' without options
                        targetDirMenu.InsertSeparatorFirst(TSTRING(DOWNLOAD_WHOLE_DIR_TO));
                        //Append favorite download dirs
                        n = 1;
                        if (spl.size() > 0) {
                                for(StringPairIter i = spl.begin(); i != spl.end(); ++i) {
                                        const tstring tar = Text::toT(i->second); // !SMT!-S
                                        dlTargets[IDC_DOWNLOAD_WHOLE_FAVORITE_DIRS + n] = Text::toT(i->first);
                                        targetDirMenu.AppendMenu(MF_STRING, IDC_DOWNLOAD_WHOLE_FAVORITE_DIRS + n, tar.c_str());
                                        n++;
                                }
                                targetDirMenu.AppendMenu(MF_SEPARATOR);
                        }

                        dlTargets[IDC_DOWNLOAD_WHOLE_FAVORITE_DIRS + n] = _T("‭");
                        targetDirMenu.AppendMenu(MF_STRING, IDC_DOWNLOAD_WHOLE_FAVORITE_DIRS + n, CTSTRING(BROWSE));
                        n++;

                        if(WinUtil::lastDirs.size() > 0) {
                                targetDirMenu.AppendMenu(MF_SEPARATOR);
                                for(TStringIter i = WinUtil::lastDirs.begin(); i != WinUtil::lastDirs.end(); ++i) {
                                        const tstring tar = *i; // !SMT!-S
                                        dlTargets[IDC_DOWNLOAD_WHOLE_FAVORITE_DIRS + n] = tar;
                                        targetDirMenu.AppendMenu(MF_STRING, IDC_DOWNLOAD_WHOLE_FAVORITE_DIRS + n, i->c_str());
                                        n++;
                                }
                        }

                        prepareMenu(resultsMenu, UserCommand::CONTEXT_SEARCH, cs.hubs);
                        checkAdcItems(resultsMenu);

                        copyMenu.InsertSeparatorFirst(TSTRING(USERINFO));
                        resultsMenu.InsertSeparatorFirst(sr? Text::toT(sr->getFileName()) : CTSTRING(FILES));
                        resultsMenu.TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, pt.x, pt.y, m_hWnd);
                        resultsMenu.RemoveFirstItem();
                        copyMenu.RemoveFirstItem();

                        cleanMenu(resultsMenu);
                        return TRUE;
                }
        }
        bHandled = FALSE;
        return FALSE;
}

void SearchFrame::initHubs() {
        ctrlHubs.insertItem(new HubInfo(Util::emptyStringT, TSTRING(ONLY_WHERE_OP), false), 0);
        ctrlHubs.SetCheckState(0, false);

        {
                ClientManager::LockInstance l; // !PPA!
                ClientManager::getInstance()->addListener(this);
                Client::List& clients = ClientManager::getInstance()->getClients();
                Client::List::const_iterator it;
                Client::List::const_iterator endIt = clients.end();
                for(it = clients.begin(); it != endIt; ++it) {
                        Client* client = *it;
                        if (!client->isConnected())
                                continue;

                        onHubAdded(new HubInfo(Text::toT(client->getHubUrl()), Text::toT(client->getHubName()), client->getMyIdentity().isOp()));
                }
        }

        ctrlHubs.SetColumnWidth(0, LVSCW_AUTOSIZE);

}

void SearchFrame::onHubAdded(HubInfo* info) {
        int nItem = ctrlHubs.insertItem(info, 0);
        ctrlHubs.SetCheckState(nItem, (ctrlHubs.GetCheckState(0) ? info->op : true));
        ctrlHubs.SetColumnWidth(0, LVSCW_AUTOSIZE);
}

void SearchFrame::onHubChanged(HubInfo* info) {
        int nItem = 0;
        int n = ctrlHubs.GetItemCount();
        for(; nItem < n; nItem++) {
                if(ctrlHubs.getItemData(nItem)->url == info->url)
                        break;
        }
        if (nItem == n)
                return;

        delete ctrlHubs.getItemData(nItem);
        ctrlHubs.SetItemData(nItem, (DWORD_PTR)info);
        ctrlHubs.updateItem(nItem);

        if (ctrlHubs.GetCheckState(0))
                ctrlHubs.SetCheckState(nItem, info->op);

        ctrlHubs.SetColumnWidth(0, LVSCW_AUTOSIZE);
}

void SearchFrame::onHubRemoved(HubInfo* info) {
        int nItem = 0;
        int n = ctrlHubs.GetItemCount();
        for(; nItem < n; nItem++) {
                if(ctrlHubs.getItemData(nItem)->url == info->url)
                        break;
        }

        delete info;

        if (nItem == n)
                return;

        delete ctrlHubs.getItemData(nItem);
        ctrlHubs.DeleteItem(nItem);
        ctrlHubs.SetColumnWidth(0, LVSCW_AUTOSIZE);
}

LRESULT SearchFrame::onGetList(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
        ctrlResults.forEachSelected(&SearchInfo::getList);
        return 0;
}

LRESULT SearchFrame::onBrowseList(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
        ctrlResults.forEachSelected(&SearchInfo::browseList);
        return 0;
}

LRESULT SearchFrame::onItemChangedHub(int /* idCtrl */, LPNMHDR pnmh, BOOL& /* bHandled */) {
        NMLISTVIEW* lv = (NMLISTVIEW*)pnmh;
        if(lv->iItem == 0 && (lv->uNewState ^ lv->uOldState) & LVIS_STATEIMAGEMASK) {
                if (((lv->uNewState & LVIS_STATEIMAGEMASK) >> 12) - 1) {
                        for(int iItem = 0; (iItem = ctrlHubs.GetNextItem(iItem, LVNI_ALL)) != -1; ) {
                                HubInfo* client = ctrlHubs.getItemData(iItem);
                                if (!client->op)
                                        ctrlHubs.SetCheckState(iItem, false);
                        }
                }
        }

        return 0;
}

LRESULT SearchFrame::onPurge(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
        ctrlSearchBox.ResetContent();
        lastSearches.clear();
        return 0;
}


void SearchFrame::SearchInfo::update() {
        if(sr->getType() == SearchResult::TYPE_FILE) {
                if(sr->getFile().rfind(_T('\\')) == tstring::npos) {
                        columns[COLUMN_FILENAME] = Text::toT(sr->getFile());
                } else {
                        columns[COLUMN_FILENAME] = Text::toT(Util::getFileName(sr->getFile()));
                        columns[COLUMN_PATH] = Text::toT(Util::getFilePath(sr->getFile()));
                }

                columns[COLUMN_TYPE] = Text::toT(Util::getFileExt(Text::fromT(columns[COLUMN_FILENAME])));
                if(!columns[COLUMN_TYPE].empty() && columns[COLUMN_TYPE][0] == _T('.'))
                        columns[COLUMN_TYPE].erase(0, 1);
                columns[COLUMN_SIZE] = _T("‭") + Util::formatBytesW(sr->getSize());
                columns[COLUMN_EXACT_SIZE] = _T("‭") + Util::formatExactSize(sr->getSize());
        } else {
                columns[COLUMN_FILENAME] = Text::toT(sr->getFileName());
                columns[COLUMN_PATH] = Text::toT(sr->getFile());
                columns[COLUMN_TYPE] = TSTRING(DIRECTORY);
                if(sr->getSize() > 0) {
                        columns[COLUMN_SIZE] = _T("‭") + Util::formatBytesW(sr->getSize());
                        columns[COLUMN_EXACT_SIZE] = _T("‭") + Util::formatExactSize(sr->getSize());
                }
        }
        columns[COLUMN_NICK] = Text::toT(sr->getUser()->getFirstNick());
        columns[COLUMN_CONNECTION] = Text::toT(ClientManager::getInstance()->getConnection(sr->getUser()->getCID()));
        columns[COLUMN_HUB] = Text::toT(sr->getHubName());
        columns[COLUMN_SLOTS] = Text::toT(sr->getSlotString());
        columns[COLUMN_IP] = Text::toT(sr->getIP());
        columns[COLUMN_DNS] = Text::toT(Socket::nslookup(sr->getIP()));
        flagimage = 0;
        if (!columns[COLUMN_IP].empty()) {
                // Only attempt to grab a country mapping if we actually have an IP address
                string aCountry = Util::getIpCountry(sr->getIP());
                if(!aCountry.empty()) {
                        flagimage = WinUtil::getFlagImage(aCountry);
                        columns[COLUMN_LOCATION] = Text::toT(aCountry);
                }
        }
        if(sr->getType() == SearchResult::TYPE_FILE) {
                columns[COLUMN_TTH] = Text::toT(sr->getTTH().toBase32());
        }
        if (sr->getUser()->getLastDownloadSpeed() > 0) {
                columns[COLUMN_UPLOAD] = Util::formatBytesW(sr->getUser()->getLastDownloadSpeed()) + _T("/s");
        } else if(user->isSet(User::FIREBALL)) {
                columns[COLUMN_UPLOAD] = Text::toT(">=100 kB/s");
        } else {
                columns[COLUMN_UPLOAD] = Text::toT("N/A");
        }
}

LRESULT SearchFrame::onCopy(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
        string data;
        // !SMT!-UI: copy several rows
        int i = -1;
        while( (i = ctrlResults.GetNextItem(i, LVNI_SELECTED)) != -1) {
                SearchResult* sr = ctrlResults.getItemData(i)->sr;
                string sCopy;
                switch (wID) {
                        case IDC_COPY_NICK:
                                sCopy = sr->getUser()->getFirstNick();
                                break;
                        case IDC_COPY_FILENAME:
                                sCopy = Util::getFileName(sr->getFile());
                                break;
                        case IDC_COPY_PATH:
                                sCopy = Util::getFilePath(sr->getFile());
                                break;
                        case IDC_COPY_SIZE:
                                sCopy = Util::formatBytes(sr->getSize());
                                break;
                        case IDC_COPY_LINK:
                                if(sr->getType() == SearchResult::TYPE_FILE)
                                        sCopy = WinUtil::getMagnet(sr->getTTH(), Text::toT(sr->getFileName()), sr->getSize());
                                break;
                        case IDC_COPY_WMLINK: // !SMT!-UI
                                if(sr->getType() == SearchResult::TYPE_FILE)
                                        sCopy = WinUtil::getWebMagnet(sr->getTTH(), Text::toT(sr->getFileName()), sr->getSize());
                                break;
                        case IDC_COPY_HTML_LINK:
                                if(sr->getType() == SearchResult::TYPE_FILE)
                                        sCopy = WinUtil::getHtmlMagnet(sr->getTTH(), Text::toT(sr->getFileName()), sr->getSize());
                                break;
                        case IDC_COPY_TTH:
                                if(sr->getType() == SearchResult::TYPE_FILE)
                                        sCopy = sr->getTTH().toBase32();
                                break;
                        default:
                                dcdebug("SEARCHFRAME DON'T GO HERE\n");
                                return 0;
                }
                if (!sCopy.empty()) {
                        if (data.empty())
                                data = sCopy;
                        else
                                data = data + "\r\n" + sCopy;
                }
        }
        if (!data.empty())
                WinUtil::setClipboard(Text::toT(data));
        return 0;
}

LRESULT SearchFrame::onCustomDraw(int /*idCtrl*/, LPNMHDR pnmh, BOOL& /*bHandled*/) {
        CRect rc;
        LPNMLVCUSTOMDRAW cd = (LPNMLVCUSTOMDRAW)pnmh;

        switch(cd->nmcd.dwDrawStage) {
        case CDDS_PREPAINT:
                return CDRF_NOTIFYITEMDRAW;

        case CDDS_ITEMPREPAINT: {
                cd->clrText = WinUtil::textColor;
                SearchInfo* si = (SearchInfo*)cd->nmcd.lItemlParam;
                if(si->sr != NULL) {
                        targets.clear();
                        QueueManager::getInstance()->getTargets(TTHValue(si->sr->getTTH().toBase32()), targets);
                        if(si->sr->getType() == SearchResult::TYPE_FILE && targets.size() > 0) {
                                cd->clrText = SETTING(SEARCH_ALTERNATE_COLOUR);
                        }
                        if (ShareManager::getInstance()->isTTHShared(si->sr->getTTH())) // !SMT!-S
                                cd->clrTextBk = SETTING(DUPE_COLOR); // !SMT!-S
                }
                return CDRF_NEWFONT | CDRF_NOTIFYSUBITEMDRAW;
        }
        case CDDS_SUBITEM | CDDS_ITEMPREPAINT: {
                if(BOOLSETTING(GET_USER_COUNTRY) && (ctrlResults.findColumn(cd->iSubItem) == COLUMN_LOCATION)) { // !SMT!-S
                        SearchInfo* si = (SearchInfo*)cd->nmcd.lItemlParam;
                        ctrlResults.GetSubItemRect((int)cd->nmcd.dwItemSpec, cd->iSubItem, LVIR_BOUNDS, rc);
                        COLORREF color;
                        if(ctrlResults.GetItemState((int)cd->nmcd.dwItemSpec, LVIS_SELECTED) & LVIS_SELECTED) {
                                if(ctrlResults.m_hWnd == ::GetFocus()) {
                                        color = GetSysColor(COLOR_HIGHLIGHT);
                                        SetBkColor(cd->nmcd.hdc, GetSysColor(COLOR_HIGHLIGHT));
                                        SetTextColor(cd->nmcd.hdc, GetSysColor(COLOR_HIGHLIGHTTEXT));
                                } else {
                                        color = GetBkColor(cd->nmcd.hdc);
                                        SetBkColor(cd->nmcd.hdc, color);
                                }
                        } else {
                                color = WinUtil::bgColor;
                                SetBkColor(cd->nmcd.hdc, WinUtil::bgColor);
                                SetTextColor(cd->nmcd.hdc, WinUtil::textColor);
                        }
                        CRect rc2 = rc;
                        rc2.left += 2;
                        HGDIOBJ oldpen = ::SelectObject(cd->nmcd.hdc, CreatePen(PS_SOLID,0, color));
                        HGDIOBJ oldbr = ::SelectObject(cd->nmcd.hdc, CreateSolidBrush(color));
                        Rectangle(cd->nmcd.hdc,rc.left, rc.top, rc.right, rc.bottom);

                        DeleteObject(::SelectObject(cd->nmcd.hdc, oldpen));
                        DeleteObject(::SelectObject(cd->nmcd.hdc, oldbr));

                        TCHAR buf[256];
                        ctrlResults.GetItemText((int)cd->nmcd.dwItemSpec, cd->iSubItem, buf, 255);
                        buf[255] = 0;
                        if(_tcslen(buf) > 0) {
                                LONG top = rc2.top + (rc2.Height() - 15)/2;
                                if((top - rc2.top) < 2)
                                        top = rc2.top + 1;

                                POINT p = { rc2.left, top };
                                WinUtil::flagImages.Draw(cd->nmcd.hdc, si->getFlagImage(), p, LVSIL_SMALL);
                                top = rc2.top + (rc2.Height() - WinUtil::getTextHeight(cd->nmcd.hdc) - 1)/2;
                                ::ExtTextOut(cd->nmcd.hdc, rc2.left + 30, top + 1, ETO_CLIPPED, rc2, buf, _tcslen(buf), NULL);
                                return CDRF_SKIPDEFAULT;
                        }
                }
        }
        default:
                return CDRF_DODEFAULT;
        }
}

LRESULT SearchFrame::onFilterChar(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& bHandled) {
        if(!BOOLSETTING(FILTER_ENTER) || (wParam == VK_RETURN)) {
                TCHAR *buf = new TCHAR[ctrlFilter.GetWindowTextLength()+1];
                ctrlFilter.GetWindowText(buf, ctrlFilter.GetWindowTextLength()+1);
                filter = buf;
                delete[] buf;

                updateSearchList();
        }

        bHandled = false;

        return 0;
}

bool SearchFrame::parseFilter(FilterModes& mode, int64_t& size) {
        tstring::size_type start = (tstring::size_type)tstring::npos;
        tstring::size_type end = (tstring::size_type)tstring::npos;
        int64_t multiplier = 1;

        if(filter.compare(0, 2, _T(">=")) == 0) {
                mode = GREATER_EQUAL;
                start = 2;
        } else if(filter.compare(0, 2, _T("<=")) == 0) {
                mode = LESS_EQUAL;
                start = 2;
        } else if(filter.compare(0, 2, _T("==")) == 0) {
                mode = EQUAL;
                start = 2;
        } else if(filter.compare(0, 2, _T("!=")) == 0) {
                mode = NOT_EQUAL;
                start = 2;
        } else if(filter[0] == _T('<')) {
                mode = LESS;
                start = 1;
        } else if(filter[0] == _T('>')) {
                mode = GREATER;
                start = 1;
        } else if(filter[0] == _T('=')) {
                mode = EQUAL;
                start = 1;
        }

        if(start == tstring::npos)
                return false;
        if(filter.length() <= start)
                return false;

        if((end = Util::findSubString(filter, _T("TiB"))) != tstring::npos) {
                multiplier = 1024LL * 1024LL * 1024LL * 1024LL;
        } else if((end = Util::findSubString(filter, _T("GiB"))) != tstring::npos) {
                multiplier = 1024*1024*1024;
        } else if((end = Util::findSubString(filter, _T("MiB"))) != tstring::npos) {
                multiplier = 1024*1024;
        } else if((end = Util::findSubString(filter, _T("KiB"))) != tstring::npos) {
                multiplier = 1024;
        } else if((end = Util::findSubString(filter, _T("TB"))) != tstring::npos) {
                multiplier = 1000LL * 1000LL * 1000LL * 1000LL;
        } else if((end = Util::findSubString(filter, _T("GB"))) != tstring::npos) {
                multiplier = 1000*1000*1000;
        } else if((end = Util::findSubString(filter, _T("MB"))) != tstring::npos) {
                multiplier = 1000*1000;
        } else if((end = Util::findSubString(filter, _T("kB"))) != tstring::npos) {
                multiplier = 1000;
        } else if((end = Util::findSubString(filter, _T("B"))) != tstring::npos) {
                multiplier = 1;
        }


        if(end == tstring::npos) {
                end = filter.length();
        }

        tstring tmpSize = filter.substr(start, end-start);
        size = static_cast<int64_t>(Util::toDouble(Text::fromT(tmpSize)) * multiplier);

        return true;
}

bool SearchFrame::matchFilter(SearchInfo* si, int sel, bool doSizeCompare, FilterModes mode, int64_t size, PME* reg) {
        if(filter.empty())
                return true;

        if(doSizeCompare) {
                bool insert = true;
                switch(mode) {
                        case EQUAL: insert = (size == si->sr->getSize()); break;
                        case GREATER_EQUAL: insert = (size <=  si->sr->getSize()); break;
                        case LESS_EQUAL: insert = (size >=  si->sr->getSize()); break;
                        case GREATER: insert = (size < si->sr->getSize()); break;
                        case LESS: insert = (size > si->sr->getSize()); break;
                        case NOT_EQUAL: insert = (size != si->sr->getSize()); break;
                }
                return insert;
        } else if (reg) {
                return reg->match(Text::toLower(si->getText(sel))) > 0; // !SMT!-fix
        }
        return true;
}


void SearchFrame::updateSearchList(SearchInfo* si) {
        int64_t size = -1;
        FilterModes mode = NONE;

        int sel = ctrlFilterSel.GetCurSel();
        bool doSizeCompare = sel == COLUMN_SIZE && parseFilter(mode, size);

        PME reg(Text::toLower(filter), _T("")); // !SMT!-fix
        PME* aReg = NULL;

        if(reg.IsValid() && !filter.empty())
                aReg = &reg;

        if(si != NULL) {
                if(!matchFilter(si, sel, doSizeCompare, mode, size, aReg))
                        ctrlResults.deleteItem(si);
        } else {
                ctrlResults.SetRedraw(FALSE);
                ctrlResults.DeleteAllItems();

                for(vector<SearchInfo*>::const_iterator i = ctrlResults.mainItems.begin(); i != ctrlResults.mainItems.end(); ++i) {
                        SearchInfo* si = *i;
                        si->collapsed = true;
                        if(matchFilter(si, sel, doSizeCompare, mode, size, aReg)) {
                                dcassert(ctrlResults.findItem(si) == -1);
                                int k = ctrlResults.insertItem(si, si->imageIndex());
                                if(si->subItems.size() > 0) {
                                        if(si->collapsed) {
                                                ctrlResults.SetItemState(k, INDEXTOSTATEIMAGEMASK(1), LVIS_STATEIMAGEMASK);
                                        } else {
                                                ctrlResults.SetItemState(k, INDEXTOSTATEIMAGEMASK(2), LVIS_STATEIMAGEMASK);
                                        }
                                } else {
                                        ctrlResults.SetItemState(k, INDEXTOSTATEIMAGEMASK(0), LVIS_STATEIMAGEMASK);
                                }
                        }
                }
                ctrlResults.SetRedraw(TRUE);
        }
}

LRESULT SearchFrame::onSelChange(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& bHandled) {
        TCHAR *buf = new TCHAR[ctrlFilter.GetWindowTextLength()+1];
        ctrlFilter.GetWindowText(buf, ctrlFilter.GetWindowTextLength()+1);
        filter = buf;
        delete[] buf;

        updateSearchList();

        bHandled = false;

        return 0;
}

// This and related changes taken from apex-sppedmod by SMT
LRESULT SearchFrame::onEditChange(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
    unsigned sz = ctrlSearchBox.GetWindowTextLength();
    if (sz == 39) {
       TCHAR *buf = (TCHAR*)alloca((sz+1)*sizeof(TCHAR));
       ctrlSearchBox.GetWindowText(buf, sz+1);
       for (int i = 0; i < 39; i++)
         if (!((buf[i] >= '0' && buf[i] <='9') || (buf[i] >= 'A' && buf[i] <='Z')))
            return 0;
       ctrlFiletype.SetCurSel(8);
    }
    return 0;
}

void SearchFrame::on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw() {
        bool refresh = false;
        if(ctrlResults.GetBkColor() != WinUtil::bgColor) {
                ctrlResults.SetBkColor(WinUtil::bgColor);
                ctrlResults.SetTextBkColor(WinUtil::bgColor);
                ctrlResults.setFlickerFree(WinUtil::bgBrush);
                ctrlHubs.SetBkColor(WinUtil::bgColor);
                ctrlHubs.SetTextBkColor(WinUtil::bgColor);
                refresh = true;
        }
        if(ctrlResults.GetTextColor() != WinUtil::textColor) {
                ctrlResults.SetTextColor(WinUtil::textColor);
                ctrlHubs.SetTextColor(WinUtil::textColor);
                refresh = true;
        }
        ctrlOneSearchWindow.SetCheck(BOOLSETTING(ONE_SEARCH_WINDOW));
        if(refresh == true) {
                RedrawWindow(NULL, NULL, RDW_ERASE | RDW_INVALIDATE | RDW_UPDATENOW | RDW_ALLCHILDREN);
        }
}

LRESULT SearchFrame::onFocusDropDown1(UINT uMsg, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled){
    dd1_Focus = (uMsg == WM_SETFOCUS) ? true : false;
    UpdateLayout();
    bHandled = FALSE;
    return 0;
}

LRESULT SearchFrame::onFocusDropDown2(UINT uMsg, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled){
    dd2_Focus = (uMsg == WM_SETFOCUS) ? true : false;
    UpdateLayout();
    bHandled = FALSE;
    return 0;
}

LRESULT SearchFrame::onFocusDropDown3(UINT uMsg, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled){
    dd3_Focus = (uMsg == WM_SETFOCUS) ? true : false;
    UpdateLayout();
    bHandled = FALSE;
    return 0;
}

LRESULT SearchFrame::onFocusDropDown4(UINT uMsg, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled){
    dd4_Focus = (uMsg == WM_SETFOCUS) ? true : false;
    UpdateLayout();
    bHandled = FALSE;
    return 0;
}

LRESULT SearchFrame::onFocusDropDown5(UINT uMsg, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled){
    dd5_Focus = (uMsg == WM_SETFOCUS) ? true : false;
    UpdateLayout();
    bHandled = FALSE;
    return 0;
}

LRESULT SearchFrame::onLButtonDownDropDown1(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/) {
    dropDown1.SetFocus();
    dd1_Pressed = true;
    UpdateLayout();
    return 0;
}

LRESULT SearchFrame::onLButtonDownDropDown2(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/) {
    dropDown2.SetFocus();
    dd2_Pressed = true;
    UpdateLayout();
    return 0;
}

LRESULT SearchFrame::onLButtonDownDropDown3(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/) {
    dropDown3.SetFocus();
    dd3_Pressed = true;
    UpdateLayout();
    return 0;
}

LRESULT SearchFrame::onLButtonDownDropDown4(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/) {
    dropDown4.SetFocus();
    dd4_Pressed = true;
    UpdateLayout();
    return 0;
}

LRESULT SearchFrame::onLButtonDownDropDown5(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/) {
    dropDown5.SetFocus();
    dd5_Pressed = true;
    UpdateLayout();
    return 0;
}

LRESULT SearchFrame::onLButtonUpDropDown1(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/) {
    if(dd1_Pressed ){
        if(ctrlAutoClose.GetCheck()){
            if(dd3_Checked) dd3_Checked = false;
        }
        dd1_Checked = !dd1_Checked;
        dd1_Pressed = false;
    }
    UpdateLayout();
    return 0;
}

LRESULT SearchFrame::onLButtonUpDropDown2(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/) {
    if(dd2_Pressed){
        if(ctrlAutoClose.GetCheck()){
            if(dd4_Checked) dd4_Checked = false;
            if(dd5_Checked) dd5_Checked = false;
        }
        dd2_Checked = !dd2_Checked;
        dd2_Pressed = false;
    }
    UpdateLayout();
    return 0;
}

LRESULT SearchFrame::onLButtonUpDropDown3(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/) {
    if(dd3_Pressed){
        if(ctrlAutoClose.GetCheck()){
            if(dd1_Checked) dd1_Checked = false;
        }
        dd3_Checked = !dd3_Checked;
        dd3_Pressed = false;
    }
    UpdateLayout();
    return 0;
}

LRESULT SearchFrame::onLButtonUpDropDown4(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/) {
    if(dd4_Pressed){
        if(ctrlAutoClose.GetCheck()){
            if(dd2_Checked) dd2_Checked = false;
            if(dd5_Checked) dd5_Checked = false;
        }
        dd4_Checked = !dd4_Checked;
        dd4_Pressed = false;
        UpdateLayout();
    }
    UpdateLayout();
    return 0;
}

LRESULT SearchFrame::onLButtonUpDropDown5(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/) {
    if(dd5_Pressed){
        if(ctrlAutoClose.GetCheck()){
            if(dd2_Checked) dd2_Checked = false;
            if(dd4_Checked) dd4_Checked = false;
        }
        dd5_Checked = !dd5_Checked;
        dd5_Pressed = false;
    }
    UpdateLayout();
    return 0;
}

LRESULT SearchFrame::onDrawItem(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
    HWND hwnd = 0;
    if(wParam == IDC_FILETYPES) {
        ListDraw(hwnd, wParam, (DRAWITEMSTRUCT*)lParam);
    } else {
        OMenu::onDrawItem(hwnd, uMsg, wParam, lParam, bHandled);
    }
	
	DRAWITEMSTRUCT* dis = (DRAWITEMSTRUCT*)lParam;
	bHandled = FALSE;

    if(dis->CtlType == ODT_STATIC) {
		bHandled = TRUE;
        if(dis->CtlID == IDC_DROP1) {
			paintDropDown(dis, CTSTRING(SEARCH_IN_RESULTS), dd1_Checked, dd1_Pressed, dd1_Focus);
        } else if(dis->CtlID == IDC_DROP2){
			paintDropDown(dis, CTSTRING(HUBLIST), dd2_Checked, dd2_Pressed, dd2_Focus);
        } else if(dis->CtlID == IDC_DROP3) {
			paintDropDown(dis, CTSTRING(SETTINGS_SEARCH), dd3_Checked, dd3_Pressed, dd3_Focus);
        } else if(dis->CtlID == IDC_DROP4) {
			paintDropDown(dis, CTSTRING(SEARCH_OPTIONS), dd4_Checked, dd4_Pressed, dd4_Focus);
        } else if(dis->CtlID == IDC_DROP5) {
			paintDropDown(dis, CTSTRING(DOWNLOAD_OPTIONS), dd5_Checked, dd5_Pressed, dd5_Focus);
        } else if(dis->CtlID == IDC_SINK1 || dis->CtlID == IDC_SINK2 || dis->CtlID == IDC_SINK3 || dis->CtlID == IDC_SINK4 || dis->CtlID == IDC_SINK5) {
            paintBorder(dis);
        } else if(dis->CtlID == IDC_SEARCH_SLIDER) {
			paintSlider(dis);
        }
    } else if(dis->CtlID == IDC_STATUSBAR && dis->itemID == STATUSBAR_PROGRESS_TEXT){
        bHandled = TRUE;
        paintStatusProgress(dis);
    }
	return S_OK;
}

void SearchFrame::paintStatusProgress(DRAWITEMSTRUCT* dis) {
    CDC dc(dis->hDC);
    RECT rc = dis->rcItem;
	int borders[3];
	ctrlStatus.GetBorders(borders);
    int length = 0;

    if( !bStopped ){
		HGDIOBJ oldpen = ::SelectObject(dc, ::CreatePen(PS_SOLID, 0, RGB(255,255,255)));
		HGDIOBJ oldbr = ::SelectObject(dc, ::CreateSolidBrush(RGB(128,128,128)));
		int now = GET_TICK();
		if ( maxResult == 0 ) {
            length = int((1.0/(searchEndTime - searchStartTime))*(rc.right - rc.left) * (now - searchStartTime) );
		}
		else {
			length = int((1.0/maxResult)*(rc.right - rc.left)*(resultsCount+droppedResults));
		}

		dc.Rectangle(rc.left, rc.top, rc.left + length, rc.bottom);

		::DeleteObject(::SelectObject(dc, oldpen));
		::DeleteObject(::SelectObject(dc, oldbr));
    }
	
    dc.SetBkMode(TRANSPARENT);		
	int textHeight = WinUtil::getTextHeight(dc);
	int top = rc.top + (rc.bottom - rc.top - textHeight) / 2;

	dc.SetTextColor(RGB(255, 255, 255));
	RECT rc2 = rc;
	rc2.right = rc.left + length;
	dc.ExtTextOut(rc.left + borders[2], top, ETO_CLIPPED, &rc2, statusText.c_str(), statusText.size(), NULL);
		
	dc.SetTextColor(WinUtil::textColor);
	rc2 = rc;
	rc2.left = rc.left + length;
	dc.ExtTextOut(rc.left + borders[2], top, ETO_CLIPPED, &rc2, statusText.c_str(), statusText.size(), NULL);
    dc.Detach();
}

void SearchFrame::paintDropDown(DRAWITEMSTRUCT* dis, LPCTSTR szTitle, bool checked, bool pressed, bool focus){
	CDC dc;

	dc.Attach(dis->hDC);
	CRect rect;
	CRect rcClient(dis->rcItem);
	rect = rcClient;

	HFONT hOldFont = dc.SelectFont(GetFont());

	// draw text
	rect.top += ::GetSystemMetrics(SM_CXBORDER);

	dc.DrawFrameControl(&rcClient, DFC_BUTTON, DFCS_BUTTONPUSH | DFCS_FLAT |
		(pressed ? DFCS_PUSHED : 0) | (checked ? DFCS_CHECKED : 0) | DFCS_INACTIVE);
	
	if(pressed)
		::OffsetRect(&rect, 1, 1);

	dc.SetBkMode(TRANSPARENT);
	dc.DrawText(szTitle, -1, &rect, DT_CENTER | DT_VCENTER | DT_TOP | DT_SINGLELINE);
	
	// draw expand/collapse button
	if(checked)
		szTitle = _T("-");
	else
		szTitle = _T("+");

	rect.left += 2 * ::GetSystemMetrics(SM_CXEDGE);
	
	dc.DrawText(szTitle, -1, &rect, DT_LEFT | DT_VCENTER | DT_TOP | DT_SINGLELINE);
	dc.SelectFont(hOldFont);

	// draw focus rect
	if(focus){
		::CopyRect(&rect, &rcClient);
		::InflateRect(&rect, -::GetSystemMetrics(SM_CXEDGE), -::GetSystemMetrics(SM_CYEDGE));
		dc.DrawFocusRect(&rect);
	}

	dc.Detach();
}

void SearchFrame::paintBorder(DRAWITEMSTRUCT* dis){
	// draw border
	CDC dc;

	dc.Attach(dis->hDC);
	CRect rcClient(dis->rcItem);
	
	dc.FillRect(&rcClient, ::GetSysColorBrush(COLOR_3DFACE));
	dc.DrawEdge(&rcClient, BDR_RAISEDINNER | BDR_SUNKENOUTER, BF_RECT);
	
	dc.Detach();
}

LRESULT SearchFrame::onSetCursorSlider(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/){
    SetCursor(LoadCursor(NULL, IDC_SIZEWE));     
	return 1; 
}

LRESULT SearchFrame::onLButtonDownSlider(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM lParam, BOOL& /*bHandled*/){

	beginPos = GET_X_LPARAM(lParam); 

	slider.SetCapture();
	moving = true;
	return 0;
}

LRESULT SearchFrame::onLButtonUpSlider(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM lParam, BOOL& /*bHandled*/){

	beginPos = GET_X_LPARAM(lParam); 

	::ReleaseCapture();
	moving = false;
	return 0;
}

LRESULT SearchFrame::onMouseMoveSlider(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM lParam, BOOL& /*bHandled*/){

	if(!moving)
		return 0;

	int nowPos = GET_X_LPARAM(lParam); 
	
	if(nowPos > beginPos){
		width += nowPos - beginPos;
		UpdateLayout();
		return 1;
	}

	if(nowPos < beginPos){
		width -= beginPos - nowPos;
		UpdateLayout();
		return 1;
	}
	
	return 0;
}

void SearchFrame::paintSlider(DRAWITEMSTRUCT* dis){
	// draw border
	CDC dc;

	dc.Attach(dis->hDC);
	CRect rcClient(dis->rcItem);
	
	dc.FillRect(&rcClient, ::GetSysColorBrush(COLOR_ACTIVEBORDER));
	dc.DrawEdge(&rcClient, BDR_RAISEDINNER | BDR_RAISEDOUTER, BF_RECT);
	
	dc.Detach();
}


void SearchFrame::updateStatusTexts(bool updateProgressText,bool updateOther,const tstring& str){
    //if(storedResults == 0)
    //	resultsLabel.SetWindowText(Util::emptyStringT.c_str());
    //else
    //	resultsLabel.SetWindowText(Text::toT(Util::toString(storedResults)).c_str());
    if ( updateProgressText ) {
        statusText = str;    
        SetWindowText(str.c_str());
    }

    if (updateOther) {
        tstring ss;
        if ( !bStopped && updateOther) {
            ss.clear();
            ss = ss.append(Text::tformat("%s %s",Util::toString(resultsCount).c_str(),STRING(ITEMS).c_str()));
            ctrlStatus.SetText(STATUSBAR_COUNT, ss.c_str());

            ss.clear();
	        if ( maxResult == 0 ) {
    		    int now = GET_TICK();
                int percent = int(((now- searchStartTime) * 100) / (searchEndTime - searchStartTime));
		        int secondsLeft = int(searchEndTime > now ? (searchEndTime - now)/1000 : 0);
                if ( percent >= 100 ) {
    		        ss.append(TSTRING(DONE));
                } else {
                    ss.append(Text::tformat("%s [%d%%]",Text::fromT(Util::formatSeconds(secondsLeft)).c_str(),percent));
                }
	        }	
	        else {
    		    int total = int(resultsCount+droppedResults);
                int percent = int(  (100.0*total)/maxResult ) ;
                if ( percent >= 100 ) {
                    ss.append(TSTRING(DONE));
                } else {
    			    ss.append(Text::tformat("%d/%d [%d%%]",total,maxResult,percent));
                }
	        }
            ctrlStatus.SetText(STATUSBAR_PROGRESS,ss.c_str());
        
            ss.clear();
            ss.append(Text::tformat("%s %s",Util::toString(droppedResults).c_str(),STRING(FILTERED).c_str()));
            ctrlStatus.SetText(STATUSBAR_COUNT_FILTER, ss.c_str());
        }
    }
	ctrlStatus.RedrawWindow(0,0,RDW_INVALIDATE|RDW_UPDATENOW);
}


/**
 * @file
 * $Id: SearchFrm.cpp,v 1.114 2006/11/12 20:32:08 bigmuscle Exp $
 */
