/*
 * Copyright (C) 2001-2006 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#if !defined(SEARCH_FRM_H)
#define SEARCH_FRM_H

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "FlatTabCtrl.h"
#include "TypedListViewCtrl.h"
#include "WinUtil.h"
#include "UserInfoBase.h"

#include "../client/Client.h"
#include "../client/SearchManager.h"
#include "../client/CriticalSection.h"
#include "../client/ClientManagerListener.h"
#include "../client/FavoriteManager.h"
#include "../client/QueueManager.h"
#include "../client/pme.h"

#include "UCHandler.h"

#define SEARCH_MESSAGE_MAP 6            // This could be any number, really...
#define SHOWUI_MESSAGE_MAP 7
#define FILTER_MESSAGE_MAP 8
#define SLIDER_MESSAGE_MAP 9

#define DROPDOWN_MESSAGE_MAP1 21
#define DROPDOWN_MESSAGE_MAP2 22
#define DROPDOWN_MESSAGE_MAP3 23
#define DROPDOWN_MESSAGE_MAP4 24
#define DROPDOWN_MESSAGE_MAP5 25

class SearchFrame : public MDITabChildWindowImpl<SearchFrame, RGB(127, 127, 255), IDR_SEARCH>, public StaticFrame<SearchFrame, ResourceManager::SEARCH> ,
        private SearchManagerListener, private ClientManagerListener,
        public UCHandler<SearchFrame>, public UserInfoBaseHandler<SearchFrame>,
        private SettingsManagerListener, private TimerManagerListener
{
public:
        static void openNewWindow(const tstring& str = Util::emptyStringW, LONGLONG size = 0, SearchManager::SizeModes mode = SearchManager::SIZE_ATLEAST, SearchManager::TypeModes type = SearchManager::TYPE_ANY);
        static void closeAll();

        DECLARE_FRAME_WND_CLASS_EX(_T("SearchFrame"), IDR_SEARCH, 0, COLOR_3DFACE)

        typedef MDITabChildWindowImpl<SearchFrame, RGB(127, 127, 255), IDR_SEARCH> baseClass;
        typedef UCHandler<SearchFrame> ucBase;
        typedef UserInfoBaseHandler<SearchFrame> uicBase;

        BEGIN_MSG_MAP(SearchFrame)
                NOTIFY_HANDLER(IDC_RESULTS, LVN_GETDISPINFO, ctrlResults.onGetDispInfo)
                NOTIFY_HANDLER(IDC_RESULTS, LVN_COLUMNCLICK, ctrlResults.onColumnClick)
                NOTIFY_HANDLER(IDC_RESULTS, LVN_GETINFOTIP, ctrlResults.onInfoTip)
                NOTIFY_HANDLER(IDC_HUB, LVN_GETDISPINFO, ctrlHubs.onGetDispInfo)
                NOTIFY_HANDLER(IDC_RESULTS, NM_DBLCLK, onDoubleClickResults)
                NOTIFY_HANDLER(IDC_RESULTS, LVN_KEYDOWN, onKeyDown)
                NOTIFY_HANDLER(IDC_HUB, LVN_ITEMCHANGED, onItemChangedHub)
                NOTIFY_HANDLER(IDC_RESULTS, NM_CUSTOMDRAW, onCustomDraw)
                MESSAGE_HANDLER(WM_CREATE, onCreate)
                MESSAGE_HANDLER(WM_SETFOCUS, onFocus)
                MESSAGE_HANDLER(WM_CONTEXTMENU, onContextMenu)
                MESSAGE_HANDLER(WM_SPEAKER, onSpeaker)
                MESSAGE_HANDLER(WM_CTLCOLOREDIT, onCtlColor)
                MESSAGE_HANDLER(WM_CTLCOLORSTATIC, onCtlColor)
                MESSAGE_HANDLER(WM_CTLCOLORLISTBOX, onCtlColor)
                MESSAGE_HANDLER(WM_CLOSE, onClose)
                MESSAGE_HANDLER(WM_DRAWITEM, onDrawItem)
                MESSAGE_HANDLER(WM_MEASUREITEM, onMeasure)
                COMMAND_ID_HANDLER(IDC_VIEW_AS_TEXT, onViewAsText)
                COMMAND_ID_HANDLER(IDC_REMOVE, onRemove)
                COMMAND_ID_HANDLER(IDC_SEARCH, onSearch)
                COMMAND_ID_HANDLER(IDC_SEARCH_STOP, onStop)
                COMMAND_ID_HANDLER(IDC_COPY_NICK, onCopy)
                COMMAND_ID_HANDLER(IDC_COPY_FILENAME, onCopy)
                COMMAND_ID_HANDLER(IDC_COPY_PATH, onCopy)
                COMMAND_ID_HANDLER(IDC_COPY_SIZE, onCopy)
                COMMAND_ID_HANDLER(IDC_FREESLOTS, onFreeSlots)
                COMMAND_ID_HANDLER(IDC_ONESEARCHWINDOW, onOneSearchWindow)
                COMMAND_ID_HANDLER(IDC_COLLAPSED, onCollapsed)
                COMMAND_ID_HANDLER(IDC_GETLIST, onGetList)
                COMMAND_ID_HANDLER(IDC_BROWSELIST, onBrowseList)
                COMMAND_ID_HANDLER(IDC_SEARCH_ALTERNATES, onSearchByTTH)
                COMMAND_ID_HANDLER(IDC_BITZI_LOOKUP, onBitziLookup)
                COMMAND_ID_HANDLER(IDC_COPY_LINK, onCopy)
                COMMAND_ID_HANDLER(IDC_COPY_WMLINK, onCopy) // !SMT!-UI
                COMMAND_ID_HANDLER(IDC_COPY_HTML_LINK, onCopy) // !SMT!-UI
                COMMAND_ID_HANDLER(IDC_COPY_TTH, onCopy)
                COMMAND_ID_HANDLER(IDC_PURGE, onPurge)
                COMMAND_CODE_HANDLER(CBN_EDITCHANGE, onEditChange)

                COMMAND_ID_HANDLER(IDC_DOWNLOADTO, onDownloadTo)
                COMMAND_RANGE_HANDLER(IDC_DOWNLOAD_FAVORITE_DIRS, IDC_DOWNLOAD_FAVORITE_DIRS + 499, onDownload)
                COMMAND_RANGE_HANDLER(IDC_DOWNLOAD_WHOLE_FAVORITE_DIRS, IDC_DOWNLOAD_WHOLE_FAVORITE_DIRS + 499, onDownloadWhole)
                COMMAND_RANGE_HANDLER(IDC_DOWNLOAD_TARGET, IDC_DOWNLOAD_TARGET + 499, onDownloadTarget)

                CHAIN_COMMANDS(ucBase)
                CHAIN_COMMANDS(uicBase)
                CHAIN_MSG_MAP(baseClass)
        ALT_MSG_MAP(SEARCH_MESSAGE_MAP)
                MESSAGE_HANDLER(WM_CHAR, onChar)
                MESSAGE_HANDLER(WM_KEYDOWN, onChar)
                MESSAGE_HANDLER(WM_KEYUP, onChar)
        ALT_MSG_MAP(SHOWUI_MESSAGE_MAP)
                MESSAGE_HANDLER(BM_SETCHECK, onShowUI)
        ALT_MSG_MAP(FILTER_MESSAGE_MAP)
                MESSAGE_HANDLER(WM_CTLCOLORLISTBOX, onCtlColor)
                MESSAGE_HANDLER(WM_KEYUP, onFilterChar)
                COMMAND_CODE_HANDLER(CBN_SELCHANGE, onSelChange)
        ALT_MSG_MAP(DROPDOWN_MESSAGE_MAP1)
            MESSAGE_HANDLER(WM_SETFOCUS, onFocusDropDown1)
            MESSAGE_HANDLER(WM_KILLFOCUS, onFocusDropDown1)
            MESSAGE_HANDLER(WM_LBUTTONDOWN, onLButtonDownDropDown1)
            MESSAGE_HANDLER(WM_LBUTTONUP, onLButtonUpDropDown1)
        ALT_MSG_MAP(DROPDOWN_MESSAGE_MAP2)
            MESSAGE_HANDLER(WM_SETFOCUS, onFocusDropDown2)
            MESSAGE_HANDLER(WM_KILLFOCUS, onFocusDropDown2)
            MESSAGE_HANDLER(WM_LBUTTONDOWN, onLButtonDownDropDown2)
            MESSAGE_HANDLER(WM_LBUTTONUP, onLButtonUpDropDown2)
        ALT_MSG_MAP(DROPDOWN_MESSAGE_MAP3)
            MESSAGE_HANDLER(WM_SETFOCUS, onFocusDropDown3)
            MESSAGE_HANDLER(WM_KILLFOCUS, onFocusDropDown3)
            MESSAGE_HANDLER(WM_LBUTTONDOWN, onLButtonDownDropDown3)
            MESSAGE_HANDLER(WM_LBUTTONUP, onLButtonUpDropDown3)
        ALT_MSG_MAP(DROPDOWN_MESSAGE_MAP4)
            MESSAGE_HANDLER(WM_SETFOCUS, onFocusDropDown4)
            MESSAGE_HANDLER(WM_KILLFOCUS, onFocusDropDown4)
            MESSAGE_HANDLER(WM_LBUTTONDOWN, onLButtonDownDropDown4)
            MESSAGE_HANDLER(WM_LBUTTONUP, onLButtonUpDropDown4)
        ALT_MSG_MAP(DROPDOWN_MESSAGE_MAP5)
            MESSAGE_HANDLER(WM_SETFOCUS, onFocusDropDown5)
            MESSAGE_HANDLER(WM_SETFOCUS, onFocusDropDown5)
            MESSAGE_HANDLER(WM_KILLFOCUS, onFocusDropDown5)
            MESSAGE_HANDLER(WM_LBUTTONDOWN, onLButtonDownDropDown5)
            MESSAGE_HANDLER(WM_LBUTTONUP, onLButtonUpDropDown5)
        ALT_MSG_MAP(SLIDER_MESSAGE_MAP)
            MESSAGE_HANDLER(WM_SETCURSOR, onSetCursorSlider)
            MESSAGE_HANDLER(WM_MOUSEMOVE, onMouseMoveSlider)
            MESSAGE_HANDLER(WM_LBUTTONDOWN, onLButtonDownSlider)
            MESSAGE_HANDLER(WM_LBUTTONUP, onLButtonUpSlider)
        END_MSG_MAP()

        SearchFrame() :
        searchBoxContainer(WC_COMBOBOX, this, SEARCH_MESSAGE_MAP),
                searchContainer(WC_EDIT, this, SEARCH_MESSAGE_MAP),
                purgeContainer(WC_EDIT, this, SEARCH_MESSAGE_MAP),
                sizeContainer(WC_EDIT, this, SEARCH_MESSAGE_MAP),
                modeContainer(WC_COMBOBOX, this, SEARCH_MESSAGE_MAP),
                sizeModeContainer(WC_COMBOBOX, this, SEARCH_MESSAGE_MAP),
                fileTypeContainer(WC_COMBOBOX, this, SEARCH_MESSAGE_MAP),
                showUIContainer(WC_COMBOBOX, this, SHOWUI_MESSAGE_MAP),
                slotsContainer(WC_COMBOBOX, this, SEARCH_MESSAGE_MAP),
                collapsedContainer(WC_COMBOBOX, this, SEARCH_MESSAGE_MAP),
                doSearchContainer(WC_COMBOBOX, this, SEARCH_MESSAGE_MAP),
                resultsContainer(WC_LISTVIEW, this, SEARCH_MESSAGE_MAP),
                hubsContainer(WC_LISTVIEW, this, SEARCH_MESSAGE_MAP),
                ctrlFilterContainer(WC_EDIT, this, FILTER_MESSAGE_MAP),
                ctrlFilterSelContainer(WC_COMBOBOX, this, FILTER_MESSAGE_MAP),
                initialSize(0), initialMode(SearchManager::SIZE_ATLEAST), initialType(SearchManager::TYPE_ANY),
                showUI(true), onlyFree(false), closed(false), isHash(false), droppedResults(0), resultsCount(0),
                expandSR(false), exactSize1(false), exactSize2(0), searches(0),
                dropDown1Container(WC_STATIC, this, DROPDOWN_MESSAGE_MAP1),
                dropDown2Container(WC_STATIC, this, DROPDOWN_MESSAGE_MAP2),
                dropDown3Container(WC_STATIC, this, DROPDOWN_MESSAGE_MAP3),
                dropDown4Container(WC_STATIC, this, DROPDOWN_MESSAGE_MAP4),
                dropDown5Container(WC_STATIC, this, DROPDOWN_MESSAGE_MAP5),
                autoCloseContainer(WC_COMBOBOX, this, SEARCH_MESSAGE_MAP),
                sliderContainer(WC_STATIC, this, SLIDER_MESSAGE_MAP),
                resultLimitContainer(WC_EDIT, this, SEARCH_MESSAGE_MAP),
                oneSearchWindowContainer(WC_COMBOBOX, this, SEARCH_MESSAGE_MAP),
                dd1_Checked(false), dd1_Focus(false), dd1_Pressed(false),
                dd2_Checked(false), dd2_Focus(false), dd2_Pressed(false),
                dd3_Checked(true), dd3_Focus(false), dd3_Pressed(false),
                dd4_Checked(false), dd4_Focus(false), dd4_Pressed(false),
                dd5_Checked(false), dd5_Focus(false), dd5_Pressed(false),
                beginPos(0), moving(false),width(200),
                maxResult(SETTING(RESULT_THERSHOLD))
        {
                SearchManager::getInstance()->addListener(this);
                useGrouping = BOOLSETTING(GROUP_SEARCH_RESULTS);
        }

        virtual ~SearchFrame() {
                images.Destroy();
                searchTypes.Destroy();
        }

        LRESULT onChar(UINT uMsg, WPARAM wParam, LPARAM /*lParam*/, BOOL& bHandled);
        LRESULT onClose(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
        LRESULT onDrawItem(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
        LRESULT onMeasure(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
        LRESULT onContextMenu(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM lParam, BOOL& bHandled);
        LRESULT onCreate(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled);
        LRESULT onCtlColor(UINT uMsg, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/);
        LRESULT onDoubleClickResults(int idCtrl, LPNMHDR pnmh, BOOL& bHandled);
        LRESULT onSpeaker(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/);
        LRESULT onSearchByTTH(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
        LRESULT onCopy(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
        LRESULT onBitziLookup(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
        LRESULT onCustomDraw(int /*idCtrl*/, LPNMHDR pnmh, BOOL& bHandled);
        LRESULT onFilterChar(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
        LRESULT onSelChange(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
        LRESULT onPurge(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
        LRESULT onGetList(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
        LRESULT onBrowseList(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
        LRESULT onEditChange(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);

        LRESULT onDownloadTo(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
        LRESULT onDownload(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
        LRESULT onDownloadWhole(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
        LRESULT onDownloadTarget(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
        
        LRESULT onFocusDropDown1(UINT uMsg, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled);
        LRESULT onFocusDropDown2(UINT uMsg, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled);
        LRESULT onFocusDropDown3(UINT uMsg, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled);
        LRESULT onFocusDropDown4(UINT uMsg, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled);
        LRESULT onFocusDropDown5(UINT uMsg, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled);
        LRESULT onLButtonDownDropDown1(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
        LRESULT onLButtonDownDropDown2(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
        LRESULT onLButtonDownDropDown3(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
        LRESULT onLButtonDownDropDown4(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
        LRESULT onLButtonDownDropDown5(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);        
        LRESULT onLButtonUpDropDown1(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
        LRESULT onLButtonUpDropDown2(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);        
        LRESULT onLButtonUpDropDown3(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
        LRESULT onLButtonUpDropDown4(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
        LRESULT onLButtonUpDropDown5(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);

        LRESULT onLButtonDownSlider(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
        LRESULT onLButtonUpSlider(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
        LRESULT onSetCursorSlider(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
        LRESULT onMouseMoveSlider(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& bHandled);

        void UpdateLayout(BOOL bResizeBars = TRUE);
        void runUserCommand(UserCommand& uc);

        void removeSelected() {
                int i = -1;
                Lock l(cs);
                while( (i = ctrlResults.GetNextItem(-1, LVNI_SELECTED)) != -1) {
                        ctrlResults.removeGroupedItem(ctrlResults.getItemData(i));
                }
        }

        LRESULT onViewAsText(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
                ctrlResults.forEachSelected(&SearchInfo::view);
                return 0;
        }

        LRESULT onRemove(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
                removeSelected();
                return 0;
        }

        LRESULT onFreeSlots(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
                onlyFree = (ctrlSlots.GetCheck() == 1);
                SettingsManager::getInstance()->set(SettingsManager::FREE_SLOTS_DEFAULT, onlyFree);
                return 0;
        }

        LRESULT onOneSearchWindow(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
            SettingsManager::getInstance()->set(SettingsManager::ONE_SEARCH_WINDOW, ctrlOneSearchWindow.GetCheck() == 1);
            SettingsManager::getInstance()->save();
            return 0;
        }

        LRESULT onCollapsed(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
                expandSR = (ctrlCollapsed.GetCheck() == 1);
                return 0;
        }

        LRESULT onSearch(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
                onEnter();
                return 0;
        }

        LRESULT onKeyDown(int /*idCtrl*/, LPNMHDR pnmh, BOOL& /*bHandled*/) {
                NMLVKEYDOWN* kd = (NMLVKEYDOWN*) pnmh;

                if(kd->wVKey == VK_DELETE) {
                        removeSelected();
                }
                return 0;
        }

        LRESULT onFocus(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/) {
                if(::IsWindow(ctrlSearch))
                        ctrlSearch.SetFocus();
                return 0;
        }

        LRESULT onShowUI(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& bHandled) {
                bHandled = FALSE;
                showUI = (wParam == BST_CHECKED);
                UpdateLayout(FALSE);
                return 0;
        }

        void SearchFrame::setInitial(const tstring& str, LONGLONG size, SearchManager::SizeModes mode, SearchManager::TypeModes type) {
                initialString = str; initialSize = size; initialMode = mode; initialType = type; bStopped = true;
        }

        LRESULT onStop(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
			stopSearch();
			return 0;
        }
		void stopSearch() {
                ctrlDoSearch.EnableWindow(true);
                ctrlStopSearch.EnableWindow(false);
                updateStatusTexts(true,false,TSTRING(SEARCH_STOPPED));
                bStopped = true;
                SearchManager::getInstance()->stopSearch((int*)this);
                if(searches != 0) {
                        searches--;
                        TimerManager::getInstance()->removeListener(this);
                }
		}

private:
        tstring getDownloadDirectory(WORD wID);
        class SearchInfo;

public:
        TypedTreeListViewCtrl<SearchInfo, IDC_RESULTS>& getUserList() { return ctrlResults; }

private:
        enum {
                COLUMN_FIRST,
                COLUMN_FILENAME = COLUMN_FIRST,
                COLUMN_HITS,
                COLUMN_NICK,
                COLUMN_TYPE,
                COLUMN_SIZE,
                COLUMN_PATH,
                COLUMN_SLOTS,
                COLUMN_CONNECTION,
                COLUMN_HUB,
                COLUMN_EXACT_SIZE,
                COLUMN_UPLOAD,
                COLUMN_LOCATION, // !SMT!-IP
                COLUMN_IP,
                COLUMN_DNS, // !SMT!-IP
                COLUMN_TTH,
                COLUMN_LAST
        };

        enum Images {
                IMAGE_UNKOWN,
                IMAGE_SLOW,
                IMAGE_NORMAL,
                IMAGE_FAST
        };

        enum FilterModes{
                NONE,
                EQUAL,
                GREATER_EQUAL,
                LESS_EQUAL,
                GREATER,
                LESS,
                NOT_EQUAL
        };

        class SearchInfo : public UserInfoBase {
        public:
                typedef SearchInfo* Ptr;
                typedef vector<Ptr> List;
                typedef List::const_iterator Iter;

                SearchInfo::List subItems;

                SearchInfo(SearchResult* aSR) : UserInfoBase(aSR->getUser()), sr(aSR), collapsed(true), main(NULL) {
                        sr->incRef(); update();
                }
                ~SearchInfo() {
                        sr->decRef();
                }

                bool collapsed;
                SearchInfo* main;

                void getList();
                void browseList();

                void view();
                struct Download {
                        Download(const tstring& aTarget) : tgt(aTarget) { }
                        void operator()(SearchInfo* si);
                        const tstring& tgt;
                };
                struct DownloadWhole {
                        DownloadWhole(const tstring& aTarget) : tgt(aTarget) { }
                        void operator()(SearchInfo* si);
                        const tstring& tgt;
                };
                struct DownloadTarget {
                        DownloadTarget(const tstring& aTarget) : tgt(aTarget) { }
                        void operator()(SearchInfo* si);
                        const tstring& tgt;
                };
                struct CheckTTH {
                        CheckTTH() : op(true), firstHubs(true), hasTTH(false), firstTTH(true) { }
                        void operator()(SearchInfo* si);
                        bool firstHubs;
                        StringList hubs;
                        bool op;
                        bool hasTTH;
                        bool firstTTH;
                        tstring tth;
                };

                const tstring& getText(int col) const { return columns[col]; }

                static int compareItems(SearchInfo* a, SearchInfo* b, int col) {
                        if(!a->sr || !b->sr)
                                return 0;

                        switch(col) {
                                case COLUMN_TYPE:
                                        if(a->sr->getType() == b->sr->getType())
                                                return lstrcmpi(a->columns[COLUMN_TYPE].c_str(), b->columns[COLUMN_TYPE].c_str());
                                        else
                                                return(a->sr->getType() == SearchResult::TYPE_DIRECTORY) ? -1 : 1;
                                case COLUMN_HITS: return compare(a->subItems.size(), b->subItems.size());
                                case COLUMN_SLOTS:
                                        if(a->sr->getFreeSlots() == b->sr->getFreeSlots())
                                                return compare(a->sr->getSlots(), b->sr->getSlots());
                                        else
                                                return compare(a->sr->getFreeSlots(), b->sr->getFreeSlots());
                                case COLUMN_SIZE:
                                case COLUMN_EXACT_SIZE: return compare(a->sr->getSize(), b->sr->getSize());
                                case COLUMN_UPLOAD: return compare(a->columns[COLUMN_UPLOAD],b->columns[COLUMN_UPLOAD]);
                                default: return lstrcmpi(a->getText(col).c_str(), b->getText(col).c_str());
                        }
                }

                int imageIndex() const {
                        int image = 0;
                        if (BOOLSETTING(USE_SYSTEM_ICONS)) {
                                image = sr->getType() == SearchResult::TYPE_FILE ? WinUtil::getIconIndex(Text::toT(sr->getFile())) : WinUtil::getDirIconIndex();
                        } else {
                                string tmp = ClientManager::getInstance()->getConnection(sr->getUser()->getCID());
                                if( (tmp == "28.8Kbps") ||
                                        (tmp == "33.6Kbps") ||
                                        (tmp == "56Kbps") ||
                                        (tmp == "Modem") ||
                                        (tmp == "Satellite") ||
                                        (tmp == "Wireless") ||
                                        (tmp == "ISDN") ) {
                                        image = 1;
                                } else if( (tmp == "Cable") ||
                                        (tmp == "DSL") ) {
                                        image = 2;
                                } else if( (tmp == "LAN(T1)") ||
                                        (tmp == "LAN(T3)") ) {
                                        image = 3;
                                }
                                if(sr->getType() == SearchResult::TYPE_FILE)
                                        image+=4;
                        }
                        return image;
                }

                void update();

                SearchInfo* createMainItem() { return this; }
                const tstring& getGroupingString() const { return columns[COLUMN_TTH]; }
                void updateMainItem() {
                        uint32_t total = main->subItems.size();
                        if(total != 0) {
                                TCHAR buf[256];
                                snwprintf(buf, sizeof(buf), _T("%d %s"), total + 1, CTSTRING(USERS));

                                main->columns[COLUMN_HITS] = buf;
                                if(total == 1)
                                        main->columns[COLUMN_SIZE] = columns[COLUMN_SIZE];
                        } else {
                                main->columns[COLUMN_HITS] = Util::emptyStringT;
                        }
                }

                GETSET(int, flagimage, FlagImage);
                SearchResult* sr;
                tstring columns[COLUMN_LAST];
        };

        struct HubInfo : public FastAlloc<HubInfo> {
                HubInfo(const tstring& aUrl, const tstring& aName, bool aOp) : url(aUrl),
                        name(aName), op(aOp) { }

                const tstring& getText(int col) const {
                        return (col == 0) ? name : Util::emptyStringT;
                }
                static int compareItems(HubInfo* a, HubInfo* b, int col) {
                        return (col == 0) ? lstrcmpi(a->name.c_str(), b->name.c_str()) : 0;
                }
                int imageIndex() {
                        return 0;
                }

                tstring url;
                tstring name;
                bool op;
        };

        // WM_SPEAKER
        enum Speakers {
                ADD_RESULT,
                FILTER_RESULT,
                HUB_ADDED,
                HUB_CHANGED,
                HUB_REMOVED,
                QUEUE_STATS,
                SEARCH_START,
                NEXT_SEARCH
        };

        tstring initialString;
        int64_t initialSize;
        SearchManager::SizeModes initialMode;
        SearchManager::TypeModes initialType;

        CStatusBarCtrl ctrlStatus;
        CEdit ctrlSearch;
        CComboBox ctrlSearchBox;
        CEdit ctrlSize;
        CComboBox ctrlMode;
        CComboBox ctrlSizeMode;
        CComboBox ctrlFiletype;
        CImageList searchTypes;
        CButton ctrlDoSearch;
        CButton ctrlStopSearch;
        CButton ctrlPurge;

        BOOL ListMeasure(HWND hwnd, UINT uCtrlId, MEASUREITEMSTRUCT *mis);
        BOOL ListDraw(HWND hwnd, UINT uCtrlId, DRAWITEMSTRUCT *dis);

        CContainedWindow searchContainer;
        CContainedWindow searchBoxContainer;
        CContainedWindow sizeContainer;
        CContainedWindow modeContainer;
        CContainedWindow sizeModeContainer;
        CContainedWindow fileTypeContainer;
        CContainedWindow slotsContainer;
        CContainedWindow collapsedContainer;
        CContainedWindow showUIContainer;
        CContainedWindow doSearchContainer;
        CContainedWindow resultsContainer;
        CContainedWindow hubsContainer;
        CContainedWindow purgeContainer;
        CContainedWindow ctrlFilterContainer;
        CContainedWindow ctrlFilterSelContainer;
        CContainedWindow dropDown1Container;
        CContainedWindow dropDown2Container;
        CContainedWindow dropDown3Container;
        CContainedWindow dropDown4Container;
        CContainedWindow dropDown5Container;
        CContainedWindow autoCloseContainer;
        CContainedWindow sliderContainer;
        CContainedWindow resultLimitContainer;
        CContainedWindow oneSearchWindowContainer;

        CButton ctrlAutoClose;
        CButton ctrlOneSearchWindow;
        CStatic border1,border2,border3,border4,border5;
        CStatic dropDown1,dropDown2,dropDown3,dropDown4,dropDown5;

        CStatic searchLabel, sizeLabel, optionLabel, typeLabel, hubsLabel, srLabel;
        CButton ctrlSlots, ctrlShowUI, ctrlCollapsed;        
        CStatic slider;
        CStatic resultLimitLabel;
        CEdit ctrlResultLimit;

        tstring filter;

        bool showUI;

        CImageList images;
        TypedTreeListViewCtrl<SearchInfo, IDC_RESULTS> ctrlResults;
        TypedListViewCtrl<HubInfo, IDC_HUB> ctrlHubs;

        // OMenu grantMenu; // !SMT!-UI
        OMenu resultsMenu;
        OMenu targetMenu;
        OMenu targetDirMenu;
        OMenu copyMenu;

        TStringList search;
        StringList targets;
        StringList wholeTargets;

        CEdit ctrlFilter;
        CComboBox ctrlFilterSel;

        bool onlyFree;
        bool isHash;
        bool expandSR;
        bool bStopped;
        bool exactSize1;
        bool useGrouping;
        int64_t exactSize2;
        int64_t resultsCount;
        bool dd1_Focus, dd2_Focus, dd3_Focus, dd4_Focus, dd5_Focus  ;
        bool dd1_Checked, dd2_Checked, dd3_Checked, dd4_Checked, dd5_Checked;        
        bool dd1_Pressed, dd2_Pressed, dd3_Pressed, dd4_Pressed, dd5_Pressed;

        int width;
        bool moving;
        int beginPos;
        int maxResult;
        bool search_done;
        int64_t searchEndTime,searchStartTime;
        tstring statusText;
        CriticalSection cs;

        static TStringList lastSearches;
        size_t droppedResults;

        bool closed;
        int searches;

        StringMap ucLineParams;

        static int columnIndexes[];
        static int columnSizes[];

        typedef map<HWND, SearchFrame*> FrameMap;
        typedef FrameMap::const_iterator FrameIter;
        typedef pair<HWND, SearchFrame*> FramePair;

        static FrameMap frames;

        typedef map<int,tstring> TargetsMap; // !SMT!-S
        TargetsMap dlTargets; // !SMT!-S

        void downloadSelected(const tstring& aDir, bool view = false);
        void downloadWholeSelected(const tstring& aDir);
        void onEnter();
        void onTab(bool shift);

        void download(SearchResult* aSR, const tstring& aDir, bool view);

        virtual void on(SearchManagerListener::SR, SearchResult* aResult) throw();
        virtual void on(SearchManagerListener::Searching, SearchQueueItem* aSearch) throw();

        virtual void on(TimerManagerListener::Second, uint32_t aTick) throw();

        // ClientManagerListener
        virtual void on(ClientConnected, Client* c) throw() { speak(HUB_ADDED, c); }
        virtual void on(ClientUpdated, Client* c) throw() { speak(HUB_CHANGED, c); }
        virtual void on(ClientDisconnected, Client* c) throw() { speak(HUB_REMOVED, c); }
        virtual void on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw();

        void initHubs();
        void onHubAdded(HubInfo* info);
        void onHubChanged(HubInfo* info);
        void onHubRemoved(HubInfo* info);
        bool matchFilter(SearchInfo* si, int sel, bool doSizeCompare = false, FilterModes mode = NONE, int64_t size = 0, PME* filter = NULL);
        bool parseFilter(FilterModes& mode, int64_t& size);
        void updateSearchList(SearchInfo* si = NULL);

        LRESULT onItemChangedHub(int idCtrl, LPNMHDR pnmh, BOOL& bHandled);

        void speak(Speakers s, Client* aClient) {
                HubInfo* hubInfo = new HubInfo(Text::toT(aClient->getHubUrl()), Text::toT(aClient->getHubName()), aClient->getMyIdentity().isOp());
                PostMessage(WM_SPEAKER, WPARAM(s), LPARAM(hubInfo));
        }

        void paintDropDown(DRAWITEMSTRUCT* dis, LPCTSTR szTitle, bool checked, bool pressed, bool focus);
        void paintBorder(DRAWITEMSTRUCT* dis);
        void paintSlider(DRAWITEMSTRUCT* dis);
        void paintStatusProgress(DRAWITEMSTRUCT* dis);
        void updateStatusTexts(bool updateProgressText=false,bool updateOther=true,const tstring& str=Util::emptyStringT);
        
        enum {
            STATUSBAR_SHOWUI=0,
            STATUSBAR_PROGRESS_TEXT,
            STATUSBAR_PROGRESS,
            STATUSBAR_COUNT,
            STATUSBAR_COUNT_FILTER,
            STATUSBAR_NEXT_SEARCH
        };
};

#endif // !defined(SEARCH_FRM_H)

/**
 * @file
 * $Id: SearchFrm.h,v 1.79 2006/10/22 18:57:56 bigmuscle Exp $
 */

