/*
 * Copyright (C) 2001-2006 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#if !defined(WIN_UTIL_H)
#define WIN_UTIL_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "../client/Util.h"
#include "../client/SettingsManager.h"
#include "../client/Client.h"
#include "../client/User.h"
#include "../client/MerkleTree.h"
#include "../client/version.h"

#include "resource.h"
#include "OMenu.h"

// Some utilities for handling HLS colors, taken from Jean-Michel LE FOL's codeproject
// article on WTL OfficeXP Menus
typedef DWORD HLSCOLOR;
#define HLS(h,l,s) ((HLSCOLOR)(((BYTE)(h)|((WORD)((BYTE)(l))<<8))|(((DWORD)(BYTE)(s))<<16)))
#define HLS_H(hls) ((BYTE)(hls))
#define HLS_L(hls) ((BYTE)(((WORD)(hls)) >> 8))
#define HLS_S(hls) ((BYTE)((hls)>>16))

HLSCOLOR RGB2HLS (COLORREF rgb);
COLORREF HLS2RGB (HLSCOLOR hls);

COLORREF HLS_TRANSFORM (COLORREF rgb, int percent_L, int percent_S);

class FlatTabCtrl;
class UserCommand;

template<class T, int title, int ID = -1>
class StaticFrame {
public:
        virtual ~StaticFrame() { frame = NULL; }

        static T* frame;
        static void openWindow() {
                if(frame == NULL) {
                        frame = new T();
                        frame->CreateEx(WinUtil::mdiClient, frame->rcDefault, CTSTRING_I(ResourceManager::Strings(title)));
                } else {
                        // match the behavior of MainFrame::onSelected()
                        HWND hWnd = frame->m_hWnd;
                        if(frame->MDIGetActive() != hWnd) {
                                MainFrame::getMainFrame()->MDIActivate(hWnd);
                        } else if(BOOLSETTING(TOGGLE_ACTIVE_WINDOW)) {
                                ::SetWindowPos(hWnd, HWND_BOTTOM, 0, 0, 0, 0, SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE);
                                frame->MDINext(hWnd);
                                hWnd = frame->MDIGetActive();
                        }
                        if(::IsIconic(hWnd))
                                ::ShowWindow(hWnd, SW_RESTORE);
                }
        }
        static bool isMDIChildActive(HWND hWnd) {
                HWND wnd = MainFrame::getMainFrame()->MDIGetActive();
                dcassert(wnd != NULL);
                return (hWnd == wnd);
        }
        // !SMT!-S
        static void closeWindow() {
           if (frame) ::PostMessage(frame->m_hWnd, WM_CLOSE, NULL, NULL);
        }
};

template<class T, int title, int ID>
T* StaticFrame<T, title, ID>::frame = NULL;

class WinUtil {
public:
        static CImageList fileImages;
        static int fileImageCount;
        static CImageList userImages;
        static CImageList flagImages;
        static int flagImageCount; // !SMT!-IP

        // !SMT!-UI search user by exact share size
        typedef HASH_MULTIMAP<uint64_t,User::Ptr> ShareMap;
        typedef ShareMap::iterator ShareIter;
        //static ShareMap UsersShare;
        //static mutable CriticalSection usercs;

        struct TextItem {
                WORD itemID;
                ResourceManager::Strings translatedString;
        };

        typedef HASH_MAP<tstring, int> ImageMap;
        typedef ImageMap::const_iterator ImageIter;
        static ImageMap fileIndexes;
        static HBRUSH bgBrush;
        static COLORREF textColor;
        static COLORREF bgColor;
        static HFONT font;
        static int fontHeight;
        static HFONT boldFont;
        static HFONT systemFont;
        static HFONT smallBoldFont;
        static CMenu mainMenu;
        static CMenu grantMenu;
        static CMenu speedMenu; // !SMT!-S
        static CMenu privateMenu; // !SMT!-PSW
        static OMenu userSummaryMenu; // !SMT!-UI
        static HICON banIconOnline; // !SMT!-UI
        static HICON banIconOffline; // !SMT!-UI
        static int dirIconIndex;
        static int dirMaskedIndex;
        static TStringList lastDirs;
        static HWND mainWnd;
        static HWND mdiClient;
        static FlatTabCtrl* tabCtrl;
        static tstring commands;
        static HHOOK hook;
        static tstring tth;
        static StringPairList initialDirs;
        static tstring exceptioninfo;
        static DWORD helpCookie;
        static bool isAppActive;
        static CHARFORMAT2 m_TextStyleTimestamp;
        static CHARFORMAT2 m_ChatTextGeneral;
        static CHARFORMAT2 m_TextStyleMyNick;
        static CHARFORMAT2 m_ChatTextMyOwn;
        static CHARFORMAT2 m_ChatTextServer;
        static CHARFORMAT2 m_ChatTextSystem;
        static CHARFORMAT2 m_TextStyleBold;
        static CHARFORMAT2 m_TextStyleFavUsers;
        static CHARFORMAT2 m_TextStyleOPs;
        static CHARFORMAT2 m_TextStyleURL;
        static CHARFORMAT2 m_ChatTextPrivate;
        static CHARFORMAT2 m_ChatTextLog;
        static bool mutesounds;
        static DWORD comCtlVersion;

        static void init(HWND hWnd);
        static void uninit();

        static void initColors();
        static void reLoadImages(); // User Icon Begin / User Icon End

        static bool getUserColor(const User::Ptr& aUser, COLORREF &fg, COLORREF &bg, Client *hub = 0); // !SMT!-UI

        static string getAppName() {
                TCHAR buf[MAX_PATH+1];
                DWORD x = GetModuleFileName(NULL, buf, MAX_PATH);
                return Text::fromT(tstring(buf, x));
        }

        static void clearSummaryMenu(); // !SMT!-UI
        static void decodeFont(const tstring& setting, LOGFONT &dest);

        static void addInitalDir(const User::Ptr& user, string dir) {
                // Clear out previos initial dirs, just in case
                /// @todo clean up
                getInitialDir(user);
                while(initialDirs.size() > 30) {
                        initialDirs.erase(initialDirs.begin());
                }
                initialDirs.push_back(make_pair(user->getCID().toBase32(), dir));
        }

        static string getInitialDir(const User::Ptr& user) {
                for(StringPairIter i = initialDirs.begin(); i != initialDirs.end(); ++i) {
                        if(i->first == user->getCID().toBase32()) {
                                string dir = i->second;
                                initialDirs.erase(i);
                                return dir;
                        }
                }
                return Util::emptyString;
        }

        static bool getVersionInfo(OSVERSIONINFOEX& ver);

        /**
         * Check if this is a common /-command.
         * @param cmd The whole text string, will be updated to contain only the command.
         * @param param Set to any parameters.
         * @param message Message that should be sent to the chat.
         * @param status Message that should be shown in the status line.
         * @return True if the command was processed, false otherwise.
         */
        static bool checkCommand(tstring& cmd, tstring& param, tstring& message, tstring& status);

        static int getTextWidth(const tstring& str, HWND hWnd) {
                HDC dc = ::GetDC(hWnd);
                int sz = getTextWidth(str, dc);
                ::ReleaseDC(mainWnd, dc);
                return sz;
        }
        static int getTextWidth(const tstring& str, HDC dc) {
                SIZE sz = { 0, 0 };
                ::GetTextExtentPoint32(dc, str.c_str(), str.length(), &sz);
                return sz.cx;
        }

        static int getTextHeight(HWND wnd, HFONT fnt) {
                HDC dc = ::GetDC(wnd);
                int h = getTextHeight(dc, fnt);
                ::ReleaseDC(wnd, dc);
                return h;
        }

        static int getTextHeight(HDC dc, HFONT fnt) {
                HGDIOBJ old = ::SelectObject(dc, fnt);
                int h = getTextHeight(dc);
                ::SelectObject(dc, old);
                return h;
        }

        static int getTextHeight(HDC dc) {
                TEXTMETRIC tm;
                ::GetTextMetrics(dc, &tm);
                return tm.tmHeight;
        }

        static void setClipboard(const tstring& str);

        static void addLastDir(const tstring& dir) {
                if(find(lastDirs.begin(), lastDirs.end(), dir) != lastDirs.end()) {
                        return;
                }
                if(lastDirs.size() == 10) {
                        lastDirs.erase(lastDirs.begin());
                }
                lastDirs.push_back(dir);
        }

        static void UnlinkStaticMenus(CMenu &menu); // !SMT!-UI

        static tstring encodeFont(LOGFONT const& font);

        static bool browseFile(tstring& target, HWND owner = NULL, bool save = true, const tstring& initialDir = Util::emptyStringW, const TCHAR* types = NULL, const TCHAR* defExt = NULL);
        static bool browseDirectory(tstring& target, HWND owner = NULL);

        // Hash related
        static void bitziLink(const TTHValue& /*aHash*/);
        static void copyMagnet(const TTHValue& /*aHash*/, const tstring& /*aFile*/, int64_t);
        static string getMagnet(const TTHValue& aHash, const tstring& aFile, int64_t aSize);
        static string getWebMagnet(const TTHValue& aHash, const tstring& aFile, int64_t aSize); // !SMT!-UI
        static string getHtmlMagnet(const TTHValue& aHash, const tstring& aFile, int64_t aSize);

        static void searchHash(const TTHValue& /*aHash*/);

        // URL related
        static void registerDchubHandler();
        static void registerADChubHandler();
        static void registerMagnetHandler();
        static void unRegisterDchubHandler();
        static void unRegisterADChubHandler();
        static void unRegisterMagnetHandler();
        static void parseDchubUrl(const tstring& /*aUrl*/);
        static void parseADChubUrl(const tstring& /*aUrl*/);
        static void parseMagnetUri(const tstring& /*aUrl*/, bool aOverride = false);
        static bool parseDBLClick(const tstring& /*aString*/, string::size_type start, string::size_type end);
        static bool urlDcADCRegistered;
        static bool urlMagnetRegistered;
        static int textUnderCursor(POINT p, CEdit& ctrl, tstring& x);
        static void openLink(const tstring& url,bool internal=false);
        static void openFile(const tstring& file) {
                ::ShellExecute(NULL, NULL, file.c_str(), NULL, NULL, SW_SHOWNORMAL);
        }
        static void openFolder(const tstring& file);

        static int getIconIndex(const tstring& aFileName);

        static int getDirIconIndex() { return dirIconIndex; }
        static int getDirMaskedIndex() { return dirMaskedIndex; }

        static double toBytes(TCHAR* aSize);

        static int getOsMajor();
        static int getOsMinor();

        //returns the position where the context menu should be
        //opened if it was invoked from the keyboard.
        //aPt is relative to the screen not the control.
        static void getContextMenuPos(CListViewCtrl& aList, POINT& aPt);
        static void getContextMenuPos(CTreeViewCtrl& aTree, POINT& aPt);
        static void getContextMenuPos(CEdit& aEdit,                     POINT& aPt);

        static bool getUCParams(HWND parent, const UserCommand& cmd, StringMap& sm) throw();

        /*static tstring getNicks(const CID& cid) throw();
        static tstring getNicks(const User::Ptr& u) {
                if(u->isSet(User::NMDC)) {
                        return Text::toT(u->getFirstNick());
                } else  {
                        return getNicks(u->getCID());
                }
        }*/
        /** @return Pair of hubnames as a string and a bool representing the user's online status */
        static pair<tstring, bool> getHubNames(const CID& cid) throw();
        static pair<tstring, bool> getHubNames(const User::Ptr& u) { return getHubNames(u->getCID()); }

        static void splitTokens(int* array, const string& tokens, int maxItems = -1) throw();
        static void saveHeaderOrder(CListViewCtrl& ctrl, SettingsManager::StrSetting order,
                SettingsManager::StrSetting widths, int n, int* indexes, int* sizes) throw(); // !SMT!-UI todo: disable - this routine does not save column visibility

        static bool isShift() { return (GetKeyState(VK_SHIFT) & 0x8000) > 0; }
        static bool isAlt() { return (GetKeyState(VK_MENU) & 0x8000) > 0; }
        static bool isCtrl() { return (GetKeyState(VK_CONTROL) & 0x8000) > 0; }

        static tstring escapeMenu(tstring str) {
                string::size_type i = 0;
                while( (i = str.find(_T('&'), i)) != string::npos) {
                        str.insert(str.begin()+i, 1, _T('&'));
                        i += 2;
                }
                return str;
        }
        template<class T> static HWND hiddenCreateEx(T& p) throw() {
                HWND active = (HWND)::SendMessage(mdiClient, WM_MDIGETACTIVE, 0, 0);
                ::LockWindowUpdate(mdiClient);
                HWND ret = p.CreateEx(mdiClient);
                if(active && ::IsWindow(active))
                        ::SendMessage(mdiClient, WM_MDIACTIVATE, (WPARAM)active, 0);
                ::LockWindowUpdate(0);
                return ret;
        }
        template<class T> static HWND hiddenCreateEx(T* p) throw() {
                return hiddenCreateEx(*p);
        }

        static void translate(HWND page, TextItem* textItems)
        {
                if (textItems != NULL) {
                        for(int i = 0; textItems[i].itemID != 0; i++) {
                                ::SetDlgItemText(page, textItems[i].itemID,
                                        Text::toT(ResourceManager::getInstance()->getString(textItems[i].translatedString)).c_str());
                        }
                }
        }

        struct tbIDImage {
                int id, image;
                bool check;
                ResourceManager::Strings tooltip;
        };
        static tbIDImage ToolbarButtons[];

        static void ClearPreviewMenu(OMenu &previewMenu);
        static int SetupPreviewMenu(CMenu &previewMenu, string extension);
        static void RunPreviewCommand(unsigned int index, string target);
        static string formatTime(long rest);
        static uint8_t getImage(const Identity& u);
        static int getFlagImage(string& country, bool fullname = false); // !SMT!-IP
        static string generateStats();
        static string disableCzChars(string message);
        static bool shutDown(int action);
        static int getFirstSelectedIndex(CListViewCtrl& list);
        static int setButtonPressed(int nID, bool bPressed = true);
        static string getWMPSpam(HWND playerWnd = NULL);
        static string getItunesSpam(HWND playerWnd = NULL);
        static string getMPCSpam();
        static void offsetDlgItem(CWindow* dlg,int id,int cx ,int cy);
        // !SMT!-UI
        static COLORREF blendColors(COLORREF a, COLORREF b) {
                return ((uint32_t)a & 0xFEFEFE)/2 + ((uint32_t)b & 0xFEFEFE)/2;
        }
        static int MessageBox(LPCTSTR lpText,LPCTSTR lpCaption = 0 ,UINT uType=0 ){
            if(ResourceManager::getInstance()->isRTL()){
                uType |= MB_RIGHT | MB_RTLREADING ;
            }
            return ::MessageBox( WinUtil::mainWnd , lpText , lpCaption != 0 ? lpCaption : T_FULL_APPNAME , uType );
        }

// FDM extension
        static string findNickInTString(const tstring aLine) {
                // set to 0, to fix warning C4701
                tstring::size_type i = 0;
                tstring::size_type j = 0;

                //Check For <Nick>
                if (((i = aLine.find_first_of('<')) != string::npos) && ((j = aLine.find_first_of('>')) != string::npos && j > i))
                        return Text::fromT(aLine.substr(i + 1, j - i - 1).c_str());
                //Check for * Nick
                if ((aLine.find(_T("***")) == string::npos) && ((i = Util::findSubString(Text::fromT(aLine).c_str(), "* ")) != string::npos) && ((j = aLine.find(' ', 2)) != string::npos && j > i))
                        return Text::fromT(aLine.substr(i + 2, j - i - 2).c_str());
                return "";
        }

        static string getIpCountryForChat(string ip, bool ts) {
                if (!ip.empty() /* && Util::fileExists(Util::getConfigPath() + "GeoIPCountryWhois.csv") !SMT!-IP */ && BOOLSETTING(COUNTRY_IN_CHAT) && BOOLSETTING(IP_IN_CHAT)) {
                        return ts ? " | " + ip + " | " + Util::getIpCountry(ip) : " " + ip + " | " + Util::getIpCountry(ip);
                } else if (!ip.empty() /* && Util::fileExists(Util::getConfigPath() + "GeoIPCountryWhois.csv") !SMT!-IP */ && BOOLSETTING(COUNTRY_IN_CHAT)) {
                        return ts ? " | " + Util::getIpCountry(ip) : Util::getIpCountry(ip);
                } else if (!ip.empty() && BOOLSETTING(IP_IN_CHAT)) {
                        return ts ? " | " + ip + " " : " " + ip + " ";
                }
                return "";
        }
// FDM extension
        static bool WinUtil::hasNetLimiter();
private:
        static int CALLBACK browseCallbackProc(HWND hwnd, UINT uMsg, LPARAM /*lp*/, LPARAM pData);

};

/*
// Not working as expected ...
class SimpleDlg : public CDialogImpl<SimpleDlg> {
public: 
	static DWORD GetWndExStyle(DWORD dwExStyle) {
		return dwExStyle | WS_EX_LAYOUTRTL;
	}
    const int IDD;
    SimpleDlg ( int resource_id = 0 ) : IDD(resource_id) {};    
protected: 
    BEGIN_MSG_MAP(SimpleDlg)
        MESSAGE_HANDLER(WM_INITDIALOG, OnInitSimpleDlg)
    END_MSG_MAP()
    
    LRESULT OnInitSimpleDlg(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
        if(ResourceManager::getInstance()->isRTL()){
            doRtlLayout(m_hWnd);
        }
        bHandled = FALSE;
        return FALSE;
    }

    void doRtlLayout(HWND hwnd){
        CWindow parent;
        parent.Attach(hwnd);
        parent.ModifyStyleEx(0,WS_EX_LAYOUTRTL);
        
        CRect rcParent;
        parent.GetClientRect(&rcParent);

        for(HWND hWndChild = ::GetTopWindow(hwnd); hWndChild != NULL;hWndChild = ::GetNextWindow(hWndChild, GW_HWNDNEXT)){
            CWindow child;
            child.Attach(hWndChild);
            
            CRect rcChild;
            child.GetClientRect(&rcChild);
            child.MapWindowPoints(hwnd,&rcChild);
            int w = rcChild.Width();
            rcChild.right = rcParent.right - rcChild.left;
            rcChild.left = rcChild.right - w ;
            child.MoveWindow(rcChild);
            child.Detach();         
            doRtlLayout(hWndChild);            
        }
        parent.Detach();
    }
};
*/

#endif // !defined(WIN_UTIL_H)

/**
 * @file
 * $Id: WinUtil.h,v 1.62 2006/11/12 20:32:08 bigmuscle Exp $
 */
