/*
 * Copyright (C) 2001-2006 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#if !defined(ABOUT_DLG_H)
#define ABOUT_DLG_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "../client/HttpConnection.h"
#include "../client/SimpleXML.h"
#include <atlctrlx.h>

static const TCHAR thanks[] =
_T(" - DC++,ApexDC++,ApexDC++ mod,StrongDC++,revconnect \r\n")
_T(" - All testers and DC community \r\n")
_T(" - Spread The Word :-)\r\n");


class AboutDlg : public CDialogImpl<AboutDlg>, private HttpConnectionListener
{

public:
        enum { IDD = IDD_ABOUTBOX };
        enum { WM_VERSIONDATA = WM_APP + 53 };

        AboutDlg() { }
        virtual ~AboutDlg() { }

        BEGIN_MSG_MAP(AboutDlg)
                MESSAGE_HANDLER(WM_INITDIALOG, OnInitDialog)
                MESSAGE_HANDLER(WM_VERSIONDATA, onVersionData)
                COMMAND_ID_HANDLER(IDOK, OnCloseCmd)
                COMMAND_ID_HANDLER(IDCANCEL, OnCloseCmd)
                COMMAND_ID_HANDLER(IDC_LINK, onLink)
        END_MSG_MAP()

        LRESULT OnInitDialog(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/) {
                SetWindowText(CTSTRING(ABOUT));
                SetDlgItemText(IDC_VERSION,Text::tformat(
                  "%s\r\n"
                  "Development/coding: themaestro\r\n"
				  ,FULL_APPNAME).c_str());

                SetDlgItemText(IDC_ABOUT_THANKS_BOX,CTSTRING(ABOUT_THANKS_BOX));
                CEdit ctrlThanks(GetDlgItem(IDC_ABOUT_THANKS));
                ctrlThanks.FmtLines(TRUE);
                ctrlThanks.AppendText(thanks, TRUE);
                ctrlThanks.Detach();
                
                SetDlgItemText(IDC_TTH, WinUtil::tth.c_str());
                                
                SetDlgItemText(IDC_ABOUT_LATEST_BOX, CTSTRING(ABOUT_LATEST_BOX));
                SetDlgItemText(IDC_ABOUT_LATEST, CTSTRING(DOWNLOADING));
                                
                double ratio= 0;
                if(SETTING(TOTAL_DOWNLOAD) > 0) {
                    ratio = ((double)SETTING(TOTAL_UPLOAD)) / ((double)SETTING(TOTAL_DOWNLOAD));
                }
                SetDlgItemText(IDC_ABOUT_TOTAL, Text::tformat(CSTRING(ABOUT_TOTAL),
                    Util::formatBytes(SETTING(TOTAL_DOWNLOAD)).c_str(),
                    Util::formatBytes(SETTING(TOTAL_UPLOAD)).c_str(),
                    ratio,WinUtil::formatTime(Util::getUptime()).c_str()).c_str());
                SetDlgItemText(IDC_ABOUT_TOTAL_BOX, CTSTRING(ABOUT_TOTAL_BOX));
                SetDlgItemText(IDC_LINK, Text::toT(LINK_WEBSITE).c_str());
                SetDlgItemText(IDC_ABOUT_COMPILED, Text::tformat(CSTRING(ABOUT_COMPILED),BUILDID,Util::getCompileDate().c_str()).c_str());

                url.SubclassWindow(GetDlgItem(IDC_LINK));
                url.SetHyperLinkExtendedStyle(HLINK_COMMANDBUTTON|HLINK_UNDERLINEHOVER);
                url.m_tip.AddTool(url, Text::toT(LINK_WEBSITE).c_str(), &url.m_rcLink, 1);

                CenterWindow(GetParent());
                c.addListener(this);
                c.downloadFile(LINK_VERSIONFILE);
                return TRUE;
        }

        LRESULT onVersionData(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/) {
                tstring* x = (tstring*) wParam;
                SetDlgItemText(IDC_ABOUT_LATEST, x->c_str());
                delete x;
                return 0;
        }

        LRESULT onLink (WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
                WinUtil::openLink(Text::toT(LINK_WEBSITE).c_str());
                return 0;
        }

        LRESULT OnCloseCmd(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
                EndDialog(wID);
                return 0;
        }

private:
        HttpConnection c;
        CHyperLink url;

        AboutDlg(const AboutDlg&) { dcassert(0); }

        virtual void on(HttpConnectionListener::Data, HttpConnection* /*conn*/, const uint8_t* buf, size_t len) throw() {
                downBuf.append((char*)buf, len);
        }

        virtual void on(HttpConnectionListener::Complete, HttpConnection* conn, const string&) throw() {
                
                if(!downBuf.empty()) {
                        SimpleXML xml;
                        xml.fromXML(downBuf);
                        xml.stepIn();
                        if(xml.findChild("CurrentVersion")) {
                            tstring* x = new tstring(Text::toT(xml.getChildAttrib("version")));
                            PostMessage(WM_VERSIONDATA, (WPARAM) x);
                        }
                }
                conn->removeListener(this);
        }

        virtual void on(HttpConnectionListener::Failed, HttpConnection* conn, const string& aLine) throw() {
                tstring* x = new tstring(Text::toT(aLine));
                PostMessage(WM_VERSIONDATA, (WPARAM) x);
                conn->removeListener(this);
        }

        string downBuf;
};

#endif // !defined(ABOUT_DLG_H)

/**
 * @file
 * $Id: AboutDlg.h,v 1.47 2006/10/28 20:25:31 bigmuscle Exp $
 */
