/*
 * Copyright (C) 2001-2006 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "Resource.h"

#include "AppearancePage.h"
#include "../client/SettingsManager.h"
#include "../client/StringTokenizer.h"

#include "WinUtil.h"

static const TCHAR types[] = _T("DC++ Theme Files\0*.dctheme;\0DC++ Settings Files\0*.xml;\0All Files\0*.*\0");
static const TCHAR defExt[] = _T(".dctheme");

PropPage::TextItem AppearancePage::texts[] = {
        { IDC_SETTINGS_APPEARANCE_OPTIONS, ResourceManager::SETTINGS_OPTIONS },
        { IDC_SETTINGS_BOLD_CONTENTS, ResourceManager::SETTINGS_BOLD_OPTIONS },
        { IDC_SETTINGS_TIME_STAMPS_FORMAT, ResourceManager::SETTINGS_TIME_STAMPS_FORMAT },
        { IDC_SETTINGS_LANGUAGE_FILE, ResourceManager::SETTINGS_LANGUAGE_FILE },
        { IDC_BROWSE, ResourceManager::BROWSE_ACCEL },
        { IDC_SETTINGS_REQUIRES_RESTART, ResourceManager::SETTINGS_REQUIRES_RESTART },
        { IDC_SETTINGS_GET_USER_COUNTRY, ResourceManager::SETTINGS_GET_USER_COUNTRY },        
        { IDC_SETTINGS_THEME, ResourceManager::SETTINGS_THEME },
        { IDC_THEME_FILE_LABEL, ResourceManager::SETTINGS_THEME_FILE_LABEL }, 
        { IDC_THEME_IMPORT, ResourceManager::SETTINGS_THEME_IMPORT },
        { IDC_THEME_OPTIONS_LABEL , ResourceManager::SETTINGS_THEME_OPTIONS_LABEL },
        { IDC_THEME_EXPORT , ResourceManager::SETTINGS_THEME_EXPORT },   
        { IDC_THEME_UPDATE , ResourceManager::SETTINGS_THEME_UPDATE },   
        { 0, ResourceManager::SETTINGS_AUTO_AWAY }
};

PropPage::Item AppearancePage::items[] = {
        { IDC_TIME_STAMPS_FORMAT, SettingsManager::TIME_STAMPS_FORMAT, PropPage::T_STR },
        { IDC_LANGUAGE, SettingsManager::LANGUAGE_FILE, PropPage::T_STR },
        { 0, 0, PropPage::T_END }
};

PropPage::ListItem AppearancePage::listItems[] = {
        { SettingsManager::SHOW_PROGRESS_BARS, ResourceManager::SETTINGS_SHOW_PROGRESS_BARS },
        { SettingsManager::SHOW_INFOTIPS, ResourceManager::SETTINGS_SHOW_INFO_TIPS },
        { SettingsManager::FILTER_MESSAGES, ResourceManager::SETTINGS_FILTER_MESSAGES },
        { SettingsManager::MINIMIZE_TRAY, ResourceManager::SETTINGS_MINIMIZE_TRAY },
        { SettingsManager::EXPAND_QUEUE , ResourceManager::SETTINGS_EXPAND_QUEUE },
        { SettingsManager::SORT_FAVUSERS_FIRST, ResourceManager::SETTINGS_SORT_FAVUSERS_FIRST },
        { SettingsManager::USE_SYSTEM_ICONS, ResourceManager::SETTINGS_USE_SYSTEM_ICONS },
        { SettingsManager::GET_USER_COUNTRY, ResourceManager::SETTINGS_GET_USER_COUNTRY },
        { SettingsManager::USE_OLD_SHARING_UI, ResourceManager::SETTINGS_USE_OLD_SHARING_UI },
        { SettingsManager::USE_VERTICAL_VIEW, ResourceManager::SETTINGS_USE_VERTICAL_VIEW },
        { SettingsManager::TABS_ON_TOP, ResourceManager::TABS_ON_TOP },
        { 0, ResourceManager::SETTINGS_AUTO_AWAY }
};

PropPage::ListItem AppearancePage::boldItems[] = {
        { SettingsManager::BOLD_FINISHED_DOWNLOADS, ResourceManager::FINISHED_DOWNLOADS },
        { SettingsManager::BOLD_FINISHED_UPLOADS, ResourceManager::FINISHED_UPLOADS },
        { SettingsManager::BOLD_QUEUE, ResourceManager::DOWNLOAD_QUEUE },
        { SettingsManager::BOLD_HUB, ResourceManager::HUB },
        { SettingsManager::BOLD_PM, ResourceManager::PRIVATE_MESSAGE },
        { SettingsManager::BOLD_SEARCH, ResourceManager::SEARCH },
        { SettingsManager::BOLD_WAITING_USERS, ResourceManager::WAITING_USERS },
        { 0, ResourceManager::SETTINGS_AUTO_AWAY }
};

AppearancePage::~AppearancePage(){ }

const int ThemeSounds[] = {
    SettingsManager::BEGINFILE, SettingsManager::FINISHFILE, 
    SettingsManager::SOURCEFILE, SettingsManager::UPLOADFILE, 
    SettingsManager::FAKERFILE, SettingsManager::BEEPFILE, 
    SettingsManager::CHATNAMEFILE, SettingsManager::SOUND_TTH,
    SettingsManager::SOUND_EXC, SettingsManager::SOUND_HUBCON,
    SettingsManager::SOUND_HUBDISCON, SettingsManager::SOUND_FAVUSER,
    SettingsManager::SOUND_FAVUSER_OFFLINE, SettingsManager::SOUND_TYPING_NOTIFY,
    SettingsManager::PRIVATE_MESSAGE_BEEP_OPEN, SettingsManager::PRIVATE_MESSAGE_BEEP,

    SettingsManager::STR_LAST
};

const int ThemeFontColor[] = {
    SettingsManager::BOLD_AUTHOR_MESS,
    SettingsManager::BACKGROUND_COLOR,SettingsManager::SEARCH_ALTERNATE_COLOUR,
    SettingsManager::ERROR_COLOR,SettingsManager::PROGRESS_BACK_COLOR, 
    SettingsManager::PROGRESS_COMPRESS_COLOR, SettingsManager::PROGRESS_SEGMENT_COLOR,
    SettingsManager::COLOR_DOWNLOADED,SettingsManager::COLOR_RUNNING,
    SettingsManager::COLOR_VERIFIED,SettingsManager::COLOR_AVOIDING,
    SettingsManager::BAN_COLOR,SettingsManager::DUPE_COLOR,
    SettingsManager::TEXT_GENERAL_BACK_COLOR,SettingsManager::TEXT_GENERAL_FORE_COLOR,
    SettingsManager::TEXT_GENERAL_BOLD, SettingsManager::TEXT_GENERAL_ITALIC,
    SettingsManager::TEXT_MYNICK_BACK_COLOR, SettingsManager::TEXT_MYNICK_FORE_COLOR,
    SettingsManager::TEXT_MYNICK_BOLD, SettingsManager::TEXT_MYNICK_ITALIC,
    SettingsManager::TEXT_MYOWN_BACK_COLOR, SettingsManager::TEXT_MYOWN_FORE_COLOR,
    SettingsManager::TEXT_MYOWN_BOLD, SettingsManager::TEXT_MYOWN_ITALIC,
    SettingsManager::TEXT_PRIVATE_BACK_COLOR, SettingsManager::TEXT_PRIVATE_FORE_COLOR,
    SettingsManager::TEXT_PRIVATE_BOLD, SettingsManager::TEXT_PRIVATE_ITALIC,
    SettingsManager::TEXT_SYSTEM_BACK_COLOR, SettingsManager::TEXT_SYSTEM_FORE_COLOR,
    SettingsManager::TEXT_SYSTEM_BOLD, SettingsManager::TEXT_SYSTEM_ITALIC,
    SettingsManager::TEXT_SERVER_BACK_COLOR, SettingsManager::TEXT_SERVER_FORE_COLOR,
    SettingsManager::TEXT_SERVER_BOLD, SettingsManager::TEXT_SERVER_ITALIC,
    SettingsManager::TEXT_TIMESTAMP_BACK_COLOR, SettingsManager::TEXT_TIMESTAMP_FORE_COLOR,
    SettingsManager::TEXT_TIMESTAMP_BOLD, SettingsManager::TEXT_TIMESTAMP_ITALIC,
    SettingsManager::TEXT_URL_BACK_COLOR, SettingsManager::TEXT_URL_FORE_COLOR,
    SettingsManager::TEXT_URL_BOLD, SettingsManager::TEXT_URL_ITALIC,
    SettingsManager::TEXT_FAV_BACK_COLOR, SettingsManager::TEXT_FAV_FORE_COLOR,
    SettingsManager::TEXT_FAV_BOLD, SettingsManager::TEXT_FAV_ITALIC,
    SettingsManager::TEXT_OP_BACK_COLOR, SettingsManager::TEXT_OP_FORE_COLOR,
    SettingsManager::TEXT_OP_BOLD, SettingsManager::TEXT_OP_ITALIC,
    SettingsManager::TEXT_FONT, SettingsManager::TEXT_COLOR,
    SettingsManager::BACKGROUND_COLOR, 
    // progressbar
	SettingsManager::PROGRESSBAR_ODC_STYLE, SettingsManager::STEALTHY_STYLE,
	SettingsManager::PROGRESSBAR_ODC_BUMPED, SettingsManager::PROGRESS_OVERRIDE_COLORS,
    SettingsManager::PROGRESS_OVERRIDE_COLORS2, SettingsManager::PROGRESS_3DDEPTH,
	SettingsManager::TOP_SPEED, SettingsManager::TOP_UP_SPEED, 
	SettingsManager::MENUBAR_LEFT_COLOR, SettingsManager::MENUBAR_RIGHT_COLOR, 
	SettingsManager::MENUBAR_BUMPED, SettingsManager::MENUBAR_TWO_COLORS, 
    SettingsManager::DOWNLOAD_BAR_COLOR,SettingsManager::UPLOAD_BAR_COLOR,
	SettingsManager::PROGRESS_TEXT_COLOR_DOWN,SettingsManager::PROGRESS_TEXT_COLOR_UP,
    // user list
	SettingsManager::NORMAL_COLOUR,SettingsManager::RESERVED_SLOT_COLOR, 
    SettingsManager::IGNORED_COLOR, SettingsManager::FIREBALL_COLOR, 
    SettingsManager::SERVER_COLOR, SettingsManager::PASIVE_COLOR, 
    SettingsManager::OP_COLOR, SettingsManager::CLIENT_CHECKED_COLOUR, 
    SettingsManager::FILELIST_CHECKED_COLOUR, SettingsManager::FULL_CHECKED_COLOUR, 
    SettingsManager::BAD_CLIENT_COLOUR, SettingsManager::BAD_FILELIST_COLOUR, 
    SettingsManager::USERLIST_IMAGE,

    SettingsManager::STR_LAST
};
const int ThemeBallonPopups[] = {
    SettingsManager::POPUP_TIME, SettingsManager::POPUPFILE,
    SettingsManager::MAX_MSG_LENGTH,SettingsManager::POPUP_HUB_CONNECTED,
    SettingsManager::POPUP_HUB_DISCONNECTED, SettingsManager::POPUP_FAVORITE_CONNECTED, 
    SettingsManager::POPUP_FAVORITE_DISCONNECTED,SettingsManager::POPUP_CHEATING_USER, 
    SettingsManager::POPUP_DOWNLOAD_START, SettingsManager::POPUP_DOWNLOAD_FAILED,
    SettingsManager::POPUP_DOWNLOAD_FINISHED, SettingsManager::POPUP_UPLOAD_FINISHED,
    SettingsManager::POPUP_PM, SettingsManager::POPUP_NEW_PM,
    SettingsManager::POPUP_AWAY,SettingsManager::POPUP_MINIMIZED,
    SettingsManager::PM_PREVIEW,SettingsManager::POPUP_TYPE
};

const int ThemeToolbar[] = {
    SettingsManager::TOOLBARIMAGE, SettingsManager::TOOLBARHOTIMAGE, 
	SettingsManager::TB_IMAGE_SIZE, SettingsManager::TB_IMAGE_SIZE_HOT, 
    SettingsManager::TOOLBAR,

    SettingsManager::STR_LAST
};
const int ThemeWindows[] = {
	SettingsManager::OPEN_PUBLIC, SettingsManager::OPEN_FAVORITE_HUBS, 
	SettingsManager::OPEN_FAVORITE_USERS,SettingsManager::OPEN_QUEUE,
	SettingsManager::OPEN_FINISHED_DOWNLOADS,SettingsManager::OPEN_WAITING_USERS,
	SettingsManager::OPEN_FINISHED_UPLOADS,SettingsManager::OPEN_SEARCH_SPY,
	SettingsManager::OPEN_NETWORK_STATISTICS, SettingsManager::OPEN_NOTEPAD,
	SettingsManager::OPEN_CDMDEBUG,

	SettingsManager::POPUP_PMS,SettingsManager::POPUP_HUB_PMS,
    SettingsManager::POPUP_BOT_PMS,SettingsManager::POPUNDER_FILELIST,
    SettingsManager::POPUNDER_PM,SettingsManager::JOIN_OPEN_NEW_WINDOW,
    SettingsManager::IGNORE_HUB_PMS,SettingsManager::IGNORE_BOT_PMS,
    SettingsManager::TOGGLE_ACTIVE_WINDOW,SettingsManager::PROMPT_PASSWORD,
    SettingsManager::ONE_SEARCH_WINDOW,SettingsManager::AUTO_HIDE_TABS_BAR,
    SettingsManager::AUTO_HIDE_TRANSFER_BAR,
	SettingsManager::CONFIRM_EXIT,SettingsManager::CONFIRM_HUB_REMOVAL,SettingsManager::CONFIRM_DELETE,
    
    SettingsManager::STR_LAST
};

const int ThemeAppearnce[] = {
    SettingsManager::TIME_STAMPS_FORMAT,
    SettingsManager::SHOW_PROGRESS_BARS, SettingsManager::SHOW_INFOTIPS, 
    SettingsManager::FILTER_MESSAGES, SettingsManager::MINIMIZE_TRAY, 
    SettingsManager::EXPAND_QUEUE, SettingsManager::SORT_FAVUSERS_FIRST, 
    SettingsManager::USE_SYSTEM_ICONS, SettingsManager::GET_USER_COUNTRY, 
    SettingsManager::USE_OLD_SHARING_UI, SettingsManager::USE_VERTICAL_VIEW, 
    SettingsManager::TABS_ON_TOP, SettingsManager::BOLD_FINISHED_DOWNLOADS, 
    SettingsManager::BOLD_FINISHED_UPLOADS, SettingsManager::BOLD_QUEUE, 
    SettingsManager::BOLD_HUB, SettingsManager::BOLD_PM, 
    SettingsManager::BOLD_SEARCH, SettingsManager::BOLD_WAITING_USERS,

    SettingsManager::STR_LAST
};

int const ThemeAll[] = {
    // sounds
    SettingsManager::BEGINFILE, SettingsManager::FINISHFILE, 
    SettingsManager::SOURCEFILE, SettingsManager::UPLOADFILE, 
    SettingsManager::FAKERFILE, SettingsManager::BEEPFILE, 
    SettingsManager::CHATNAMEFILE, SettingsManager::SOUND_TTH,
    SettingsManager::SOUND_EXC, SettingsManager::SOUND_HUBCON,
    SettingsManager::SOUND_HUBDISCON, SettingsManager::SOUND_FAVUSER,
    SettingsManager::SOUND_FAVUSER_OFFLINE, SettingsManager::SOUND_TYPING_NOTIFY,
    SettingsManager::PRIVATE_MESSAGE_BEEP_OPEN, SettingsManager::PRIVATE_MESSAGE_BEEP,
    //font & color
    SettingsManager::BOLD_AUTHOR_MESS,
    SettingsManager::BACKGROUND_COLOR,SettingsManager::SEARCH_ALTERNATE_COLOUR,
    SettingsManager::ERROR_COLOR,SettingsManager::PROGRESS_BACK_COLOR, 
    SettingsManager::PROGRESS_COMPRESS_COLOR, SettingsManager::PROGRESS_SEGMENT_COLOR,
    SettingsManager::COLOR_DOWNLOADED,SettingsManager::COLOR_RUNNING,
    SettingsManager::COLOR_VERIFIED,SettingsManager::COLOR_AVOIDING,
    SettingsManager::BAN_COLOR,SettingsManager::DUPE_COLOR,
    SettingsManager::TEXT_GENERAL_BACK_COLOR,SettingsManager::TEXT_GENERAL_FORE_COLOR,
    SettingsManager::TEXT_GENERAL_BOLD, SettingsManager::TEXT_GENERAL_ITALIC,
    SettingsManager::TEXT_MYNICK_BACK_COLOR, SettingsManager::TEXT_MYNICK_FORE_COLOR,
    SettingsManager::TEXT_MYNICK_BOLD, SettingsManager::TEXT_MYNICK_ITALIC,
    SettingsManager::TEXT_MYOWN_BACK_COLOR, SettingsManager::TEXT_MYOWN_FORE_COLOR,
    SettingsManager::TEXT_MYOWN_BOLD, SettingsManager::TEXT_MYOWN_ITALIC,
    SettingsManager::TEXT_PRIVATE_BACK_COLOR, SettingsManager::TEXT_PRIVATE_FORE_COLOR,
    SettingsManager::TEXT_PRIVATE_BOLD, SettingsManager::TEXT_PRIVATE_ITALIC,
    SettingsManager::TEXT_SYSTEM_BACK_COLOR, SettingsManager::TEXT_SYSTEM_FORE_COLOR,
    SettingsManager::TEXT_SYSTEM_BOLD, SettingsManager::TEXT_SYSTEM_ITALIC,
    SettingsManager::TEXT_SERVER_BACK_COLOR, SettingsManager::TEXT_SERVER_FORE_COLOR,
    SettingsManager::TEXT_SERVER_BOLD, SettingsManager::TEXT_SERVER_ITALIC,
    SettingsManager::TEXT_TIMESTAMP_BACK_COLOR, SettingsManager::TEXT_TIMESTAMP_FORE_COLOR,
    SettingsManager::TEXT_TIMESTAMP_BOLD, SettingsManager::TEXT_TIMESTAMP_ITALIC,
    SettingsManager::TEXT_URL_BACK_COLOR, SettingsManager::TEXT_URL_FORE_COLOR,
    SettingsManager::TEXT_URL_BOLD, SettingsManager::TEXT_URL_ITALIC,
    SettingsManager::TEXT_FAV_BACK_COLOR, SettingsManager::TEXT_FAV_FORE_COLOR,
    SettingsManager::TEXT_FAV_BOLD, SettingsManager::TEXT_FAV_ITALIC,
    SettingsManager::TEXT_OP_BACK_COLOR, SettingsManager::TEXT_OP_FORE_COLOR,
    SettingsManager::TEXT_OP_BOLD, SettingsManager::TEXT_OP_ITALIC,
    SettingsManager::TEXT_FONT, SettingsManager::TEXT_COLOR,
    SettingsManager::BACKGROUND_COLOR, 
    // progressbar
	SettingsManager::PROGRESSBAR_ODC_STYLE, SettingsManager::STEALTHY_STYLE,
	SettingsManager::PROGRESSBAR_ODC_BUMPED, SettingsManager::PROGRESS_OVERRIDE_COLORS,
    SettingsManager::PROGRESS_OVERRIDE_COLORS2, SettingsManager::PROGRESS_3DDEPTH,
	SettingsManager::TOP_SPEED, SettingsManager::TOP_UP_SPEED, 
	SettingsManager::MENUBAR_LEFT_COLOR, SettingsManager::MENUBAR_RIGHT_COLOR, 
	SettingsManager::MENUBAR_BUMPED, SettingsManager::MENUBAR_TWO_COLORS, 
    SettingsManager::DOWNLOAD_BAR_COLOR,SettingsManager::UPLOAD_BAR_COLOR,
	SettingsManager::PROGRESS_TEXT_COLOR_DOWN,SettingsManager::PROGRESS_TEXT_COLOR_UP,
    // user list
	SettingsManager::NORMAL_COLOUR,SettingsManager::RESERVED_SLOT_COLOR, 
    SettingsManager::IGNORED_COLOR, SettingsManager::FIREBALL_COLOR, 
    SettingsManager::SERVER_COLOR, SettingsManager::PASIVE_COLOR, 
    SettingsManager::OP_COLOR, SettingsManager::CLIENT_CHECKED_COLOUR, 
    SettingsManager::FILELIST_CHECKED_COLOUR, SettingsManager::FULL_CHECKED_COLOUR, 
    SettingsManager::BAD_CLIENT_COLOUR, SettingsManager::BAD_FILELIST_COLOUR, 
    SettingsManager::USERLIST_IMAGE,
    // popups
    SettingsManager::POPUP_TIME, SettingsManager::POPUPFILE,
    SettingsManager::MAX_MSG_LENGTH,SettingsManager::POPUP_HUB_CONNECTED,
    SettingsManager::POPUP_HUB_DISCONNECTED, SettingsManager::POPUP_FAVORITE_CONNECTED, 
    SettingsManager::POPUP_FAVORITE_DISCONNECTED,SettingsManager::POPUP_CHEATING_USER, 
    SettingsManager::POPUP_DOWNLOAD_START, SettingsManager::POPUP_DOWNLOAD_FAILED,
    SettingsManager::POPUP_DOWNLOAD_FINISHED, SettingsManager::POPUP_UPLOAD_FINISHED,
    SettingsManager::POPUP_PM, SettingsManager::POPUP_NEW_PM,
    SettingsManager::POPUP_AWAY,SettingsManager::POPUP_MINIMIZED,
    SettingsManager::PM_PREVIEW,SettingsManager::POPUP_TYPE,
    // toolbar 
    SettingsManager::TOOLBARIMAGE, SettingsManager::TOOLBARHOTIMAGE, 
	SettingsManager::TB_IMAGE_SIZE, SettingsManager::TB_IMAGE_SIZE_HOT, 
    SettingsManager::TOOLBAR,
    // windows
	SettingsManager::OPEN_PUBLIC, SettingsManager::OPEN_FAVORITE_HUBS, 
	SettingsManager::OPEN_FAVORITE_USERS,SettingsManager::OPEN_QUEUE,
	SettingsManager::OPEN_FINISHED_DOWNLOADS,SettingsManager::OPEN_WAITING_USERS,
	SettingsManager::OPEN_FINISHED_UPLOADS,SettingsManager::OPEN_SEARCH_SPY,
	SettingsManager::OPEN_NETWORK_STATISTICS, SettingsManager::OPEN_NOTEPAD,
	SettingsManager::OPEN_CDMDEBUG,

	SettingsManager::POPUP_PMS,SettingsManager::POPUP_HUB_PMS,
    SettingsManager::POPUP_BOT_PMS,SettingsManager::POPUNDER_FILELIST,
    SettingsManager::POPUNDER_PM,SettingsManager::JOIN_OPEN_NEW_WINDOW,
    SettingsManager::IGNORE_HUB_PMS,SettingsManager::IGNORE_BOT_PMS,
    SettingsManager::TOGGLE_ACTIVE_WINDOW,SettingsManager::PROMPT_PASSWORD,
    SettingsManager::ONE_SEARCH_WINDOW,SettingsManager::AUTO_HIDE_TABS_BAR,
    SettingsManager::AUTO_HIDE_TRANSFER_BAR,
	SettingsManager::CONFIRM_EXIT,SettingsManager::CONFIRM_HUB_REMOVAL,SettingsManager::CONFIRM_DELETE,
    
    // appearance
    SettingsManager::TIME_STAMPS_FORMAT, SettingsManager::LANGUAGE_FILE, 
    SettingsManager::SHOW_PROGRESS_BARS, SettingsManager::SHOW_INFOTIPS, 
    SettingsManager::FILTER_MESSAGES, SettingsManager::MINIMIZE_TRAY, 
    SettingsManager::EXPAND_QUEUE, SettingsManager::SORT_FAVUSERS_FIRST, 
    SettingsManager::USE_SYSTEM_ICONS, SettingsManager::GET_USER_COUNTRY, 
    SettingsManager::USE_OLD_SHARING_UI, SettingsManager::USE_VERTICAL_VIEW, 
    SettingsManager::TABS_ON_TOP, SettingsManager::BOLD_FINISHED_DOWNLOADS, 
    SettingsManager::BOLD_FINISHED_UPLOADS, SettingsManager::BOLD_QUEUE, 
    SettingsManager::BOLD_HUB, SettingsManager::BOLD_PM, 
    SettingsManager::BOLD_SEARCH, SettingsManager::BOLD_WAITING_USERS,

    SettingsManager::STR_LAST
};


void AppearancePage::write()
{
        PropPage::write((HWND)*this, items, listItems, GetDlgItem(IDC_APPEARANCE_BOOLEANS));
        PropPage::write((HWND)*this, items, boldItems, GetDlgItem(IDC_BOLD_BOOLEANS));
}

LRESULT AppearancePage::onInitDialog(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
        PropPage::translate((HWND)(*this), texts);

        PropPage::read((HWND)*this, items, listItems, GetDlgItem(IDC_APPEARANCE_BOOLEANS));
        PropPage::read((HWND)*this, items, boldItems, GetDlgItem(IDC_BOLD_BOOLEANS));
        
        SetDlgItemText(IDC_THEME_FILE,Text::toT(SettingsManager::getInstance()->get(SettingsManager::THEME_FILE)).c_str());

        themeFile.Attach(GetDlgItem(IDC_THEME_FILE));        
        fillThemeFile(Util::emptyStringT);

        themeOptions.Attach(GetDlgItem(IDC_THEME_OPTIONS));
        
        int index;
        index = themeOptions.AddString(CTSTRING(ALL));
        themeOptions.SetItemDataPtr(index,(void*)ThemeAll);

        index = themeOptions.AddString(CTSTRING(SETTINGS_SOUNDS));
        themeOptions.SetItemDataPtr(index,(void*)ThemeSounds);

        index = themeOptions.AddString(CTSTRING(SETTINGS_TEXT_STYLES));
        themeOptions.SetItemDataPtr(index,(void*)ThemeFontColor);

        index = themeOptions.AddString(CTSTRING(BALLOON_POPUPS));
        themeOptions.SetItemDataPtr(index,(void*)ThemeBallonPopups);

        index = themeOptions.AddString(CTSTRING(SETTINGS_TOOLBAR));
        themeOptions.SetItemDataPtr(index,(void*)ThemeToolbar);
        
        index = themeOptions.AddString(CTSTRING(SETTINGS_WINDOWS));
        themeOptions.SetItemDataPtr(index,(void*)ThemeWindows);

        index = themeOptions.AddString(CTSTRING(SETTINGS_APPEARANCE));
        themeOptions.SetItemDataPtr(index,(void*)ThemeAppearnce);


        themeOptions.SetCurSel(0);
        // Do specialized reading here
        return TRUE;
}

LRESULT AppearancePage::onDestroy(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/) {
    themeFile.Detach();
    themeOptions.Detach();
    return 0;
}

LRESULT AppearancePage::onBrowse(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
        TCHAR buf[MAX_PATH];
        static const TCHAR types[] = _T("Language Files\0*.xml\0All Files\0*.*\0");

        GetDlgItemText(IDC_LANGUAGE, buf, MAX_PATH);
        tstring x = buf;

        if(WinUtil::browseFile(x, m_hWnd, false, Text::toT(Util::getDataPath()), types) == IDOK) {
                SetDlgItemText(IDC_LANGUAGE, x.c_str());
        }
        return 0;
}

LRESULT AppearancePage::onClickedHelp(WORD /* wNotifyCode */, WORD /*wID*/, HWND /* hWndCtl */, BOOL& /* bHandled */) {
        WinUtil::MessageBox(CTSTRING(TIMESTAMP_HELP), CTSTRING(TIMESTAMP_HELP_DESC), MB_OK | MB_ICONINFORMATION);
        return S_OK;
}
LRESULT AppearancePage::onImport(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
    tstring x = _T("");
    if(WinUtil::browseFile(x, m_hWnd, false, x, types, defExt) == IDOK) {
        SetDlgItemText(IDC_THEME_FILE, x.c_str());
        fileThemeName = Text::fromT(x);
    }
    return 0;
}

LRESULT AppearancePage::onUpdate(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
    unsigned sz = themeFile.GetWindowTextLength();
    TCHAR *buf = (TCHAR*)alloca((sz+1)*sizeof(TCHAR));
    themeFile.GetWindowText(buf, sz+1);
    string fileThemeName = Text::fromT(tstring(buf));        
    SettingsManager::getInstance()->set(SettingsManager::THEME_FILE,fileThemeName);
    SettingsManager::getInstance()->save();
    
    LPARAM subsettings = (LPARAM)themeOptions.GetItemData(themeOptions.GetCurSel());
    
    SendMessage(GetParent(),WM_COMMAND,IDCANCEL,0);
    SendMessage(WinUtil::mainWnd,WM_COMMAND,IDC_THEME_UPDATE,subsettings);
    return 0;
}

LRESULT AppearancePage::onExport(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
    tstring x = _T("");
    if(WinUtil::browseFile(x, m_hWnd, true, x, types, defExt) == IDOK) {        
        SettingsManager::getInstance()->save(Text::fromT(x),(int*)themeOptions.GetItemData(themeOptions.GetCurSel()));
    }
    return 0;
}

LRESULT AppearancePage::onThemeFile(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
    unsigned sz = themeFile.GetWindowTextLength();
    TCHAR *buf = (TCHAR*)alloca((sz+1)*sizeof(TCHAR));
    themeFile.GetWindowText(buf, sz+1);
    string fileThemeName = Text::fromT(tstring(buf));
    return 0;
}
LRESULT AppearancePage::onThemeFileSelectionChange(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
    int index = themeFile.GetCurSel();
    int sz = themeFile.GetLBTextLen(index);
    TCHAR *buf = (TCHAR*)alloca((sz+1)*sizeof(TCHAR));
    themeFile.GetLBText(index,buf);
    fileThemeName = Text::fromT(tstring(buf));
    return 0;
}

void AppearancePage::fillThemeFile(const tstring& dir) {
    tstring sFile = Text::toT(Util::getThemePath()) + dir + (dir.empty() ? _T("") : _T("\\")) + tstring(_T("*.dctheme"));
    WIN32_FIND_DATA fData;
    HANDLE hFind;
    hFind = FindFirstFile(sFile.c_str(), &fData);
    if(hFind != INVALID_HANDLE_VALUE)
    {
            do
            {
                tstring cFileName = _T("${ThemesDir}\\")+ dir + (dir.empty() ? _T("") : _T("\\")) + fData.cFileName ;
                themeFile.AddString(cFileName.c_str());
            } while (FindNextFile(hFind, &fData));
    }
    FindClose(hFind);


    sFile = Text::toT(Util::getThemePath()) + dir + (dir.empty() ? _T("") : _T("\\")) + tstring(_T("*"));
    hFind = FindFirstFile(sFile.c_str(), &fData);
    if(hFind != INVALID_HANDLE_VALUE)
    {
            do
            {
                    tstring filename = fData.cFileName;
                    if((fData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) &&
                            (filename != _T(".")) && (filename != _T("..")))
                    {
                        fillThemeFile( dir + (dir.empty() ? _T("") : _T("\\")) + filename );
                    }
            } while (FindNextFile(hFind, &fData));
    }
    FindClose(hFind);

}
/**
 * @file
 * $Id: AppearancePage.cpp,v 1.26 2006/12/13 10:16:32 bigmuscle Exp $
 */
