/*
 * Copyright (C) 2001-2006 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "Resource.h"

#include "MainFrm.h"
#include "AboutDlg.h"
#include "HubFrame.h"
#include "SearchFrm.h"
#include "PublicHubsFrm.h"
#include "PropertiesDlg.h"
#include "UsersFrame.h"
#include "DirectoryListingFrm.h"
#include "RecentsFrm.h"
#include "FavoritesFrm.h"
#include "NotepadFrame.h"
#include "QueueFrame.h"
#include "SpyFrame.h"
#include "FinishedFrame.h"
#include "ADLSearchFrame.h"
#include "FinishedULFrame.h"
#include "TextFrame.h"
#include "UpdateDlg.h"
#include "StatsFrame.h"
#include "WaitingUsersFrame.h"
#include "HashProgressDlg.h"
#include "UPnP.h"
#include "PrivateFrame.h"
#include "WinUtil.h"
#include "CDMDebugFrame.h"
#include "InputBox.h"
#include "PopupManager.h"
#include "AGEmotionSetup.h"
#include "Winamp.h"
#include "Players.h"
#include "iTunesCOMInterface.h"
#include "Browser.h"
#ifdef BETA
#include "BetaDlg.h"
#endif

#include "../client/ConnectionManager.h"
#include "../client/DownloadManager.h"
#include "../client/HashManager.h"
#include "../client/UploadManager.h"
#include "../client/StringTokenizer.h"
#include "../client/SimpleXML.h"
#include "../client/ShareManager.h"
#include "../client/LogManager.h"
#include "../client/WebServerManager.h"
#include "../client/Thread.h"
#include "../client/FavoriteManager.h"
#ifdef BETA
#include <atlcomtime.h>
#endif


MainFrame* MainFrame::anyMF = NULL;
bool MainFrame::bShutdown = false;
uint32_t MainFrame::iCurrentShutdownTime = 0;
bool MainFrame::isShutdownStatus = false;
CAGEmotionSetup* g_pEmotionsSetup = NULL;

MainFrame::MainFrame() : trayMessage(0), maximized(false), lastUpload(-1), lastUpdate(0),
lastUp(0), lastDown(0), oldshutdown(false), stopperThread(NULL), c(new HttpConnection()),
closing(false), awaybyminimize(false), missedAutoConnect(false), lastTTHdir(Util::emptyStringT), tabsontop(false),
bTrayIcon(false), bAppMinimized(false), bIsPM(false), UPnP_TCPConnection(NULL), UPnP_UDPConnection(NULL) ,endThrottleTime(0) {
                memset(statusSizes, 0, sizeof(statusSizes));  
                SettingsManager::getInstance()->set(SettingsManager::THROTTLE_ENABLE,false);
                anyMF = this;
}

MainFrame::~MainFrame() {
        m_CmdBar.m_hImageList = NULL;

        delete g_pEmotionsSetup;
        g_pEmotionsSetup = NULL;

        images.Destroy();
        largeImages.Destroy();
        largeImagesHot.Destroy();
        largeImagesExtra.Destroy();
        largeImagesHotExtra.Destroy();
        winampImages.Destroy();
        winampImagesHot.Destroy();

        WinUtil::uninit();
}

DWORD WINAPI MainFrame::stopper(void* p) {
        MainFrame* mf = (MainFrame*)p;
        HWND wnd, wnd2 = NULL;

        while( (wnd=::GetWindow(mf->m_hWndMDIClient, GW_CHILD)) != NULL) {
                if(wnd == wnd2)
                        Sleep(100);
                else {
                        ::PostMessage(wnd, WM_CLOSE, 0, 0);
                        wnd2 = wnd;
                }
        }

        mf->PostMessage(WM_CLOSE);
        return 0;
}

class ListMatcher : public Thread {
public:
        ListMatcher(StringList files_) : files(files_) {

        }
        virtual int run() {
                for(StringIter i = files.begin(); i != files.end(); ++i) {
                        User::Ptr u = DirectoryListing::getUserFromFilename(*i);
                        if(!u)
                                continue;
                        DirectoryListing dl(u);
                        try {
                                dl.loadFile(*i);
                                const size_t BUF_SIZE = STRING(MATCHED_FILES).size() + 16;
                                AutoArray<char> tmp(BUF_SIZE);
                                snprintf(tmp, BUF_SIZE, CSTRING(MATCHED_FILES), QueueManager::getInstance()->matchListing(dl));
                                LogManager::getInstance()->message(u->getFirstNick() + ": " + string(tmp));
                        } catch(const Exception&) {

                        }
                }
                delete this;
                return 0;
        }
        StringList files;
};

LRESULT MainFrame::onMatchAll(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
        ListMatcher* matcher = new ListMatcher(File::findFiles(Util::getListPath(), "*.xml*"));
        try {
                matcher->start();
        } catch(const ThreadException&) {
                ///@todo add error message
                delete matcher;
        }

        return 0;
}

LRESULT MainFrame::OnCreate(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled) {
        g_pEmotionsSetup = new CAGEmotionSetup;
        if ((g_pEmotionsSetup == NULL)||
                (!g_pEmotionsSetup->Create())){
                dcassert(FALSE);
                return -1;
        }

        TimerManager::getInstance()->addListener(this);
        QueueManager::getInstance()->addListener(this);
        LogManager::getInstance()->addListener(this);
        WebServerManager::getInstance()->addListener(this);

        if(BOOLSETTING(WEBSERVER)) {
                try {
                        WebServerManager::getInstance()->Start();
                } catch(const Exception& e) {
                        WinUtil::MessageBox(Text::toT(e.getError()).c_str(), 0, MB_ICONSTOP | MB_OK);
                }
        }

        WinUtil::init(m_hWnd);

        trayMessage = RegisterWindowMessage(_T("TaskbarCreated"));

        TimerManager::getInstance()->start();


// Shareaza Extension Start

#ifdef BETA
        // ***********
        // Beta expiry. Remember to re-compile to update the time, and remove this
        // section for final releases and public betas.
        COleDateTime tCompileDate;
        tCompileDate.ParseDateTime( _T( __DATE__ ), LOCALE_NOUSEROVERRIDE, 1033 );
        COleDateTime tCurrent = COleDateTime::GetCurrentTime();
        COleDateTimeSpan tTimeOut(30, 0, 0, 0); // 30 days for default
        if ( ( tCompileDate + tTimeOut )  < tCurrent )
        {
            WinUtil::MessageBox( CTSTRING(MSG_BETA_TIME_PASSED), 
                Util::Text::toT(Util::vformat(CTSTRING(MSG_BETA_TIME_PASSED_TITLE),Util::getCompileDate()).c_str(), MB_ICONSTOP | MB_OK );
                ExitProcess(1);
        }
#endif

// Shareaza Extension End


        // Set window name
        SetWindowText(T_FULL_APPNAME);

        // Load images
        // create command bar window
        hWndCmdBar = m_CmdBar.Create(m_hWnd, rcDefault, NULL, ATL_SIMPLE_CMDBAR_PANE_STYLE);

        m_hMenu = WinUtil::mainMenu;

        hShutdownIcon = (HICON)::LoadImage(GetModuleHandle(NULL), MAKEINTRESOURCE(IDB_SHUTDOWN), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
        images.CreateFromImage(IDB_TOOLBAR, 16, 16, CLR_DEFAULT, IMAGE_BITMAP, LR_CREATEDIBSECTION | LR_SHARED);

        m_CmdBar.AttachMenu(m_hMenu);
        m_CmdBar.m_hImageList = images;
        m_CmdBar.m_arrCommand.Add(ID_FILE_CONNECT);
        m_CmdBar.m_arrCommand.Add(ID_FILE_RECONNECT);
        m_CmdBar.m_arrCommand.Add(IDC_FOLLOW);
        m_CmdBar.m_arrCommand.Add(IDC_RECENTS);
        m_CmdBar.m_arrCommand.Add(IDC_FAVORITES);
        m_CmdBar.m_arrCommand.Add(IDC_FAVUSERS);
        m_CmdBar.m_arrCommand.Add(IDC_QUEUE);
        m_CmdBar.m_arrCommand.Add(IDC_FINISHED);
        m_CmdBar.m_arrCommand.Add(IDC_UPLOAD_QUEUE);
        m_CmdBar.m_arrCommand.Add(IDC_FINISHED_UL);
        m_CmdBar.m_arrCommand.Add(ID_FILE_SEARCH);
        m_CmdBar.m_arrCommand.Add(IDC_FILE_ADL_SEARCH);
        m_CmdBar.m_arrCommand.Add(IDC_SEARCH_SPY);
        m_CmdBar.m_arrCommand.Add(IDC_OPEN_FILE_LIST);
        m_CmdBar.m_arrCommand.Add(ID_FILE_SETTINGS);
        m_CmdBar.m_arrCommand.Add(IDC_NOTEPAD);
        m_CmdBar.m_arrCommand.Add(IDC_NET_STATS);
        m_CmdBar.m_arrCommand.Add(IDC_CDMDEBUG_WINDOW);
        m_CmdBar.m_arrCommand.Add(ID_WINDOW_CASCADE);
        m_CmdBar.m_arrCommand.Add(ID_WINDOW_TILE_HORZ);
        m_CmdBar.m_arrCommand.Add(ID_WINDOW_TILE_VERT);
        m_CmdBar.m_arrCommand.Add(ID_WINDOW_MINIMIZE_ALL);
        m_CmdBar.m_arrCommand.Add(ID_WINDOW_RESTORE_ALL);
        m_CmdBar.m_arrCommand.Add(ID_GET_TTH);
        m_CmdBar.m_arrCommand.Add(IDC_UPDATE);
        m_CmdBar.m_arrCommand.Add(IDC_OPEN_MY_LIST);
        m_CmdBar.m_arrCommand.Add(IDC_MATCH_ALL);
        m_CmdBar.m_arrCommand.Add(IDC_REFRESH_FILE_LIST);
        m_CmdBar.m_arrCommand.Add(IDC_OPEN_DOWNLOADS);
        m_CmdBar.m_arrCommand.Add(ID_FILE_QUICK_CONNECT);
        m_CmdBar.m_arrCommand.Add(ID_APP_EXIT);
        m_CmdBar.m_arrCommand.Add(IDC_HASH_PROGRESS);
        m_CmdBar.m_arrCommand.Add(ID_WINDOW_ARRANGE);
        m_CmdBar.m_arrCommand.Add(ID_APP_ABOUT);
        m_CmdBar.m_arrCommand.Add(IDC_HELP_HOMEPAGE);
        m_CmdBar.m_arrCommand.Add(IDC_HELP_DISCUSS);
        m_CmdBar.m_arrCommand.Add(IDC_HELP_DONATE);
        m_CmdBar.m_arrCommand.Add(IDC_GUIDE);
        m_CmdBar.m_arrCommand.Add(IDC_HELP_GEOIPFILE);

        // remove old menu
        SetMenu(NULL);

        tbarcreated = false;
        HWND hWndToolBar = createToolbar();
        HWND hWndToolBarMain = createMainToolbar();
        HWND hWndWinampBar = createWinampToolbar();
        CAxWindow browser;
        CRect rect( 0 , 0 , 0 ,60);
        browser.Create(m_hWnd, rect , Text::toT(LINK_MAIN).c_str() , WS_CHILD | WS_VISIBLE | CCS_NODIVIDER  , 0, 1);
        CreateSimpleReBar( ATL_SIMPLE_REBAR_NOBORDER_STYLE ); 
        
        AddSimpleReBarBand(hWndCmdBar,NULL,TRUE ,0,TRUE);
        AddSimpleReBarBand(hWndToolBarMain, NULL, TRUE ,780,TRUE);
        AddSimpleReBarBand(hWndToolBar, NULL, TRUE ,780,TRUE);        
        AddSimpleReBarBand(hWndWinampBar,NULL,TRUE,780,TRUE);
        AddSimpleReBarBand(browser.m_hWnd,NULL,TRUE,780,TRUE);
        
        
        CreateSimpleStatusBar();

        ctrlStatus.Attach(m_hWndStatusBar);
        ctrlStatus.SetSimple(FALSE);
        int w[STATUTBAR_NUM_PANES] = { 0, 0, 0, 0, 0, 0, 0, 0, 0};
        ctrlStatus.SetParts(STATUTBAR_NUM_PANES, w);
        statusSizes[STATUSBAR_AWAY] = WinUtil::getTextWidth(TSTRING(AWAY), ::GetDC(ctrlStatus.m_hWnd)); // for "AWAY" segment

        CToolInfo ti(TTF_SUBCLASS, ctrlStatus.m_hWnd);

        ctrlLastLines.Create(ctrlStatus.m_hWnd, rcDefault, NULL, WS_POPUP | TTS_NOPREFIX | TTS_ALWAYSTIP | TTS_BALLOON, WS_EX_TOPMOST);
        ctrlLastLines.SetWindowPos(HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);
        ctrlLastLines.AddTool(&ti);
        ctrlLastLines.SetDelayTime(TTDT_AUTOPOP, 15000);

        CreateMDIClient();
        m_CmdBar.SetMDIClient(m_hWndMDIClient);
        WinUtil::mdiClient = m_hWndMDIClient;

        ctrlTab.Create(m_hWnd, rcDefault);
        WinUtil::tabCtrl = &ctrlTab;
        tabsontop = BOOLSETTING(TABS_ON_TOP);

        transferView.Create(m_hWnd);

        SetSplitterPanes(m_hWndMDIClient, transferView.m_hWnd);
        SetSplitterExtendedStyle(SPLIT_PROPORTIONAL);
        m_nProportionalPos = SETTING(TRANSFER_SPLIT_SIZE);
        UIAddToolBar(hWndToolBar);
        UIAddToolBar(hWndWinampBar);
        UISetCheck(ID_VIEW_TOOLBAR, 1);
        UISetCheck(ID_VIEW_EXTRA_TOOLBAR, 1);
        UISetCheck(ID_VIEW_STATUS_BAR, 1);
        UISetCheck(ID_VIEW_TRANSFER_VIEW, 1);
        UISetCheck(ID_VIEW_TABS, 1);
        UISetCheck(ID_VIEW_WINAMP_TOOLBAR, 1);

        UISetCheck(IDC_TRAY_LIMITER, BOOLSETTING(THROTTLE_ENABLE));

        // register object for message filtering and idle updates
        CMessageLoop* pLoop = _Module.GetMessageLoop();
        ATLASSERT(pLoop != NULL);
        pLoop->AddMessageFilter(this);
        pLoop->AddIdleHandler(this);

        trayMenu.CreatePopupMenu();
        trayMenu.AppendMenu(MF_STRING, IDC_TRAY_SHOW, CTSTRING(MENU_SHOW));        
        trayMenu.AppendMenu(MF_STRING, IDC_OPEN_DOWNLOADS, CTSTRING(MENU_OPEN_DOWNLOADS_DIR));
        trayMenu.AppendMenu(MF_SEPARATOR);
        trayMenu.AppendMenu(MF_STRING, IDC_REFRESH_FILE_LIST, CTSTRING(MENU_REFRESH_FILE_LIST));
        trayMenu.AppendMenu(MF_STRING, IDC_TRAY_LIMITER, CTSTRING(TRAY_LIMITER));
        trayMenu.AppendMenu(MF_SEPARATOR);
        trayMenu.AppendMenu(MF_STRING, IDC_HOMEPAGE, CTSTRING(MENU_HOMEPAGE));
        trayMenu.AppendMenu(MF_STRING, IDC_LOCAL_SITE, CTSTRING(MENU_LOCAL_SITE));
        trayMenu.AppendMenu(MF_STRING, ID_APP_ABOUT, CTSTRING(MENU_ABOUT));
        trayMenu.AppendMenu(MF_SEPARATOR);
        trayMenu.AppendMenu(MF_STRING, ID_APP_EXIT, CTSTRING(MENU_EXIT));
        trayMenu.SetMenuDefaultItem(IDC_TRAY_SHOW);

        if(SETTING(PROTECT_START) && Text::toT(SETTING(PASSWORD)) != _T("LWPNACQDBZRYXW3VHJVCJ64QBZNGHOHHHZWCLNQ") && Text::toT(SETTING(PASSWORD)) != _T("")) {
                LineDlg dlg;
                dlg.description = TSTRING(PASSWORD_DESC);
                dlg.title = TSTRING(PASSWORD_TITLE);
                dlg.password = true;
                dlg.disable = true;
                if(dlg.DoModal(m_hWnd) == IDOK ) {
                        tstring tmp = dlg.line;
                        TigerTree mytth(TigerTree::calcBlockSize(tmp.size(), 1));
                        mytth.update(tmp.c_str(), tmp.size());
                        mytth.finalize();
                        if (Text::toT(mytth.getRoot().toBase32().c_str()) != Text::toT(SETTING(PASSWORD))) {
                                ExitProcess(1);
                        }
                }
        }

#ifndef BETA
        c->addListener(this);
        c->downloadFile(LINK_VERSIONFILE);
        enabled = true;
#else
        #error This block is removed due to security reasons :)
#endif

        // ZoneAlarm - ref: http://www.unixwiz.net/backstealth/
        if (BOOLSETTING(DETECT_BADSOFT)) {
                int times_detected = SETTING(BADSOFT_DETECTIONS);
                if (times_detected % 5 == 0) {
                        if (FindWindow(NULL, _T("ZoneAlarm")) || FindWindow(NULL, _T("ZoneAlarm Pro"))) {
                                WinUtil::MessageBox(CTSTRING(ZONEALARM_WARNING), 0, MB_OK | MB_ICONINFORMATION | MB_TOPMOST);
                        }
                        if(GetModuleHandle(_T("nl_lsp.dll")) > 0)
                                WinUtil::MessageBox(CTSTRING(NETLIMITER_WARNING), 0, MB_OK | MB_ICONINFORMATION | MB_TOPMOST);
                }
                SettingsManager::getInstance()->set(SettingsManager::BADSOFT_DETECTIONS, ++times_detected);
        }

        if(FindWindow(NULL, _T("PeerGuardian")) || FindWindow(NULL, _T("PeerGuardian 2")) || FindWindow(NULL, _T("ProtoWall")) || !Util::fileExists(Util::getDataPath() + "PeerGuardian.dll")) {
                SettingsManager::getInstance()->set(SettingsManager::PG_ENABLE, false);
        }

        PostMessage(WM_COMMAND, IDC_BROWSER);
        if(BOOLSETTING(OPEN_PUBLIC)) PostMessage(WM_COMMAND, ID_FILE_CONNECT);
        if(BOOLSETTING(OPEN_FAVORITE_HUBS)) PostMessage(WM_COMMAND, IDC_FAVORITES);
        if(BOOLSETTING(OPEN_FAVORITE_USERS)) PostMessage(WM_COMMAND, IDC_FAVUSERS);
        if(BOOLSETTING(OPEN_QUEUE)) PostMessage(WM_COMMAND, IDC_QUEUE);
        if(BOOLSETTING(OPEN_FINISHED_DOWNLOADS)) PostMessage(WM_COMMAND, IDC_FINISHED);
        if(BOOLSETTING(OPEN_WAITING_USERS)) PostMessage(WM_COMMAND, IDC_UPLOAD_QUEUE);
        if(BOOLSETTING(OPEN_FINISHED_UPLOADS)) PostMessage(WM_COMMAND, IDC_FINISHED_UL);
        if(BOOLSETTING(OPEN_SEARCH_SPY)) PostMessage(WM_COMMAND, IDC_SEARCH_SPY);
        if(BOOLSETTING(OPEN_NETWORK_STATISTICS)) PostMessage(WM_COMMAND, IDC_NET_STATS);
        if(BOOLSETTING(OPEN_NOTEPAD)) PostMessage(WM_COMMAND, IDC_NOTEPAD);
        if(BOOLSETTING(OPEN_CDMDEBUG)) PostMessage(WM_COMMAND, IDC_CDMDEBUG_WINDOW);

        if(!BOOLSETTING(SHOW_STATUSBAR)) PostMessage(WM_COMMAND, ID_VIEW_STATUS_BAR);
        if(!BOOLSETTING(SHOW_TOOLBAR)) PostMessage(WM_COMMAND, ID_VIEW_TOOLBAR);
        if(!BOOLSETTING(SHOW_EXTRA_TOOLBAR)) PostMessage(WM_COMMAND, ID_VIEW_EXTRA_TOOLBAR);
		PostMessage(WM_COMMAND, ID_VIEW_TRANSFER_VIEW);
		if(BOOLSETTING(SHOW_TRANSFERVIEW)) {
			PostMessage(WM_COMMAND, ID_VIEW_TRANSFER_VIEW);
		}
		PostMessage(WM_COMMAND, ID_VIEW_TABS);
		if(BOOLSETTING(SHOW_TABS)) {
			PostMessage(WM_COMMAND, ID_VIEW_TABS);
		}        
		if(!BOOLSETTING(SHOW_WINAMP_TOOLBAR)) PostMessage(WM_COMMAND, ID_VIEW_WINAMP_TOOLBAR);

#ifndef BETA
        //if(!WinUtil::isShift())
        //        PostMessage(WM_SPEAKER, AUTO_CONNECT);
#endif

        PostMessage(WM_SPEAKER, PARSE_COMMAND_LINE);

        try {
                File::ensureDirectory(SETTING(LOG_DIRECTORY));
        } catch (const FileException) { }

        startSocket();

        normalicon.hIcon = (HICON)::LoadImage(GetModuleHandle(NULL), MAKEINTRESOURCE(IDR_MAINFRAME), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
        pmicon.hIcon = (HICON)::LoadImage(GetModuleHandle(NULL), MAKEINTRESOURCE(IDR_TRAY_PM), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);

        updateTray( BOOLSETTING( MINIMIZE_TRAY ) );

        Util::setAway(BOOLSETTING(AWAY));

        ctrlToolbar.CheckButton(IDC_AWAY,BOOLSETTING(AWAY));
        ctrlToolbar.CheckButton(IDC_LIMITER,BOOLSETTING(THROTTLE_ENABLE));
        ctrlToolbar.CheckButton(IDC_DISABLE_SOUNDS, BOOLSETTING(SOUNDS_DISABLED));
        ctrlToolbar.CheckButton(ID_VIEW_WINAMP_TOOLBAR, BOOLSETTING(SHOW_WINAMP_TOOLBAR));

#ifndef BETA
        if(SETTING(NICK).empty()) {
                PostMessage(WM_COMMAND, IDC_FIRST_TIME_RUN);
        }
#endif        
        // We want to pass this one on to the splitter...hope it get's there...
        PostMessage(WM_COMMAND, IDC_BROWSER);
        bHandled = FALSE;
        return 0;
}

HWND MainFrame::createWinampToolbar() {
        CToolBarCtrl ctrlSmallToolbar;
        winampImages.CreateFromImage(IDB_WINAMP_CONTROL, 20, 20, CLR_DEFAULT, IMAGE_BITMAP, LR_CREATEDIBSECTION | LR_SHARED);
        winampImagesHot.CreateFromImage(IDB_WINAMP_CONTROL_HOT, 20, 20, CLR_DEFAULT, IMAGE_BITMAP, LR_CREATEDIBSECTION | LR_SHARED);

        ctrlSmallToolbar.Create(m_hWnd, NULL, NULL, ATL_SIMPLE_CMDBAR_PANE_STYLE | TBSTYLE_FLAT | TBSTYLE_TOOLTIPS, 0, ATL_IDW_TOOLBAR);
        ctrlSmallToolbar.SetImageList(winampImages);
        ctrlSmallToolbar.SetHotImageList(winampImagesHot);

        const int numButtons = 11;

        TBBUTTON tb[numButtons];
        memset(tb, 0, sizeof(tb));
        int n = 0, bitmap = 0;

#define ADD_BUTTON(a) \
        tb[n].iBitmap = bitmap++; \
        tb[n].idCommand = a; \
        tb[n].fsState = TBSTATE_ENABLED; \
        tb[n].fsStyle = TBSTYLE_BUTTON | TBSTYLE_AUTOSIZE; \
        n++;

#define ADD_SEPERATOR() \
        tb[n].fsStyle = TBSTYLE_SEP; \
        n++;

        //Fix buttons
        // back / play / pause / next / stop / volume up / volume 50% / volume down
        ADD_BUTTON(IDC_WINAMP_SPAM)
        ADD_SEPERATOR()
        ADD_BUTTON(IDC_WINAMP_BACK)
        ADD_BUTTON(IDC_WINAMP_PLAY)
        ADD_BUTTON(IDC_WINAMP_PAUSE)
        ADD_BUTTON(IDC_WINAMP_NEXT)
        ADD_BUTTON(IDC_WINAMP_STOP)
        ADD_BUTTON(IDC_WINAMP_VOL_UP)
        ADD_BUTTON(IDC_WINAMP_VOL_HALF)
        ADD_BUTTON(IDC_WINAMP_VOL_DOWN)
        ADD_SEPERATOR()

        ctrlSmallToolbar.SetButtonStructSize();
        ctrlSmallToolbar.AddButtons(numButtons, tb);
        ctrlSmallToolbar.AutoSize();

        return ctrlSmallToolbar.m_hWnd;
}

LRESULT MainFrame::onWinampButton(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
        if(SETTING(MEDIA_PLAYER) == 0) {
        HWND hwndWinamp = FindWindow(_T("Winamp v1.x"), NULL);
        if (::IsWindow(hwndWinamp)) {
                switch(wID) {
                        case IDC_WINAMP_BACK: SendMessage(hwndWinamp, WM_COMMAND, WINAMP_BUTTON1, 0); break;
                        case IDC_WINAMP_PLAY: SendMessage(hwndWinamp, WM_COMMAND, WINAMP_BUTTON2, 0); break;
                        case IDC_WINAMP_STOP: SendMessage(hwndWinamp, WM_COMMAND, WINAMP_BUTTON4, 0); break;
                        case IDC_WINAMP_PAUSE: SendMessage(hwndWinamp, WM_COMMAND, WINAMP_BUTTON3, 0); break;
                        case IDC_WINAMP_NEXT: SendMessage(hwndWinamp, WM_COMMAND, WINAMP_BUTTON5, 0); break;
                        case IDC_WINAMP_VOL_UP: SendMessage(hwndWinamp, WM_COMMAND, WINAMP_VOLUMEUP, 0); break;
                        case IDC_WINAMP_VOL_DOWN: SendMessage(hwndWinamp, WM_COMMAND, WINAMP_VOLUMEDOWN, 0); break;
            case IDC_WINAMP_VOL_HALF: SendMessage(hwndWinamp, WM_WA_IPC, 255/2, IPC_SETVOLUME); break;
                }
        }
        } else if(SETTING(MEDIA_PLAYER) == 1) {
                HWND hwndWMP = FindWindow(_T("WMPlayerApp"), NULL);
                if (::IsWindow(hwndWMP)) {
                        switch(wID) {
                                case IDC_WINAMP_BACK: SendMessage(hwndWMP, WM_COMMAND, WMP_PREV, 0); break;
                                case IDC_WINAMP_PLAY: SendMessage(hwndWMP, WM_COMMAND, WMP_PLAY, 0); break;
                                case IDC_WINAMP_STOP: SendMessage(hwndWMP, WM_COMMAND, WMP_STOP, 0); break;
                                case IDC_WINAMP_PAUSE: SendMessage(hwndWMP, WM_COMMAND, WMP_PLAY, 0); break;
                                case IDC_WINAMP_NEXT: SendMessage(hwndWMP, WM_COMMAND, WMP_NEXT, 0); break;
                                case IDC_WINAMP_VOL_UP: SendMessage(hwndWMP, WM_COMMAND, WMP_VOLUP, 0); break;
                                case IDC_WINAMP_VOL_DOWN: SendMessage(hwndWMP, WM_COMMAND, WMP_VOLDOWN, 0); break;
                                case IDC_WINAMP_VOL_HALF: SendMessage(hwndWMP, WM_COMMAND, WMP_MUTE, 0); break;
                        }
                }
        } else if(SETTING(MEDIA_PLAYER) == 2) {
                // Since i couldn't find out the appropriate window messages, we doing this  la COM
                HWND hwndiTunes = FindWindow(_T("iTunes"), _T("iTunes"));
                if (::IsWindow(hwndiTunes)) {
                        IiTunes *iITunes;
                        CoInitialize(NULL);
                        if (SUCCEEDED(::CoCreateInstance(CLSID_iTunesApp, NULL, CLSCTX_LOCAL_SERVER, IID_IiTunes, (PVOID *)&iITunes))) {
                                long currVol;
                                switch(wID) {
                                        case IDC_WINAMP_BACK: iITunes->PreviousTrack(); break;
                                        case IDC_WINAMP_PLAY: iITunes->Play(); break;
                                        case IDC_WINAMP_STOP: iITunes->Stop();  break;
                                        case IDC_WINAMP_PAUSE: iITunes->Pause(); break;
                                        case IDC_WINAMP_NEXT: iITunes->NextTrack(); break;
                                        case IDC_WINAMP_VOL_UP: iITunes->get_SoundVolume(&currVol); iITunes->put_SoundVolume(currVol+10); break;
                                        case IDC_WINAMP_VOL_DOWN: iITunes->get_SoundVolume(&currVol); iITunes->put_SoundVolume(currVol-10); break;
                                        case IDC_WINAMP_VOL_HALF: iITunes->put_SoundVolume(50); break;
                                }
                        }
                        iITunes->Release();
                        CoUninitialize();
                }
        } else if(SETTING(MEDIA_PLAYER) == 3) {
                HWND hwndMPC = FindWindow(_T("MediaPlayerClassicW"), NULL);
                if (::IsWindow(hwndMPC)) {
                        switch(wID) {
                                case IDC_WINAMP_BACK: SendMessage(hwndMPC, WM_COMMAND, MPC_PREV, 0); break;
                                case IDC_WINAMP_PLAY: SendMessage(hwndMPC, WM_COMMAND, MPC_PLAY, 0); break;
                                case IDC_WINAMP_STOP: SendMessage(hwndMPC, WM_COMMAND, MPC_STOP, 0); break;
                                case IDC_WINAMP_PAUSE: SendMessage(hwndMPC, WM_COMMAND, MPC_PAUSE, 0); break;
                                case IDC_WINAMP_NEXT: SendMessage(hwndMPC, WM_COMMAND, MPC_NEXT, 0); break;
                                case IDC_WINAMP_VOL_UP: SendMessage(hwndMPC, WM_COMMAND, MPC_VOLUP, 0); break;
                                case IDC_WINAMP_VOL_DOWN: SendMessage(hwndMPC, WM_COMMAND, MPC_VOLDOWN, 0); break;
                                case IDC_WINAMP_VOL_HALF: SendMessage(hwndMPC, WM_COMMAND, MPC_MUTE, 0); break;
                        }
                }
        }
       return 0;
}

void MainFrame::startSocket() {
        SearchManager::getInstance()->disconnect();
        ConnectionManager::getInstance()->disconnect();

//      if(ClientManager::getInstance()->isActive()) {
                try {
                        ConnectionManager::getInstance()->listen();
                } catch(const Exception&) {
                        WinUtil::MessageBox(CTSTRING(TCP_PORT_BUSY), 0, MB_ICONSTOP | MB_OK);
                }
                try {
                        SearchManager::getInstance()->listen();
                } catch(const Exception&) {
                        WinUtil::MessageBox(CTSTRING(TCP_PORT_BUSY), 0, MB_ICONSTOP | MB_OK);
                }
//      }

        startUPnP();
		if ( BOOLSETTING(IPUPDATE) ) {
			new ExternalIP();
		}
}

void MainFrame::startUPnP() {
        stopUPnP();

        if( SETTING(INCOMING_CONNECTIONS) == SettingsManager::INCOMING_FIREWALL_UPNP ) {
                UPnP_TCPConnection = new UPnP( Util::getLocalIp(), "TCP", APPNAME " Download Port (" + Util::toString(ConnectionManager::getInstance()->getPort()) + " TCP)", ConnectionManager::getInstance()->getPort() );
                UPnP_UDPConnection = new UPnP( Util::getLocalIp(), "UDP", APPNAME " Search Port (" + Util::toString(SearchManager::getInstance()->getPort()) + " UDP)", SearchManager::getInstance()->getPort() );

                if ( FAILED(UPnP_UDPConnection->OpenPorts()) || FAILED(UPnP_TCPConnection->OpenPorts()) )
                {
                        LogManager::getInstance()->message(STRING(UPNP_FAILED_TO_CREATE_MAPPINGS));
                        WinUtil::MessageBox(CTSTRING(UPNP_FAILED_TO_CREATE_MAPPINGS), 0, MB_OK | MB_ICONWARNING);

                        // We failed! thus reset the objects
                        delete UPnP_TCPConnection;
                        delete UPnP_UDPConnection;
                        UPnP_TCPConnection = UPnP_UDPConnection = NULL;
                }
                else
                {
                        if(!BOOLSETTING(NO_IP_OVERRIDE)) {
                                // now lets configure the external IP (connect to me) address
                                string ExternalIP = UPnP_TCPConnection->GetExternalIP();
                                if ( !ExternalIP.empty() ) {
                                        // woohoo, we got the external IP from the UPnP framework
                                        SettingsManager::getInstance()->set(SettingsManager::EXTERNAL_IP, ExternalIP );
                                } else {
                                        //:-(  Looks like we have to rely on the user setting the external IP manually
                                        // no need to do cleanup here because the mappings work
                                        LogManager::getInstance()->message(STRING(UPNP_FAILED_TO_GET_EXTERNAL_IP));
                                        WinUtil::MessageBox(CTSTRING(UPNP_FAILED_TO_GET_EXTERNAL_IP), 0, MB_OK | MB_ICONWARNING);
                                }
                        }
                }
        }
}

void MainFrame::stopUPnP() {
        // Just check if the port mapping objects are initialized (NOT NULL)
        if ( UPnP_TCPConnection != NULL )
        {
                if (FAILED(UPnP_TCPConnection->ClosePorts()) )
                {
                        LogManager::getInstance()->message(STRING(UPNP_FAILED_TO_REMOVE_MAPPINGS));
                }
                delete UPnP_TCPConnection;
        }
        if ( UPnP_UDPConnection != NULL )
        {
                if (FAILED(UPnP_UDPConnection->ClosePorts()) )
                {
                        LogManager::getInstance()->message(STRING(UPNP_FAILED_TO_REMOVE_MAPPINGS));
                }
                delete UPnP_UDPConnection;
        }
        // Not sure this is required (i.e. Objects are checked later in execution)
        // But its better being on the save side :P
        UPnP_TCPConnection = UPnP_UDPConnection = NULL;
}

HWND MainFrame::createMainToolbar() {
	CToolBarCtrl ctrlToolbar;
    largeImages.CreateFromImage(IDB_TOOLBAR_MAIN, 90, 15, CLR_DEFAULT, IMAGE_BITMAP, LR_CREATEDIBSECTION | LR_SHARED);
    largeImagesHot.CreateFromImage(IDB_TOOLBAR_MAIN, 90, 15, CLR_DEFAULT, IMAGE_BITMAP, LR_CREATEDIBSECTION | LR_SHARED);

	ctrlToolbar.Create(m_hWnd, NULL, NULL, ATL_SIMPLE_CMDBAR_PANE_STYLE | TBSTYLE_FLAT | TBSTYLE_TOOLTIPS, 0, ATL_IDW_TOOLBAR);
	ctrlToolbar.SetImageList(largeImages);
	ctrlToolbar.SetHotImageList(largeImagesHot);

	const int numButtons = 7;

	TBBUTTON tb[numButtons];
	memset(tb, 0, sizeof(tb));
	int n = 0, bitmap = 0;

	tb[n].iBitmap = bitmap++;
	tb[n].idCommand = IDC_BROWSER;
	tb[n].fsState = TBSTATE_ENABLED;
	tb[n].fsStyle = TBSTYLE_BUTTON | TBSTYLE_AUTOSIZE ;

	n++;
	tb[n].iBitmap = bitmap++;
	tb[n].idCommand = ID_FILE_CONNECT;
	tb[n].fsState = TBSTATE_ENABLED;
	tb[n].fsStyle = TBSTYLE_BUTTON | TBSTYLE_AUTOSIZE;

	n++;
	tb[n].iBitmap = bitmap++;
	tb[n].idCommand = ID_FILE_SEARCH;
	tb[n].fsState = TBSTATE_ENABLED;
	tb[n].fsStyle = TBSTYLE_BUTTON | TBSTYLE_AUTOSIZE;

	n++;
	tb[n].iBitmap = bitmap++;
	tb[n].idCommand = IDC_QUEUE;
	tb[n].fsState = TBSTATE_ENABLED;
	tb[n].fsStyle = TBSTYLE_BUTTON | TBSTYLE_AUTOSIZE;

	n++;
	tb[n].iBitmap = bitmap++;
	tb[n].idCommand = IDC_FINISHED;
	tb[n].fsState = TBSTATE_ENABLED;
	tb[n].fsStyle = TBSTYLE_BUTTON | TBSTYLE_AUTOSIZE;

	n++;
	tb[n].iBitmap = bitmap++;
	tb[n].idCommand = IDC_NET_STATS ;
	tb[n].fsState = TBSTATE_ENABLED;
	tb[n].fsStyle = TBSTYLE_BUTTON | TBSTYLE_AUTOSIZE;
	
	n++;
	tb[n].iBitmap = bitmap++;
	tb[n].idCommand = IDC_SEARCH_SPY ;
	tb[n].fsState = TBSTATE_ENABLED;
	tb[n].fsStyle = TBSTYLE_BUTTON | TBSTYLE_AUTOSIZE;

	ctrlToolbar.SetButtonStructSize();
	ctrlToolbar.AddButtons(numButtons, tb);
	ctrlToolbar.AutoSize();

	return ctrlToolbar.m_hWnd;
}

HWND MainFrame::createToolbar() {
        if(!tbarcreated) {
                if(SETTING(TOOLBARIMAGE) == "")
                        largeImagesExtra.CreateFromImage(IDB_TOOLBAR20, 32, 32, CLR_DEFAULT, IMAGE_BITMAP, LR_CREATEDIBSECTION | LR_SHARED);
                else
                        largeImagesExtra.CreateFromImage(Text::toT(Util::getThemeFile(SETTING(TOOLBARIMAGE))).c_str(), SETTING(TB_IMAGE_SIZE), 0, CLR_DEFAULT, IMAGE_BITMAP, LR_CREATEDIBSECTION | LR_SHARED | LR_LOADFROMFILE);

                if(SETTING(TOOLBARHOTIMAGE) == "")
                        largeImagesHotExtra.CreateFromImage(IDB_TOOLBAR20_HOT, 32, 32, CLR_DEFAULT, IMAGE_BITMAP, LR_CREATEDIBSECTION | LR_SHARED);
                else
                        largeImagesHotExtra.CreateFromImage(Text::toT(Util::getThemeFile(SETTING(TOOLBARHOTIMAGE))).c_str(), SETTING(TB_IMAGE_SIZE), 0, CLR_DEFAULT, IMAGE_BITMAP, LR_CREATEDIBSECTION | LR_SHARED | LR_LOADFROMFILE);

                ctrlToolbar.Create(m_hWnd, NULL, NULL, ATL_SIMPLE_CMDBAR_PANE_STYLE | TBSTYLE_FLAT | TBSTYLE_TOOLTIPS, 0, ATL_IDW_TOOLBAR);
                ctrlToolbar.SetImageList(largeImagesExtra);
                ctrlToolbar.SetHotImageList(largeImagesHotExtra);
                tbarcreated = true;
        }

        while(ctrlToolbar.GetButtonCount()>0)
                ctrlToolbar.DeleteButton(0);

        ctrlToolbar.SetButtonStructSize();
        StringTokenizer<string> t(SETTING(TOOLBAR), ',');
        StringList& l = t.getTokens();

        for(StringList::const_iterator k = l.begin(); k != l.end(); ++k) {
                int i = Util::toInt(*k);

                TBBUTTON nTB;
                memset(&nTB, 0, sizeof(TBBUTTON));

                if((i == INT_MAX) || (i == -1)) {
                        nTB.fsStyle = TBSTYLE_SEP;
                } else {
                        nTB.iBitmap = WinUtil::ToolbarButtons[i].image;
                        nTB.idCommand = WinUtil::ToolbarButtons[i].id;
                        nTB.fsState = TBSTATE_ENABLED;
                        nTB.fsStyle = TBSTYLE_AUTOSIZE | ((WinUtil::ToolbarButtons[i].check == true)? TBSTYLE_CHECK : TBSTYLE_BUTTON);
                        nTB.iString = WinUtil::ToolbarButtons[i].tooltip;
                }
                ctrlToolbar.AddButtons(1, &nTB);
        }

        ctrlToolbar.AutoSize();

        return ctrlToolbar.m_hWnd;
}



LRESULT MainFrame::onSpeaker(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/) {

        if(wParam == DOWNLOAD_LISTING) {
                auto_ptr<DirectoryListInfo> i(reinterpret_cast<DirectoryListInfo*>(lParam));
                DirectoryListingFrame::openWindow(i->file, i->user, i->speed);
        } else if(wParam == BROWSE_LISTING) {
                auto_ptr<DirectoryBrowseInfo> i(reinterpret_cast<DirectoryBrowseInfo*>(lParam));
                DirectoryListingFrame::openWindow(i->user, i->text, 0);
        } else if(wParam == VIEW_FILE_AND_DELETE) {
                auto_ptr<tstring> file(reinterpret_cast<tstring*>(lParam));
                if (BOOLSETTING(EXTERNAL_PREVIEW)) // !SMT!-UI
                        ShellExecute(NULL, NULL, file->c_str(), NULL, NULL, SW_SHOW); // !SMT!-UI
                else {
                        TextFrame::openWindow(*file);
                        File::deleteFile(Text::fromT(*file));
                }
        } else if(wParam == STATS) {
                auto_ptr<TStringList> pstr(reinterpret_cast<TStringList*>(lParam));
                const TStringList& str = *pstr;
                if(ctrlStatus.IsWindow()) {
                        HDC dc = ::GetDC(ctrlStatus.m_hWnd);
                        bool u = false;
                        for(int ii = STATUSBAR_AWAY ; ii < STATUTBAR_NUM_PANES ; ii++ ) {
                            int w = min( WinUtil::getTextWidth(str[ii], dc), 130 );
                            if(statusSizes[ii] < w) {
                                statusSizes[ii] = w;
                                u = true;
                            }
                            ctrlStatus.SetText(ii, str[ii].c_str());
                            ctrlStatus.SetTipText(ii, str[ii].c_str());
                        }
                        ::ReleaseDC(ctrlStatus.m_hWnd, dc);
                        if(u)
                                UpdateLayout(TRUE);
                }
                if (bShutdown) {
                        uint32_t iSec = GET_TICK() / 1000;
                        if (ctrlStatus.IsWindow()) {
                                if(!isShutdownStatus) {
                                        ctrlStatus.SetIcon(STATUSBAR_TIME, hShutdownIcon);
                                        isShutdownStatus = true;
                                }
                                if (DownloadManager::getInstance()->getDownloadCount() > 0) {
                                        iCurrentShutdownTime = iSec;
                                        ctrlStatus.SetText(STATUSBAR_TIME, _T(""));
                                } else {
                                        int64_t timeLeft = SETTING(SHUTDOWN_TIMEOUT) - (iSec - iCurrentShutdownTime);
                                        ctrlStatus.SetText(STATUSBAR_TIME, (_T(" ") + Util::formatSeconds(timeLeft, timeLeft < 3600)).c_str(), SBT_POPOUT);
                                        if (iCurrentShutdownTime + SETTING(SHUTDOWN_TIMEOUT) <= iSec) {
                                                bool bDidShutDown = false;
                                                bDidShutDown = WinUtil::shutDown(SETTING(SHUTDOWN_ACTION));
                                                if (bDidShutDown) {
                                                        // Should we go faster here and force termination?
                                                        // We "could" do a manual shutdown of this app...
                                                } else {
                                                        ctrlStatus.SetText(STATUSBAR_MESSAGE, CTSTRING(FAILED_TO_SHUTDOWN));
                                                        ctrlStatus.SetText(STATUSBAR_TIME, _T(""));
                                                }
                                                // We better not try again. It WON'T work...
                                                bShutdown = false;
                                        }
                                }
                        }
                } else {
                        if (ctrlStatus.IsWindow()) {
                                if(isShutdownStatus) {
                                        ctrlStatus.SetText(STATUSBAR_TIME, _T(""));
                                        ctrlStatus.SetIcon(STATUSBAR_TIME, NULL);
                                        isShutdownStatus = false;
                                }
                        }
                }
        } else if(wParam == AUTO_CONNECT) {
                autoConnect(FavoriteManager::getInstance()->getFavoriteHubs());
        } else if(wParam == PARSE_COMMAND_LINE) {
                parseCommandLine(GetCommandLine());
        } else if(wParam == STATUS_MESSAGE) {
                tstring* msg = (tstring*)lParam;
                if(ctrlStatus.IsWindow()) {
                        tstring line = Text::toT("[" + Util::getShortTimeString() + "] ") + *msg;

                        ctrlStatus.SetText(STATUSBAR_MESSAGE, line.c_str());
                        while(lastLinesList.size() + 1 > MAX_CLIENT_LINES)
                                lastLinesList.erase(lastLinesList.begin());
                        if (line.find(_T('\r')) == tstring::npos) {
                                lastLinesList.push_back(line);
                        } else {
                                lastLinesList.push_back(line.substr(0, line.find(_T('\r'))));
                        }
                }
                delete msg;
        } else if (wParam == CONNECT_HUB ) {
            tstring* hubname = (tstring*)lParam;
            FavoriteHubEntry *entry = FavoriteManager::getInstance()->getFavoriteHubEntry(Text::fromT(*hubname));
            if ( entry == NULL ) {
                HubFrame::openWindow(*hubname,true);
            }
            else {
        		HubFrame::openWindow(Text::toT(entry->getServer()),true
		    	    , Text::toT(entry->getRawOne())
			        , Text::toT(entry->getRawTwo())
			        , Text::toT(entry->getRawThree())
			        , Text::toT(entry->getRawFour())
			        , Text::toT(entry->getRawFive())
			        , entry->getWindowPosX(), entry->getWindowPosY(), entry->getWindowSizeX(), entry->getWindowSizeY(), entry->getWindowType(), 
			        entry->getChatUserSplit(), entry->getUserListState());
            }
        } else if(wParam == SHOW_POPUP) {
                Popup* msg = (Popup*)lParam;
                PopupManager::getInstance()->Show(Text::fromT(msg->Message), Text::fromT(msg->Title), msg->Icon);
                delete msg;
        } else if(wParam == WM_CLOSE) {
                PopupManager::getInstance()->Remove((int)lParam);
        } else if(wParam == REMOVE_POPUP){
                PopupManager::getInstance()->AutoRemove();
        } else if(wParam == SET_PM_TRAY_ICON) {
                if(bIsPM == false && (!WinUtil::isAppActive || bAppMinimized) && bTrayIcon == true) {
                        setIcon(pmicon.hIcon);
                        bIsPM = true;
                }
    }

        return 0;
}

void MainFrame::parseCommandLine(const tstring& cmdLine)
{
        string::size_type i = 0;
        string::size_type j;

        if( (j = cmdLine.find(_T("dchub://"), i)) != string::npos) {
                WinUtil::parseDchubUrl(cmdLine.substr(j));
                }
        if( (j = cmdLine.find(_T("adc://"), i)) != string::npos) {
                WinUtil::parseADChubUrl(cmdLine.substr(j));
        }
        if( (j = cmdLine.find(_T("magnet:?"), i)) != string::npos) {
                WinUtil::parseMagnetUri(cmdLine.substr(j));
        }
}

LRESULT MainFrame::onCopyData(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM lParam, BOOL& /*bHandled*/) {
        if (!getPassword()) return false; // !SMT!-f
        ShowWindow(SW_RESTORE); // !SMT!-f
        tstring cmdLine = (LPCTSTR) (((COPYDATASTRUCT *)lParam)->lpData);
        parseCommandLine(Text::toT(WinUtil::getAppName() + " ") + cmdLine);
        return true;
}

LRESULT MainFrame::onHashProgress(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
        HashProgressDlg(false).DoModal(m_hWnd);
        return 0;
}

LRESULT MainFrame::OnAppAbout(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
        AboutDlg dlg;
        dlg.DoModal(m_hWnd);
        return 0;
}

LRESULT MainFrame::onOpenWindows(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
        switch(wID) {
                case ID_FILE_SEARCH: 
                    if( BOOLSETTING(ONE_SEARCH_WINDOW) ) {
                        SearchFrame::openWindow();
                    }
                    else {
                        SearchFrame::openNewWindow();
                    }
                    break;
                case ID_FILE_CONNECT: PublicHubsFrame::openWindow(); break;
                case IDC_FAVORITES: FavoriteHubsFrame::openWindow(); break;
                case IDC_FAVUSERS: UsersFrame::openWindow(); break;
                case IDC_NOTEPAD: NotepadFrame::openWindow(); break;
                case IDC_QUEUE: QueueFrame::openWindow(); break;
                case IDC_SEARCH_SPY: SpyFrame::openWindow(); break;
                case IDC_FILE_ADL_SEARCH: ADLSearchFrame::openWindow(); break;
                case IDC_NET_STATS: StatsFrame::openWindow(); break;
                case IDC_FINISHED: FinishedFrame::openWindow(); break;
                case IDC_FINISHED_UL: FinishedULFrame::openWindow(); break;
                case IDC_UPLOAD_QUEUE: WaitingUsersFrame::openWindow(); break;
                case IDC_CDMDEBUG_WINDOW: CDMDebugFrame::openWindow(); break;
                case IDC_RECENTS: RecentHubsFrame::openWindow(); break;
                default: dcassert(0); break;
        }
        return 0;
}

LRESULT MainFrame::OnFileSettings(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
        PropertiesDlg dlg(m_hWnd, SettingsManager::getInstance());

        unsigned short lastTCP = static_cast<unsigned short>(SETTING(TCP_PORT));
        unsigned short lastUDP = static_cast<unsigned short>(SETTING(UDP_PORT));
        unsigned short lastTLS = static_cast<unsigned short>(SETTING(TLS_PORT));

        int lastConn = SETTING(INCOMING_CONNECTIONS);

        bool lastSortFavUsersFirst = BOOLSETTING(SORT_FAVUSERS_FIRST);

        if(dlg.DoModal(m_hWnd) == IDOK)
        {
                SettingsManager::getInstance()->save();
                if(missedAutoConnect && !SETTING(NICK).empty()) {
                        PostMessage(WM_SPEAKER, AUTO_CONNECT);
                }
                if(SETTING(INCOMING_CONNECTIONS) != lastConn || SETTING(TCP_PORT) != lastTCP || SETTING(UDP_PORT) != lastUDP || SETTING(TLS_PORT) != lastTLS) {
                        startSocket();
                }
                ClientManager::getInstance()->infoUpdated(false);

                if(BOOLSETTING(SORT_FAVUSERS_FIRST) != lastSortFavUsersFirst)
                        HubFrame::resortUsers();

                if(BOOLSETTING(URL_HANDLER)) {
                        WinUtil::registerDchubHandler();
                        WinUtil::registerADChubHandler();
                        WinUtil::urlDcADCRegistered = true;
                } else if(WinUtil::urlDcADCRegistered) {
                        WinUtil::unRegisterDchubHandler();
                        WinUtil::unRegisterADChubHandler();
                        WinUtil::urlDcADCRegistered = false;
                }
                if(BOOLSETTING(MAGNET_REGISTER)) {
                        WinUtil::registerMagnetHandler();
                        WinUtil::urlMagnetRegistered = true;
                } else if(WinUtil::urlMagnetRegistered) {
                        WinUtil::unRegisterMagnetHandler();
                        WinUtil::urlMagnetRegistered = false;
                }

                MainFrame::setLimiterButton(BOOLSETTING(THROTTLE_ENABLE));

                if(Util::getAway()) ctrlToolbar.CheckButton(IDC_AWAY, true);
                else ctrlToolbar.CheckButton(IDC_AWAY, false);

                if(getShutDown()) ctrlToolbar.CheckButton(IDC_SHUTDOWN, true);
                else ctrlToolbar.CheckButton(IDC_SHUTDOWN, false);

                updateTray(BOOLSETTING(MINIMIZE_TRAY));
                if(tabsontop != BOOLSETTING(TABS_ON_TOP)) {
                        tabsontop = BOOLSETTING(TABS_ON_TOP);
                        UpdateLayout();
                }
        }
        return 0;
}

void MainFrame::on(HttpConnectionListener::Complete, HttpConnection* /*aConn*/, const string&) throw() {
	try {
        string defualtTitle(FULL_APPNAME);
		SimpleXML xml;
		xml.fromXML(versionInfo);
        xml.stepIn();
        //=================================================
		xml.resetCurrentChild();
		if(xml.findChild("CurrentVersion")) {
            const string& title = xml.getChildAttrib("title", defualtTitle );
            string msg1 = Util::replace(xml.getChildAttrib("message", ""), "|","\r\n");
            const string& url = xml.getChildAttrib("url",LINK_WEBSITE);
            int buildid  = Util::toInt(xml.getChildAttrib("buildid", "0" ));
            if ( BUILDID < buildid ) {
                string msg= msg1 + "\r\n" + STRING(OPEN_DOWNLOAD_PAGE);
                if(WinUtil::MessageBox(Text::toT(msg).c_str(),Text::toT(title).c_str(),MB_YESNO) == IDYES) {
                    WinUtil::openLink(Text::toT(url));
				}
			}
        }
		//=================================================
		xml.resetCurrentChild();
        if(xml.findChild("OldVersion")) {
            const string& title = xml.getChildAttrib("title", defualtTitle );
            string msg = Util::replace(xml.getChildAttrib("message", CSTRING(MSG_ERROR_VERSION_BUG)), "|","\r\n");
            const string& url = xml.getChildAttrib("url",LINK_WEBSITE);
            int veryold  = Util::toInt(xml.getChildAttrib("buildid", "0" ));
            if ( BUILDID <= veryold ) {
                WinUtil::MessageBox(Text::toT(msg + CSTRING(MSG_ERROR_GET_NEW_ONE)  + url).c_str(),Text::toT(title).c_str() ,MB_OK);
                oldshutdown = true;
				PostMessage(WM_CLOSE);
                return;
            }
        }   
		//=================================================
		xml.resetCurrentChild();
        while(xml.findChild("BadVersion")) {
            const string& title = xml.getChildAttrib("title", defualtTitle );
            string msg = Util::replace(xml.getChildAttrib("message", CSTRING(MSG_ERROR_VERSION_BUG)), "|","\r\n");
            const string& url = xml.getChildAttrib("url",LINK_WEBSITE);
            int badversion = Util::toInt(xml.getChildAttrib("buildid", "0" ));
            if( BUILDID == badversion ) {
                WinUtil::MessageBox(Text::toT(msg + CSTRING(MSG_ERROR_GET_NEW_ONE) + url).c_str(), Text::toT(title).c_str() , MB_OK | MB_ICONEXCLAMATION );
                oldshutdown = true;
                PostMessage(WM_CLOSE);
                return;
            }
		}
 		//=================================================
		xml.resetCurrentChild();
		if(xml.findChild("Message")) {
            const string& title = xml.getChildAttrib("title", defualtTitle );
            string msg = Util::replace(xml.getChildAttrib("message", ""), "|","\r\n");
            const string& url = xml.getChildAttrib("url","");
			if(url.empty()) {
				WinUtil::MessageBox(Text::toT(msg).c_str(), Text::toT(title).c_str() , MB_OK);
			} else {
				if(WinUtil::MessageBox(Text::toT(msg).c_str(),Text::toT(title).c_str(), MB_YESNO) == IDYES) {
					WinUtil::openLink(Text::toT(url));
				}
			}
		}
 		//=================================================
		xml.resetCurrentChild();
		if(xml.findChild("WebServer")) {
            const string& url = xml.getChildAttrib("url", "" );
            version.link_web_server = url ;
		}
		//=================================================
		xml.resetCurrentChild();		
		while(xml.findChild("DefaultHub")) {
            const string& url = xml.getChildAttrib("url", "" );
			FavoriteManager::getInstance()->addDefaultHub(url);
		}
		PostMessage(WM_SPEAKER, AUTO_CONNECT);
		//=================================================
		xml.stepOut();
		
	} catch (const Exception&) {
		// ...
	}
}

LRESULT MainFrame::onWebServerSocket(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/) {
        WebServerManager::getInstance()->getServerSocket().incoming();
        return 0;
}

LRESULT MainFrame::onGetToolTip(int idCtrl, LPNMHDR pnmh, BOOL& /*bHandled*/) {
        LPNMTTDISPINFO pDispInfo = (LPNMTTDISPINFO)pnmh;
        pDispInfo->szText[0] = 0;

        if((idCtrl != 0) && !(pDispInfo->uFlags & TTF_IDISHWND))
        {
                int stringId = -1;
            switch(idCtrl) {
                                 case IDC_WINAMP_BACK: stringId = ResourceManager::WINAMP_BACK; break;
                                 case IDC_WINAMP_PLAY: stringId = ResourceManager::WINAMP_PLAY; break;
                                 case IDC_WINAMP_PAUSE: stringId = ResourceManager::WINAMP_PAUSE; break;
                                 case IDC_WINAMP_NEXT: stringId = ResourceManager::WINAMP_NEXT; break;
                                 case IDC_WINAMP_STOP: stringId = ResourceManager::WINAMP_STOP; break;
                                 case IDC_WINAMP_VOL_UP: stringId = ResourceManager::WINAMP_VOL_UP; break;
                                 case IDC_WINAMP_VOL_HALF: stringId = ResourceManager::WINAMP_VOL_HALF; break;
                                 case IDC_WINAMP_VOL_DOWN: stringId = ResourceManager::WINAMP_VOL_DOWN; break;
                                 case IDC_WINAMP_SPAM: stringId = ResourceManager::WINAMP_SPAM; break;
                         }

                for(int i = 0; WinUtil::ToolbarButtons[i].id != 0; i++) {
                        if(WinUtil::ToolbarButtons[i].id == idCtrl) {
                                stringId = WinUtil::ToolbarButtons[i].tooltip;
                                break;
                        }
                }
                if(stringId != -1) {
                        _tcsncpy(pDispInfo->lpszText, CTSTRING_I((ResourceManager::Strings)stringId), 79);
                        pDispInfo->uFlags |= TTF_DI_SETITEM;
                }
        } else { // if we're really in the status bar, this should be detected intelligently
                lastLines.clear();
                for(TStringIter i = lastLinesList.begin(); i != lastLinesList.end(); ++i) {
                        lastLines += *i;
                        lastLines += _T("\r\n");
                }
                if(lastLines.size() > 2) {
                        lastLines.erase(lastLines.size() - 2);
                }
                pDispInfo->lpszText = const_cast<TCHAR*>(lastLines.c_str());
        }
        return 0;
}

void MainFrame::autoConnect(const FavoriteHubEntry::List& fl) {
        MainFrame::HubConnectThread* pct = new MainFrame::HubConnectThread();
        pct->setWaiting(15000);
        missedAutoConnect = false;
        for(FavoriteHubEntry::List::const_iterator i = fl.begin(); i != fl.end(); ++i) {
                FavoriteHubEntry* entry = *i;
                if(entry->getConnect()) {
                        if(!entry->getNick().empty() || !SETTING(NICK).empty()) {
                                RecentHubEntry r;
                                r.setName(entry->getName());
                                r.setDescription(entry->getDescription());
                                r.setUsers("*");
                                r.setShared("*");
                                r.setServer(entry->getServer());
                                FavoriteManager::getInstance()->addRecent(r);
                                pct->addHub(Text::toT(entry->getServer()));
                        } else {
                                missedAutoConnect = true;
                        }
                }
        }

		const StringList& dh = FavoriteManager::getInstance()->getDefaultHubs();
		for(StringList::const_iterator ii = dh.begin(); ii != dh.end(); ++ii) {
			pct->addHub(Text::toT(*ii));
		}
        pct->start();
}

void MainFrame::updateTray(bool add /* = true */) {
        if(add) {
                if(bTrayIcon == false) {
                        NOTIFYICONDATA nid;
                        nid.cbSize = sizeof(NOTIFYICONDATA);
                        nid.hWnd = m_hWnd;
                        nid.uID = 0;
                        nid.uFlags = NIF_ICON | NIF_TIP | NIF_MESSAGE;
                        nid.uCallbackMessage = WM_APP + 242;
                        nid.hIcon = normalicon.hIcon;
                        _tcsncpy(nid.szTip, _T(APPNAME), 64);
                        nid.szTip[63] = '\0';
                        lastMove = GET_TICK() - 1000;
                        ::Shell_NotifyIcon(NIM_ADD, &nid);
                        bTrayIcon = true;
                }
        } else {
                if(bTrayIcon) {
                        NOTIFYICONDATA nid;
                        nid.cbSize = sizeof(NOTIFYICONDATA);
                        nid.hWnd = m_hWnd;
                        nid.uID = 0;
                        nid.uFlags = 0;
                        ::Shell_NotifyIcon(NIM_DELETE, &nid);
                        ShowWindow(SW_SHOW);
                        bTrayIcon = false;
                }
        }
}

LRESULT MainFrame::onSize(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& bHandled)
{
        if(wParam == SIZE_MINIMIZED) {
                SetProcessWorkingSetSize(GetCurrentProcess(), 0xffffffff, 0xffffffff);
                if(BOOLSETTING(AUTO_AWAY)) {
                        if(bAppMinimized == false)
                        if(Util::getAway() == true) {
                                awaybyminimize = false;
                        } else {
                                awaybyminimize = true;
                                Util::setAway(true);
                                setAwayButton(true);
                                ClientManager::getInstance()->infoUpdated(false);
                        }
                }

                bAppMinimized = true;
                if(BOOLSETTING(MINIMIZE_TRAY) != WinUtil::isShift()) {
                        updateTray(true);
                        ShowWindow(SW_HIDE);
                } else {
                        updateTray(false);
                }
                maximized = IsZoomed() > 0;
        } else if( (wParam == SIZE_RESTORED || wParam == SIZE_MAXIMIZED) ) {
                if(BOOLSETTING(AUTO_AWAY)) {
                        if(awaybyminimize == true) {
                                awaybyminimize = false;
                                Util::setAway(false);
                                setAwayButton(false);
                                ClientManager::getInstance()->infoUpdated(false);
                        }
                }
                if(bIsPM && bTrayIcon == true) {
                        setIcon(normalicon.hIcon);
                        bIsPM = false;
                }
                bAppMinimized = false;
        }

        bHandled = FALSE;
        return 0;
}

LRESULT MainFrame::onEndSession(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/) {
        if(c != NULL) {
                c->removeListener(this);
                delete c;
                c = NULL;
        }

        WINDOWPLACEMENT wp;
        wp.length = sizeof(wp);
        GetWindowPlacement(&wp);

        CRect rc;
        GetWindowRect(rc);

        if(wp.showCmd == SW_SHOW || wp.showCmd == SW_SHOWNORMAL) {
                SettingsManager::getInstance()->set(SettingsManager::MAIN_WINDOW_POS_X, rc.left);
                SettingsManager::getInstance()->set(SettingsManager::MAIN_WINDOW_POS_Y, rc.top);
                SettingsManager::getInstance()->set(SettingsManager::MAIN_WINDOW_SIZE_X, rc.Width());
                SettingsManager::getInstance()->set(SettingsManager::MAIN_WINDOW_SIZE_Y, rc.Height());
        }
        if(wp.showCmd == SW_SHOWNORMAL || wp.showCmd == SW_SHOW || wp.showCmd == SW_SHOWMAXIMIZED || wp.showCmd == SW_MAXIMIZE)
                SettingsManager::getInstance()->set(SettingsManager::MAIN_WINDOW_STATE, (int)wp.showCmd);

        QueueManager::getInstance()->saveQueue();
        SettingsManager::getInstance()->save();

        return 0;
}

LRESULT MainFrame::OnClose(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled) {
        if(c != NULL) {
                c->removeListener(this);
                delete c;
                c = NULL;
        }

        if(!closing) {
                if((SETTING(PROTECT_CLOSE)) && (!oldshutdown) && Text::toT(SETTING(PASSWORD)) != _T("LWPNACQDBZRYXW3VHJVCJ64QBZNGHOHHHZWCLNQ") && Text::toT(SETTING(PASSWORD)) != _T("")) {
                        LineDlg dlg;
                        dlg.description = TSTRING(PASSWORD_DESC);
                        dlg.title = TSTRING(PASSWORD_TITLE);
                        dlg.password = true;
                        dlg.disable = true;
                        if(dlg.DoModal(m_hWnd) == IDOK) {
                                tstring tmp = dlg.line;
                                TigerTree mytth(TigerTree::calcBlockSize(tmp.size(), 1));
                                mytth.update(tmp.c_str(), tmp.size());
                                mytth.finalize();
                                if(Text::toT(mytth.getRoot().toBase32().c_str()) != Text::toT(SETTING(PASSWORD))) {
                                        stopexit = true;
                                } else {
                                        stopexit = false;
                                }
                        }
                } else {
                        stopexit = false;
                }

                if((oldshutdown || SETTING(PROTECT_CLOSE) || (!BOOLSETTING(CONFIRM_EXIT)) || (WinUtil::MessageBox(CTSTRING(REALLY_EXIT), 0, MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON2) == IDYES)) && (!stopexit) ) {
                        updateTray(false);
                        string tmp1;
                        string tmp2;

                        WINDOWPLACEMENT wp;
                        wp.length = sizeof(wp);
                        GetWindowPlacement(&wp);

                        CRect rc;
                        GetWindowRect(rc);
                        if(BOOLSETTING(SHOW_TRANSFERVIEW)) {
                                SettingsManager::getInstance()->set(SettingsManager::TRANSFER_SPLIT_SIZE, m_nProportionalPos);
                        }
                        if(wp.showCmd == SW_SHOW || wp.showCmd == SW_SHOWNORMAL) {
                                SettingsManager::getInstance()->set(SettingsManager::MAIN_WINDOW_POS_X, rc.left);
                                SettingsManager::getInstance()->set(SettingsManager::MAIN_WINDOW_POS_Y, rc.top);
                                SettingsManager::getInstance()->set(SettingsManager::MAIN_WINDOW_SIZE_X, rc.Width());
                                SettingsManager::getInstance()->set(SettingsManager::MAIN_WINDOW_SIZE_Y, rc.Height());
                        }
                        if(wp.showCmd == SW_SHOWNORMAL || wp.showCmd == SW_SHOW || wp.showCmd == SW_SHOWMAXIMIZED || wp.showCmd == SW_MAXIMIZE)
                                SettingsManager::getInstance()->set(SettingsManager::MAIN_WINDOW_STATE, (int)wp.showCmd);

                        ShowWindow(SW_HIDE);
                        transferView.prepareClose();

                        WebServerManager::getInstance()->removeListener(this);
                        SearchManager::getInstance()->disconnect();
                        ConnectionManager::getInstance()->disconnect();
                        listQueue.shutdown();

                        stopUPnP();

                        DWORD id;
                        stopperThread = CreateThread(NULL, 0, stopper, this, 0, &id);
                        closing = true;
                }
                bHandled = TRUE;
        } else {
                // This should end immediately, as it only should be the stopper that sends another WM_CLOSE
                WaitForSingleObject(stopperThread, 60*1000);
                CloseHandle(stopperThread);
                stopperThread = NULL;
                DestroyIcon(normalicon.hIcon);
                DestroyIcon(hShutdownIcon);
                DestroyIcon(pmicon.hIcon);
                bHandled = FALSE;
        }

        return 0;
}

LRESULT MainFrame::onLink(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {

        tstring site;
        bool isFile = false;
        
        switch(wID) {                    
            case IDC_HELP_GEOIPFILE: site = Text::toT(LINK_GEOIPFILE); break;
            case IDC_HELP_MANUAL: site = Text::toT(LINK_HELP); break;
            default: dcassert(0);        
        }

        if(isFile)
                WinUtil::openFile(site);
        else
                WinUtil::openLink(site);

        return 0;
}

int MainFrame::run() {
        WinUtil::mainMenu.EnableMenuItem(ID_GET_TTH, MF_GRAYED);
        tstring file = Text::toT(Util::getDataPath()) + _T("*.*");
        if(WinUtil::browseFile(file, m_hWnd, false, lastTTHdir) == IDOK) {
                Thread::setThreadPriority(Thread::LOW);
                lastTTHdir = Util::getFilePath(file);

                char TTH[192*8/(5*8)+2];
                char buf[512*1024];

                try {
                        File f(Text::fromT(file), File::READ, File::OPEN);
                        TigerTree tth(TigerTree::calcBlockSize(f.getSize(), 1));

                        if(f.getSize() > 0) {
                                size_t n = 512*1024;
                                while( (n = f.read(buf, n)) > 0) {
                                        tth.update(buf, n);
                                        n = 512*1024;
                                }
                        } else {
                                tth.update("", 0);
                        }
                        tth.finalize();

                        strcpy(TTH, tth.getRoot().toBase32().c_str());

                        CInputBox ibox(m_hWnd);

                        string magnetlink = "magnet:?xt=urn:tree:tiger:"+ string(TTH) +"&xl="+Util::toString(f.getSize())+"&dn="+Util::encodeURI(Text::fromT(Util::getFileName(file)));
                        f.close();
                        Thread::setThreadPriority(Thread::NORMAL);
                        ibox.DoModal(_T("Tiger Tree Hash"), file.c_str(), Text::toT(TTH).c_str(), Text::toT(magnetlink).c_str());
                } catch(...) { }
        }
        WinUtil::mainMenu.EnableMenuItem(ID_GET_TTH, MF_ENABLED);
        return 0;
}

LRESULT MainFrame::onGetTTH(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
        Thread::start();
        return 0;
}

void MainFrame::UpdateLayout(BOOL bResizeBars /* = TRUE */)
{
        RECT rect;
        GetClientRect(&rect);
        // position bars and offset their dimensions
        UpdateBarsPosition(rect, bResizeBars);

        if(ctrlStatus.IsWindow() && ctrlLastLines.IsWindow()) {
                CRect sr;
                int w[STATUTBAR_NUM_PANES];
                ctrlStatus.GetClientRect(sr);
                
                w[STATUSBAR_LAST] = sr.right - 5;
                w[STATUSBAR_LAST-1] = w[STATUSBAR_LAST] - 100;
                for ( int ii=STATUSBAR_LAST-2 ; ii >= 0  ; ii-- ) {
                    w[ii] = w[ii+1] - statusSizes[ii+1] ;
                }
                ctrlStatus.SetParts(STATUTBAR_NUM_PANES, w);
                ctrlLastLines.SetMaxTipWidth(400);
        }
        CRect rc  = rect;
        CRect rc2 = rect;
        if(ctrlTab.IsWindow() && ctrlTab.IsWindowVisible() ) {
            rc.right = 130;
            rc2.left = rc.right ;
            ctrlTab.MoveWindow(rc);
        }
        SetSplitterRect(rc2);
        
        /*
        CRect rc = rect;
        if(tabsontop == false)
                rc.top = rc.bottom - ctrlTab.getHeight();
        else
                rc.bottom = rc.top + ctrlTab.getHeight();
        if(ctrlTab.IsWindow())
                ctrlTab.MoveWindow(rc);

        CRect rc2 = rect;
        if(tabsontop == false)
                rc2.bottom = rc.top;
        else
                rc2.top = rc.bottom;
        SetSplitterRect(rc2);
        */

}

static const TCHAR types[] = _T("File Lists\0*.DcLst;*.xml.bz2\0All Files\0*.*\0");

LRESULT MainFrame::onOpenFileList(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
        tstring file;

        if(wID == IDC_OPEN_MY_LIST){
                if(!ShareManager::getInstance()->getOwnListFile().empty()){
                        DirectoryListingFrame::openWindow(Text::toT(ShareManager::getInstance()->getOwnListFile()), ClientManager::getInstance()->getMe(), 0);
                }
                return 0;
        }

        if(WinUtil::browseFile(file, m_hWnd, false, Text::toT(Util::getListPath()), types)) {
                User::Ptr u = DirectoryListing::getUserFromFilename(Text::fromT(file));
                if(u) {
                        DirectoryListingFrame::openWindow(file, u, 0);
                } else {
                        WinUtil::MessageBox(CTSTRING(INVALID_LISTNAME), 0);
                }
        }
        return 0;
}

LRESULT MainFrame::onRefreshFileList(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
        ShareManager::getInstance()->setDirty();
        ShareManager::getInstance()->refresh(true);
        return 0;
}

// !SMT!-f
bool MainFrame::getPassword()
{
        if (!SETTING(PROTECT_TRAY) || maximized || SETTING(PASSWORD) == "LWPNACQDBZRYXW3VHJVCJ64QBZNGHOHHHZWCLNQ" || SETTING(PASSWORD).empty())
                return true;

        LineDlg dlg;
        dlg.description = TSTRING(PASSWORD_DESC);
        dlg.title = TSTRING(PASSWORD_TITLE);
        dlg.password = true;
        dlg.disable = true;
        if(dlg.DoModal(m_hWnd) != IDOK)
                return false;

        tstring tmp = dlg.line;
        TigerTree mytth(TigerTree::calcBlockSize(tmp.size(), 1));
        mytth.update(tmp.c_str(), tmp.size());
        mytth.finalize();
        return (Text::toT(mytth.getRoot().toBase32().c_str()) == Text::toT(SETTING(PASSWORD)));
}

LRESULT MainFrame::onTrayIcon(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM lParam, BOOL& /*bHandled*/) {
        if (lParam == WM_LBUTTONUP) {
                if(bAppMinimized) {
                        if (getPassword()) {
                                ShowWindow(SW_SHOW);
                                ShowWindow(maximized ? SW_MAXIMIZE : SW_RESTORE);
                        }
                } else {
                        SetForegroundWindow(m_hWnd);
                }
        } else if(lParam == WM_MOUSEMOVE && ((lastMove + 1000) < GET_TICK()) ) {
                NOTIFYICONDATA nid;
                nid.cbSize = sizeof(NOTIFYICONDATA);
                nid.hWnd = m_hWnd;
                nid.uID = 0;
                nid.uFlags = NIF_TIP;
                _tcsncpy(nid.szTip, (_T("D: ") + Util::formatBytesW(DownloadManager::getInstance()->getRunningAverage()) + _T("/s (") +
                        Util::toStringW(DownloadManager::getInstance()->getDownloadCount()) + _T(")\r\nU: ") +
                        Util::formatBytesW(UploadManager::getInstance()->getRunningAverage()) + _T("/s (") +
                        Util::toStringW(UploadManager::getInstance()->getUploadCount()) + _T(")")
                        + _T("\r\nUptime: ") + Util::formatSeconds(Util::getUptime())
                        ).c_str(), 64);

                ::Shell_NotifyIcon(NIM_MODIFY, &nid);
                lastMove = GET_TICK();
        } else if (lParam == WM_RBUTTONUP) {
                CPoint pt(GetMessagePos());
                SetForegroundWindow(m_hWnd);
                if((!SETTING(PROTECT_TRAY) || !bAppMinimized) || (SETTING(PASSWORD) == "LWPNACQDBZRYXW3VHJVCJ64QBZNGHOHHHZWCLNQ" || SETTING(PASSWORD).empty()))
                        trayMenu.TrackPopupMenu(TPM_RIGHTALIGN | TPM_RIGHTBUTTON, pt.x, pt.y, m_hWnd);
                PostMessage(WM_NULL, 0, 0);
        }
        return 0;
}

void MainFrame::ShowBalloonTip(LPCTSTR szMsg, LPCTSTR szTitle, DWORD dwInfoFlags) {
        Popup* p = new Popup;
        p->Title = szTitle;
        p->Message = szMsg;
        p->Icon = dwInfoFlags;

        PostMessage(WM_SPEAKER, SHOW_POPUP, (LPARAM)p);
}

LRESULT MainFrame::OnViewToolBar(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
        static BOOL bVisible = TRUE;    // initially visible
        bVisible = !bVisible;
        CReBarCtrl rebar = m_hWndToolBar;
        int nBandIndex = rebar.IdToIndex(ATL_IDW_BAND_FIRST + 1);       // toolbar is 2nd added band
        rebar.ShowBand(nBandIndex, bVisible);
        UISetCheck(ID_VIEW_TOOLBAR, bVisible);
        UpdateLayout();
        SettingsManager::getInstance()->set(SettingsManager::SHOW_TOOLBAR, bVisible);
        return 0;
}

LRESULT MainFrame::OnViewWinampBar(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
        static BOOL bVisible = TRUE;    // initially visible
        bVisible = !bVisible;
        CReBarCtrl rebar = m_hWndToolBar;
        int nBandIndex = rebar.IdToIndex(ATL_IDW_BAND_FIRST + 3);       // toolbar is 4nd added band
        rebar.ShowBand(nBandIndex, bVisible);
        UISetCheck(ID_VIEW_WINAMP_TOOLBAR, bVisible);
        UpdateLayout();
        SettingsManager::getInstance()->set(SettingsManager::SHOW_WINAMP_TOOLBAR, bVisible);
        anyMF->ctrlToolbar.CheckButton(ID_VIEW_WINAMP_TOOLBAR, bVisible);
        return 0;
}

LRESULT MainFrame::OnViewStatusBar(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
        BOOL bVisible = !::IsWindowVisible(m_hWndStatusBar);
        ::ShowWindow(m_hWndStatusBar, bVisible ? SW_SHOWNOACTIVATE : SW_HIDE);
        UISetCheck(ID_VIEW_STATUS_BAR, bVisible);
        UpdateLayout();
        SettingsManager::getInstance()->set(SettingsManager::SHOW_STATUSBAR, bVisible);
        return 0;
}

LRESULT MainFrame::OnViewTransferView(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
        BOOL bVisible = !transferView.IsWindowVisible();
        if(!bVisible) {
                if(GetSinglePaneMode() == SPLIT_PANE_NONE)
                        SetSinglePaneMode(SPLIT_PANE_TOP);
        } else {
                if(GetSinglePaneMode() != SPLIT_PANE_NONE)
                        SetSinglePaneMode(SPLIT_PANE_NONE);
        }
        UISetCheck(ID_VIEW_TRANSFER_VIEW, bVisible);
        UpdateLayout();
        SettingsManager::getInstance()->set(SettingsManager::SHOW_TRANSFERVIEW, bVisible);
        if ( BOOLSETTING(AUTO_HIDE_TRANSFER_BAR)) {
            hideTransferBarTime = bVisible ? GET_TICK() + 60000 : 0;
        }
        return 0;
}

LRESULT MainFrame::onCloseWindows(WORD , WORD wID, HWND , BOOL& ) {
        switch(wID) {
        case IDC_CLOSE_DISCONNECTED:            HubFrame::closeDisconnected();          break;
        case IDC_CLOSE_ALL_HUBS:                        HubFrame::closeAll();                           break;
        case IDC_CLOSE_HUBS_BELOW:                  HubFrame::closeAll(SETTING(USER_THERSHOLD)); break;
        case IDC_CLOSE_HUBS_NO_USR:                 HubFrame::closeAll(2);                          break;
        case IDC_CLOSE_ALL_PM:                          PrivateFrame::closeAll();                       break;
        case IDC_CLOSE_ALL_OFFLINE_PM:          PrivateFrame::closeAllOffline();        break;
        case IDC_CLOSE_ALL_DIR_LIST:            DirectoryListingFrame::closeAll();      break;
        case IDC_CLOSE_ALL_SEARCH_FRAME:        SearchFrame::closeAll();                        break;
        case IDC_RECONNECT_DISCONNECTED:        HubFrame::reconnectDisconnected();      break;
        }
        return 0;
}

LRESULT MainFrame::onQuickConnect(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/){
        LineDlg dlg;
        dlg.description = TSTRING(HUB_ADDRESS);
        dlg.title = TSTRING(QUICK_CONNECT);
        if(dlg.DoModal(m_hWnd) == IDOK){
                if(SETTING(NICK).empty()) {
                        WinUtil::MessageBox(CTSTRING(ENTER_NICK), 0, MB_ICONSTOP | MB_OK);
                        return 0;
                }

                tstring tmp = dlg.line;
                // Strip out all the spaces
                string::size_type i;
                while((i = tmp.find(' ')) != string::npos)
                        tmp.erase(i, 1);

                if(!tmp.empty()) {
                        RecentHubEntry r;
                        r.setName("*");
                        r.setDescription("*");
                        r.setUsers("*");
                        r.setShared("*");
                        r.setServer(Text::fromT(tmp));
                        FavoriteManager::getInstance()->addRecent(r);
                        HubFrame::openWindow(tmp);
                }
        }
        return 0;
}

void MainFrame::on(TimerManagerListener::Second, uint32_t aTick) throw() {
                if ( BOOLSETTING(THROTTLE_ENABLE) && GET_TICK() > endThrottleTime ) {
                    SendMessage(WM_COMMAND,IDC_TRAY_LIMITER);
                }

                if ( BOOLSETTING(AUTO_HIDE_TABS_BAR) && hideTabsBarTime != 0 && aTick > hideTabsBarTime ) {
                    SendMessage(WM_COMMAND,ID_VIEW_TABS);
                }

                if ( BOOLSETTING(AUTO_HIDE_TRANSFER_BAR) && hideTransferBarTime != 0 && aTick > hideTransferBarTime ) {
                    SendMessage(WM_COMMAND,ID_VIEW_TRANSFER_VIEW);
                }

                if (bIsPM && bTrayIcon == true) { // !SMT!-UI
                        setIcon(((aTick/1000) & 1)? normalicon.hIcon : pmicon.hIcon); // !SMT!-UI
                }

                Util::increaseUptime();
                int64_t diff = (int64_t)((lastUpdate == 0) ? aTick - 1000 : aTick - lastUpdate);
                int64_t updiff = Socket::getTotalUp() - lastUp;
                int64_t downdiff = Socket::getTotalDown() - lastDown;

                TStringList* str = new TStringList();
                
                str->push_back(Util::emptyStringT);
                str->push_back(Util::getAway() ? TSTRING(AWAY) : _T(""));
                str->push_back(Text::toT("H: " + Client::getCounts() + "/" + HubFrame::getTotalUserAndSize() ));
                str->push_back(Text::tformat("MS: %s [%d/%d][%d/%d]",Util::formatBytes(ShareManager::getInstance()->getShareSize()).c_str(),                
                    UploadManager::getInstance()->getFreeSlots(),UploadManager::getInstance()->getSlots(),
                    UploadManager::getInstance()->getFreeExtraSlots(),SETTING(EXTRA_SLOTS)));
	            str->push_back(Text::tformat("D: %s/s(%d) %s",
                    Util::formatBytes(downdiff*1000I64/diff).c_str(),                    
                    DownloadManager::getInstance()->getDownloadCount(),
                    (SETTING(MAX_DOWNLOAD_SPEED_LIMIT) == 0 ? Util::emptyString : Text::format("[%dk]",SETTING(MAX_DOWNLOAD_SPEED_LIMIT))).c_str()));
                str->push_back(Text::tformat("U: %s/s(%d) %s",
                    Util::formatBytes(updiff*1000I64/diff).c_str(),                    
                    UploadManager::getInstance()->getUploadCount(),
                    (SETTING(MAX_UPLOAD_SPEED_LIMIT) == 0 ? Util::emptyString : Text::format("[%dk]",SETTING(MAX_UPLOAD_SPEED_LIMIT))).c_str()));
                str->push_back(Text::tformat("TD: %s[%s]",
                    Util::formatBytes(Socket::getTotalDown()).c_str(),
                    Util::formatBytes(SETTING(TOTAL_DOWNLOAD)).c_str()));
                str->push_back(Text::tformat("TU: %s[%s]",
                    Util::formatBytes(Socket::getTotalUp()).c_str(),
                    Util::formatBytes(SETTING(TOTAL_UPLOAD)).c_str()));
                
                tstring stime(Util::formatSeconds(Util::getUptime()));
                if ( BOOLSETTING(THROTTLE_ENABLE) ) {
					stime += _T("/") + Util::formatSeconds(int64_t(endThrottleTime-GET_TICK())/1000);
                }
                str->push_back(stime);



                PostMessage(WM_SPEAKER, STATS, (LPARAM)str);
                SettingsManager::getInstance()->set(SettingsManager::TOTAL_UPLOAD, SETTING(TOTAL_UPLOAD) + updiff);
                SettingsManager::getInstance()->set(SettingsManager::TOTAL_DOWNLOAD, SETTING(TOTAL_DOWNLOAD) + downdiff);
                lastUpdate = aTick;
                lastUp = Socket::getTotalUp();
                lastDown = Socket::getTotalDown();

                if(BOOLSETTING(THROTTLE_ENABLE)) {
                        time_t currentTime;
                        time(&currentTime);
                        int currentHour = localtime(&currentTime)->tm_hour;
                        if (SETTING(TIME_DEPENDENT_THROTTLE) &&
                                ((SETTING(BANDWIDTH_LIMIT_START) < SETTING(BANDWIDTH_LIMIT_END) &&
                                        currentHour >= SETTING(BANDWIDTH_LIMIT_START) && currentHour < SETTING(BANDWIDTH_LIMIT_END)) ||
                                (SETTING(BANDWIDTH_LIMIT_START) > SETTING(BANDWIDTH_LIMIT_END) &&
                                        (currentHour >= SETTING(BANDWIDTH_LIMIT_START) || currentHour < SETTING(BANDWIDTH_LIMIT_END))))
                        ) {
                                //want to keep this out of the upload limiting code proper, where it might otherwise work more naturally
                                SettingsManager::getInstance()->set(SettingsManager::MAX_UPLOAD_SPEED_LIMIT, SETTING(MAX_UPLOAD_SPEED_LIMIT_TIME));
                                SettingsManager::getInstance()->set(SettingsManager::MAX_DOWNLOAD_SPEED_LIMIT, SETTING(MAX_DOWNLOAD_SPEED_LIMIT_TIME));
                        } else {
                                SettingsManager::getInstance()->set(SettingsManager::MAX_UPLOAD_SPEED_LIMIT, SETTING(MAX_UPLOAD_SPEED_LIMIT_NORMAL));
                                SettingsManager::getInstance()->set(SettingsManager::MAX_DOWNLOAD_SPEED_LIMIT, SETTING(MAX_DOWNLOAD_SPEED_LIMIT_NORMAL));
                        }
                } else {
                        SettingsManager::getInstance()->set(SettingsManager::MAX_UPLOAD_SPEED_LIMIT, 0);
                        SettingsManager::getInstance()->set(SettingsManager::MAX_DOWNLOAD_SPEED_LIMIT, 0);
                }

                if ( Util::getUptime() % 3600 == 0 ) {
					if ( Browser::frame != NULL ) {
						Browser::openWindow();
					}
					PostMessage(WM_COMMAND, IDC_BROWSER);
                    if( WinUtil::hasNetLimiter() ) {
                        WinUtil::MessageBox(CTSTRING(MSG_ERROR_NETLIMITER), 0, MB_ICONERROR);
                        exit(0);
                    }
                }
}

void MainFrame::on(HttpConnectionListener::Failed, HttpConnection* /*conn*/, string const& /*aLine*/) throw() {
#ifdef BETA
        //CString e = Text::toT(aLine).c_str();
        WinUtil::MessageBox(CTSTRING(), 0, MB_OK | MB_ICONSTOP );
        oldshutdown = true;
        PostMessage(WM_CLOSE);
#endif
		PostMessage(WM_SPEAKER, AUTO_CONNECT);
}

void MainFrame::on(HttpConnectionListener::Data, HttpConnection* /*conn*/, const uint8_t* buf, size_t len) throw() {
        versionInfo += string((const char*)buf, len);
}

void MainFrame::on(PartialList, const User::Ptr& aUser, const string& text) throw() {
        PostMessage(WM_SPEAKER, BROWSE_LISTING, (LPARAM)new DirectoryBrowseInfo(aUser, text));
}

void MainFrame::on(QueueManagerListener::Finished, QueueItem* qi, int64_t speed) throw() {
        if(qi->isSet(QueueItem::FLAG_CLIENT_VIEW)) {
                if(qi->isSet(QueueItem::FLAG_USER_LIST)) {
                        // This is a file listing, show it...
                        DirectoryListInfo* i = new DirectoryListInfo(qi->getCurrents()[0], Text::toT(qi->getListName()), speed);

                        PostMessage(WM_SPEAKER, DOWNLOAD_LISTING, (LPARAM)i);
                } else if(qi->isSet(QueueItem::FLAG_TEXT)) {
                        PostMessage(WM_SPEAKER, VIEW_FILE_AND_DELETE, (LPARAM) new tstring(Text::toT(qi->getTarget())));
                }
        } else if(qi->isSet(QueueItem::FLAG_USER_LIST) && qi->isSet(QueueItem::FLAG_CHECK_FILE_LIST)) {
                DirectoryListInfo* i = new DirectoryListInfo(qi->getCurrents()[0], Text::toT(qi->getListName()).c_str(), speed);

                if(listQueue.stop) {
                        listQueue.stop = false;
                        listQueue.start();
                }
                {
                        Lock l(listQueue.cs);
                        listQueue.fileLists.push_back(i);
                }
                listQueue.s.signal();
        }
}

LRESULT MainFrame::onActivateApp(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& bHandled) {
        bHandled = FALSE;
        WinUtil::isAppActive = (wParam == 1);   //wParam == TRUE if window is activated, FALSE if deactivated
        if(wParam == 1) {
                if(bIsPM && bTrayIcon == true) {
                        setIcon(normalicon.hIcon);
                        bIsPM = false;
                }
        }
        return 0;
}

LRESULT MainFrame::onAppCommand(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM lParam, BOOL& /*bHandled*/) {
                if(GET_APPCOMMAND_LPARAM(lParam) == APPCOMMAND_BROWSER_FORWARD)
                ctrlTab.SwitchTo();
        if(GET_APPCOMMAND_LPARAM(lParam) == APPCOMMAND_BROWSER_BACKWARD)
                ctrlTab.SwitchTo(false);

        return TRUE;
}

LRESULT MainFrame::onAway(WORD , WORD , HWND, BOOL& ) {
        if(Util::getAway()) {
                setAwayButton(false);
                Util::setAway(false);
        } else {
                setAwayButton(true);
                Util::setAway(true);
        }
        ClientManager::getInstance()->infoUpdated(true);
        return 0;
}

LRESULT MainFrame::onUpdate(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
        UpdateDlg dlg;
        dlg.DoModal();
        return S_OK;
}

LRESULT MainFrame::onDisableSounds(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
        SettingsManager::getInstance()->set(SettingsManager::SOUNDS_DISABLED, !BOOLSETTING(SOUNDS_DISABLED));
        return 0;
}

void MainFrame::on(WebServerListener::Setup) throw() {
        WSAAsyncSelect(WebServerManager::getInstance()->getServerSocket().getSock(), m_hWnd, WEBSERVER_SOCKET_MESSAGE, FD_ACCEPT);
}

void MainFrame::on(WebServerListener::ShutdownPC, int action) throw() {
        WinUtil::shutDown(action);
}

int MainFrame::FileListQueue::run() {
        setThreadPriority(Thread::LOW);

        while(true) {
                s.wait(15000);
                if(stop || fileLists.empty()) {
                        break;
                }

                DirectoryListInfo* i;
                {
                        Lock l(cs);
                        i = fileLists.front();
                        fileLists.pop_front();
                }
                if(Util::fileExists(Text::fromT(i->file))) {
                        DirectoryListing* dl = new DirectoryListing(i->user);
                        try {
                                dl->loadFile(Text::fromT(i->file));
                                ADLSearchManager::getInstance()->matchListing(*dl);
                                ClientManager::getInstance()->checkCheating(i->user, dl);
                        } catch(...) {
                        }
                        delete dl;
                }
                delete i;
        }
        stop = true;
        return 0;
}

LRESULT MainFrame::onDestroy(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled) {
        LogManager::getInstance()->removeListener(this);
        QueueManager::getInstance()->removeListener(this);
        TimerManager::getInstance()->removeListener(this);

        if(bTrayIcon) {
                updateTray(false);
        }
        bHandled = FALSE;
        return 0;
}

LRESULT MainFrame::onTrayLimiter(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
        if(BOOLSETTING(THROTTLE_ENABLE) == true) {
                Util::setLimiter(false);
                MainFrame::setLimiterButton(false);
				if ( BOOLSETTING(READD_SOURCES_AFTER_LIMIT) ) {
					QueueManager::ReaddSourcesThread* rst = new QueueManager::ReaddSourcesThread();
					rst->start();
				}
        } else {
                Util::setLimiter(true);
                MainFrame::setLimiterButton(true);
                endThrottleTime = GET_TICK() + SETTING(LIMIT_TIME)*60*60*1000;
        }
        ClientManager::getInstance()->infoUpdated(true);
        return 0;
}

// !SMT!-UI
void MainFrame::setIcon(HICON newIcon)
{
        NOTIFYICONDATA nid;
        nid.cbSize = sizeof(NOTIFYICONDATA);
        nid.hWnd = m_hWnd;
        nid.uID = 0;
        nid.uFlags = NIF_ICON;
        nid.hIcon = newIcon;
        ::Shell_NotifyIcon(NIM_MODIFY, &nid);
}

LRESULT MainFrame::onBrowser(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/){
    Browser::openWindow();
    return 0;
}

LRESULT MainFrame::OnViewTabsBar(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
    BOOL bVisible = !::IsWindowVisible(ctrlTab.m_hWnd);
    ::ShowWindow( ctrlTab.m_hWnd , bVisible ? SW_SHOWNOACTIVATE : SW_HIDE);
    UISetCheck(ID_VIEW_TABS, bVisible);
    UpdateLayout();
    SettingsManager::getInstance()->set(SettingsManager::SHOW_TABS, bVisible);
    if ( BOOLSETTING(AUTO_HIDE_TABS_BAR)) {
        hideTabsBarTime = bVisible ? GET_TICK() + 60000 : 0;
    }
    return 0;
}

LRESULT MainFrame::OnViewExtraToolBar(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
        static BOOL bVisible = TRUE;    // initially visible
        bVisible = !bVisible;
        CReBarCtrl rebar = m_hWndToolBar;
        int nBandIndex = rebar.IdToIndex(ATL_IDW_BAND_FIRST + 2);       // toolbar is 3nd added band
        rebar.ShowBand(nBandIndex, bVisible);
        UISetCheck(ID_VIEW_EXTRA_TOOLBAR, bVisible);
        UpdateLayout();
        SettingsManager::getInstance()->set(SettingsManager::SHOW_EXTRA_TOOLBAR, bVisible);
        return 0;
}

LRESULT MainFrame::onConnectFavorites(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/){
    FavoriteHubEntry::List& favoriteHubs = FavoriteManager::getInstance()->getFavoriteHubs();
    MainFrame::HubConnectThread* pct = new MainFrame::HubConnectThread();

    for(FavoriteHubEntry::Iter i = favoriteHubs.begin(); i != favoriteHubs.end(); ++i) {	
        pct->addHub(Text::toT((*i)->getServer()));
    }
    pct->start();
    return 0;
}

LRESULT MainFrame::onRemoveAllFavorites(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/){
    FavoriteHubEntry::List& favoriteHubs = FavoriteManager::getInstance()->getFavoriteHubs();

    FavoriteHubEntry::Iter i = favoriteHubs.begin();
    while ( i != favoriteHubs.end() ) {
        FavoriteManager::getInstance()->removeFavorite(*i);
        i = favoriteHubs.begin();
    }
    return 0;
}

LRESULT MainFrame::onAddOpenToFavorites(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/){
    HubFrame::addOpenToFavirtes();
    return 0;
}

LRESULT MainFrame::onFirstTimeRun(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	// display public hubs list
	PublicHubsFrame::openWindow();

    // set a random nick
	SettingsManager::getInstance()->set(SettingsManager::NICK, Util::randomNick());
    SettingsManager::getInstance()->set(SettingsManager::DESCRIPTION, "SababaDC.com");

	// choose connection type
    SettingsManager::getInstance()->set(SettingsManager::INCOMING_CONNECTIONS, SettingsManager::INCOMING_FIREWALL_PASSIVE);
	if(Util::isPrivateIp(Util::getLocalIp())){
		if (( WinUtil::getOsMajor() >= 5 && WinUtil::getOsMinor() >= 1 ) //WinXP & WinSvr2003
			 || WinUtil::getOsMajor() >= 6 )  //Longhorn
		{
			UPnP_TCPConnection = new UPnP( Util::getLocalIp(), "TCP", APPNAME " Download Port (" + Util::toString(ConnectionManager::getInstance()->getPort()) + " TCP)", ConnectionManager::getInstance()->getPort() );
			UPnP_UDPConnection = new UPnP( Util::getLocalIp(), "UDP", APPNAME " Search Port (" + Util::toString(SearchManager::getInstance()->getPort()) + " UDP)", SearchManager::getInstance()->getPort() );
			if ( FAILED(UPnP_UDPConnection->OpenPorts()) || FAILED(UPnP_TCPConnection->OpenPorts()) || UPnP_TCPConnection->GetExternalIP().empty()) 
            {
                delete UPnP_TCPConnection;
                delete UPnP_UDPConnection;
                UPnP_TCPConnection = UPnP_UDPConnection = NULL;
            }else{
				SettingsManager::getInstance()->set(SettingsManager::EXTERNAL_IP, UPnP_TCPConnection->GetExternalIP() );
                SettingsManager::getInstance()->set(SettingsManager::INCOMING_CONNECTIONS,SettingsManager::INCOMING_FIREWALL_UPNP);
			}
        }	
	}

    // display upload setting dialog
    PropertiesDlg dlg(m_hWnd,SettingsManager::getInstance(),true);
	dlg.DoModal(m_hWnd);

	SettingsManager::getInstance()->save();

	return 0;
}

LRESULT MainFrame::onThemeUpdate(WORD /*wNotifyCode*/, WORD /*wID*/, HWND hWndCtl, BOOL& /*bHandled*/){
    SettingsManager::getInstance()->load(Util::getThemeFile(SETTING(THEME_FILE)),(int*)hWndCtl);    
    WinUtil::initColors();
    SettingsManager::getInstance()->save();
    return 0;
}
LRESULT MainFrame::onLocalSite(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/){
    try {
        WebServerManager::getInstance()->Start();
        Sleep(1000);
    } catch(const Exception& e) {
        WinUtil::MessageBox(Text::toT(e.getError()).c_str(), 0, MB_ICONSTOP | MB_OK);
    }
    
    WinUtil::openLink(Text::tformat("http://127.0.0.1:%d/",SETTING(WEBSERVER_PORT)));
    return 0;
}

LRESULT MainFrame::onHomePage(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/){
    WinUtil::openLink(Text::toT(LINK_WEBSITE));
    return 0;
}

void MainFrame::HubConnectThread::addHub(const tstring& hub){
	hubs.push_back(hub);
}
void MainFrame::HubConnectThread::setWaiting(int milisecods){
	this->waiting = milisecods ;
}
int MainFrame::HubConnectThread::run(){
    ::SendMessage(WinUtil::mainWnd,WM_COMMAND, IDC_BROWSER,0 );
	Sleep(waiting);
 	for(std::vector<tstring>::const_iterator i = hubs.begin(); i != hubs.end(); ++i) {
		Sleep(1000);
		::SendMessage(WinUtil::mainWnd,WM_SPEAKER, CONNECT_HUB , (LPARAM)&(*i));
	}
	::SendMessage(WinUtil::mainWnd,WM_COMMAND, IDC_BROWSER,0 );
	delete this;	
	return 0;
}



/**
  * @file
  * $Id: MainFrm.cpp,v 1.90 2005/04/09 15:31:08 arnetheduck Exp $

/**
 * @file
 * $Id: MainFrm.cpp,v 1.20 2004/07/21 13:15:15 bigmuscle Exp
 */
