/*
 * Copyright (C) 2001-2006 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#if !defined(MISC_PAGE_H)
#define MISC_PAGE_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <atlcrack.h>
#include "PropPage.h"
#include <atlctrlx.h>

#include "../client/SimpleXML.h"
#include "ExListViewCtrl.h"

class MiscPage : public CPropertyPage<IDD_MISCPAGE>, public PropPage
{
public:
	MiscPage(SettingsManager *s) : PropPage(s) {
		title = _tcsdup((TSTRING(SETTINGS_ADVANCED) + _T('\\') + TSTRING(SETTINGS_MISC)).c_str());
		SetTitle(title);
		m_psp.dwFlags |= PSP_RTLREADING;
	}
	virtual ~MiscPage() {
		ignoreListCtrl.Detach(); // !BugMaster!
		free(title); }

	BEGIN_MSG_MAP_EX(MiscPage)
		MESSAGE_HANDLER(WM_INITDIALOG, onInitDialog)
		COMMAND_ID_HANDLER(IDC_TIME_AWAY, onFixControls)
		COMMAND_ID_HANDLER(IDC_IGNORE_ADD, onIgnoreAdd)
		COMMAND_ID_HANDLER(IDC_IGNORE_REMOVE, onIgnoreRemove)
		COMMAND_ID_HANDLER(IDC_IGNORE_CLEAR, onIgnoreClear)
	END_MSG_MAP()

	LRESULT onInitDialog(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT onFixControls(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT onIgnoreAdd(WORD /* wNotifyCode */, WORD /*wID*/, HWND /* hWndCtl */, BOOL& /* bHandled */);
	LRESULT onIgnoreRemove(WORD /* wNotifyCode */, WORD /*wID*/, HWND /* hWndCtl */, BOOL& /* bHandled */);
	LRESULT onIgnoreClear(WORD /* wNotifyCode */, WORD /*wID*/, HWND /* hWndCtl */, BOOL& /* bHandled */);

	// Common PropPage interface
	PROPSHEETPAGE *getPSP() { return (PROPSHEETPAGE *)*this; }
	virtual void write();
	
private:
	typedef hash_set<tstring> TStringHash;
	typedef TStringHash::iterator TStringHashIter;
	CComboBox timeCtrlBegin, timeCtrlEnd;
	static Item items[];
	static TextItem texts[];
	void fixControls();
	TCHAR* title;
	TStringHash ignoreList;
	ExListViewCtrl ignoreListCtrl;

	void onIgnoreLoad() {
		try {
			SimpleXML xml;
			xml.fromXML(File(Util::getConfigPath() + "Ignores.xml", File::READ, File::OPEN).read());
			if(xml.findChild("IgnoreList")) {
				xml.stepIn();
				while(xml.findChild("User"))
				{	
					ignoreList.insert(Text::toT(xml.getChildAttrib("Nick")).c_str());
				}
				xml.stepOut();
			}
		} catch(const Exception&) {
			// ...
		}
	}

	void onIgnoreSave() {
	  SimpleXML xml;
	  xml.addTag("IgnoreList");
	  xml.stepIn();

	  for(TStringHash::iterator i = ignoreList.begin(); i != ignoreList.end(); ++i)
	  {
		 xml.addTag("User");
		 xml.addChildAttrib("Nick", (Text::fromT(*i)));
      }
	  xml.stepOut();

	  string fname = Util::getConfigPath() + "Ignores.xml";

	  File f(fname + ".tmp", File::WRITE, File::CREATE | File::TRUNCATE);
	  f.write(SimpleXML::utf8Header);
	  f.write(xml.toXML());
	  f.close();
	  File::deleteFile(fname);
	  File::renameFile(fname + ".tmp", fname);
	}
};

#endif // !defined(MISC_PAGE_H)
