/*
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef USERINFO_H
#define USERINFO_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "TypedListViewCtrl.h"
#include "WinUtil.h"
#include "UserInfoBase.h"

#include "../client/FastAlloc.h"
#include "../client/TaskQueue.h"

        enum Tasks { UPDATE_USER_JOIN, UPDATE_USER, REMOVE_USER, ADD_CHAT_LINE,
                ADD_STATUS_LINE, ADD_SILENT_STATUS_LINE, SET_WINDOW_TITLE, GET_PASSWORD,
                PRIVATE_MESSAGE, STATS, CONNECTED, DISCONNECTED, CHEATING_USER,
                GET_SHUTDOWN, SET_SHUTDOWN, KICK_MSG
        };

        struct UserTask : public Task {
                //UserTask(const User::Ptr& u, Client* client_) : user(u), client(client_) { }
                UserTask(const User::Ptr& u, Client* client_) : user(u), client(client_) { }

                // !SMT!-S
                virtual void beforeExecute() {
                        if (user == ClientManager::getInstance()->getMe())
                                identity = client->getMyIdentity();
                        else
                                ClientManager::getInstance()->getIdentity(user, identity);
                }

                User::Ptr user;
                Identity identity;
                Client* client; // !SMT!-S: client required for distincting self on different hubs
        };

        struct MessageTask : public StringTask {
                MessageTask(const User::Ptr& from_, const User::Ptr& to_, const User::Ptr& replyTo_, const string& m, bool annoying_) : StringTask(m), // !SMT!-S
                        from(from_), to(to_), replyTo(replyTo_), annoying(annoying_) { }

                // !SMT!-S
                virtual void beforeExecute() {
                        ClientManager::getInstance()->getIdentity(from, fromId);
                        ClientManager::getInstance()->getIdentity(replyTo, replyToId);
                }

                User::Ptr from;
                User::Ptr to;
                User::Ptr replyTo;
                bool annoying; // !SMT!-S
                Identity fromId;
                Identity replyToId;
        };

        enum {
                COLUMN_FIRST,
                COLUMN_NICK = COLUMN_FIRST,
                COLUMN_SHARED,
                COLUMN_EXACT_SHARED,
                COLUMN_DESCRIPTION,
                COLUMN_TAG,
                COLUMN_CONNECTION,
                COLUMN_EMAIL,
                COLUMN_VERSION,
                COLUMN_MODE,
                COLUMN_HUBS,
                COLUMN_SLOTS,
                COLUMN_UPLOAD_SPEED,
                COLUMN_IP,
                COLUMN_DNS, // !SMT!-IP
                COLUMN_PK,
                COLUMN_LAST
        };

class UserInfo : public UserInfoBase, public FastAlloc<UserInfo>, public ColumnBase {
public:
        UserInfo(const UserTask& u) : UserInfoBase(u.user) {
                update(u.identity, -1);
        };
        static int compareItems(const UserInfo* a, const UserInfo* b, int col);
        uint8_t imageIndex() const { return WinUtil::getImage(identity); }

        bool update(const Identity& identity, int sortCol);

        string getNick() const { return identity.getNick(); }
        bool isHidden() const { return identity.isHidden(); }

        GETSET(Identity, identity, Identity);

        // !SMT!-S
        typedef HASH_MAP<User::Ptr, UserInfo*, User::HashFunction> UserMapBase;
        typedef UserMapBase::const_iterator UserMapIter;
        class UserMap : public UserMapBase {
                public:
                UserInfo* findUser(const User::Ptr& user) {
                        UserMapIter i = find(user);
                        return (i == end())? NULL : i->second;
                }
        };
};


#endif //USERINFO_H
