/*
 * Copyright (C) 2001-2006 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "Resource.h"

#include "../client/QueueManager.h"
#include "../client/QueueItem.h"

#include "WinUtil.h"
#include "TransferView.h"
#include "MainFrm.h"
#include "UsersFrame.h" // !SMT!-S
#include "PrivateFrame.h" // !SMT!-UI

#include "BarShader.h"

int TransferView::columnIndexes[] = { COLUMN_USER, COLUMN_HUB, COLUMN_STATUS, COLUMN_TIMELEFT,
COLUMN_SPEED, COLUMN_FILE, COLUMN_SIZE, COLUMN_PATH, COLUMN_LOCATION, COLUMN_IP, COLUMN_DNS, // !SMT!-IP
COLUMN_SHARE, COLUMN_SLOTS, // !SMT!-UI
COLUMN_RATIO };
int TransferView::columnSizes[] = { 150, 100, 250, 75, 75, 175, 100, 200, 50, 50, 75, 75, 75, 75 }; // !SMT!-IP

static ResourceManager::Strings columnNames[] = { ResourceManager::USER, ResourceManager::HUB_SEGMENTS, ResourceManager::STATUS,
ResourceManager::TIME_LEFT, ResourceManager::SPEED, ResourceManager::FILENAME, ResourceManager::SIZE, ResourceManager::PATH,
ResourceManager::LOCATION_BARE, ResourceManager::IP_BARE, ResourceManager::DNS_BARE, // !SMT!-IP
ResourceManager::SHARED, ResourceManager::SLOTS, // !SMT!-UI
ResourceManager::RATIO};

TransferView::~TransferView() {
        arrows.Destroy();
        speedImages.Destroy();
        DestroyIcon(user);
}

LRESULT TransferView::onCreate(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/) {
        arrows.CreateFromImage(IDB_ARROWS, 16, 3, CLR_DEFAULT, IMAGE_BITMAP, LR_CREATEDIBSECTION | LR_SHARED);
        user = (HICON)LoadImage((HINSTANCE)::GetWindowLong(::GetParent(m_hWnd), GWL_HINSTANCE), MAKEINTRESOURCE(IDR_TUSER), IMAGE_ICON, 16, 16, LR_DEFAULTSIZE);
        speedImages.CreateFromImage(IDB_TSPEEDS, 16, 5, CLR_DEFAULT, IMAGE_BITMAP, LR_CREATEDIBSECTION | LR_SHARED);
        speedImagesBW.CreateFromImage(IDB_TSPEEDSBW, 16, 5, CLR_DEFAULT, IMAGE_BITMAP, LR_CREATEDIBSECTION | LR_SHARED);

        ctrlTransfers.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN |
                WS_HSCROLL | WS_VSCROLL | LVS_REPORT | LVS_SHOWSELALWAYS | LVS_SHAREIMAGELISTS, WS_EX_CLIENTEDGE, IDC_TRANSFERS);
        ctrlTransfers.SetExtendedListViewStyle(LVS_EX_LABELTIP | LVS_EX_HEADERDRAGDROP | LVS_EX_FULLROWSELECT | 0x00010000 | LVS_EX_INFOTIP);

        WinUtil::splitTokens(columnIndexes, SETTING(MAINFRAME_ORDER), COLUMN_LAST);
        WinUtil::splitTokens(columnSizes, SETTING(MAINFRAME_WIDTHS), COLUMN_LAST);

        for(int j=0; j<COLUMN_LAST; ++j) {
                int fmt = (j == COLUMN_SIZE || j == COLUMN_TIMELEFT || j == COLUMN_SPEED) ? LVCFMT_RIGHT : LVCFMT_LEFT;
                ctrlTransfers.InsertColumn(j, CTSTRING_I(columnNames[j]), fmt, columnSizes[j], j);
        }

        ctrlTransfers.setColumnOrderArray(COLUMN_LAST, columnIndexes);
        ctrlTransfers.setVisible(SETTING(MAINFRAME_VISIBLE));

        ctrlTransfers.SetBkColor(WinUtil::bgColor);
        ctrlTransfers.SetTextBkColor(WinUtil::bgColor);
        ctrlTransfers.SetTextColor(WinUtil::textColor);
        ctrlTransfers.setFlickerFree(WinUtil::bgBrush);

        ctrlTransfers.SetImageList(arrows, LVSIL_SMALL);
        ctrlTransfers.setSortColumn(COLUMN_USER);

        transferMenu.CreatePopupMenu();
        appendUserItems(transferMenu);
        //transferMenu.AppendMenu(MF_POPUP, (UINT)(HMENU)WinUtil::speedMenu, CTSTRING(UPLOAD_SPEED_LIMIT)); // !SMT!-S
        transferMenu.AppendMenu(MF_SEPARATOR);
        transferMenu.AppendMenu(MF_STRING, IDC_FORCE, CTSTRING(FORCE_ATTEMPT));
        transferMenu.AppendMenu(MF_SEPARATOR);
        transferMenu.AppendMenu(MF_STRING, IDC_SEARCH_ALTERNATES, CTSTRING(SEARCH_FOR_ALTERNATES));

        // !SMT!-UI
        copyMenu.CreatePopupMenu();
        copyMenu.InsertSeparatorFirst(TSTRING(COPY));
        for(int i = 0; i < COLUMN_LAST; ++i)
                copyMenu.AppendMenu(MF_STRING, IDC_COPY + i, CTSTRING_I(columnNames[i]));
        copyMenu.AppendMenu(MF_SEPARATOR);
        copyMenu.AppendMenu(MF_STRING, IDC_COPY_LINK, CTSTRING(COPY_MAGNET_LINK));
        copyMenu.AppendMenu(MF_STRING, IDC_COPY_WMLINK, CTSTRING(COPY_MLINK_TEMPL));
        copyMenu.AppendMenu(MF_STRING, IDC_COPY_HTML_LINK, CTSTRING(COPY_HTML_LINK));

        previewMenu.CreatePopupMenu();
        previewMenu.AppendMenu(MF_SEPARATOR);
        usercmdsMenu.CreatePopupMenu();

        transferMenu.AppendMenu(MF_POPUP, (UINT)(HMENU)previewMenu, CTSTRING(PREVIEW_MENU));
        transferMenu.AppendMenu(MF_POPUP, (UINT)(HMENU)usercmdsMenu, CTSTRING(SETTINGS_USER_COMMANDS));
        transferMenu.AppendMenu(MF_POPUP, (UINT_PTR)(HMENU)copyMenu, CTSTRING(COPY)); // !SMT!-UI
        transferMenu.AppendMenu(MF_SEPARATOR);
        transferMenu.AppendMenu(MF_STRING, IDC_MENU_SLOWDISCONNECT, CTSTRING(SETCZDC_DISCONNECTING_ENABLE));
        transferMenu.AppendMenu(MF_SEPARATOR);
        transferMenu.AppendMenu(MF_STRING, IDC_REMOVE, CTSTRING(CLOSE_CONNECTION));
        transferMenu.SetMenuDefaultItem(IDC_PRIVATEMESSAGE);

        segmentedMenu.CreatePopupMenu();
        segmentedMenu.AppendMenu(MF_STRING, IDC_SEARCH_ALTERNATES, CTSTRING(SEARCH_FOR_ALTERNATES));
        segmentedMenu.AppendMenu(MF_POPUP, (UINT)(HMENU)previewMenu, CTSTRING(PREVIEW_MENU));
        segmentedMenu.AppendMenu(MF_STRING, IDC_MENU_SLOWDISCONNECT, CTSTRING(SETCZDC_DISCONNECTING_ENABLE));
        segmentedMenu.AppendMenu(MF_POPUP, (UINT_PTR)(HMENU)copyMenu, CTSTRING(COPY)); // !SMT!-UI
        segmentedMenu.AppendMenu(MF_STRING, IDC_ASK_SLOT, CTSTRING(ASK_SLOT)); // !SMT!-UI
        segmentedMenu.AppendMenu(MF_SEPARATOR);
        segmentedMenu.AppendMenu(MF_STRING, IDC_CONNECT_ALL, CTSTRING(CONNECT_ALL));
        segmentedMenu.AppendMenu(MF_STRING, IDC_DISCONNECT_ALL, CTSTRING(DISCONNECT_ALL));
        segmentedMenu.AppendMenu(MF_SEPARATOR);
        segmentedMenu.AppendMenu(MF_STRING, IDC_EXPAND_ALL, CTSTRING(EXPAND_ALL));
        segmentedMenu.AppendMenu(MF_STRING, IDC_COLLAPSE_ALL, CTSTRING(COLLAPSE_ALL));
        segmentedMenu.AppendMenu(MF_SEPARATOR);
        segmentedMenu.AppendMenu(MF_STRING, IDC_REMOVEALL, CTSTRING(REMOVE_ALL));

        ConnectionManager::getInstance()->addListener(this);
        DownloadManager::getInstance()->addListener(this);
        UploadManager::getInstance()->addListener(this);
        SettingsManager::getInstance()->addListener(this);

        return 0;
}

void TransferView::prepareClose() {
        ctrlTransfers.saveHeaderOrder(SettingsManager::MAINFRAME_ORDER, SettingsManager::MAINFRAME_WIDTHS,
                SettingsManager::MAINFRAME_VISIBLE);

        ConnectionManager::getInstance()->removeListener(this);
        DownloadManager::getInstance()->removeListener(this);
        UploadManager::getInstance()->removeListener(this);
        SettingsManager::getInstance()->removeListener(this);
        WinUtil::UnlinkStaticMenus(transferMenu); // !SMT!-UI
}

LRESULT TransferView::onSize(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/) {
        RECT rc;
        GetClientRect(&rc);
        ctrlTransfers.MoveWindow(&rc);

        return 0;
}

LRESULT TransferView::onContextMenu(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
        if (reinterpret_cast<HWND>(wParam) == ctrlTransfers && ctrlTransfers.GetSelectedCount() > 0) {
                POINT pt = { GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam) };

                if(pt.x == -1 && pt.y == -1) {
                        WinUtil::getContextMenuPos(ctrlTransfers, pt);
                }

                if(ctrlTransfers.GetSelectedCount() > 0) {
                        int i = -1;
                        ItemInfo* itemI;
                        bool bCustomMenu = false;
                        ItemInfo* ii = ctrlTransfers.getItemData(ctrlTransfers.GetNextItem(-1, LVNI_SELECTED));
                        bool main = ii->subItems.size() > 1;

                        updateSummary(); // !SMT!-UI

                        if(!main && (i = ctrlTransfers.GetNextItem(i, LVNI_SELECTED)) != -1) {
                                itemI = ctrlTransfers.getItemData(i);
                                bCustomMenu = true;

                                usercmdsMenu.InsertSeparatorFirst(TSTRING(SETTINGS_USER_COMMANDS));

                                // !SMT!-S
                                selectedUser = itemI->user;
                                if(selectedUser != (User::Ptr)NULL) {
                                        prepareMenu(usercmdsMenu, UserCommand::CONTEXT_CHAT, ClientManager::getInstance()->getHubs(selectedUser->getCID()));
                                        // !SMT!-S
                                        for (int j = 0; j < WinUtil::speedMenu.GetMenuItemCount(); j++)
                                           WinUtil::speedMenu.CheckMenuItem(j, MF_BYPOSITION | MF_UNCHECKED);
                                        FavoriteManager::FavoriteMap favUsers = FavoriteManager::getInstance()->getFavoriteUsers();
                                        FavoriteManager::FavoriteMap::const_iterator ui = favUsers.find(selectedUser->getCID());
                                        if (ui != favUsers.end()) {
                                           const FavoriteUser & u = ui->second;
                                           int id = IDC_SPEED_NONE;
                                           switch (u.getUploadLimit()) {
                                              case FavoriteUser::UL_SU:  id = IDC_SPEED_SUPER; break;
                                              case FavoriteUser::UL_BAN: id = IDC_SPEED_BAN; break;
                                              case FavoriteUser::UL_2:   id = IDC_SPEED_02K; break;
                                              case FavoriteUser::UL_5:   id = IDC_SPEED_05K; break;
                                              case FavoriteUser::UL_8:   id = IDC_SPEED_08K; break;
                                              case FavoriteUser::UL_12:  id = IDC_SPEED_12K; break;
                                              case FavoriteUser::UL_16:  id = IDC_SPEED_16K; break;
                                              case FavoriteUser::UL_24:  id = IDC_SPEED_24K; break;
                                              case FavoriteUser::UL_32:  id = IDC_SPEED_32K; break;
                                              case FavoriteUser::UL_64:  id = IDC_SPEED_64K; break;
                                              case FavoriteUser::UL_128: id = IDC_SPEED_128K; break;
                                              case FavoriteUser::UL_256: id = IDC_SPEED_256K; break;
                                           }
                                           WinUtil::speedMenu.CheckMenuItem(id, MF_BYCOMMAND | MF_CHECKED);
                                        }
                                }
                                // end !SMT!-S
                        }
                        WinUtil::ClearPreviewMenu(previewMenu);

                        segmentedMenu.CheckMenuItem(IDC_MENU_SLOWDISCONNECT, MF_BYCOMMAND | MF_UNCHECKED);
                        transferMenu.CheckMenuItem(IDC_MENU_SLOWDISCONNECT, MF_BYCOMMAND | MF_UNCHECKED);
                        transferMenu.EnableMenuItem(IDC_ASK_SLOT, MFS_DISABLED); // !SMT!-UI

                        if(ii->download) {
                                transferMenu.EnableMenuItem(IDC_MENU_SLOWDISCONNECT, MFS_ENABLED);
                                transferMenu.EnableMenuItem(IDC_ASK_SLOT, MFS_ENABLED); // !SMT!-UI
                                if(!ii->Target.empty()) {
                                        string target = Text::fromT(ii->Target);
                                        string ext = Util::getFileExt(target);
                                        if(ext.size()>1) ext = ext.substr(1);
                                        PreviewAppsSize = WinUtil::SetupPreviewMenu(previewMenu, ext);

                                        QueueItem::StringMap& queue = QueueManager::getInstance()->lockQueue();

                                        QueueItem::StringIter qi = queue.find(&target);

                                        bool slowDisconnect = false;
                                        if(qi != queue.end())
                                                slowDisconnect = qi->second->isSet(QueueItem::FLAG_AUTODROP);

                                        QueueManager::getInstance()->unlockQueue();

                                        if(slowDisconnect) {
                                                segmentedMenu.CheckMenuItem(IDC_MENU_SLOWDISCONNECT, MF_BYCOMMAND | MF_CHECKED);
                                                transferMenu.CheckMenuItem(IDC_MENU_SLOWDISCONNECT, MF_BYCOMMAND | MF_CHECKED);
                                        }
                                }
                        } else {
                                transferMenu.EnableMenuItem(IDC_MENU_SLOWDISCONNECT, MFS_DISABLED);
                        }

                        if(previewMenu.GetMenuItemCount() > 0) {
                                segmentedMenu.EnableMenuItem((UINT)(HMENU)previewMenu, MFS_ENABLED);
                                transferMenu.EnableMenuItem((UINT)(HMENU)previewMenu, MFS_ENABLED);
                        } else {
                                segmentedMenu.EnableMenuItem((UINT)(HMENU)previewMenu, MFS_DISABLED);
                                transferMenu.EnableMenuItem((UINT)(HMENU)previewMenu, MFS_DISABLED);
                        }

                        previewMenu.InsertSeparatorFirst(TSTRING(PREVIEW_MENU));

                        if(!main) {
                                checkAdcItems(transferMenu);
                                transferMenu.InsertSeparatorFirst(TSTRING(MENU_TRANSFERS));
                                transferMenu.TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, pt.x, pt.y, m_hWnd);
                                transferMenu.RemoveFirstItem();
                        } else {
                                segmentedMenu.InsertSeparatorFirst(TSTRING(SETTINGS_SEGMENT));
                                segmentedMenu.TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, pt.x, pt.y, m_hWnd);
                                segmentedMenu.RemoveFirstItem();
                        }

                        if ( bCustomMenu ) {
                                WinUtil::ClearPreviewMenu(usercmdsMenu);
                        }
                        return TRUE;
                }
        }
        bHandled = FALSE;
        return FALSE;
}

// !SMT!-UI
static bool _vector_find(vector<uint64_t> v, uint64_t value) {
    for (vector<uint64_t>::const_iterator i = v.begin(); i != v.end(); ++i)
        if ((*i) == value)
            return true;
    return false;
}
// !SMT!-UI
LRESULT TransferView::onAskSlot(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
        ItemInfo* ii = ctrlTransfers.getItemData(ctrlTransfers.GetNextItem(-1, LVNI_SELECTED));
        vector<uint64_t> sentShare;
        StringMap params;
        params["filename"] = Text::fromT(Util::getFileName(ii->Target));
        for(ItemInfo::Iter i = ii->subItems.begin(); i != ii->subItems.end(); i++) {

                const User::Ptr& u = (*i)->getUser();

                SimpleIdentity id;
                if (!ClientManager::getInstance()->getIdentity(u, id)) continue;

                if (_vector_find(sentShare, id.share)) // don't send to same users (on other hubs)
                        continue;

                params["nick"] = u->getFirstNick();
                PrivateFrame::openWindow(u, Text::toT(Util::formatParams(SETTING(SLOT_ASK), params, false)));
                sentShare.push_back(id.share);
        }
        return 0;
}

// !SMT!-UI. todo: move same code to template CopyBaseHandler
LRESULT TransferView::onCopy(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
        tstring data;
        int i = -1, columnId = wID-IDC_COPY; // !SMT!-UI: copy several rows
        while( (i = ctrlTransfers.GetNextItem(i, LVNI_SELECTED)) != -1) {
                ItemInfo* ii = ctrlTransfers.getItemData(i);
                tstring sdata;

                TTHValue tth;
                QueueManager::getInstance()->getTTH(Text::fromT(ii->Target), tth);

                if (wID == IDC_COPY_LINK)
                        sdata = Text::toT(WinUtil::getMagnet(tth, Util::getFileName(ii->Target), ii->size));
                else if (wID == IDC_COPY_WMLINK)
                        sdata = Text::toT(WinUtil::getWebMagnet(tth, Util::getFileName(ii->Target), ii->size));
                else if (wID == IDC_COPY_HTML_LINK)
                        sdata = Text::toT(WinUtil::getHtmlMagnet(tth, Util::getFileName(ii->Target), ii->size));
                else
                        sdata = ii->getText(columnId);

                if (data.empty())
                        data = sdata;
                else
                        data = data + L"\r\n" + sdata;
        }
        WinUtil::setClipboard(data);
        return 0;
}

void TransferView::runUserCommand(UserCommand& uc) {
        if(!WinUtil::getUCParams(m_hWnd, uc, ucLineParams))
                return;

        StringMap ucParams = ucLineParams;

        int i = -1;
        while((i = ctrlTransfers.GetNextItem(i, LVNI_SELECTED)) != -1) {
                ItemInfo* itemI = ctrlTransfers.getItemData(i);
                if(!itemI->user->isOnline())
                        continue;

                StringMap tmp = ucParams;
                ucParams["fileFN"] = Text::fromT(itemI->Target);

                // compatibility with 0.674 and earlier
                ucParams["file"] = ucParams["fileFN"];

                ClientManager::getInstance()->userCommand(itemI->user, uc, tmp, true);
        }
}

LRESULT TransferView::onForce(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
        int i = -1;
        while( (i = ctrlTransfers.GetNextItem(i, LVNI_SELECTED)) != -1) {
                ItemInfo *ii = ctrlTransfers.getItemData(i);
                ii->columns[COLUMN_STATUS] = CTSTRING(CONNECTING_FORCED);
                ctrlTransfers.updateItem( ii );
                ClientManager::getInstance()->connect(ii->user);
        }
        return 0;
}

void TransferView::ItemInfo::removeAll() {
        if(subItems.size() <= 1) {
                QueueManager::getInstance()->removeSource(user, QueueItem::Source::FLAG_REMOVED);
        } else {
                if(!BOOLSETTING(CONFIRM_DELETE) || WinUtil::MessageBox( CTSTRING(MSG_REALLY_REMOVE), 0, MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON2) == IDYES)
                        QueueManager::getInstance()->remove(Text::fromT(Target));
        }
}

LRESULT TransferView::onCustomDraw(int /*idCtrl*/, LPNMHDR pnmh, BOOL& bHandled) {
        CRect rc;
        NMLVCUSTOMDRAW* cd = (NMLVCUSTOMDRAW*)pnmh;

        switch(cd->nmcd.dwDrawStage) {
        case CDDS_PREPAINT:
                return CDRF_NOTIFYITEMDRAW;

        case CDDS_ITEMPREPAINT:
                return CDRF_NOTIFYSUBITEMDRAW;

        case CDDS_SUBITEM | CDDS_ITEMPREPAINT:{
                ItemInfo* ii = reinterpret_cast<ItemInfo*>(cd->nmcd.lItemlParam);

                int colIndex = ctrlTransfers.findColumn(cd->iSubItem);
                cd->clrTextBk = WinUtil::bgColor;

                if((ii->status == ItemInfo::STATUS_RUNNING) && (colIndex == COLUMN_STATUS) ) {
                        if(!BOOLSETTING(SHOW_PROGRESS_BARS)) {
                                bHandled = FALSE;
                                return 0;
                        }

                        // Get the color of this bar
                        COLORREF clr = SETTING(PROGRESS_OVERRIDE_COLORS) ?
                                (ii->download ? (!ii->main ? SETTING(DOWNLOAD_BAR_COLOR) : SETTING(PROGRESS_SEGMENT_COLOR)) : SETTING(UPLOAD_BAR_COLOR)) :
                                GetSysColor(COLOR_HIGHLIGHT);

                        //this is just severely broken, msdn says GetSubItemRect requires a one based index
                        //but it wont work and index 0 gives the rect of the whole item
                        if(cd->iSubItem == 0) {
                                //use LVIR_LABEL to exclude the icon area since we will be painting over that
                                //later
                                ctrlTransfers.GetItemRect((int)cd->nmcd.dwItemSpec, &rc, LVIR_LABEL);
                        } else {
                                ctrlTransfers.GetSubItemRect((int)cd->nmcd.dwItemSpec, cd->iSubItem, LVIR_BOUNDS, &rc);
                        }

                        /* Thanks & credits for Stealthy style go to phaedrus */
                        bool useStealthyStyle = BOOLSETTING(STEALTHY_STYLE);
                        //bool isMain = (!ii->main || !ii->download);
                        //bool isSmaller = (!ii->main && ii->collapsed == false);

                        // fixes issues with double border
                        if(useStealthyStyle) {
                                rc.top -= 1;
                                //if(isSmaller)
                                        //rc.bottom -= 1;
                        }

                        // Real rc, the original one.
                        CRect real_rc = rc;
                        // We need to offset the current rc to (0, 0) to paint on the New dc
                        rc.MoveToXY(0, 0);

                        CRect rc4;
                        CRect rc2;
                        rc2 = rc;

                        // Text rect
                        if(useStealthyStyle){
                                rc2.left += 23; // indented for icon and text
                                rc2.right -= 2; // and without messing with the border of the cell
                                rc2.top -= 1; // and fix text vertical alignment
                                // Background rect
                                rc4 = rc;
                        }else{
                                rc2 = rc;
                                rc2.left += 6; // indented with 6 pixels
                                rc2.right -= 2; // and without messing with the border of the cell
                                // Background rect
                                rc4 = rc;
                                rc2.left += 9;
                        }

                        CDC cdc;
                        cdc.CreateCompatibleDC(cd->nmcd.hdc);
                        HBITMAP hBmp = CreateCompatibleBitmap(cd->nmcd.hdc,  real_rc.Width(),  real_rc.Height());

                        HBITMAP pOldBmp = cdc.SelectBitmap(hBmp);
                        HDC& dc = cdc.m_hDC;

                        COLORREF barPal[3] = { HLS_TRANSFORM(clr, -40, 50), clr, HLS_TRANSFORM(clr, 20, -30) };
                        COLORREF barPal2[3] = { HLS_TRANSFORM(clr, -15, 0), clr, HLS_TRANSFORM(clr, 15, 0) };
                        COLORREF oldcol;
                        // The value throws off, usually with about 8-11 (usually negatively f.ex. in src use 190, the change might actually happen already at aprox 180)
                        HLSCOLOR hls = RGB2HLS(clr);
                        LONG top = rc2.top + (rc2.Height() - WinUtil::getTextHeight(cd->nmcd.hdc) - 1)/2 + 1;

                        HFONT oldFont = (HFONT)SelectObject(dc, WinUtil::font);
                        SetBkMode(dc, TRANSPARENT);

                        // Get the color of this text bar - this way it ends up looking nice imo.
                        if(!useStealthyStyle) {
                        oldcol = ::SetTextColor(dc, SETTING(PROGRESS_OVERRIDE_COLORS2) ?
                                (ii->download ? SETTING(PROGRESS_TEXT_COLOR_DOWN) : SETTING(PROGRESS_TEXT_COLOR_UP)) :
                                OperaColors::TextFromBackground(clr));
                        } else {
                                if(clr == RGB(255, 255, 255)) // see if user is using white as clr, rare but you may never know
                                        oldcol = ::SetTextColor(dc, RGB(0, 0, 0));
                                else
                                        oldcol = ::SetTextColor(dc, barPal2[1]);
                        }


                        // !BUGMASTER!
                        if(ii->size == 0)
                                ii->size = 1;

                        if(BOOLSETTING(PROGRESSBAR_ODC_STYLE) || useStealthyStyle) {
                                // New style progressbar tweaks the current colors
                                HLSTRIPLE hls_bk = OperaColors::RGB2HLS(cd->clrTextBk);

                                // Create pen (ie outline border of the cell)
                                HPEN penBorder = ::CreatePen(PS_SOLID, 1, OperaColors::blendColors(cd->clrTextBk, clr, (hls_bk.hlstLightness > 0.75) ? 0.6 : 0.4));
                                HGDIOBJ pOldPen = ::SelectObject(dc, penBorder);

                                // Draw the outline (but NOT the background) using pen
                                HBRUSH hBrOldBg = CreateSolidBrush(cd->clrTextBk);
                                hBrOldBg = (HBRUSH)::SelectObject(dc, hBrOldBg);

                                if(useStealthyStyle)
                                        ::Rectangle(dc, rc4.left, rc4.top, rc4.right, rc4.bottom);
                                else
                                        ::Rectangle(dc, rc.left, rc.top, rc.right, rc.bottom);

                                DeleteObject(::SelectObject(dc, hBrOldBg));

                                // Set the background color, by slightly changing it
                                HBRUSH hBrDefBg = CreateSolidBrush(OperaColors::blendColors(cd->clrTextBk, clr, (hls_bk.hlstLightness > 0.75) ? 0.85 : 0.70));
                                HGDIOBJ oldBg = ::SelectObject(dc, hBrDefBg);

                                // Draw the outline AND the background using pen+brush
                                if(useStealthyStyle)
                                        ::Rectangle(dc, rc4.left, rc4.top, rc4.left + (LONG)(rc4.Width() * ii->getRatio() + 0.5), rc4.bottom);
                                else
                                        ::Rectangle(dc, rc.left, rc.top, rc.left + (LONG)(rc.Width() * ii->getRatio() + 0.5), rc.bottom);

                                if(useStealthyStyle) {
                                        // Draw the text over entire item
                                        ::ExtTextOut(dc, rc2.left, top, ETO_CLIPPED, rc2, ii->getText(COLUMN_STATUS).c_str(), ii->getText(COLUMN_STATUS).length(), NULL);

                                        rc.right = rc.left + (int) (((int64_t)rc.Width()) * ii->actual / ii->size);

                                        if(ii->pos != 0)
                                                rc.bottom -= 1;

                                        rc.top += 1;

                                        //create bar pen
                                        if(HLS_S(hls) <= 30) // good values would be 20-30
                                                penBorder = ::CreatePen(PS_SOLID, 1, barPal2[0]);
                                        else
                                                penBorder = ::CreatePen(PS_SOLID, 1, barPal[0]);

                                        DeleteObject(::SelectObject(dc, penBorder));

                                        //create bar brush
                                        hBrDefBg = CreateSolidBrush(barPal[1]);

                                        DeleteObject(::SelectObject(dc, hBrDefBg));

                                        //draw bar
                                        ::Rectangle(dc, rc.left, rc.top, rc.right, rc.bottom);

                                        //draw bar highlight
                                        if(rc.Width() > 4){
                                                DeleteObject(SelectObject(cdc, CreatePen(PS_SOLID,1,barPal[2])));
                                                rc.top += 2;
                                                ::MoveToEx(cdc,rc.left+2,rc.top,(LPPOINT)NULL);
                                                ::LineTo(cdc,rc.right-2,rc.top);
                                        }
                                }
                                // Reset pen
                                DeleteObject(::SelectObject(dc, pOldPen));
                                // Reset bg (brush)
                                DeleteObject(::SelectObject(dc, oldBg));
                        }

                        // Draw the background and border of the bar
                        if(!BOOLSETTING(PROGRESSBAR_ODC_STYLE) && !useStealthyStyle) {
                                CBarShader statusBar(rc.bottom - rc.top, rc.right - rc.left, SETTING(PROGRESS_BACK_COLOR), ii->size);

                                rc.right = rc.left + (int) (rc.Width() * ii->pos / ii->size);
                                if(!ii->download) {
                                        statusBar.FillRange(0, ii->start, HLS_TRANSFORM(clr, -20, 30));
                                        statusBar.FillRange(ii->start, ii->actual,  clr);
                                } else {
                                        statusBar.FillRange(0, ii->actual, clr);
                                        if(ii->main)
                                                statusBar.FillRange(ii->start, ii->actual, SETTING(PROGRESS_SEGMENT_COLOR));
                                }
                                if(ii->pos > ii->actual)
                                        statusBar.FillRange(ii->actual, ii->pos, SETTING(PROGRESS_COMPRESS_COLOR));

                                statusBar.Draw(cdc, rc.top, rc.left, SETTING(PROGRESS_3DDEPTH));
                        } else {
                                if(!useStealthyStyle) {
                                        int right = rc.left + (int) ((int64_t)rc.Width() * ii->actual / ii->size);

                                        COLORREF a, b;
                                        OperaColors::EnlightenFlood(clr, a, b);
                                        OperaColors::FloodFill(cdc, rc.left+1, rc.top+1, right, rc.bottom-1, a, b, BOOLSETTING(PROGRESSBAR_ODC_BUMPED));
                                }
                        }

                        if(useStealthyStyle) {
                                // Draw icon - Nasty way to do the filelist icon, but couldn't get other ways to work well, TODO: do separating filelists from other transfers the proper way...
                                if(ii->getText(COLUMN_PATH).find(Text::toT(Util::getListPath())) != string::npos || ii->getText(COLUMN_PATH).find(Text::toT(Util::getConfigPath())) != string::npos) {
                                        DrawIconEx(dc, rc2.left - 20, rc2.top + 2, user, 16, 16, NULL, NULL, DI_NORMAL | DI_COMPAT);
                                } else if(ii->status == ItemInfo::STATUS_RUNNING) {
                                        RECT rc9 = rc2;
                                        rc9.left -= 20;
                                        rc9.top += 4;
                                        rc9.right = rc9.left + 16;
                                        rc9.bottom = rc9.top + 12;

                                        int64_t speedkb = ii->speed / 1000;
                                        int64_t speedmark;
                                        if(!BOOLSETTING(THROTTLE_ENABLE)) {
                                                if(!ii->download) {
                                                        speedmark = SETTING(TOP_UP_SPEED) / 5;
                                                } else {
                                                        speedmark = SETTING(TOP_SPEED) / 5;
                                                }
                                        } else {
                                                if(!ii->download) {
                                                        speedmark = SETTING(MAX_UPLOAD_SPEED_LIMIT_NORMAL) / 5;
                                                } else {
                                                        speedmark = SETTING(MAX_DOWNLOAD_SPEED_LIMIT_NORMAL) / 5;
                                                }
                                        }

                                        if((HLS_S(hls) > 30) || (HLS_L(hls) < 70)) {
                                                if(speedkb >= speedmark * 5)
                                                        speedImages.DrawEx(4, dc, rc9, CLR_DEFAULT, CLR_DEFAULT, ILD_IMAGE);
                                                else if(speedkb >= speedmark * 4)
                                                        speedImages.DrawEx(3, dc, rc9, CLR_DEFAULT, CLR_DEFAULT, ILD_IMAGE);
                                                else if(speedkb >= speedmark * 3)
                                                        speedImages.DrawEx(2, dc, rc9, CLR_DEFAULT, CLR_DEFAULT, ILD_IMAGE);
                                                else if(speedkb >= speedmark * 2)
                                                        speedImages.DrawEx(1, dc, rc9, CLR_DEFAULT, CLR_DEFAULT, ILD_IMAGE);
                                                else
                                                        speedImages.DrawEx(0, dc, rc9, CLR_DEFAULT, CLR_DEFAULT, ILD_IMAGE);
                                        } else { // color can be assumed to be a shade of grey, use greyscale speedImages
                                                if(speedkb >= speedmark * 5)
                                                        speedImagesBW.DrawEx(4, dc, rc9, CLR_DEFAULT, CLR_DEFAULT, ILD_IMAGE);
                                                else if(speedkb >= speedmark * 4)
                                                        speedImagesBW.DrawEx(3, dc, rc9, CLR_DEFAULT, CLR_DEFAULT, ILD_IMAGE);
                                                else if(speedkb >= speedmark * 3)
                                                        speedImagesBW.DrawEx(2, dc, rc9, CLR_DEFAULT, CLR_DEFAULT, ILD_IMAGE);
                                                else if(speedkb >= speedmark * 2)
                                                        speedImagesBW.DrawEx(1, dc, rc9, CLR_DEFAULT, CLR_DEFAULT, ILD_IMAGE);
                                                else
                                                        speedImagesBW.DrawEx(0, dc, rc9, CLR_DEFAULT, CLR_DEFAULT, ILD_IMAGE);
                                        }
                                }
                                // use white to as many colors as possible (values might need some tweaking), I didn't like TextFromBackground...
                                if(((HLS_L(hls) > 190) && (HLS_S(hls) <= 30)) || (HLS_L(hls) > 211))
                                        oldcol = ::SetTextColor(dc, HLS_TRANSFORM(clr, -40, 0));
                                else
                                        oldcol = ::SetTextColor(dc, RGB(255, 255, 255));
                                rc2.right = rc.right;
                        }

                        // Draw the text, the other stuff here was moved upwards due to stealthy style being added
                        ::ExtTextOut(dc, rc2.left, top, ETO_CLIPPED, rc2, ii->getText(COLUMN_STATUS).c_str(), ii->getText(COLUMN_STATUS).length(), NULL);

                        SelectObject(dc, oldFont);
                        ::SetTextColor(dc, oldcol);

                        // New way:
                        BitBlt(cd->nmcd.hdc, real_rc.left, real_rc.top, real_rc.Width(), real_rc.Height(), dc, 0, 0, SRCCOPY);
                        DeleteObject(cdc.SelectBitmap(pOldBmp));

                        //bah crap, if we return CDRF_SKIPDEFAULT windows won't paint the icons
                        //so we have to do it
                        if(cd->iSubItem == 0){
                                LVITEM lvItem;
                                lvItem.iItem = cd->nmcd.dwItemSpec;
                                lvItem.iSubItem = 0;
                                lvItem.mask = LVIF_IMAGE | LVIF_STATE;
                                lvItem.stateMask = LVIS_SELECTED;
                                ctrlTransfers.GetItem(&lvItem);

                                HIMAGELIST imageList = (HIMAGELIST)::SendMessage(ctrlTransfers.m_hWnd, LVM_GETIMAGELIST, LVSIL_SMALL, 0);
                                if(imageList) {
                                        //let's find out where to paint it
                                        //and draw the background to avoid having
                                        //the selection color as background
                                        CRect iconRect;
                                        ctrlTransfers.GetSubItemRect((int)cd->nmcd.dwItemSpec, 0, LVIR_ICON, iconRect);
                                        ImageList_Draw(imageList, lvItem.iImage, cd->nmcd.hdc, iconRect.left, iconRect.top, ILD_TRANSPARENT);
                                }
                        }
                        return CDRF_SKIPDEFAULT;
                } else if(BOOLSETTING(GET_USER_COUNTRY) && (colIndex == COLUMN_LOCATION)) { // !SMT!-IP
                        ItemInfo* ii = (ItemInfo*)cd->nmcd.lItemlParam;
                        ctrlTransfers.GetSubItemRect((int)cd->nmcd.dwItemSpec, cd->iSubItem, LVIR_BOUNDS, rc);
                        COLORREF color;
                        if(ctrlTransfers.GetItemState((int)cd->nmcd.dwItemSpec, LVIS_SELECTED) & LVIS_SELECTED) {
                                if(ctrlTransfers.m_hWnd == ::GetFocus()) {
                                        color = GetSysColor(COLOR_HIGHLIGHT);
                                        SetBkColor(cd->nmcd.hdc, GetSysColor(COLOR_HIGHLIGHT));
                                        SetTextColor(cd->nmcd.hdc, GetSysColor(COLOR_HIGHLIGHTTEXT));
                                } else {
                                        color = GetBkColor(cd->nmcd.hdc);
                                        SetBkColor(cd->nmcd.hdc, color);
                                }
                        } else {
                                color = WinUtil::bgColor;
                                SetBkColor(cd->nmcd.hdc, WinUtil::bgColor);
                                SetTextColor(cd->nmcd.hdc, WinUtil::textColor);
                        }
                        CRect rc2 = rc;
                        rc2.left += 2;
                        HGDIOBJ oldpen = ::SelectObject(cd->nmcd.hdc, CreatePen(PS_SOLID,0, color));
                        HGDIOBJ oldbr = ::SelectObject(cd->nmcd.hdc, CreateSolidBrush(color));
                        Rectangle(cd->nmcd.hdc,rc.left, rc.top, rc.right, rc.bottom);

                        DeleteObject(::SelectObject(cd->nmcd.hdc, oldpen));
                        DeleteObject(::SelectObject(cd->nmcd.hdc, oldbr));

                        TCHAR buf[256];
                        ctrlTransfers.GetItemText((int)cd->nmcd.dwItemSpec, cd->iSubItem, buf, 255);
                        buf[255] = 0;
                        if(_tcslen(buf) > 0) {
                                LONG top = rc2.top + (rc2.Height() - 15)/2;
                                if((top - rc2.top) < 2)
                                        top = rc2.top + 1;

                                POINT p = { rc2.left, top };
                                WinUtil::flagImages.Draw(cd->nmcd.hdc, ii->flagImage, p, LVSIL_SMALL);
                                top = rc2.top + (rc2.Height() - WinUtil::getTextHeight(cd->nmcd.hdc) - 1)/2;
                                ::ExtTextOut(cd->nmcd.hdc, rc2.left + 30, top + 1, ETO_CLIPPED, rc2, buf, _tcslen(buf), NULL);
                                return CDRF_SKIPDEFAULT;
                        }
                } else if((colIndex != COLUMN_USER) && (colIndex != COLUMN_HUB) && (colIndex != COLUMN_STATUS) && (colIndex != COLUMN_LOCATION) && // !SMT!-IP
                        (ii->status != ItemInfo::STATUS_RUNNING)) {
                        cd->clrText = OperaColors::blendColors(WinUtil::bgColor, WinUtil::textColor, 0.4);
                        return CDRF_NEWFONT;
                }
                // Fall through
        }
        default:
                return CDRF_DODEFAULT;
        }
}

LRESULT TransferView::onDoubleClickTransfers(int /*idCtrl*/, LPNMHDR pnmh, BOOL& /*bHandled*/) {
    NMITEMACTIVATE* item = (NMITEMACTIVATE*)pnmh;
        if (item->iItem != -1) {
                CRect rect;
                ctrlTransfers.GetItemRect(item->iItem, rect, LVIR_ICON);

                // if double click on state icon, ignore...
                if (item->ptAction.x < rect.left)
                        return 0;

                ItemInfo* i = ctrlTransfers.getItemData(item->iItem);
                if(/*!i->multiSource || */i->subItems.size() <= 1) {
                        switch(SETTING(TRANSFERLIST_DBLCLICK)) {
                                case 0:
                                        i->pm();
                                        break;
                                case 1:
                                        i->getList();
                                        break;
                                case 2:
                                        i->matchQueue();
                                        break;
                                case 4:
                                        i->addFav();
                                        break;
                                case 5: // !SMT!-UI
                                        i->columns[COLUMN_STATUS] = CTSTRING(CONNECTING_FORCED);
                                        ctrlTransfers.updateItem(i);
                                        ClientManager::getInstance()->connect(i->user);
                                        break;
                        }
                }
        }
        return 0;
}

int TransferView::ItemInfo::compareItems(ItemInfo* a, ItemInfo* b, int col) {
        if(a->status == b->status) {
                if(a->download != b->download) {
                        return a->download ? -1 : 1;
                }
        } else {
                return (a->status == ItemInfo::STATUS_RUNNING) ? -1 : 1;
        }

        switch(col) {
                case COLUMN_USER:
                        {
                                if(a->subItems.size() == b->subItems.size())
                                        return lstrcmpi(a->columns[COLUMN_USER].c_str(), b->columns[COLUMN_USER].c_str());
                                return compare(a->subItems.size(), b->subItems.size());
                        }
                case COLUMN_STATUS: return 0;
                case COLUMN_TIMELEFT: return compare(a->timeLeft, b->timeLeft);
                case COLUMN_SPEED: return compare(a->speed, b->speed);
                case COLUMN_SIZE: return compare(a->size, b->size);
                case COLUMN_RATIO: return compare(a->getRatio(), b->getRatio());
                default: return lstrcmpi(a->columns[col].c_str(), b->columns[col].c_str());
        }
}

LRESULT TransferView::onSpeaker(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/) {
        TaskQueue::List t;
        tasks.get(t);
        if(t.size() > 2) {
                ctrlTransfers.SetRedraw(FALSE);
        }

        for(TaskQueue::Iter i = t.begin(); i != t.end(); ++i) {
                if(i->first == ADD_ITEM) {
                        auto_ptr<UpdateInfo> ui(reinterpret_cast<UpdateInfo*>(i->second));
                        ItemInfo* ii = new ItemInfo(ui->user, ui->download);
                        transferItems.push_back(ii);
                        ii->update(*ui);
                        if(ui->download) {
                                ctrlTransfers.insertGroupedItem(ii, false, ui->fileList);
                                ii->main->multiSource = ii->multiSource;
                        } else {
                                ctrlTransfers.insertItem(ii, IMAGE_UPLOAD);
                        }
                } else if(i->first == REMOVE_ITEM) {
                        auto_ptr<UpdateInfo> ui(reinterpret_cast<UpdateInfo*>(i->second));
                        for(ItemInfo::List::iterator i = transferItems.begin(); i != transferItems.end(); ++i) {
                                ItemInfo* ii = *i;
                                if(*ui == *ii) {
                                        transferItems.erase(i);
                                        if(ui->download) {
                                                ctrlTransfers.removeGroupedItem(ii);
                                        } else {
                                                ctrlTransfers.deleteItem(ii);
                                                delete ii;
                                        }
                                        break;
                                }
                        }
                } else if(i->first == UPDATE_ITEM) {
                        auto_ptr<UpdateInfo> ui(reinterpret_cast<UpdateInfo*>(i->second));
                        for(ItemInfo::Iter i = transferItems.begin(); i != transferItems.end(); ++i) {
                                ItemInfo* ii = *i;
                                if(*ui == *ii) {
                                        if(ui->download) {
                                                ii->update(*ui);
                                                if(ii->main) {
                                                        if(ui->updateMask && UpdateInfo::MASK_FILE) setMainItem(ii);
                                                        ItemInfo* main = ii->main;

                                                        main->multiSource = ii->multiSource;
                                                        bool defString = false;

                                                        switch(ii->status) {
                                                                case ItemInfo::DOWNLOAD_STARTING:
                                                                        ii->status = ItemInfo::STATUS_RUNNING;
                                                                        if(main->status != ItemInfo::STATUS_RUNNING) {
                                                                                main->fileBegin = GET_TICK();

                                                                                if ((!SETTING(BEGINFILE).empty()) && (!BOOLSETTING(SOUNDS_DISABLED)))
                                                                                    PlaySound(Text::toT(Util::getThemeFile(SETTING(BEGINFILE))).c_str(), NULL, SND_FILENAME | SND_ASYNC);

                                                                                if(BOOLSETTING(POPUP_DOWNLOAD_START)) {
                                                                                        MainFrame::getMainFrame()->ShowBalloonTip((
                                                                                                TSTRING(FILE) + _T(": ")+ Util::getFileName(ii->Target) + _T("\n")+
                                                                                                TSTRING(USER) + _T(": ") + Text::toT(ii->user->getFirstNick())).c_str(), CTSTRING(DOWNLOAD_STARTING));
                                                                                }
                                                                                main->start = 0;
                                                                                main->actual = 0;
                                                                                main->pos = 0;
                                                                                main->status = ItemInfo::DOWNLOAD_STARTING;
                                                                        }
                                                                        break;
                                                                case ItemInfo::DOWNLOAD_FINISHED:
                                                                        ii->status = ItemInfo::STATUS_WAITING;
                                                                        main->status = ItemInfo::STATUS_WAITING;
                                                                        main->columns[COLUMN_STATUS] = TSTRING(DOWNLOAD_FINISHED_IDLE);
                                                                        break;
                                                                default:
                                                                        defString = true;
                                                        }
                                                        if(mainItemTick(main, ii->status != ItemInfo::STATUS_RUNNING) && defString && ui->transferFailed)
                                                                main->columns[COLUMN_STATUS] = ii->columns[COLUMN_STATUS];

                                                        if(!main->collapsed)
                                                                ctrlTransfers.updateItem(ii);
                                                        ctrlTransfers.updateItem(main);
                                                }
                                        } else {
                                                ii->update(*ui);
                                                ctrlTransfers.updateItem(ii);
                                        }
                                        break;
                                }
                        }
                } /*else if(wParam == UPDATE_ITEMS) {
                        auto_ptr<vector<UpdateInfo*> > v(reinterpret_cast<vector<UpdateInfo*>* >(lParam));
                        set<ItemInfo*> mainItems;
                        ctrlTransfers.SetRedraw(FALSE);
                        for(ItemInfo::Iter i = transferItems.begin(); i != transferItems.end(); ++i) {
                                ItemInfo* ii = *i;
                                for(vector<UpdateInfo*>::const_iterator j = v->begin(); j != v->end(); ++j) {
                                        UpdateInfo* ui = *j;
                                        if(*ui == *ii) {
                                                if(ii->main)
                                                        mainItems.insert(ii->main);
                                                ii->update(*ui);
                                                ctrlTransfers.updateItem(ii);
                                        }
                                }
                        }

                        for(set<ItemInfo*>::const_iterator i = mainItems.begin(); i != mainItems.end(); ++i) {
                                ItemInfo* main = *i;
                                mainItemTick(main, false);
                                ctrlTransfers.updateItem(main);
                        }

                        if(ctrlTransfers.getSortColumn() != COLUMN_STATUS)
                                ctrlTransfers.resort();
                        ctrlTransfers.SetRedraw(TRUE);

                        for_each(v->begin(), v->end(), DeleteFunction());
                }*/
        }

        if(!t.empty()) {
                ctrlTransfers.resort();
                if(t.size() > 2) {
                        ctrlTransfers.SetRedraw(TRUE);
                }
        }

        return 0;
}

LRESULT TransferView::onSearchAlternates(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
        int i = -1;
        while((i = ctrlTransfers.GetNextItem(i, LVNI_SELECTED)) != -1) {
                ItemInfo *ii = ctrlTransfers.getItemData(i);

                string target = Text::fromT(ii->getText(COLUMN_PATH) + ii->getText(COLUMN_FILE));

                TTHValue tth;
                if(QueueManager::getInstance()->getTTH(Text::fromT(ii->Target), tth)) {
                        WinUtil::searchHash(tth);
                }
        }

        return 0;
}

TransferView::ItemInfo::ItemInfo(const User::Ptr& u, bool aDownload) : UserInfoBase(u), download(aDownload), transferFailed(false),
        status(STATUS_WAITING), pos(0), size(0), start(0), actual(0), speed(0), timeLeft(0),
        Target(Util::emptyStringT), flagImage(0), collapsed(true), main(NULL)
{
        columns[COLUMN_USER] = Text::toT(u->getFirstNick());
        columns[COLUMN_HUB] = WinUtil::getHubNames(u).first;
        // !SMT!-UI
        SimpleIdentity id;
        if (ClientManager::getInstance()->getIdentity(user, id)) {
                columns[COLUMN_SHARE] = Util::formatBytesW(id.share);
                columns[COLUMN_SLOTS] = Util::toStringW(id.slots);
        }
}

void TransferView::ItemInfo::update(const UpdateInfo& ui) {
        if(ui.updateMask & UpdateInfo::MASK_STATUS) {
                status = ui.status;
        }
        if(ui.updateMask & UpdateInfo::MASK_STATUS_STRING) {
                // No slots etc from transfermanager better than disconnected from connectionmanager
                if(!transferFailed)
                        columns[COLUMN_STATUS] = ui.statusString;
                transferFailed = ui.transferFailed;
        }
        if(ui.updateMask & UpdateInfo::MASK_SIZE) {
                size = ui.size;
                columns[COLUMN_SIZE] = Util::formatBytesW(size);
        }
        if(ui.updateMask & UpdateInfo::MASK_START) {
                start = ui.start;
        }
        if(ui.updateMask & UpdateInfo::MASK_POS) {
                pos = start + ui.pos;
        }
        if(ui.updateMask & UpdateInfo::MASK_ACTUAL) {
                actual = start + ui.actual;
                columns[COLUMN_RATIO] = Util::toStringW(getRatio());
        }
        if(ui.updateMask & UpdateInfo::MASK_SPEED) {
                speed = ui.speed;
                if (status == STATUS_RUNNING) {
                        columns[COLUMN_SPEED] =Util::formatBytesW(speed) + _T("/s");
                } else {
                        columns[COLUMN_SPEED] = Util::emptyStringT;
                }
        }
        if(ui.updateMask & UpdateInfo::MASK_FILE) {
                if(ui.download) Target = ui.target;
                columns[COLUMN_FILE] = ui.file;
                columns[COLUMN_PATH] = ui.path;
        }
        if(ui.updateMask & UpdateInfo::MASK_TIMELEFT) {
                timeLeft = ui.timeLeft;
                if (status == STATUS_RUNNING) {
                        columns[COLUMN_TIMELEFT] = Util::formatSeconds(timeLeft);
                } else {
                        columns[COLUMN_TIMELEFT] = Util::emptyStringT;
                }
        }
        if(ui.updateMask & UpdateInfo::MASK_IP) {
                flagImage = ui.flagImage;
                columns[COLUMN_IP] = ui.IP;
                columns[COLUMN_LOCATION] = ui.location; // !SMT!-IP
                columns[COLUMN_DNS] = ui.dns; // !SMT!-IP
                if(main && main->subItems.size() == 1) {
                        main->flagImage = flagImage;
                        main->columns[COLUMN_IP] = ui.IP;
                        main->columns[COLUMN_DNS] = ui.dns; // !SMT!-IP
                        main->columns[COLUMN_LOCATION] = ui.location; // !SMT!-IP
                }
        }
        if(ui.updateMask & UpdateInfo::MASK_SEGMENT) {
                multiSource = ui.multiSource;
        }
}

void TransferView::on(ConnectionManagerListener::Added, ConnectionQueueItem* aCqi) {
        UpdateInfo* ui = new UpdateInfo(aCqi->getUser(), aCqi->getDownload());

        if(ui->download) {
                string aTarget; int64_t aSize; int aFlags; bool segmented;
                if(QueueManager::getInstance()->getQueueInfo(aCqi->getUser(), aTarget, aSize, aFlags, ui->fileList, segmented)) {
                        ui->setMultiSource(segmented);
                        ui->setFile(Text::toT(aTarget));
                        ui->setSize(aSize);
                }
        }

        ui->setStatus(ItemInfo::STATUS_WAITING);
        ui->setStatusString(TSTRING(CONNECTING));

        speak(ADD_ITEM, ui);
}

void TransferView::on(ConnectionManagerListener::StatusChanged, ConnectionQueueItem* aCqi) {
        UpdateInfo* ui = new UpdateInfo(aCqi->getUser(), aCqi->getDownload());
        string aTarget; int64_t aSize; int aFlags = 0; bool segmented;

        if(QueueManager::getInstance()->getQueueInfo(aCqi->getUser(), aTarget, aSize, aFlags, ui->fileList, segmented)) {
                ui->setMultiSource(segmented);
                ui->setFile(Text::toT(aTarget));
                ui->setSize(aSize);
        }

        ui->setStatusString((aFlags & QueueItem::FLAG_TESTSUR) ? TSTRING(CHECKING_CLIENT) : TSTRING(CONNECTING));
        ui->setStatus(ItemInfo::STATUS_WAITING);

        speak(UPDATE_ITEM, ui);
}

void TransferView::on(ConnectionManagerListener::Removed, ConnectionQueueItem* aCqi) {
        speak(REMOVE_ITEM, new UpdateInfo(aCqi->getUser(), aCqi->getDownload()));
}

void TransferView::on(ConnectionManagerListener::Failed, ConnectionQueueItem* aCqi, const string& aReason) {
        UpdateInfo* ui = new UpdateInfo(aCqi->getUser(), aCqi->getDownload());
        if(aCqi->getUser()->isSet(User::OLD_CLIENT)) {
                ui->setStatusString(TSTRING(SOURCE_TOO_OLD));
        } else if(aCqi->getUser()->isSet(User::PG_BLOCK)) {
                ui->setStatusString(TSTRING(PG_BLOCKED));
        } else {
                ui->setStatusString(Text::toT(aReason));
        }
        ui->setStatus(ItemInfo::STATUS_WAITING);
        speak(UPDATE_ITEM, ui);
}

// !SMT!-IP
void TransferView::UpdateInfo::setIP(const string& aIP)
{
    ip = aIP;
    dns = Text::toT(Socket::nslookup(aIP));
    IP = Text::toT(aIP);
    string country = Util::getIpCountry(aIP);
    if (!country.empty())
        flagImage = WinUtil::getFlagImage(country);
    location = Text::toT(country);
    updateMask |= MASK_IP;
}


void TransferView::on(DownloadManagerListener::Starting, Download* aDownload) {
        UpdateInfo* ui = new UpdateInfo(aDownload->getUser(), true);
        bool chunkInfo = aDownload->isSet(Download::FLAG_MULTI_CHUNK) && !aDownload->isSet(Download::FLAG_TREE_DOWNLOAD);

        ui->setStatus(ItemInfo::DOWNLOAD_STARTING);
        ui->setPos(chunkInfo ? 0 : aDownload->getTotal());
        ui->setActual(chunkInfo ? 0 : aDownload->getActual());
        ui->setStart(chunkInfo ? 0 : aDownload->getPos());
        ui->setSize(chunkInfo ? aDownload->getChunkSize() : aDownload->getSize());
        ui->setFile(Text::toT(aDownload->getTarget()));
        ui->setStatusString(TSTRING(DOWNLOAD_STARTING));
        ui->setMultiSource(aDownload->isSet(Download::FLAG_MULTI_CHUNK));
        ui->setIP(aDownload->getUserConnection().getRemoteIp()); // !SMT!-IP
        if(aDownload->isSet(Download::FLAG_TREE_DOWNLOAD)) {
                ui->file = _T("TTH: ") + ui->file;
                ui->setStatus(ItemInfo::TREE_DOWNLOAD);
        }

        speak(UPDATE_ITEM, ui);
}

void TransferView::on(DownloadManagerListener::Tick, const Download::List& dl) {
        AutoArray<TCHAR> buf(TSTRING(DOWNLOADED_BYTES).size() + 64);

        for(Download::List::const_iterator j = dl.begin(); j != dl.end(); ++j) {
                Download* d = *j;

                UpdateInfo* ui = new UpdateInfo(d->getUser(), true);
                ui->setActual(d->getActual());
                ui->setPos(d->getTotal());
                ui->setTimeLeft(d->getSecondsLeft());
                ui->setSpeed(d->getRunningAverage());

                if (!ui->ip.empty() && ui->dns.empty()) // !SMT!-IP (delayed DNS updates)
                    ui->setIP(ui->ip);

                if(d->isSet(Download::FLAG_MULTI_CHUNK) && !d->isSet(Download::FLAG_TREE_DOWNLOAD)) {
                        ui->setSize(d->isSet(Download::FLAG_MULTI_CHUNK) ? d->getChunkSize() : d->getSize());
                        ui->timeLeft = (ui->speed > 0) ? ((ui->size - d->getTotal()) / ui->speed) : 0;

                        double progress = (double)(d->getTotal())*100.0/(double)ui->size;
                        _stprintf(buf, CTSTRING(DOWNLOADED_BYTES), Util::formatBytesW(d->getTotal()).c_str(),
                                progress, Util::formatSeconds((GET_TICK() - d->getStart())/1000).c_str());
                        if(progress > 100) {
                                // workaround to fix > 100% percentage
                                d->getUserConnection().disconnect();
                                continue;
                        }
                } else {
                        _stprintf(buf, CTSTRING(DOWNLOADED_BYTES), Util::formatBytesW(d->getPos()).c_str(),
                                (double)d->getPos()*100.0/(double)d->getSize(), Util::formatSeconds((GET_TICK() - d->getStart())/1000).c_str());
                }

                tstring statusString;

                if(d->isSet(Download::FLAG_PARTIAL)) {
                        statusString += _T("[P]");
                }
                if(d->getUserConnection().isSecure()) {
                        if(d->getUserConnection().isTrusted()) {
                                statusString += _T("[S]");
                        } else {
                                statusString += _T("[U]");
                        }
                }
                if(d->isSet(Download::FLAG_TTH_CHECK)) {
                        statusString += _T("[T]");
                }
                if(d->isSet(Download::FLAG_ZDOWNLOAD)) {
                        statusString += _T("[Z]");
                }
                if(d->isSet(Download::FLAG_CHUNKED)) {
                        statusString += _T("[C]");
                }
                if(d->isSet(Download::FLAG_ROLLBACK)) {
                        statusString += _T("[R]");
                }
                if(!statusString.empty()) {
                        statusString += _T(" ");
                }
                statusString += buf;
                ui->setStatusString(statusString);
                if((d->getRunningAverage() == 0) && ((GET_TICK() - d->getStart()) > 1000)) {
                        d->getUserConnection().disconnect();
                }

                tasks.add(UPDATE_ITEM, ui);
        }

        PostMessage(WM_SPEAKER);
}

void TransferView::on(DownloadManagerListener::Failed, Download* aDownload, const string& aReason) {
        UpdateInfo* ui = new UpdateInfo(aDownload->getUser(), true, true);
        ui->setStatus(ItemInfo::STATUS_WAITING);
        ui->setPos(0);
        ui->setStatusString(Text::toT(aReason));
        ui->setSize(aDownload->getSize());
        ui->setFile(Text::toT(aDownload->getTarget()));
        ui->setIP(aDownload->getUserConnection().getRemoteIp()); // !SMT!-IP
        if(BOOLSETTING(POPUP_DOWNLOAD_FAILED)) {
                MainFrame::getMainFrame()->ShowBalloonTip((
                        TSTRING(FILE)+_T(": ") + ui->file + _T("\n")+
                        TSTRING(USER)+_T(": ") + Text::toT(ui->user->getFirstNick()) + _T("\n")+
                        TSTRING(REASON)+_T(": ") + Text::toT(aReason)).c_str(), CTSTRING(DOWNLOAD_FAILED), NIIF_WARNING);
        }
        if(aDownload->isSet(Download::FLAG_TREE_DOWNLOAD)) {
                ui->file = _T("TTH: ") + ui->file;
        }

        speak(UPDATE_ITEM, ui);
}

void TransferView::on(DownloadManagerListener::Status, const User::Ptr& aUser, const string& aMessage) {
        {
                //Lock l(cs);
                for(ItemInfo::List::iterator i = transferItems.begin(); i != transferItems.end(); ++i) {
                        ItemInfo* ii = *i;
                        if(ii->download && (ii->user == aUser)) {
                                if(ii->status == ItemInfo::STATUS_WAITING) return;
                                break;
                        }
                }
        }
        UpdateInfo* ui = new UpdateInfo(aUser, true, aMessage != STRING(WAITING_TO_RETRY));
        ui->setStatus(ItemInfo::STATUS_WAITING);
        ui->setStatusString(Text::toT(aMessage));
        speak(UPDATE_ITEM, ui);
}

void TransferView::on(UploadManagerListener::Starting, Upload* aUpload) {
        UpdateInfo* ui = new UpdateInfo(aUpload->getUser(), false);

        ui->setStatus(ItemInfo::STATUS_RUNNING);
        ui->setPos(aUpload->getTotal());
        ui->setActual(aUpload->getActual());
        ui->setStart(aUpload->getPos());
        ui->setSize(aUpload->isSet(Upload::FLAG_TTH_LEAVES) ? aUpload->getSize() : aUpload->getFileSize());
        ui->setFile(Text::toT(aUpload->getSourceFile()));

        if(!aUpload->isSet(Upload::FLAG_RESUMED)) {
                ui->setStatusString(TSTRING(UPLOAD_STARTING));
        }

        ui->setIP(aUpload->getUserConnection().getRemoteIp()); // !SMT!-IP
        if(aUpload->isSet(Upload::FLAG_TTH_LEAVES)) {
                ui->file = _T("TTH: ") + ui->file;
        }

        speak(UPDATE_ITEM, ui);
}

void TransferView::on(UploadManagerListener::Tick, const Upload::List& ul) {
        AutoArray<TCHAR> buf(TSTRING(UPLOADED_BYTES).size() + 64);

        for(Upload::List::const_iterator j = ul.begin(); j != ul.end(); ++j) {
                Upload* u = *j;

                if (u->getTotal() == 0) continue;

                UpdateInfo* ui = new UpdateInfo(u->getUser(), false);
                ui->setActual(u->getActual());
                ui->setPos(u->getTotal());
                ui->setTimeLeft(u->getSecondsLeft(true)); // we are interested when whole file is finished and not only one chunk
                ui->setSpeed(u->getRunningAverage());

                _stprintf(buf, CTSTRING(UPLOADED_BYTES), Util::formatBytesW(u->getPos()).c_str(),
                        (double)u->getPos()*100.0/(double)(u->isSet(Upload::FLAG_TTH_LEAVES) ? u->getSize() : u->getFileSize()), Util::formatSeconds((GET_TICK() - u->getStart())/1000).c_str());

                tstring statusString;

                if(u->isSet(Upload::FLAG_PARTIAL_SHARE)) {
                        statusString += _T("[P]");
                }
                if(u->isSet(Upload::FLAG_CHUNKED)) {
                        statusString += _T("[C]");
                }
                if(u->getUserConnection().isSecure()) {
                        if(u->getUserConnection().isTrusted()) {
                                statusString += _T("[S]");
                        } else {
                                statusString += _T("[U]");
                        }
                }
                if(u->isSet(Upload::FLAG_ZUPLOAD)) {
                        statusString += _T("[Z]");
                }
                if(!statusString.empty()) {
                        statusString += _T(" ");
                }
                statusString += buf;

                ui->setStatusString(statusString);

                tasks.add(UPDATE_ITEM, ui);
                if((u->getRunningAverage() == 0) && ((GET_TICK() - u->getStart()) > 1000)) {
                        u->getUserConnection().disconnect(true);
                }
        }

        PostMessage(WM_SPEAKER);
}

void TransferView::onTransferComplete(Transfer* aTransfer, bool isUpload, const string& aFileName, bool isTree) {
        UpdateInfo* ui = new UpdateInfo(aTransfer->getUser(), !isUpload);
        if(!isUpload) {
                ui->setStatus(isTree ? ItemInfo::STATUS_WAITING : ItemInfo::DOWNLOAD_FINISHED);
                if(BOOLSETTING(POPUP_DOWNLOAD_FINISHED) && !isTree) {
                        MainFrame::getMainFrame()->ShowBalloonTip((
                                TSTRING(FILE) + _T(": ") + Text::toT(aFileName) + _T("\n")+
                                TSTRING(USER) + _T(": ") + Text::toT(aTransfer->getUser()->getFirstNick())).c_str(), CTSTRING(DOWNLOAD_FINISHED_IDLE));
                }
        } else {
                ui->setStatus(ItemInfo::STATUS_WAITING);
                if(BOOLSETTING(POPUP_UPLOAD_FINISHED) && !isTree) {
                        MainFrame::getMainFrame()->ShowBalloonTip((
                                TSTRING(FILE) + _T(": ") + Text::toT(aFileName) + _T("\n")+
                                TSTRING(USER) + _T(": ") + Text::toT(aTransfer->getUser()->getFirstNick())).c_str(), CTSTRING(UPLOAD_FINISHED_IDLE));
                }
        }

        ui->setPos(0);
        ui->setStatusString(isUpload ? TSTRING(UPLOAD_FINISHED_IDLE) : TSTRING(DOWNLOAD_FINISHED_IDLE));

        speak(UPDATE_ITEM, ui);
}

void TransferView::ItemInfo::disconnect() {
        ConnectionManager::getInstance()->disconnect(user, download);
}

LRESULT TransferView::onPreviewCommand(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/){
        int i = -1;
        while((i = ctrlTransfers.GetNextItem(i, LVNI_SELECTED)) != -1) {
                ItemInfo *ii = ctrlTransfers.getItemData(i);

                QueueItem::StringMap& queue = QueueManager::getInstance()->lockQueue();

                string tmp = Text::fromT(ii->Target);
                QueueItem::StringIter qi = queue.find(&tmp);

                string aTempTarget;
                if(qi != queue.end())
                        aTempTarget = qi->second->getTempTarget();

                QueueManager::getInstance()->unlockQueue();

                WinUtil::RunPreviewCommand(wID - IDC_PREVIEW_APP, aTempTarget);
        }

        return 0;
}

void TransferView::CollapseAll() {
        for(int q = ctrlTransfers.GetItemCount()-1; q != -1; --q) {
                ItemInfo* m = (ItemInfo*)ctrlTransfers.getItemData(q);
                if(m->download && m->main) {
                        ctrlTransfers.deleteItem(m);
                }
                if(m->download && !m->main) {
                        m->collapsed = true;
                        ctrlTransfers.SetItemState(ctrlTransfers.findItem(m), INDEXTOSTATEIMAGEMASK(1), LVIS_STATEIMAGEMASK);
                 }
        }
}

void TransferView::ExpandAll() {
        for(vector<ItemInfo*>::const_iterator i = ctrlTransfers.mainItems.begin(); i != ctrlTransfers.mainItems.end(); ++i) {
                if((*i)->collapsed) {
                        ctrlTransfers.Expand(*i, ctrlTransfers.findItem(*i));
                }
        }
}

LRESULT TransferView::onConnectAll(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
        int i = -1;
        while((i = ctrlTransfers.GetNextItem(i, LVNI_SELECTED)) != -1) {
                ItemInfo* ii = ctrlTransfers.getItemData(i);
                ctrlTransfers.SetItemText(i, COLUMN_STATUS, CTSTRING(CONNECTING_FORCED));
                for(ItemInfo::Iter j = ii->subItems.begin(); j != ii->subItems.end(); ++j) {
                        int h = ctrlTransfers.findItem(*j);
                        if(h != -1)
                                ctrlTransfers.SetItemText(h, COLUMN_STATUS, CTSTRING(CONNECTING_FORCED));
                        ClientManager::getInstance()->connect((*j)->user);
                }
        }
        return 0;
}

LRESULT TransferView::onDisconnectAll(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
        int i = -1;
        while((i = ctrlTransfers.GetNextItem(i, LVNI_SELECTED)) != -1) {
                ItemInfo* ii = ctrlTransfers.getItemData(i);
                ctrlTransfers.SetItemText(i, COLUMN_STATUS, CTSTRING(DISCONNECTED));
                for(ItemInfo::Iter j = ii->subItems.begin(); j != ii->subItems.end(); ++j) {
                        int h = ctrlTransfers.findItem(*j);
                        if(h != -1)
                                ctrlTransfers.SetItemText(h, COLUMN_STATUS, CTSTRING(DISCONNECTED));
                        (*j)->disconnect();
                }
        }
        return 0;
}

LRESULT TransferView::onSlowDisconnect(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
        int i = -1;
        while((i = ctrlTransfers.GetNextItem(i, LVNI_SELECTED)) != -1) {
                ItemInfo *ii = ctrlTransfers.getItemData(i);

                QueueItem::StringMap& queue = QueueManager::getInstance()->lockQueue();

                string tmp = Text::fromT(ii->Target);
                QueueItem::StringIter qi = queue.find(&tmp);

                if(qi != queue.end()) {
                        if(qi->second->isSet(QueueItem::FLAG_AUTODROP)) {
                                qi->second->unsetFlag(QueueItem::FLAG_AUTODROP);
                        } else {
                                qi->second->setFlag(QueueItem::FLAG_AUTODROP);
                        }
                }

                QueueManager::getInstance()->unlockQueue();
        }

        return 0;
}

void TransferView::on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw() {
        bool refresh = false;
        if(ctrlTransfers.GetBkColor() != WinUtil::bgColor) {
                ctrlTransfers.SetBkColor(WinUtil::bgColor);
                ctrlTransfers.SetTextBkColor(WinUtil::bgColor);
                ctrlTransfers.setFlickerFree(WinUtil::bgBrush);
                refresh = true;
        }
        if(ctrlTransfers.GetTextColor() != WinUtil::textColor) {
                ctrlTransfers.SetTextColor(WinUtil::textColor);
                refresh = true;
        }
        if(refresh == true) {
                RedrawWindow(NULL, NULL, RDW_ERASE | RDW_INVALIDATE | RDW_UPDATENOW | RDW_ALLCHILDREN);
        }
}

bool TransferView::mainItemTick(ItemInfo* main, bool smallUpdate) {
        size_t totalSpeed = 0;  double ratio = 0; uint8_t segs = 0;
        ItemInfo* l = NULL;

        for(ItemInfo::Iter k = main->subItems.begin(); k != main->subItems.end(); ++k) {
                l = *k;
                if(l->status == ItemInfo::STATUS_RUNNING && main->Target == l->Target) {
                        segs++;
                        if(main->multiSource) {
                                totalSpeed += (size_t)l->speed;
                                ratio += l->getRatio();
                        } else
                                break;
                }
        }

        if(!main->multiSource) {
                main->status = l->status;
                main->size = l->size;
                main->pos = l->pos;
                main->actual = l->actual;
                main->speed = l->speed;
                main->timeLeft = l->timeLeft;
                main->columns[COLUMN_HUB] = l->columns[COLUMN_HUB];
                main->columns[COLUMN_STATUS] = l->columns[COLUMN_STATUS];
                main->columns[COLUMN_SIZE] = Util::formatBytesW(l->size);
                main->columns[COLUMN_SPEED] = Util::formatBytesW(l->speed) + _T("/s");
                main->columns[COLUMN_TIMELEFT] = Util::formatSeconds(l->timeLeft);
                main->columns[COLUMN_RATIO] = Util::toStringW(l->getRatio());
                return false;
        } else if(segs == 0) {
                main->pos = 0;
                main->actual = 0;
                main->status = ItemInfo::STATUS_WAITING;
                main->fileBegin = 0;
                main->timeLeft = 0;
                main->speed = 0;
                main->columns[COLUMN_TIMELEFT] = Util::emptyStringT;
                main->columns[COLUMN_SPEED] = Util::emptyStringT;
                main->columns[COLUMN_RATIO] = Util::emptyStringT;
                if(main->multiSource && main->subItems.size() > 1)
                        main->columns[COLUMN_HUB] = _T("0 ") + TSTRING(NUMBER_OF_SEGMENTS);

                return true;
        } else {
                if(smallUpdate) return false;

                ratio = ratio / segs;

                int64_t total = 0;
                int64_t fileSize = -1;

                string tmp = Text::fromT(main->Target);
                QueueItem::StringMap& queue = QueueManager::getInstance()->lockQueue();
                QueueItem::StringIter qi = queue.find(&tmp);
                if(qi != queue.end()) {
                        total = qi->second->getDownloadedBytes();
                        fileSize = qi->second->getSize();
                        qi->second->setAverageSpeed(totalSpeed);
                }
                QueueManager::getInstance()->unlockQueue();

                if(fileSize == -1) return true;

                if(main->status == ItemInfo::DOWNLOAD_STARTING) {
                        main->status = ItemInfo::STATUS_RUNNING;
                        main->columns[COLUMN_STATUS] = TSTRING(DOWNLOAD_STARTING);
                } else {
                        AutoArray<TCHAR> buf(TSTRING(DOWNLOADED_BYTES).size() + 64);
                                _stprintf(buf, CTSTRING(DOWNLOADED_BYTES), Util::formatBytesW(total).c_str(),
                                (double)total*100.0/(double)fileSize, Util::formatSeconds((GET_TICK() - main->fileBegin)/1000).c_str());

                        tstring statusString;
                        // TODO statusString += _T("[T]");

                        // hack to display whether file is compressed
                        if(ratio < 1.0000) {
                                statusString += _T("[Z] ");
                        }
                        statusString += buf;
                        main->columns[COLUMN_STATUS] = statusString;

                        main->actual = (int64_t)(total * ratio);
                        main->pos = total;
                        main->speed = totalSpeed;
                        main->timeLeft = (totalSpeed > 0) ? ((fileSize - main->pos) / totalSpeed) : 0;
                }
                main->size = fileSize;

                if(main->subItems.size() > 1) {
                        TCHAR buf[64];
                        snwprintf(buf, sizeof(buf), _T("%d %s"), segs, CTSTRING(NUMBER_OF_SEGMENTS));

                        main->columns[COLUMN_HUB] = buf;
                }
                main->columns[COLUMN_SIZE] = Util::formatBytesW(fileSize);
                main->columns[COLUMN_SPEED] = Util::formatBytesW(main->speed) + _T("/s");
                main->columns[COLUMN_TIMELEFT] = Util::formatSeconds(main->timeLeft);
                main->columns[COLUMN_RATIO] = Util::toStringW(ratio);
                return false;
        }
}

// !SMT!-S
LRESULT TransferView::onSetUserLimit(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
   FavoriteUser::UPLOAD_LIMIT lim;
   switch (wID) {
      case IDC_SPEED_NONE:  lim = FavoriteUser::UL_NONE; break;
      case IDC_SPEED_SUPER: lim = FavoriteUser::UL_SU;   break;
      case IDC_SPEED_BAN:   lim = FavoriteUser::UL_BAN;  break;
      case IDC_SPEED_02K:   lim = FavoriteUser::UL_2;    break;
      case IDC_SPEED_05K:   lim = FavoriteUser::UL_5;    break;
      case IDC_SPEED_08K:   lim = FavoriteUser::UL_8;    break;
      case IDC_SPEED_12K:   lim = FavoriteUser::UL_12;   break;
      case IDC_SPEED_16K:   lim = FavoriteUser::UL_16;   break;
      case IDC_SPEED_24K:   lim = FavoriteUser::UL_24;   break;
      case IDC_SPEED_32K:   lim = FavoriteUser::UL_32;   break;
      case IDC_SPEED_64K:   lim = FavoriteUser::UL_64;   break;
      case IDC_SPEED_128K:  lim = FavoriteUser::UL_128;  break;
      case IDC_SPEED_256K:  lim = FavoriteUser::UL_256;  break;
      default: return 0;
   }

   FavoriteManager::getInstance()->addFavoriteUser(selectedUser);
   FavoriteManager::getInstance()->setUploadLimit(selectedUser, lim);

   // close favusers window (it contains incorrect info, too lazy to update)
   UsersFrame::closeWindow();
   return 0;
}

/**
 * @file
 * $Id: TransferView.cpp,v 1.148 2006/11/12 20:32:08 bigmuscle Exp $
 */
